import math
import numpy
import mssputil

M=1.0033548378
P=1.007276

# --------------------------------------------------------------------------
#
class duplicated_query:
	def __init__(self):
		self.mz=0
		self.rt=0
		self.peak_list=[]

		self.mz_tol=0
		self.rt_tol=0

	# --------------------------------------------------------------------------
	#
	def __cmp__(self,duplicated):
		return (numpy.abs(self.rt-duplicated.rt)<=self.rt_tol and numpy.abs(self.mz-duplicated.mz)<=self.mz_tol)

	# --------------------------------------------------------------------------
	#
	def size(self):
		return len(self.peak_list)

	# --------------------------------------------------------------------------
	#
	def get(self,index):
		peak=None
		if index<self.size():
			peak=self.peak_list[index]
		return peak

	# --------------------------------------------------------------------------
	#
	def add(self,peak):
		if peak:
			index=self.size()
			for i in range(self.size()):
				if float(mssputil.get_score(peak))>float(mssputil.get_score(self.peak_list[i])):
					index=i
					break
			self.peak_list.insert(index,peak)
#		self.peak_list.append(peak)

	# --------------------------------------------------------------------------
	#
	def remove(self,peak):
		self.peak_list.remove(peak)

# --------------------------------------------------------------------------
#
class query:
	def __init__(self,peak=None):
		self.charge=0
		self.mz=0
		self.cmz=0
		self.rt=0
		self.massref_rt=0
		self.score=0
		self.present=1
		self.value=0
		self.peak_list=[]

		if peak!=None:
			self.charge=mssputil.get_charge(peak)
			self.mz=mssputil.get_mz(peak)
			self.cmz=mssputil.m2mz(mssputil.get_cmass(peak),self.charge)
			self.rt=mssputil.get_rt(peak)
			self.massref_rt=mssputil.get_massref_rt(peak)
			self.score=mssputil.get_score(peak)
			self.present=mssputil.is_peak_present(peak)
			self.add(peak)

	# --------------------------------------------------------------------------
	#
	#def __repr__(self):
        #        return repr((self.charge,self.mz,self.cmz,self.rt,self.massref_rt,self.score,self.present,self.value,self.peak_list))
	
	# --------------------------------------------------------------------------
	#
	def size(self):
		return len(self.peak_list)

	# --------------------------------------------------------------------------
	#
	def get(self,index):
		peak=None
		if index<self.size():
			peak=self.peak_list[index]
		return peak

	# --------------------------------------------------------------------------
	#
	def is_selected(self):
		selected=0
		for peak in self.peak_list: 
			if peak.selected==1:
				selected=1
				break

		return selected

	# --------------------------------------------------------------------------
	#
	def get_selected(self):
		selected=None
		for peak in self.peak_list:
			if peak.selected==1:
				selected=peak
				break

		return selected
	# --------------------------------------------------------------------------
	#
	def set_present(self,present):
		self.present=present
		for i in range(self.size()):
			peak=self.get(i)
			if i==0:
				mssputil.set_peak_present(peak,present)
			
			if present==1:
				mssputil.set_peak_color(peak,'white')
			else:
				mssputil.set_peak_color(peak,'red')

	# --------------------------------------------------------------------------
	#
	def set_color(self,present):
		for i in range(self.size()):
			peak=self.get(i)
			
			if present==1:
				mssputil.set_peak_color(peak,'white')
			else:
				mssputil.set_peak_color(peak,'red')

	# --------------------------------------------------------------------------
	#
	def get_peaks_count(self,data_height=0):
		peaks_count=0
		for peak in self.peak_list:
			fit_rms=mssputil.get_float_fit_residual(peak)
			if fit_rms>=0:
				height=mssputil.get_peak_height(peak,data_height)
				if height>0:
					peaks_count=peaks_count+1
			
		return peaks_count

	# --------------------------------------------------------------------------
	#
	def get_value(self,data_height=0,only_monoisotopic=1):
		if only_monoisotopic:
			peaks_count=1
		else:
			peaks_count=self.size()

		height=0
		for i in range(peaks_count):
			current_height=mssputil.get_peak_height(self.get(i),data_height)
			if current_height<0:
				height=-1
				break;

			height=height+current_height
			
		return height

	# --------------------------------------------------------------------------
	#
	def get_fit_rms(self,only_monoisotopic=1):
		if only_monoisotopic:
			peaks_count=1
		else:
			peaks_count=self.size()

		fit_rms=0
		for i in range(peaks_count):
			current_fit_rms=mssputil.get_float_fit_residual(self.get(i))
			if current_fit_rms<0:
				fit_rms=-1
				break;
			fit_rms=fit_rms+current_fit_rms
			
		return fit_rms

	# --------------------------------------------------------------------------
	#
	def set_position(self,rt_pos,mz_pos):
		
		peak=self.get(0)
		peak.set_position(rt_pos,mz_pos)

		peaks_count=len(self.peak_list)
		if peaks_count>1:
			for i in range(1,peaks_count):
				current_peak=self.peak_list[i]
				current_peak.set_position(rt_pos,current_peak.frequency[1])

	# ---------------------------------------------------------------------------
	#
	def restore_position(self):

		peak=self.get(0)

		rt_pos=-1*self.massref_rt
		if peak.spectrum.domain==mssputil.DOMAIN_FREQ:
			mz_pos=mssputil.mz2freq_s(self.cmz,peak.spectrum)
		else:
			mz_pos=-1*self.cmz
	
		peak.set_position(rt_pos,mz_pos)

		peaks_count=len(self.peak_list)
		if peaks_count>1:
			for i in range(1,peaks_count):
				current_peak=self.peak_list[i]
				current_peak.set_position(rt_pos,current_peak.frequency[1])

	# --------------------------------------------------------------------------
	#
	def add(self,peak):
		self.peak_list.append(peak)

# --------------------------------------------------------------------------
#
class peptide:

	def __init__(self,sequence,peak=None):
		self.sequence=sequence
		self.mass=0
		self.isodistrib=None
		self.proteins=''
		self.query_list=[]

		if peak!=None:
			self.isodistrib=mssputil.get_peak_isodistrib(peak)
			self.proteins=mssputil.get_protein(peak)
			self.mass=mssputil.get_cmass(peak)

	# --------------------------------------------------------------------------
	#
	#def __repr__(self):
        #        return repr((self.sequence,self.mass,self.isodistrib,self.proteins,self.query_list))

	# --------------------------------------------------------------------------
	#
	def __str__(self):
		return self.sequence

	# --------------------------------------------------------------------------
	#
	def __eq__(self,pep):
		if isinstance(pep,peptide):
			return eq(self.sequence,peptide.sequence)
		return False

	# --------------------------------------------------------------------------
	#
	def __cmp__(self,pep):
		if isinstance(pep,peptide):
			return cmp(self.sequence,peptide.sequence)
		return -1

	# --------------------------------------------------------------------------
	#
	def __len__(self):
		return len(self.query_list)
	
	# --------------------------------------------------------------------------
	#
	def size(self):
		return len(self.query_list)

	# --------------------------------------------------------------------------
	#
	def get_sequence(self,with_charge=0,query_nr=0):
		if with_charge==1:
			query=self.get(query_nr)
			if query!=None:
				return (self.sequence + ' (+' + ('%d' % query.charge) + ')')

		return self.sequence
	# --------------------------------------------------------------------------
	#
	def get(self,index):
		query=None
		if index<self.size():
			query=self.query_list[index]
		return query

	# --------------------------------------------------------------------------
	#
	def get_charge_index(self,charge):
		index=-1
		for i in range(0,len(self)):
			query=self.get(i)
			if query.charge==charge:
				index=i
				break
		return index

	# --------------------------------------------------------------------------
	#
	def get_charge(self,charge):
		query=None
		index=self.get_charge_index(charge)
		if index!=-1:
			query=self.query_list[index]
		return query

	# --------------------------------------------------------------------------
	#
	#wywalic porownanie score (dodac charge?)
	def add(self,query):
		if query:
			index=self.size()
			for i in range(self.size()):
				if query.score>self.query_list[i].score:
					index=i
					break
			self.query_list.insert(index,query)

	# --------------------------------------------------------------------------
	#
	def print_peptide(self):
		print self.sequence
		for i in range(self.size()):
			print self.get(i).score

# --------------------------------------------------------------------------
#
class protein:

	def __init__(self,id):
		self.name=''
		self.id=id
		self.value=0
		self.value_sd=0
		self.peptide_list=[]

	# --------------------------------------------------------------------------
	#
	def __str__(self):
		return self.id

	# --------------------------------------------------------------------------
	#
	def __cmp__(self,protein):
		return cmp(self.id,protein.id)

	# --------------------------------------------------------------------------
	#
	def size(self):
		return len(self.peptide_list)

	# --------------------------------------------------------------------------
	#
	def get(self,index):
		peptide=None
		if index<self.size():
			peptide=self.peptide_list[index]
		return peptide

	# --------------------------------------------------------------------------
	#
	def get_peptide(self,peptide):
		if peptide:
			try:
				index=self.peptide_list.index(peptide)
				peptide=self.peptide_list[index]
					
			except ValueError:
				peptide=None
		else:
			peptide=None

		return peptide

	# --------------------------------------------------------------------------
	#
	def add(self,peptide):
		if peptide:
			try:
				index=self.peptide_list.index(peptide)
				for query in peptide.query_list:
					self.peptide_list[index].add(query)
					
			except ValueError:
				self.peptide_list.append(peptide)

	# --------------------------------------------------------------------------
	#
	def print_protein(self):
		print self.id
		for i in range(self.size()):
			self.get(i).print_peptide()


# --------------------------------------------------------------------------
#
class sample:

	def __init__(self,name):
		self.spectrum=None
		self.name=name
		self.protein_list=[]

	# --------------------------------------------------------------------------
	#
	def __str__(self):
		return self.name

	# --------------------------------------------------------------------------
	#
	def __cmp__(self,sample):
		return cmp(self.name,sample.name)

	# --------------------------------------------------------------------------
	#
	def size(self):
		return len(self.protein_list)

	# --------------------------------------------------------------------------
	#
	def get(self,index):
		protein=None
		if index<self.size():
			protein=self.protein_list[index]
		return protein

	# --------------------------------------------------------------------------
	#
	def get_protein(self,protein):
		if protein:
			try:
				index=self.protein_list.index(protein)
				protein=self.protein_list[index]
					
			except ValueError:
				protein=None
		else:
			protein=None

		return protein

	# --------------------------------------------------------------------------
	#
	def add(self,protein):
		if protein:
			try:
				index=self.protein_list.index(protein)
				for peptide in protein.peptide_list:
					self.protein_list[index].add(peptide)
					
			except ValueError:
				index=self.size()
				self.protein_list.append(protein)
		
		return self.protein_list[index]

	# --------------------------------------------------------------------------
	#
	def print_sample(self):
		print self.name
		for i in range(self.size()):
			self.get(i).print_protein()

