import numpy

import mssputil
import msmath
import msfilter

SORT_STR=('RT','VALUE','RT_DELTA')
SORT_RT=0
SORT_VALUE=1
SORT_RT_DELTA=2

PENALITY_STR=('NONE','GAUSS','SQUARE')
PENALITY_NONE=0
PENALITY_GAUSS=1
PENALITY_SQUARE=2

# --------------------------------------------------------------------------
#
def get_penality_type(name):
	penality_type=-1
	for i in range(0,len(PENALITY_STR)):
		if name.lower()==PENALITY_STR[i].lower():
			penality_type=i
			break
	return penality_type

# --------------------------------------------------------------------------
#
def get_sort_type(name):
	sort_type=-1
	for i in range(0,len(SORT_STR)):
		if name.lower()==SORT_STR[i].lower():
			sort_type=i
			break
	return sort_type

# ----------------------------------------------------------------------------
#
class MSChromatogram:
	
	def __init__(self):

		self.mz=0
		self.peaks_mz=[]
		self.peaks_probs=[]

		self.pekas_count=0
		self.points_count=0
		self.expanded=0

		self.rt=[]
		self.data=[]

		self.rt_cost=None
		self.rt_cost_expanded=0
		
		self.max_data=0
		self.max_rt=0

		self.deriv_data=[]
		self.deriv2_data=[]

		self.peaks=[]

		self.rt_visible_range=(0,0)
		self.visible_max_data=0

	# ----------------------------------------------------------------------------
	#
	def rt2index(self,rt):
		index=-1

		for i in range(0,self.points_count):
			if self.rt[i]<rt:
				if rt-self.rt[i-1]>self.rt[i]-rt:
					index=i-1
				else:
					index=i
				break
		return index

	# ----------------------------------------------------------------------------
	#
	def index2rt(self,index):
		
		return self.rt[index]

	# ----------------------------------------------------------------------------
	#
	def write(self,filename):
		file=open(filename,'w')

		for i in range(0,self.points_count):
			if i>=self.rt_visible_range[0] and i<=self.rt_visible_range[1]:
				line=('%0.2f' % (-1*self.rt[i]))
				line=line+'\t'
				line=line + ('%0.4f' % self.data[i])
				file.write(line+'\n')
		file.close()

	
# --------------------------------------------------------------------------
#
def get_chromatogram(spectrum,isodistrib,charge,mz_correction=0,mz_margin=[20,20],peaks_count=2,data_threshold=100,require_max=False,penality_fun=PENALITY_NONE,mean_rt=0,fwmh_rt=5,rt=None,expand_penality=(1,1)):

	msg=''
	
	#tworzenie chromatogramu
	chromat=MSChromatogram()

	chromat.charge=charge
	if peaks_count==0:
		chromat.peaks_count=isodistrib.peaks_count
	else:
		chromat.peaks_count=min(peaks_count,isodistrib.peaks_count)

	#prawdopodobienstwa pikow
	chromat.peaks_prob=isodistrib.probs
	peaks_index=numpy.argsort(chromat.peaks_prob)
	peaks_index=peaks_index[::-1]
	chromat.peaks_prob=chromat.peaks_prob[peaks_index]/chromat.peaks_prob[peaks_index[0]]

	#polozenia pikow
	chromat.peaks_mz=isodistrib.get_peaks_mz(chromat.charge)+mz_correction
	chromat.mz=chromat.peaks_mz[0]
	chromat.peaks_mz=chromat.peaks_mz[peaks_index]

	#os czasu retencji - przerzucic tutaj pobieranie danych
	chromat.rt=rt
	if chromat.rt==None:
		chromat.rt=mssputil.get_rt_in_range(spectrum,native=True)
	chromat.points_count=len(chromat.rt)

	if chromat.points_count==0:
		return (None,'Zero points');

	#tworzenie chromatogramow pikow
#	get_chromat_data_points(chromat,spectrum,mz_margin,require_max,data_threshold)
	chromat.data=numpy.zeros(chromat.points_count)
	for i in range(0,chromat.points_count):
		chromat.data[i]=get_chromat_data_point(chromat,chromat.rt[i],spectrum,mz_margin,require_max,data_threshold)

	#dodawanie puktow jezeli niezerowe wartosci na skrajach chromatogramu
	if (chromat.data[0]>0 or chromat.data[-1]>0):
		expand_chromat_range(chromat,spectrum,mz_margin,require_max,data_threshold)

	#funkcja kary RT
	chromat.rt_cost=get_rt_cost(chromat,penality_fun,mean_rt,fwmh_rt)
	if chromat.rt_cost!=None:
		if expand_penality[0]!=0 or expand_penality[1]!=0:
			msg=expand_rt_cost(chromat,expand_penality)
		chromat.data=chromat.data*chromat.rt_cost

	if chromat.data==None or len(chromat.data)==0:
		return (None,'Zero points after RT cost function')

	chromat.max_data=chromat.data.max()	#przyspieszenie => najpierw argmax, a na jego podstawie max (w sumie po co ten chromat,max_rt????)
	chromat.max_rt=numpy.argmax(chromat.data)

	chromat.rt_visible_range=(0,chromat.points_count-1)

	return (chromat,msg)

# --------------------------------------------------------------------------
#
def get_chromat_data_point(chromat,rt,spectrum,mz_margin,require_max,data_threshold):

	traces=numpy.zeros(chromat.peaks_count)
	for i in range(0,chromat.peaks_count):
		mz_range=[chromat.peaks_mz[i]-mz_margin[0]*chromat.peaks_mz[i]/(1e+6),chromat.peaks_mz[i]+mz_margin[1]*chromat.peaks_mz[i]/(1e+6)]
		if spectrum.domain==mssputil.DOMAIN_FREQ:
			mz_points=mssputil.get_f_in_range(spectrum,mssputil.mz2freq_list(mz_range,spectrum))
			if require_max==True:
				#dodatkowe punkty na krancach przedzialu
				step=mssputil.get_f_step(spectrum)	#przyspieszenie => step pobierany wczesniej
				mz_points=numpy.hstack([numpy.hstack([mz_points[0]+step,mz_points]),mz_points[-1]-step])

		else:
			mz_points=mssputil.get_mz_in_range(spectrum,mz_range,native=True)
			if require_max==True:
				#dodatkowe punkty na krancach przedzialu
				step=mssputil.get_mz_step(spectrum)	#przyspieszenie => step pobierany wczesniej
				mz_points=numpy.hstack([numpy.hstack([mz_points[0]+step,mz_points]),mz_points[-1]-step])

		trace=mssputil.get_dataline(spectrum,mz_points,rt)
		if require_max==True:
			max_index=numpy.argmax(trace)
			if max_index==0 or max_index==len(trace)-1:
				traces[i]=0
			else:
				traces[i]=trace[max_index]
		else:
			traces[i]=numpy.max(trace)

		#prog dotyczy wszystkich pikow (skalowany przez wzgledne teoretyczne wysokosci pikow)
		if traces[i]<chromat.peaks_prob[i]*data_threshold:
			traces[i]=0

	chromat_point=numpy.power(numpy.prod(traces),1/float(chromat.peaks_count))

	return chromat_point

# --------------------------------------------------------------------------
#
def get_chromat_data_points(chromat,spectrum,mz_margin,require_max,data_threshold):

	traces=numpy.zeros((chromat.points_count,chromat.peaks_count))
	for i in range(0,chromat.peaks_count):
		mz_range=[chromat.peaks_mz[i]-mz_margin[0]*chromat.peaks_mz[i]/(1e+6),chromat.peaks_mz[i]+mz_margin[1]*chromat.peaks_mz[i]/(1e+6)]
		if spectrum.domain==mssputil.DOMAIN_FREQ:
			mz_points=mssputil.get_f_in_range(spectrum,mssputil.mz2freq_list(mz_range,spectrum))
			if require_max==True:
				#dodatkowe punkty na krancach przedzialu
				step=mssputil.get_f_step(spectrum)	#przyspieszenie => step pobierany wczesniej
				mz_points=numpy.hstack([numpy.hstack([mz_points[0]+step,mz_points]),mz_points[-1]-step])

		else:
			mz_points=mssputil.get_mz_in_range(spectrum,mz_range,native=True)
			if require_max==True:
				#dodatkowe punkty na krancach przedzialu
				step=mssputil.get_mz_step(spectrum)	#przyspieszenie => step pobierany wczesniej
				mz_points=numpy.hstack([numpy.hstack([mz_points[0]+step,mz_points]),mz_points[-1]-step])

		trace=mssputil.get_datalines(spectrum,mz_points,chromat.rt)
		if require_max==True:
			traces[:,i]=trace[:,1:trace.shape[1]-1].max(1)
			index=numpy.argmax(trace,1)
			traces[index==0,i]=0
			traces[index==trace.shape[1]-1,i]=0
		else:
			traces[:,i]=trace.max(1)

		#prog dotyczy wszystkich pikow (skalowany przez wzgledne teoretyczne wysokosci pikow)
		traces[traces[:,i].T<(chromat.peaks_prob[i]*data_threshold),i]=0

	#chromatogram laczny
	chromat.data=numpy.power(numpy.prod(traces,1),1/float(chromat.peaks_count))

# --------------------------------------------------------------------------
#
def expand_chromat_range(chromat,spectrum,mz_margin,require_max,data_threshold):

	rt_min=chromat.rt[0]
	rt_max=chromat.rt[-1]

	rt_range=mssputil.get_rt_range(spectrum,True)
	rt_step=mssputil.get_rt_step(spectrum)

	while chromat.data[0]>0 and chromat.rt[0]<rt_range[0]:
		data_point=get_chromat_data_point(chromat,chromat.rt[0]+rt_step,spectrum,mz_margin,require_max,data_threshold)
		chromat.data=numpy.hstack((numpy.array(data_point),chromat.data))
		chromat.rt=numpy.hstack((numpy.array(chromat.rt[0]+rt_step),chromat.rt))
		chromat.points_count=chromat.points_count+1

	while chromat.data[-1]>0 and chromat.rt[-1]>rt_range[1]:
		data_point=get_chromat_data_point(chromat,chromat.rt[-1]-rt_step,spectrum,mz_margin,require_max,data_threshold)
		chromat.data=numpy.hstack((chromat.data,numpy.array(data_point)))
		chromat.rt=numpy.hstack((chromat.rt,numpy.array(chromat.rt[-1]-rt_step)))
		chromat.points_count=chromat.points_count+1

	if rt_min!=chromat.rt[0] or rt_max!=chromat.rt[-1]:
		chromat.expanded=1
		#print 'Chromatogram range expanded from (%f, %f) to (%f, %f)' % (-rt_min,-rt_max,-chromat.rt[0],-chromat.rt[-1])
		
# --------------------------------------------------------------------------
#
def get_rt_cost(chromat,penality_fun,mean_rt,fwmh_rt):

	if penality_fun!=PENALITY_NONE and mean_rt!=0 and fwmh_rt!=0:
		if penality_fun==PENALITY_GAUSS:
			rt_cost=msmath.gauss(chromat.rt,1,-1*mean_rt,fwmh_rt/(2*numpy.sqrt(2*numpy.log(2.0))))
			rt_cost[rt_cost<1.3383022576488544e-004]=0

		elif penality_fun==PENALITY_SQUARE:
			rt_cost=numpy.ones(len(chromat.rt))
			rt_cost[chromat.rt<=-(mean_rt+fwmh_rt/2)]=0
			rt_cost[chromat.rt>=-(mean_rt-fwmh_rt/2)]=0
	else:
		rt_cost=None

	return rt_cost

# --------------------------------------------------------------------------
#
def expand_rt_cost(chromat,expand_penality):

	msg=''
	
	rt_min_index=0
	for i in range(0,len(chromat.rt_cost)):
		if chromat.rt_cost[i]!=0:
			rt_min_index=i
			break
	rt_min=chromat.rt[rt_min_index]

	if expand_penality[0]!=0:
		if rt_min_index>0 and chromat.data[rt_min_index]>0:
			rt_cost_value=chromat.rt_cost[rt_min_index]
			while chromat.data[rt_min_index]>0 and rt_min_index>0:
				rt_min_index=rt_min_index-1
				chromat.rt_cost[rt_min_index]=rt_cost_value

	rt_max_index=len(chromat.rt_cost)-1
	for i in range(len(chromat.rt_cost)-1,-1,-1):
		if chromat.rt_cost[i]!=0:
			rt_max_index=i
			break
	rt_max=chromat.rt[rt_max_index]

	if expand_penality[1]!=0:
		if rt_max_index<len(chromat.rt_cost)-1 and chromat.data[rt_max_index]>0:
			rt_cost_value=chromat.rt_cost[rt_max_index]
			while chromat.data[rt_max_index]>0 and rt_max_index<len(chromat.rt_cost)-1:
				rt_max_index=rt_max_index+1
				chromat.rt_cost[rt_max_index]=rt_cost_value

	if rt_min!=chromat.rt[rt_min_index] or rt_max!=chromat.rt[rt_max_index]:
		chromat.rt_cost_expanded=1
		msg='Penality function expanded from (%f, %f) to (%f, %f)' % (-rt_min,-rt_max,-chromat.rt[rt_min_index],-chromat.rt[rt_max_index])

	return msg

# --------------------------------------------------------------------------
#
def filter_chromatogram(chromat,filter_type=msfilter.FILTER_MAVERAGE,filter_span=7,filter_order=2):

	chromat.data=msfilter.filter_data(chromat.data,filter_type,filter_span,filter_order)
	chromat.max_data=chromat.data.max()
	chromat.max_rt=numpy.argmax(chromat.data)

# --------------------------------------------------------------------------
#
def get_chromatogram_peaks(chromat,max_threshold=0,sort_peaks=0,mean_rt=0,deriv2=False,max_peaks_count=0):
	
	if chromat.data.max()>0:
#		if chromat.data[-1]>0:
#			chromat.data=numpy.hstack([chromat.data,0])
#			chromat.data=numpy.hstack([chromat.data,0])
#			chromat.data=numpy.hstack([chromat.data,10])
#			chromat.rt=numpy.hstack([chromat.rt,chromat.rt[-1]+(chromat.rt[-1]-chromat.rt[-2])])
#			chromat.rt=numpy.hstack([chromat.rt,chromat.rt[-1]+(chromat.rt[-1]-chromat.rt[-2])])
#			chromat.rt=numpy.hstack([chromat.rt,chromat.rt[-1]+(chromat.rt[-1]-chromat.rt[-2])])
#			chromat.points_count=chromat.points_count+2

#		print chromat.rt
		
		chromat.deriv_data=numpy.diff(chromat.data)
		chromat.deriv_data=numpy.hstack([chromat.deriv_data,0])

		if deriv2==True:
			chromat.deriv2_data=numpy.diff(chromat.data,2)
			chromat.deriv2_data=numpy.hstack([chromat.deriv2_data,[0,0]])

		# szukanie maksimow pikow
		peak_max=[]
		peak_left_min=[]
		peak_right_min=[]
		for i in range(0,chromat.points_count-1):
			if chromat.deriv_data[i]>0 and chromat.deriv_data[i+1]<=0:
				max_index=i+1
				right_min=max(max_index-1,0)
				while right_min>=0 and chromat.data[right_min]<chromat.data[right_min+1]:
					right_min=right_min-1
					
				left_min=min(max_index+1,chromat.points_count-1)
				while left_min<chromat.points_count and chromat.data[left_min]<chromat.data[left_min-1]:
					left_min=left_min+1

				peak_max.append(max_index)
				peak_right_min.append(right_min+1)
				peak_left_min.append(left_min-1)

		#tworzenie listy pikow
		peaks_count=len(peak_max)
		if peaks_count>0:
			peaks=[]
			for i in range(peaks_count):
				#peak_max[index[i]]						index maksimum piku
				#peak_right_min[index[i]]				index prawego min piku
				#peak_left_min[index[i]]				index lewego min piku
				#chromat.data[peak_max[index[i]]]		wartosc max piku
				#chromat.rt[peak_max[index[i]]]			RT max
				#chromat.rt[peak_right_min[index[i]]]	RT prawego min
				#chromat.rt[peak_left_min[index[i]]]	RT lewego min


#				lmn=float(chromat.data[peak_right_min[i]])/float(chromat.max_data)
#				mx=float(chromat.data[peak_max[i]])/float(chromat.max_data)
#				h=float(peak_max[i]-peak_right_min[i])/800
#				print '%f %f %f %f' % (lmn, mx, h, (mx-lmn)/h)

				if get_shape_coeff(chromat,peak_max[i],peak_right_min[i])>max_threshold:
					peaks.append([peak_max[i],peak_right_min[i],peak_left_min[i],chromat.data[peak_max[i]],chromat.rt[peak_max[i]],chromat.rt[peak_right_min[i]],chromat.rt[peak_left_min[i]]])
				else:
					#laczenie kolejnych malo stromych pikow
					if len(peaks)!=0:
						prev_peak=peaks[-1]
						if peak_right_min[i]<=prev_peak[2]:
							prev_peak[2]=peak_left_min[i]
							prev_peak[6]=chromat.rt[peak_left_min[i]]

			chromat.peaks=[]
			#sortowanie i dodawanie pikow
			if len(peaks)!=0:
				if sort_peaks==SORT_VALUE:
					peaks=sorted(peaks,key=lambda peak: -1*peak[3])
				elif sort_peaks==SORT_RT_DELTA:
					peaks=sorted(peaks,key=lambda peak: abs(-1*peak[4]-mean_rt))

				peaks_count=len(peaks)
				if max_peaks_count>0:
					pp=min(peaks_count,max_peaks_count)

				for i in range(0,peaks_count):
					chromat.peaks.append(peaks[i])

	else:
		chromat.deriv_data=numpy.zeros(chromat.data.shape[0])
		chromat.deriv2_data=numpy.zeros(chromat.data.shape[0])

# --------------------------------------------------------------------------
#
def get_shape_coeff(chromat,peak_max,peak_right_min):

	lmn=float(chromat.data[peak_right_min])/float(chromat.max_data)
	mx=float(chromat.data[peak_max])/float(chromat.max_data)
#	h=float(peak_max[i]-peak_right_min[i])/float(chromat.points_count)
	h=float(peak_max-peak_right_min)/800
	#print '%f %f %f %f' % (lmn, mx, h, (mx-lmn)/h)

	return (mx-lmn)/h