#-*- coding: windows-1250 -*-

import math
import os
import sys

import Tix
import Tkinter
import Image,ImageDraw, ImageTk, ImageFont
import aggdraw
import ToolTip

import sparky

import mschromatogram
import mschromatogram_settings
import msenvelope
import msisoenvelope
import mscalib
import msfilter
import mssputil
import mstkutil


ctable=('#FF0000','#00FF00','#0000FF','gray','yellow','orange','#800000','#008000','#000080','#800080','#808000','#008080','#FF00FF','#00FFFF','#C0C0C0')

# -----------------------------------------------------------------------------
#
class chromatogram_view_dialog(mstkutil.Dialog):

	def __init__(self,session,settings=None):

		self.session=session
		self.title='Chromatogram: '
		if settings==None:
			self.settings=mschromatogram_settings.chromatogram_settings(self.session,os.path.join(os.environ.get('SPARKY_INSTALL'),os.path.join('config','chromatogram.cfg')))
		else:
			self.settings=settings;
			
		self.chromat=None
		self.chromat2=None
		self.peak=None
		self.peak2=None
		self.peptide=None
		self.spectrum=None

		self.plot_data=[]
		self.visible_range_max=0
		self.visible_range_min=0
		self.scale_y=1

		self.prev_visible_range=[]
		self.zoom_start=0
		self.zoom_stop=0
		self.zoom_max=0

		self.drag_line=None
		self.drag_line2=None
		self.drag_start_x=0
		self.drag_start_y=0

		self.move_text=None
		self.move_text2=None

		mstkutil.Dialog.__init__(self,session.tk,'')

		#self.top.tk.call('wm', 'iconbitmap',self.top._w,os.path.join(os.environ.get('SPARKY_INSTALL'),os.path.join('bin','msparky.ico')))

		f=Tkinter.Frame(self.top,borderwidth=2,relief='groove')
		f.pack(side='top',anchor='w',pady=2)

		image=mstkutil.load_image('save.gif')
		b=Tkinter.Button(f,borderwidth=1,image=image,command=self.save_cb)
		b.image=image
		b.grid(row=0,column=0,sticky='w',padx='1')
		ToolTip.ToolTip(b,text='Save chromatogram',delay=500)
		
		l=Tkinter.Label(f,text='',width='3')
		l.grid(row=0,column=1,sticky='w',padx='1')

		image=mstkutil.load_image('zoom_in.gif')
		b=Tkinter.Button(f,borderwidth=1,image=image,command=self.zoom_cb)
		b.image=image
		b.grid(row=0,column=2,sticky='w',padx='1')
		ToolTip.ToolTip(b,text='Zoom In',delay=500)

		image=mstkutil.load_image('zoom_out.gif')
		b=Tkinter.Button(f,borderwidth=1,image=image,command=self.mouse_spressed_cb)
		b.image=image
		b.grid(row=0,column=3,sticky='w',padx='1')
		ToolTip.ToolTip(b,text='Zoom Out',delay=500)

		image=mstkutil.load_image('zoom_100.gif')
		b=Tkinter.Button(f,borderwidth=1,image=image,command=self.mouse_sdouble_pressed_cb)
		b.image=image
		b.grid(row=0,column=4,sticky='w',padx='1')
		ToolTip.ToolTip(b,text='Full Range',delay=500)

		l=Tkinter.Label(f,text='',width='3')
		l.grid(row=0,column=5,sticky='w',padx='1')

		image=mstkutil.load_image('nav_west.gif')
		b=Tkinter.Button(f,borderwidth=1,image=image,command=self.prev_peak_cb)
		b.image=image
		b.grid(row=0,column=6,sticky='w',padx='1')
		ToolTip.ToolTip(b,text='Previous peak',delay=500)

		image=mstkutil.load_image('nav_east.gif')
		b=Tkinter.Button(f,borderwidth=1,image=image,command=self.next_peak_cb)
		b.image=image
		b.grid(row=0,column=7,sticky='w',padx='1')
		ToolTip.ToolTip(b,text='Next peak',delay=500)

		image=mstkutil.load_image('undo.gif')
		b=Tkinter.Button(f,borderwidth=1,image=image,command=self.restore_peak_cb)
		b.image=image
		b.grid(row=0,column=8,sticky='w',padx='1')
		ToolTip.ToolTip(b,text='Restore position',delay=500)

		l=Tkinter.Label(f,text='',width='3')
		l.grid(row=0,column=9,sticky='w',padx='1')

		image=mstkutil.load_image('envelope.gif')
		b=Tkinter.Button(f,borderwidth=1,image=image,command=self.isodistrib_cb)
		b.image=image
		b.grid(row=0,column=10,sticky='w',padx='1')
		ToolTip.ToolTip(b,text='Isotopic envelope',delay=500)
		
		f=Tkinter.Frame(self.top,borderwidth=2,relief='groove')
		f.pack(side='top',anchor='w',pady=2)

		canvas=Tkinter.Canvas(f,width=200,height=200,background='white',cursor='crosshair')
		canvas.grid(row=0,column=0,sticky='we')
		canvas.bind('<ButtonPress-1>',self.mouse_pressed_cb)
		canvas.bind('<ButtonRelease-1>',self.mouse_released_cb)
		canvas.bind('<Shift-ButtonPress-1>',self.mouse_spressed_cb)
		canvas.bind('<Double-Button-1>',self.mouse_double_pressed_cb)
		canvas.bind('<Shift-Double-Button-1>',self.mouse_sdouble_pressed_cb)
		canvas.bind('<B1-Motion>',self.mouse_dragged_cb)
		canvas.bind('<Button-3>',self.popup_menu)
		canvas.bind('<Motion>',self.mouse_moved_cb)
		canvas.bind('<Leave>',self.mouse_leave_cb)
		self.top.bind('<Key>', self.keypressed_cb)
		self.canvas=canvas

		f=Tkinter.Frame(self.top,borderwidth=2,relief='groove')
		f.pack(side='top',anchor='w',pady=2)

		canvas=Tkinter.Canvas(f,width=200,height=200,background='white',cursor='crosshair')
		canvas.grid(row=0,column=0,columnspan=2,sticky='we')
		canvas.bind('<ButtonPress-1>',self.mouse_pressed_cb)
		canvas.bind('<ButtonRelease-1>',self.mouse_released_cb)
		canvas.bind('<Shift-ButtonPress-1>',self.mouse_spressed_cb)
		canvas.bind('<Double-Button-1>',self.mouse_double_pressed_cb)
		canvas.bind('<Shift-Double-Button-1>',self.mouse_sdouble_pressed_cb)
		canvas.bind('<B1-Motion>',self.mouse_dragged_cb)
		canvas.bind('<Button-3>',self.popup_menu)
		canvas.bind('<Motion>',self.mouse_moved_cb)
		canvas.bind('<Leave>',self.mouse_leave_cb)
		self.canvas2=canvas

		m=mstkutil.option_menu(f,'Plot type: ',('Chromatogram','1 derivative','2 derivative'),'Chromatogram')
		m.add_callback(self.derivative_cb)
		m.frame.grid(row=1,column=0,sticky='w',pady=2,padx=2)
		self.derivative_menu=m

		m=mssputil.charge_menu(f,'')
		m.frame.grid(row=1,column=1,sticky='e',pady=2,padx=2)
#		m.add_callback(self.charge_cb)
		self.charge_menu=m

		br=mstkutil.button_row(self.top,
				('Setup', self.setup_cb),
				('Close', self.top.destroy))
		br.frame.pack(side='bottom',anchor='w',pady=2)

		self.top.geometry("+0+0")
		self.top.resizable(0,0)
		self.top.focus_set()
		
		self.removed_notice=session.notify_me('removed spectrum from project',self.spectrum_removed)
		self.dragging_notice=session.notify_me('dragged peak',self.peak_dragged_cb)
		self.top.bind('<Destroy>',self.remove_callbacks,1)

	# -----------------------------------------------------------------------------
	#
	def popup_menu(self,event):
		menu=Tkinter.Menu(self.canvas,tearoff=0)

		menu.add_command(label='Isotopic envelope plot',command=self.isodistrib_cb)
		menu.add_separator()

		processing_submenu=Tkinter.Menu(menu,tearoff=0)
		processing_submenu.add_checkbutton(label='Use calibration',variable=self.settings.spectrum_correction,command=self.use_calibration_cb)
		processing_submenu.add_separator()
		processing_submenu.add_command(label='Increase peaks count',command=self.peaks_increment_cb)
		processing_submenu.add_command(label='Decrease peaks count',command=self.peaks_decrement_cb)
		processing_submenu.add_separator()
		processing_submenu.add_checkbutton(label='Enable RT penality',variable=self.settings.rt_penality,command=self.rt_penality_cb)
		processing_submenu.add_command(label='Double penality width',command=self.rt_penality_double_cb)
		processing_submenu.add_command(label='Half penality width',command=self.rt_penality_half_cb)
		processing_submenu.add_separator()
		processing_submenu.add_command(label='Double m/z range',command=self.mz_range_double_cb)
		processing_submenu.add_command(label='Half m/z range',command=self.mz_range_half_cb)
		processing_submenu.add_separator()
		processing_submenu.add_command(label='Double data threshold',command=self.threshold_double_cb)
		processing_submenu.add_command(label='Half data threshold',command=self.threshold_half_cb)
		processing_submenu.add_separator()
		processing_submenu.add_command(label='Double filter span',command=self.filter_double_cb)
		processing_submenu.add_command(label='Half filter span',command=self.filter_half_cb)
		processing_submenu.add_separator()
		processing_submenu.add_checkbutton(label='Maxima elimination',variable=self.settings.max_elimination,command=self.rt_penality_cb)
		menu.add_cascade(label='Processing',menu=processing_submenu)

		view_submenu=Tkinter.Menu(menu,tearoff=0)
		view_submenu.add_checkbutton(label='Peptide position',variable=self.settings.view_peptide_position,command=self.update_plot)
		view_submenu.add_checkbutton(label='RT penality',variable=self.settings.view_rt_penality,command=self.update_plot)
		view_submenu.add_checkbutton(label='Shape coefficients',variable=self.settings.view_shape_coeffs,command=self.update_plot)
		view_submenu.add_checkbutton(label='Info',variable=self.settings.view_info,command=self.update_plot)
		menu.add_cascade(label='View',menu=view_submenu)
		max_submenu=Tkinter.Menu(menu,tearoff=0)
		max_submenu.add_command(label='Previous peak',command=self.prev_peak_cb)
		max_submenu.add_command(label='Next peak',command=self.next_peak_cb)
		max_submenu.add_command(label='Restore position',command=self.restore_peak_cb)
		menu.add_cascade(label='Position',menu=max_submenu)
		menu.add_separator()
		if event.widget==self.canvas:
			menu.add_command(label='Save',command=self.save_cb)
		else:
			menu.add_command(label='Save',command=self.save2_cb)

		menu.tk_popup(event.x_root,event.y_root)
		
	# ---------------------------------------------------------------------------
	#
	def spectrum_removed(self,removed_spectrum):
		if self.spectrum==removed_spectrum:
			self.top.destroy()

	# ---------------------------------------------------------------------------
	#
	def peak_dragged_cb(self,peak):
		if peak==self.peak:
			self.update_plot()

	# ---------------------------------------------------------------------------
	#
	def remove_callbacks(self,event):
		if str(self.top)==str(event.widget):
			if sparky.object_exists(self.session):
				if self.removed_notice!=None:
					self.session.dont_notify_me(self.removed_notice)
				if self.dragging_notice!=None:
					self.session.dont_notify_me(self.dragging_notice)

			self.removed_notice=None
			self.dragging_notice=None

	# ---------------------------------------------------------------------------
	#
	def keypressed_cb(self,event):
		ks=event.keysym.lower()
		if ks=='left' or ks=='p':
			self.prev_peak_cb()
		elif ks=='right' or ks=='n':
			self.next_peak_cb()
		elif ks=='r':
			self.restore_peak_cb()
		elif ks=='e':
			self.isodistrib_cb()
		
	# ---------------------------------------------------------------------------
	#
	def mouse_pressed_cb(self,event):
		self.zoom_start=int(self.chromat.rt_visible_range[0]+event.x/self.scale_x)
		self.drag_start_x=event.x
		self.drag_start_y=event.y

#		selected=self.canvas.find_withtag('current')
#		if selected and self.canvas.type(selected)=='oval':
#			print 'oval'
#			print self.canvas.gettags(selected)

	# ---------------------------------------------------------------------------
	#
	def mouse_released_cb(self,event):
		self.canvas.coords(self.drag_line,0,0,0,0)
		self.canvas2.coords(self.drag_line2,0,0,0,0)
		if self.zoom_start!=-10:
			self.zoom_stop=int(self.chromat.rt_visible_range[0]+event.x/self.scale_x)
			self.zoom_stop=min(max(self.zoom_stop,self.chromat.rt_visible_range[0]),self.chromat.rt_visible_range[1])
		
			if math.fabs(self.zoom_start-self.zoom_stop)>2:
				self.prev_visible_range.append((self.chromat.rt_visible_range[0],self.chromat.rt_visible_range[1]))

				self.chromat.rt_visible_range=(min(self.zoom_start,self.zoom_stop),max(self.zoom_start,self.zoom_stop))
				self.chromat2.rt_visible_range=(min(self.zoom_start,self.zoom_stop),max(self.zoom_start,self.zoom_stop))
				self.update_plot()
			self.zoom_start=-10

	# ---------------------------------------------------------------------------
	#
	def mouse_double_pressed_cb(self,event):
		self.zoom_start=-10
		self.zoom_stop=int(self.chromat.rt_visible_range[0]+event.x/self.scale_x)

		if event.widget==self.canvas:
			mz=self.chromat.mz
		else:
			mz=self.chromat2.mz

		if self.spectrum.domain==mssputil.DOMAIN_FREQ:
			mz=mssputil.freq2mz_s(mz,self.spectrum);
		else:
			mz=-1*mz

		mssputil.show_spectrum_position(self.spectrum,(self.chromat.rt[self.zoom_stop],mz))

	# ---------------------------------------------------------------------------
	#
	def mouse_spressed_cb(self,event=None):
		self.zoom_start=-10
		if len(self.prev_visible_range)>0:
			visible_range=self.prev_visible_range[len(self.prev_visible_range)-1]
			self.chromat.rt_visible_range=(visible_range[0],visible_range[1])
			self.chromat2.rt_visible_range=(visible_range[0],visible_range[1])
			del self.prev_visible_range[len(self.prev_visible_range)-1]
			self.update_plot()

	# ---------------------------------------------------------------------------
	#
	def mouse_sdouble_pressed_cb(self,event=None):
		self.zoom_start=-10
		self.prev_visible_range=[]
		self.chromat.rt_visible_range=(0,self.chromat.points_count-1)
		self.chromat2.rt_visible_range=(0,self.chromat.points_count-1)
		self.update_plot()

	# ---------------------------------------------------------------------------
	#
	def mouse_dragged_cb(self,event):
		self.mouse_moved_cb(event)
		if event.widget==self.canvas:
			self.canvas.coords(self.drag_line,self.drag_start_x,self.drag_start_y,event.x,self.drag_start_y)
		else:
			self.canvas2.coords(self.drag_line2,self.drag_start_x,self.drag_start_y,event.x,self.drag_start_y)

	# ---------------------------------------------------------------------------
	#
	def mouse_moved_cb(self,event):
		if event.widget==self.canvas:
		    chromat=self.chromat
		else:
		    chromat=self.chromat2
		
		entry=self.derivative_menu.get()
		if entry=='Chromatogram':
			plot_data=chromat.data
		elif entry=='1 derivative':
			plot_data=chromat.deriv_data
		elif entry=='2 derivative':
			plot_data=chromat.deriv2_data

		index=min(chromat.points_count-1,int(self.chromat.rt_visible_range[0]+event.x/self.scale_x))

		txt='(%0.2f;%0.2g)' % (-1*chromat.rt[index],plot_data[index])

		x_diff=4*len(txt)
		if event.x>int(event.widget.cget('width'))-2*x_diff:
			x_diff=-1*x_diff

		y_diff=-6
		if event.y<-y_diff:
			y_diff=-1*y_diff

		if event.widget==self.canvas:
			self.canvas.itemconfig(self.move_text,text=txt)
			self.canvas.coords(self.move_text,event.x+x_diff,event.y+y_diff)
		else:
			self.canvas2.itemconfig(self.move_text2,text=txt)
			self.canvas2.coords(self.move_text2,event.x+x_diff,event.y+y_diff)
	
	# ---------------------------------------------------------------------------
	#
	def mouse_leave_cb(self,event):

		if event.widget==self.canvas:
			self.canvas.itemconfig(self.move_text,text='')
			self.canvas.coords(self.move_text,0,0)
		else:
			self.canvas2.itemconfig(self.move_text2,text='')
			self.canvas2.coords(self.move_text2,0,0)

	# ---------------------------------------------------------------------------
	#
	def zoom_cb(self,event=None):
		start=self.chromat.rt_visible_range[0]+int((self.chromat.rt_visible_range[1]-self.chromat.rt_visible_range[0])/4)
		stop=self.chromat.rt_visible_range[0]+int(3*(self.chromat.rt_visible_range[1]-self.chromat.rt_visible_range[0])/4)

		if math.fabs(start-stop)>2:
			self.prev_visible_range.append((self.chromat.rt_visible_range[0],self.chromat.rt_visible_range[1]))

			self.chromat.rt_visible_range=(start,stop)
			self.chromat2.rt_visible_range=(start,stop)
			self.update_plot()

	# ---------------------------------------------------------------------------
	#
	def charge_cb(self,event=None):
		self.chromat2=None
		self.plot_chromatogram(self.peak)
	
	# ---------------------------------------------------------------------------
	#
	def empty_charge_cb(self,event=None):
		print 'ala'

	# ---------------------------------------------------------------------------
	#
	def derivative_cb(self,event=None):
#		self.chromat2=None
		self.plot_chromatogram(self.peak)

	# ---------------------------------------------------------------------------
	#
	def save_cb(self,event=None):
		self.save(self.canvas,self.chromat)
	# ---------------------------------------------------------------------------
	#
	def save2_cb(self,event=None):
		self.save(self.canvas2,self.chromat2)
		
	# ---------------------------------------------------------------------------
	#
	def filtering_cb(self):
		self.chromat=None
		self.chromat2=None
		self.update_plot()

	# ---------------------------------------------------------------------------
	#
	def rt_penality_cb(self):
		self.chromat=None
		self.chromat2=None
		self.update_plot()

	# ---------------------------------------------------------------------------
	#
	def use_calibration_cb(self):
		self.settings.spectrum_mz_correction=self.settings.spectrum_correction.get()
		self.settings.spectrum_rp_correction=self.settings.spectrum_correction.get()
		self.settings.spectrum_rt_correction=self.settings.spectrum_correction.get()

		self.rt_penality_cb()
		
	# ---------------------------------------------------------------------------
	#
	def peaks_increment_cb(self):
		self.settings.peaks_count=self.settings.peaks_count+1
		self.rt_penality_cb()

	# ---------------------------------------------------------------------------
	#
	def peaks_decrement_cb(self):
		if self.settings.peaks_count>0:
			self.settings.peaks_count=self.settings.peaks_count-1
		self.rt_penality_cb()

	# ---------------------------------------------------------------------------
	#
	def rt_penality_double_cb(self):
		self.settings.rt_penality_s=2*self.settings.rt_penality_s
		self.rt_penality_cb()

	# ---------------------------------------------------------------------------
	#
	def rt_penality_half_cb(self):
		self.settings.rt_penality_s=0.5*self.settings.rt_penality_s
		self.rt_penality_cb()
		
	# ---------------------------------------------------------------------------
	#
	def mz_range_double_cb(self):
		self.settings.mz_margin=(2*self.settings.mz_margin[0],2*self.settings.mz_margin[1])
		self.rt_penality_cb()

	# ---------------------------------------------------------------------------
	#
	def mz_range_half_cb(self):
		self.settings.mz_margin=(0.5*self.settings.mz_margin[0],0.5*self.settings.mz_margin[1])
		self.rt_penality_cb()

	# ---------------------------------------------------------------------------
	#
	def threshold_double_cb(self):
		self.settings.threshold=2*self.settings.threshold
		self.rt_penality_cb()

	# ---------------------------------------------------------------------------
	#
	def threshold_half_cb(self):
		self.settings.threshold=0.5*self.settings.threshold
		self.rt_penality_cb()

	# ---------------------------------------------------------------------------
	#
	def filter_double_cb(self):
		self.settings.filter_span=2*self.settings.filter_span
		self.rt_penality_cb()
		
	# ---------------------------------------------------------------------------
	#
	def filter_half_cb(self):
		self.settings.filter_span=0.5*self.settings.filter_span
		self.rt_penality_cb()
		
	# ---------------------------------------------------------------------------
	#
	def next_peak_cb(self):
		peak_index=self.chromat.rt2index(self.peak.frequency[0])
		for i in range(0,len(self.chromat.peaks)):
			current_max=self.chromat.peaks[i]
			if current_max[0]>peak_index:
				rt=self.chromat.index2rt(current_max[0])
				mssputil.shift_query(self.peak,rt)
				mssputil.show_spectrum_position(self.spectrum,(self.peak.frequency[0],self.peak.frequency[1]))
				self.top.focus_set()
				break

	# ---------------------------------------------------------------------------
	#
	def prev_peak_cb(self):
		peak_index=self.chromat.rt2index(self.peak.frequency[0])
		for i in range(len(self.chromat.peaks)-1,-1,-1):
			current_max=self.chromat.peaks[i]
			if current_max[0]<peak_index:
				rt=self.chromat.index2rt(current_max[0])
				mssputil.shift_query(self.peak,rt)
				mssputil.show_spectrum_position(self.spectrum,(self.peak.frequency[0],self.peak.frequency[1]))
				self.top.focus_set()
				break

	# ---------------------------------------------------------------------------
	#
	def restore_peak_cb(self):
		mssputil.shift_query(self.peak,self.settings.org_rt)
		mssputil.show_spectrum_position(self.spectrum,(self.peak.frequency[0],self.peak.frequency[1]))
		self.top.focus_set()

	# ---------------------------------------------------------------------------
	#
	def isodistrib_cb(self,event=None):
		msisoenvelope.show_peak_fit(self.peak,True)

	# ---------------------------------------------------------------------------
	#
	def setup_cb(self):
		psd=mssputil.the_dialog(mschromatogram_settings.chromatogram_settings_dialog,self.session)
		psd.set_parent_dialog(self,self.settings,self.new_settings)
		psd.top.title(self.title + ' settings')
		psd.show_window(1)

	# ---------------------------------------------------------------------------
	#
	def new_settings(self,settings):
		self.settings=settings
		self.chromat=None
		self.chromat2=None
		self.update_plot()

	# ---------------------------------------------------------------------------
	#
	def update_plot(self,event=None):
		if sparky.object_exists(self.peak):
			self.plot_chromatogram(self.peak)

	# ---------------------------------------------------------------------------
	#
	def plot_chromat(self,chromat,canvas,peak=None):

		width=float(canvas.cget('width'))
		height=float(canvas.cget('height'))

		entry=self.derivative_menu.get()
		if entry=='Chromatogram':
			plot_data=chromat.data
		elif entry=='1 derivative':
			plot_data=chromat.deriv_data
		elif entry=='2 derivative':
			plot_data=chromat.deriv2_data

		visible_range_max=plot_data[self.chromat.rt_visible_range[0]:self.chromat.rt_visible_range[1]].max()
		visible_range_min=plot_data[self.chromat.rt_visible_range[0]:self.chromat.rt_visible_range[1]].min()
		scale_y=0.8*height/max(1,visible_range_max-visible_range_min)

		canvas.delete('all')

		#rysowanie linii
		agg_image=Image.new("RGB",(int(width),int(height)),"white")
		agg_draw=aggdraw.Draw(agg_image)
		
		for i in range(self.chromat.rt_visible_range[0],self.chromat.rt_visible_range[1]):
			point_x=self.scale_x*(i-chromat.rt_visible_range[0])
			point_y=height-scale_y*(plot_data[i]-visible_range_min)

			next_point_x=self.scale_x*(i+1-chromat.rt_visible_range[0])
			next_point_y=height-scale_y*(plot_data[i+1]-visible_range_min)

			agg_pen=aggdraw.Pen("black",2)
			for j in range(0,len(chromat.peaks)):
				current_max=chromat.peaks[j]
				if i>=current_max[1] and i+1<=current_max[2]:
					agg_pen=aggdraw.Pen(ctable[j%len(ctable)],2)
					break
			agg_draw.line((point_x,point_y-1,next_point_x,next_point_y-1),agg_pen)

		agg_draw.flush()

		draw=ImageDraw.Draw(agg_image)
		
		#rysowanie maximow
		for i in range(0,len(chromat.peaks)):
			current_max=chromat.peaks[i]
			if current_max[0]>=chromat.rt_visible_range[0] and current_max[0]<=chromat.rt_visible_range[1]:
				point_x=self.scale_x*(current_max[0]-chromat.rt_visible_range[0])
				point_y=height-scale_y*(plot_data[current_max[0]]-visible_range_min)

				draw.ellipse((point_x-3,point_y-6,point_x+3,point_y),fill=ctable[i%len(ctable)],outline='black')

				if self.settings.view_shape_coeffs.get()==1:
					txt=('%0.1f' % (mschromatogram.get_shape_coeff(chromat,current_max[0],current_max[1])))
					#txt_size=draw.textsize(txt,font=self.settings.name_font_pil)
					draw.text((point_x+6,point_y-15),txt,font=self.settings.name_font_pil,fill=ctable[i%len(ctable)])

		if peak!=None and self.settings.view_peptide_position.get()==1:
			self.plot_peptide_position(draw,chromat,peak,visible_range_min,height,scale_y)

		if peak!=None and self.settings.rt_penality_fun!=mschromatogram.PENALITY_NONE and self.settings.view_rt_penality.get()==1:
			self.plot_penality(draw,chromat,peak,height)

		if self.settings.view_info.get()==1:
			self.plot_info(draw,chromat,visible_range_min,visible_range_max)
			
		canvas.image=agg_image
		canvas.photo_image=ImageTk.PhotoImage(agg_image)
		canvas.create_image(0,0,anchor='nw',image=canvas.photo_image)
		
	# ---------------------------------------------------------------------------
	#
	def plot_penality(self,draw,chromat,peak,height):

		massref_rt=mssputil.get_massref_rt(peak)
		if self.settings.spectrum_rt_correction==True:
			massref_rt=mscalib.get_calibrated_rt(massref_rt,self.spectrum)

		rt_cost=mschromatogram.get_rt_cost(chromat,self.settings.rt_penality_fun,massref_rt,self.settings.rt_penality_s)
		if chromat.rt_cost==None:
			chromat.rt_cost=mschromatogram.get_rt_cost(chromat,self.settings.rt_penality_fun,massref_rt,self.settings.rt_penality_s)
			
		visible_range_max=chromat.rt_cost[chromat.rt_visible_range[0]:chromat.rt_visible_range[1]].max()
		scale_y=0.85*height/visible_range_max

		#funkcja kary po ewentualnym rozszerzeniu
		index=chromat.rt2index(-1*massref_rt)
		for i in range(chromat.rt_visible_range[0],chromat.rt_visible_range[1]):
			point_x=self.scale_x*(i-chromat.rt_visible_range[0])
			point_y=height-scale_y*chromat.rt_cost[i]

			next_point_y=height-scale_y*chromat.rt_cost[i+1]
			if abs(chromat.rt_cost[i]-chromat.rt_cost[i+1])==1:
				next_point_x=point_x
				draw.line((point_x,point_y-1,next_point_x,next_point_y-1),width=1,fill='darkgray')

				point_x=next_point_x;
				point_y=next_point_y;
				next_point_x=self.scale_x*(i+2-chromat.rt_visible_range[0])
				next_point_y=height-scale_y*chromat.rt_cost[i+2]
			else:
				next_point_x=self.scale_x*(i+1-chromat.rt_visible_range[0])
				
			draw.line((point_x,point_y-1,next_point_x,next_point_y-1),width=1,fill='darkgray')

			if i==index:
				draw.line((point_x,height-1,point_x,0.15*height-1),width=1,fill='darkgray')

		#teoretyczna postac funkcji kary
		index=chromat.rt2index(-1*massref_rt)
		for i in range(chromat.rt_visible_range[0],chromat.rt_visible_range[1]):
			point_x=self.scale_x*(i-chromat.rt_visible_range[0])
			point_y=height-scale_y*rt_cost[i]

			next_point_y=height-scale_y*rt_cost[i+1]
			if abs(rt_cost[i]-rt_cost[i+1])==1:
				next_point_x=point_x
				draw.line((point_x,point_y-1,next_point_x,next_point_y-1),width=1,fill='#505050')
				
				point_x=next_point_x;
				point_y=next_point_y;
				next_point_x=self.scale_x*(i+2-chromat.rt_visible_range[0])
				next_point_y=height-scale_y*rt_cost[i+2]
			else:
				next_point_x=self.scale_x*(i+1-chromat.rt_visible_range[0])
				
			draw.line((point_x,point_y-1,next_point_x,next_point_y-1),width=1,fill='#505050')
			
			if i==index:
				draw.line((point_x,height-1,point_x,0.15*height-1),width=1,fill='#505050')
				
	# ---------------------------------------------------------------------------
	#
	def plot_peptide_position(self,draw,chromat,peak,visible_range_min,height,scale_y):
		
		peak_index=chromat.rt2index(peak.frequency[0])
		if peak_index>=chromat.rt_visible_range[0] and peak_index<=chromat.rt_visible_range[1]:
			point_x=self.scale_x*(peak_index-chromat.rt_visible_range[0])

			massref_rt=mssputil.get_massref_rt(peak)
			if self.settings.spectrum_rt_correction==True:
				massref_rt=mscalib.get_calibrated_rt(massref_rt,self.spectrum)

			txt=(u'RT=%0.2f (\u0394RT=%0.2f)' % ((-1*peak.frequency[0]),(-1*peak.frequency[0])-massref_rt))
			txt_size=draw.textsize(txt,font=self.settings.name_font_pil)

			if mssputil.is_peak_present(peak)==1:
				fill_color='#FFFFDD'
			else:
				fill_color='#FF2020'
			draw.polygon([(point_x-txt_size[0]/2-2,0.1*height-txt_size[1]-5), (point_x-txt_size[0]/2-2,0.1*height-3), (point_x+txt_size[0]/2+3,0.1*height-3),(point_x+txt_size[0]/2+3,0.1*height-txt_size[1]-5)],fill=fill_color,outline='#505050')
			draw.polygon([(point_x-4,0.1*height-3), (point_x+4,0.1*height-3), (point_x,0.1*height+10),],fill='#505050')

			draw.text((point_x-txt_size[0]/2,0.1*height-txt_size[1]-4),txt,font=self.settings.name_font_pil,fill='#505050')

	# ---------------------------------------------------------------------------
	#
	def plot_info(self,draw,chromat,visible_range_min,visible_range_max):

		txt=[]
		txt.append(('%s (+%d)' % (mssputil.get_sequence(self.peak),chromat.charge)))
		txt.append(('m/z = %0.4f' % chromat.mz))
		txt.append(('(min; max) = (%0.2g; %0.2g)' % (visible_range_min,visible_range_max)))

		frame_size=[0,0]
		for i in range(0,len(txt)):
			text_size=draw.textsize(txt[i],font=self.settings.name_font_pil)
			if text_size[0]>frame_size[0]:
				frame_size[0]=text_size[0]
			frame_size[1]=frame_size[1]+text_size[1]
		frame_size[0]=frame_size[0]+6
		frame_size[1]=frame_size[1]+1

		#draw.polygon([(2,2),(2+frame_size[0],2),(2+frame_size[0],2+frame_size[1]),(2,2+frame_size[1])],fill='#FFFFDD',outline='#505050')

		for i in range(0,len(txt)):
			draw.text((5,2+i*frame_size[1]/len(txt)),txt[i],font=self.settings.name_font_pil,fill='#505050')

	# ---------------------------------------------------------------------------
	#
	def reset_osd(self):

		self.drag_line=self.canvas.create_line(0,0,0,0,width=1,fill='gray')
		self.drag_line2=self.canvas2.create_line(0,0,0,0,width=1,fill='gray')

		self.move_text=self.canvas.create_text(0,0,text='',font=self.settings.name_font,fill='black')
		self.move_text2=self.canvas2.create_text(0,0,text='',font=self.settings.name_font,fill='black')

	# ---------------------------------------------------------------------------
	#
	def plot_chromatogram(self,peak,get_charge=0):

		self.peak=peak
		self.spectrum=self.peak.spectrum
		self.peptide=mssputil.get_peptide(self.peak)
		print self.peptide
		
		charge=mssputil.get_charge(self.peak)
		charge2=self.get_second_charge(get_charge)

		if self.peptide.get_charge_index(charge2)!=-1:
			self.peak2=self.peptide.get_charge(charge2).get(0)
		else:
			self.peak2=None

		#zapamietanie obecnej pozycji piku
		if self.settings.org_rt==111111 and self.settings.org_mz==111111:
			self.settings.org_rt=self.peak.frequency[0]
			self.settings.org_mz=self.peak.frequency[1]

		isodistrib=msenvelope.MSIsoDistrib(self.peak,0.05)

		#pobranie parametrow chromatogramu
		mz_margin=self.settings.mz_margin
		peaks_count=self.settings.peaks_count
		data_threshold=self.settings.threshold
		require_max=self.settings.require_max

		#parametry filtracji
		filter_type=self.settings.filter_type
		filter_span=self.settings.filter_span
		filter_order=self.settings.filter_order

		#parametry peak-pickingu
		max_threshold=float(self.settings.max_elimination.get())*self.settings.max_threshold
		sort_peaks=mschromatogram.SORT_RT

		#parametry kary za odstepstwo od czasu retencji
		if self.settings.rt_penality.get()==1:
			rt_penality_fun=self.settings.rt_penality_fun
			rt_fwmh=self.settings.rt_penality_s
			massref_rt=mssputil.get_massref_rt(self.peak)
			if self.settings.spectrum_rt_correction==True:
				massref_rt=mscalib.get_calibrated_rt(massref_rt,self.spectrum)
				
		else:
			rt_penality_fun=mschromatogram.PENALITY_NONE
			rt_fwmh=0
			massref_rt=0
		
		if self.chromat==None:
			if self.settings.spectrum_mz_correction==True:
				mz_correction=mssputil.get_mz_correction(self.spectrum,isodistrib.get_peak_mz(0,charge))
			else:
				mz_correction=0

			self.chromat,msg=mschromatogram.get_chromatogram(self.spectrum,isodistrib,charge,mz_correction,mz_margin,peaks_count,data_threshold,require_max,rt_penality_fun,massref_rt,rt_fwmh,None,self.settings.rt_penality_expand)
			if filter_type!=msfilter.FILTER_NONE:
				mschromatogram.filter_chromatogram(self.chromat,filter_type,filter_span,filter_order)
			mschromatogram.get_chromatogram_peaks(self.chromat,max_threshold,sort_peaks,massref_rt,True)
			
		if self.chromat2==None:
			if self.settings.spectrum_mz_correction==True:
				mz_correction=mssputil.get_mz_correction(self.spectrum,isodistrib.get_peak_mz(0,charge2))
			else:
				mz_correction=0

			self.chromat2,msg=mschromatogram.get_chromatogram(self.spectrum,isodistrib,charge2,mz_correction,mz_margin,peaks_count,data_threshold,require_max,rt_penality_fun,massref_rt,rt_fwmh,None,self.settings.rt_penality_expand)
			if filter_type!=msfilter.FILTER_NONE:
				mschromatogram.filter_chromatogram(self.chromat2,filter_type,filter_span,filter_order)
			mschromatogram.get_chromatogram_peaks(self.chromat2,max_threshold,sort_peaks,massref_rt,True)
			
			self.chromat2.rt_visible_range=self.chromat.rt_visible_range

		self.top.title(self.title + mssputil.get_sequence(self.peak))
		self.canvas.configure(width=max(self.chromat.points_count,900))
		self.canvas2.configure(width=max(self.chromat.points_count,900))
		
		width=float(self.canvas.cget('width'))
		height=float(self.canvas.cget('height'))
		self.scale_x=width/(self.chromat.rt_visible_range[1]-self.chromat.rt_visible_range[0])

		self.plot_chromat(self.chromat,self.canvas,self.peak)
		self.plot_chromat(self.chromat2,self.canvas2,self.peak2)

		self.reset_osd()

	# ---------------------------------------------------------------------------
	#
	def get_second_charge(self,get_charge):

		if get_charge==1:
			charge=3
			current_charge=mssputil.get_charge(self.peak)
			if self.peptide.size()>1:
				for i in range(0,len(self.peptide)):
					query=self.peptide.get(i)
					if query.charge!=current_charge:
						charge=query.charge
						break
			else:
				mz_range=mssputil.get_mz_range(self.spectrum)
				for i in range(1,self.charge_menu.get_max_charge()+1):
					mz=mssputil.m2mz(self.peptide.mass,i)
					if i!=current_charge and mz>mz_range[0] and mz<mz_range[1]:
						charge=i
						break
			#!!!!!!111jakos wylaczyc callback
#			self.charge_menu.add_callback(self.empty_charge_cb)
			self.charge_menu.set_value(charge)
			self.charge_menu.add_callback(self.charge_cb)
		else:
			charge=self.charge_menu.get_value()

		return(charge)
	
	# ---------------------------------------------------------------------------
	#
	def save(self,canvas,chromat):

		filetypes=[('PNG image','.png'),('JPEG image','.jpg'),('BMP image','.bmp'),('Postcript','.ps'),('Text file','.txt')]

		path=mstkutil.save_file(canvas,'Save chromatogram','save_chromatogram',filetypes)
		if path!=None and len(path)>0:
			format=0
			for i in range(0,len(filetypes)):
				if path.endswith(filetypes[i][1]):
					format=i
					break
			if not path.endswith(filetypes[format][1]):
				path=path+filetypes[format][1]

			if format==0 or format==1 or format==2:
				canvas.image.save(path)
			elif format==3:
				canvas.postscript(file=path,pageheight=int(canvas.cget('height')),pagewidth=int(canvas.cget('width')))
			elif format==4:
				chromat.write(path)
				
# -----------------------------------------------------------------------------
#
def show_chromatogram(session):

	spectrum=session.selected_spectrum()
	if spectrum!=None and hasattr(spectrum,'is_ms'):
		peaks=spectrum.selected_peaks()
		if peaks!=None and len(peaks)==1 and peaks[0].note!=None and peaks[0].note!='':
			spectrum_settings=mschromatogram_settings.get_chromatogram_settings(spectrum)

			d=chromatogram_view_dialog(session,spectrum_settings);
			d.plot_chromatogram(peaks[0],1)
			d.show_window(1)

# -----------------------------------------------------------------------------
#
def show_peak_chromatogram(peak):

	if peak!=None:
		spectrum_settings=mschromatogram_settings.get_chromatogram_settings(peak.spectrum)

		d=chromatogram_view_dialog(peak.spectrum.session,spectrum_settings);
		d.plot_chromatogram(peak,1)
		d.show_window(1)
