#-*- coding: windows-1250 -*-

import sys
import os
import Tkinter
import tkFont
import ImageFont

import Pmw

import mschromatogram
import msfindpeptides_settings
import msfilter
import mstkutil

class chromatogram_settings:

	def __init__(self,session,path=None):

		self.session=session
		self.loaded=0

		if path!=None:
			self.loaded=self.read_settings(path)

		try:
			num=int(self.session.project.saved_value('CH_SPECTRUM_RT_CORRECTION'))
			if num<0 or num>1:
				raise ValueError
			self.spectrum_rt_correction=num
		except (ValueError,TypeError):
			self.spectrum_rt_correction=True

		try:
			num=int(self.session.project.saved_value('CH_SPECTRUM_MZ_CORRECTION'))
			if num<0 or num>1:
				raise ValueError
			self.spectrum_mz_correction=num
		except (ValueError,TypeError):
			self.spectrum_mz_correction=True

		try:
			num=int(self.session.project.saved_value('CH_SPECTRUM_RP_CORRECTION'))
			if num<0 or num>1:
				raise ValueError
			self.spectrum_rp_correction=num
		except (ValueError,TypeError):
			self.spectrum_rp_correction=True

		try:
			num=int(self.session.project.saved_value('CH_PEAKS_COUNT'))
			if num<0:
				raise ValueError
			self.peaks_count=num
		except (ValueError,TypeError):
			self.peaks_count=2

		try:
			nums_str=self.session.project.saved_value('CH_MZ_RANGE')
			if nums_str!=None:
				fields=nums_str.split(',')
				if len(fields)==2:
					num1=int(fields[0])
					num2=int(fields[1])
				if num1<0 or num2<0:
					raise ValueError
				self.mz_margin=(num1,num2)
			else:
				raise ValueError
		except (ValueError,TypeError):
			self.mz_margin=(20,20)

		try:
			num=int(self.session.project.saved_value('CH_REQUIRE_MAX'))
			if num<0 or num>1:
				raise ValueError
			if num==0:
				self.require_max=False
			else:
				self.require_max=True
		except (ValueError,TypeError):
			self.require_max=True

		try:
			num=float(self.session.project.saved_value('CH_THRESHOLD'))
			if num<0:
				raise ValueError
			self.threshold=num
		except (ValueError,TypeError):
			self.threshold=5000

		try:
			str=self.session.project.saved_value('CH_FILTER_TYPE')
			filter_type=msfilter.get_filter_type(str)
			if filter_type==-1:
				raise ValueError
			self.filter_type=filter_type
		except (ValueError,TypeError):
			self.filter_type=msfilter.FILTER_GAUSS

		try:
			num=float(self.session.project.saved_value('CH_FILTER_SPAN'))
#			poprawic dla Gaussian i innych
			if num<0:
				raise ValueError
			self.filter_span=num
		except (ValueError,TypeError):
			self.filter_span=1.5

		try:
			num=int(self.session.project.saved_value('CH_FILTER_ORDER'))
#			poprawic dla Gaussian i innych
			if num<0:
				raise ValueError
			self.filter_order=num
		except (ValueError,TypeError):
			self.filter_order=0

		self.rt_penality=Tkinter.IntVar()
		try:
			num=int(self.session.project.saved_value('CH_RT_PENALITY'))
			if num<0 or num>1:
				raise ValueError
			self.rt_penality.set(num)
		except (ValueError,TypeError):
			self.rt_penality.set(1)

		try:
			num=int(self.session.project.saved_value('CH_RT_PENALITY_FUN'))
			if num<mschromatogram.PENALITY_NONE or num>mschromatogram.PENALITY_SQUARE:
				raise ValueError
			self.rt_penality_fun=num
		except (ValueError,TypeError):
			self.rt_penality_fun=mschromatogram.PENALITY_GAUSS

		try:
			num=float(self.session.project.saved_value('CH_RT_PENALITY_S'))
			if num<0:
				raise ValueError
			self.rt_penality_s=num
		except (ValueError,TypeError):
			self.rt_penality_s=5.0

		try:
			nums_str=self.session.project.saved_value('CH_RT_PENALITY_EXPAND')
			if nums_str!=None:
				fields=nums_str.split(',')
				if len(fields)==2:
					num1=int(fields[0])
					num2=int(fields[1])
				if num1<0 or num1>1 or num2<0 or num2>0:
					raise ValueError
				self.rt_penality_expand=(num1,num2)
			else:
				raise ValueError
		except (ValueError,TypeError):
			self.rt_penality_expand=(1,1)
			
		self.max_elimination=Tkinter.IntVar()
		try:
			num=int(self.session.project.saved_value('CH_MAX_ELIMINATION'))
			if num<0 or num>1:
				raise ValueError
			self.max_elimination.set(num)
		except (ValueError,TypeError):
			self.max_elimination.set(1)

		try:
			num=float(self.session.project.saved_value('CH_MAX_THRESHOLD'))
			if num<0:
				raise ValueError
			self.max_threshold=num
		except (ValueError,TypeError):
			self.max_threshold=2

		self.view_rt_penality=Tkinter.IntVar()
		try:
			num=int(self.session.project.saved_value('CH_VIEW_RT_PENALITY'))
			if num<0 or num>1:
				raise ValueError
			self.view_rt_penality.set(num)
		except (ValueError,TypeError):
			self.view_rt_penality.set(0)

		self.view_peptide_position=Tkinter.IntVar()
		try:
			num=int(self.session.project.saved_value('CH_VIEW_PEPTIDE_POSITION'))
			if num<0 or num>1:
				raise ValueError
			self.view_peptide_position.set(num)
		except (ValueError,TypeError):
			self.view_peptide_position.set(1)

		self.view_shape_coeffs=Tkinter.IntVar()
		try:
			num=int(self.session.project.saved_value('CH_VIEW_SHAPE_COEFFS'))
			if num<0 or num>1:
				raise ValueError
			self.view_shape_coeffs.set(num)
		except (ValueError,TypeError):
			self.view_shape_coeffs.set(1)

		self.view_info=Tkinter.IntVar()
		try:
			num=int(self.session.project.saved_value('CH_VIEW_INFO'))
			if num<0 or num>1:
				raise ValueError
			self.view_info.set(num)
		except (ValueError,TypeError):
			self.view_info.set(1)

			
		self.current_max=0

		self.org_rt=111111
		self.org_mz=111111

		self.spectrum_correction=Tkinter.IntVar(0)
		if self.spectrum_mz_correction==1 or self.spectrum_rp_correction==1 or self.spectrum_rt_correction==1:
			self.spectrum_correction.set(1)
			
		font_size=10
		if sys.platform=='win32':
			font_size=8
		self.name_font=tkFont.Font(family="Courier",size=font_size,weight="bold")
		self.name_font_pil=mstkutil.load_pil_font('arial',10)
		
		
	# --------------------------------------------------------------------------
	#
	def read_settings(self,path):
		result=0

		if path!=None:
			try:
				file=open(path,'r')
				lines=file.readlines()

				for i in range(0,len(lines)):
					line=lines[i].strip()
					if len(line)!=0 and line[0]!='#':
						fields=lines[i].split('=')
						if len(fields)==2:
							self.session.project.save_value(fields[0].strip(),fields[1].strip())
				file.close()
				result=1
			except IOError, (errno, strerror):
				print "I/O error(%s): %s" % (errno, strerror)
				result=0

		return result
	
# --------------------------------------------------------------------------
#
def get_chromatogram_settings(spectrum):

	settings=chromatogram_settings(spectrum.session,os.path.join(os.environ.get('SPARKY_INSTALL'),os.path.join('config','chromatogram.cfg')))

	if spectrum.processed==1 and spectrum.ms_processing!=None and len(spectrum.ms_processing)>1:
		#???????co zrobic z use_rt_correction i use_mz_correction

		search_settings=msfindpeptides_settings.get_search_settings(spectrum)

		settings.peaks_count=search_settings.ch_peaks_count
		settings.mz_margin=search_settings.ch_mz_margin
		settings.threshold=search_settings.ch_threshold
		settings.require_max=search_settings.ch_require_max

		settings.filter_type=search_settings.ch_filter_type
		settings.filter_span=search_settings.ch_filter_span
		settings.filter_order=search_settings.ch_filter_order

		settings.rt_penality.set(search_settings.ch_rt_penality)
		settings.rt_penality_fun=search_settings.ch_rt_penality_fun
		if settings.rt_penality_fun==mschromatogram.PENALITY_NONE:
			settings.rt_penality.set(0)
		settings.rt_penality_s=search_settings.ch_rt_penality_s
		settings.rt_penality_expand=search_settings.ch_rt_penality_expand
		
		settings.max_elimination.set(search_settings.ch_max_elimination)
		settings.max_threshold=search_settings.ch_max_threshold
		if settings.max_threshold==0:
			settings.max_elimination.set(0)

	return settings

# -----------------------------------------------------------------------------
#
class chromatogram_settings_dialog(mstkutil.Settings_Dialog):

	def __init__(self, session):
		self.session=session
		self.loaded=0

		mstkutil.Settings_Dialog.__init__(self,session.tk,'Settings')

		Pmw.initialise(session.tk)
		balloon=Pmw.Balloon(self.top,initwait=1500)

		#---------------------------------------------------------------------
		#
		tab_panel=Pmw.NoteBook(self.top,borderwidth=1)
		tab_panel.pack(fill='both',expand=1,padx=1,pady=3)


		#---------------------------------------------------------------------
		#
		tab=tab_panel.add(' Calibration ')
		#
		f=Pmw.Group(tab,tag_text='m/z calibration:')
		f.pack(side='top',anchor='nw',fill='x',padx=0,pady=0)

		#wprowadzic przycisk usowajacy wszystkie kalibracje
		cb=mstkutil.checkbutton(f.interior(),'Use spectrum m/z calibration',0)
		cb.button.grid(row=0,column=0,sticky='w',padx=2,pady=2)
		self.spectrum_mz_correction_check=cb

		f.interior().columnconfigure(0, weight=1)

		#
		f=Pmw.Group(tab,tag_text='RT calibration:')
		f.pack(side='top',anchor='nw',fill='x',padx=0,pady=0)


		cb=mstkutil.checkbutton(f.interior(),'Use spectrum RT calibration',0)
		cb.button.grid(row=3,column=0,sticky='w',padx=2,pady=2)
		self.spectrum_rt_correction_check=cb

		f.interior().columnconfigure(0, weight=1)

		#
		f=Pmw.Group(tab,tag_text='RP calibration:')
		f.pack(side='top',anchor='nw',fill='x',padx=0,pady=0)

		cb=mstkutil.checkbutton(f.interior(),'Use spectrum RP calibration',0)
		cb.button.grid(row=2,column=0,sticky='w',padx=2,pady=2)
		self.spectrum_rp_correction_check=cb

		f.interior().columnconfigure(0, weight=1)

		#---------------------------------------------------------------------
		#
		tab=tab_panel.add(' Chromatogram ')

		#
		f=Pmw.Group(tab,tag_text='Chromatogram:')
		f.pack(side='top',anchor='nw',fill='x',padx=0,pady=0)

		l=Tkinter.Label(f.interior(),text='Number of peaks:')
		l.grid(row=0,column=0,sticky='w',padx=2,pady=2)

		e=mstkutil.num_entry_field(f.interior(),mstkutil.ENTRY_INT,'',0,10,'',12)
		e.frame.grid(row=0,column=1,columnspan=2,sticky='e',padx=2,pady=2)
		balloon.bind(e.entry,u'Liczba pikow uwzgldnianych podczas wyznaczania chromatogramu\n(0 oznacza wszystkie piki obwiedni teoretycznej).')
		self.entries.append(e)
		self.peaks_count_edit=e

		l=Tkinter.Label(f.interior(),text='m/z range [ppm]: ')
		l.grid(row=1,column=0,sticky='w',padx=2,pady=2)

		ff=Tkinter.Frame(f.interior())
		ff.grid(row=1,column=1,columnspan=2,sticky='e')
		e=mstkutil.num_entry_field(ff,mstkutil.ENTRY_FP,'',1,1000,'',5)
		e.frame.grid(row=1,column=1,sticky='e',padx=2,pady=2)
		balloon.bind(e.entry,u'Zakres tolerancji m/z chromatogramu\ndla pojedynczego piku.')
		self.entries.append(e)
		self.mz_range_edit1=e

		e=mstkutil.num_entry_field(ff,mstkutil.ENTRY_FP,'',1,1000,'',5)
		e.frame.grid(row=1,column=2,sticky='e',padx=3,pady=2)
		balloon.bind(e.entry,u'Zakres tolerancji m/z chromatogramu\ndla pojedynczego piku.')
		self.entries.append(e)
		self.mz_range_edit2=e

		l=Tkinter.Label(f.interior(),text='Intensity threshold: ')
		l.grid(row=2,column=0,sticky='w',padx=2,pady=2)

		e=mstkutil.num_entry_field(f.interior(),mstkutil.ENTRY_FP,'',1,10000000,'',12)
		e.frame.grid(row=2,column=1,columnspan=2,sticky='e',padx=2,pady=2)
		balloon.bind(e.entry,u'Minimalna warto widma uwzgldniana podczas\nwyznaczania chromatogramu.')
		self.entries.append(e)
		self.threshold_edit=e

		cb=mstkutil.checkbutton(f.interior(),'Require local max. in m/z range',0)
		cb.button.grid(row=3,columnspan=2,sticky='w',padx=2,pady=2)
		balloon.bind(cb.button,u'Zerowanie wartoci chromatogramu gdy w zakresie tolerancji m/z\nnie wystpuje maksimum lokalne widma.')
		self.entries.append(e)
		self.require_max_check=cb

		f.interior().columnconfigure(0, weight=1)
		f.interior().columnconfigure(1, weight=1)
		f.interior().columnconfigure(2, weight=1)

		#
		f=Pmw.Group(tab,tag_text='Filtering:')
		f.pack(side='top',anchor='nw',fill='x',padx=0,pady=0)

		l=Tkinter.Label(f.interior(),text='Filter type: ')
		l.grid(row=0,column=0,sticky='w',padx=2,pady=2)

		m=mstkutil.option_menu(f.interior(),'',msfilter.FILTER_STR,msfilter.FILTER_STR[msfilter.FILTER_GAUSS])
		m.frame.grid(row=0,column=1,columnspan=2,sticky='e',pady=2,padx=2)
		balloon.bind(m.menubutton,u'Rodzaj filtru wygadzajcego chromatogram.')
		self.filter_type_menu=m
		self.filter_type_menu.add_callback(self.filter_type_cb)

		l=Tkinter.Label(f.interior(),text='Filter span: ')
		l.grid(row=1,column=0,sticky='w',padx=2,pady=2)

		e=mstkutil.num_entry_field(f.interior(),1,'',3,1000000,'',12,cb=self.filter_params_cb)
		balloon.bind(e.entry,u'Szeroko filtru wygadzajcego chromatogram.')
		e.frame.grid(row=1,column=1,sticky='e',padx=2,pady=1)
		self.entries.append(e)
		self.filter_span_edit=e

		l=Tkinter.Label(f.interior(),text='Filter order: ')
		l.grid(row=2,column=0,sticky='w',padx=2,pady=2)

		e=mstkutil.num_entry_field(f.interior(),1,'',0,1000000,'',12,cb=self.filter_params_cb)
		e.frame.grid(row=2,column=1,sticky='e',padx=2,pady=2)
		balloon.bind(e.entry,u'Rzd filtru wygadzajcego chromatogram.')
		self.entries.append(e)
		self.filter_order_edit=e

		f.interior().columnconfigure(0, weight=1)
		f.interior().columnconfigure(1, weight=1)


		#
		f=Pmw.Group(tab,tag_text='Postprocessing:')
		f.pack(side='top',anchor='nw',fill='x',padx=0,pady=0)

		cb=mstkutil.checkbutton(f.interior(),'Use RT penality function',0)
		balloon.bind(cb.button,u'Funkcja kary za odstpstwo od przewidywanego\nczasu retencji.')
		cb.button.grid(row=0,column=0,columnspan=2,sticky='w',pady=2)
		self.rt_penality_check=cb
		self.rt_penality_check.add_callback(self.rt_penality_cb)

		l=Tkinter.Label(f.interior(),text='Penality function width [min]: ')
		l.grid(row=1,column=0,sticky='w',padx=2,pady=2)

		e=mstkutil.num_entry_field(f.interior(),mstkutil.ENTRY_FP,'',0,1000000,'',12)
		balloon.bind(e.entry,u'Seroko funkcji kary za odstpstwo od przewidywanego\nczasu retencji.')
		e.frame.grid(row=1,column=1,sticky='e',padx=2,pady=2)
		self.entries.append(e)
		self.rt_penality_edit=e

		cb=mstkutil.checkbutton(f.interior(),'Eliminate insignificant peaks',0)
		balloon.bind(cb.button,u'Pomijanie mao znaczcych pikw chromatogramu.')
		cb.button.grid(row=2,column=0,columnspan=2,sticky='w',pady=2)
		self.max_elimination_check=cb
		self.max_elimination_check.add_callback(self.max_elimination_cb)

		l=Tkinter.Label(f.interior(),text='Peak elimination threshold: ')
		l.grid(row=3,column=0,sticky='w',padx=2,pady=2)

		e=mstkutil.num_entry_field(f.interior(),mstkutil.ENTRY_FP,'',0,1000000,'',12)
		balloon.bind(e.entry,u'Wspczynnik okrelajcy wymagan stromo zbocza narastajcego piku.')
		e.frame.grid(row=3,column=1,sticky='e',padx=2,pady=2)
		self.entries.append(e)
		self.max_threshold_edit=e

		f.interior().columnconfigure(0, weight=1)
		f.interior().columnconfigure(1, weight=1)

		#
		#
		#
		#
		tab_panel.selectpage(' Chromatogram ')
		tab_panel.setnaturalsize()

		#---------------------------------------------------------------------
		#
		br=mstkutil.button_row(self.top,
				('Apply', self.apply_settings_cb),
				('Load', self.load_cb),
				('Close', self.close_cb),
				)
		br.frame.pack(side='bottom',anchor='w',pady=2)

		self.top.geometry("+0+0")
		self.top.resizable(0,0)


	# ---------------------------------------------------------------------------
	#
	def load_cb(self):
		path=mstkutil.load_file(self.top,'Open configuration','open_cfg')
		if path!=None:
			settings=chromatogram_settings(self.session,path)
			self.loaded=settings.loaded
			self.show_settings(settings)


	# ---------------------------------------------------------------------------
	#
	def filter_type_cb(self,event=None):

		filter_type=msfilter.get_filter_type(self.filter_type_menu.variable.get())
		if filter_type==msfilter.FILTER_NONE:
			self.filter_span_edit.entry.configure(state='disabled')
			self.filter_order_edit.entry.configure(state='disabled')
		else:
			self.filter_span_edit.entry.configure(state='normal')
			if filter_type==msfilter.FILTER_GAUSS:
				self.filter_span_edit.entry_type=mstkutil.ENTRY_FP
				self.filter_span_edit.min_value=0
				#uzaleznic filter span od zakresu RT
				self.filter_span_edit.variable.set(1.5)
				self.filter_order_edit.min_value=0
				self.filter_order_edit.max_value=2
				self.filter_order_edit.variable.set(0)
				state='normal'
			else:
				self.filter_span_edit.entry_type=mstkutil.ENTRY_INT
				if filter_type==msfilter.FILTER_SG:
					self.filter_span_edit.min_value=5
					self.filter_span_edit.variable.set(5)
					self.filter_order_edit.min_value=2
					self.filter_order_edit.max_value=100
					self.filter_order_edit.variable.set(2)
					state='normal'
				else:
					self.filter_span_edit.min_value=3
					self.filter_span_edit.variable.set(3)
					self.filter_order_edit.min_value=0
					self.filter_order_edit.variable.set(0)
					state='disabled'
			self.filter_order_edit.entry.configure(state=state)

	# ---------------------------------------------------------------------------
	#
	def filter_params_cb(self,event=None):
		result=True
		span_error=False
		order_error=False

		if self.filter_span_edit.validate()==None:
			span_error=True
		if self.filter_order_edit.validate()==None:
			order_error=True

		if span_error==False and order_error==False:
			filter_type=self.filter_type_menu.variable.get()
			if filter_type!=msfilter.FILTER_STR[msfilter.FILTER_GAUSS]:
				span=int(self.filter_span_edit.variable.get())
				order=int(self.filter_order_edit.variable.get())

				if span%2!=1:
					span_error=True
				if filter_type==msfilter.FILTER_STR[msfilter.FILTER_SG] and span <= order+2:
					order_error=True

		if span_error==True:
			self.filter_span_edit.entry.configure(background='pink')
		else:
			self.filter_span_edit.entry.configure(background='white')

		if order_error==True:
			self.filter_order_edit.entry.configure(background='pink')
		else:
			self.filter_order_edit.entry.configure(background='white')

		result=(not span_error and not order_error)

		return result

	# ---------------------------------------------------------------------------
	#
	def rt_penality_cb(self,event):

		if self.rt_penality_check.variable.get()==1:
			self.rt_penality_edit.entry.configure(state='normal')
		else:
			self.rt_penality_edit.entry.configure(state='disabled')

	# ---------------------------------------------------------------------------
	#
	def max_elimination_cb(self,event):

		if self.max_elimination_check.variable.get()==1:
			self.max_threshold_edit.entry.configure(state='normal')
		else:
			self.max_threshold_edit.entry.configure(state='disabled')

	# ---------------------------------------------------------------------------
	#
	def show_settings(self,settings):

		self.spectrum_mz_correction_check.variable.set(settings.spectrum_mz_correction)
		self.spectrum_rp_correction_check.variable.set(settings.spectrum_rp_correction)
		self.spectrum_rt_correction_check.variable.set(settings.spectrum_rt_correction)

		self.peaks_count_edit.variable.set(settings.peaks_count)

		self.mz_range_edit1.variable.set(settings.mz_margin[0])
		self.mz_range_edit2.variable.set(settings.mz_margin[1])
		self.require_max_check.variable.set(settings.require_max)
		self.threshold_edit.variable.set(settings.threshold)

		self.filter_type_menu.variable.set((msfilter.FILTER_STR[settings.filter_type]))
		self.filter_span_edit.variable.set(settings.filter_span)
		self.filter_order_edit.variable.set(settings.filter_order)
		self.filter_params_cb()

		self.rt_penality_check.variable.set(settings.rt_penality.get())
		self.rt_penality_edit.variable.set(settings.rt_penality_s)

		self.max_elimination_check.variable.set(settings.max_elimination.get())
		self.max_threshold_edit.variable.set(settings.max_threshold)

	# ---------------------------------------------------------------------------
	#
	def get_settings(self):
		settings=chromatogram_settings(self.session)

		settings.spectrum_mz_correction=self.spectrum_mz_correction_check.variable.get()
		settings.spectrum_rp_correction=self.spectrum_rp_correction_check.variable.get()
		settings.spectrum_rt_correction=self.spectrum_rt_correction_check.variable.get()

		settings.peaks_count=int(self.peaks_count_edit.variable.get())
		settings.mz_margin=(float(self.mz_range_edit1.variable.get()),float(self.mz_range_edit2.variable.get()))
		if self.require_max_check.variable.get()==1:
			settings.require_max=True
		else:
			settings.require_max=False
		settings.threshold=float(self.threshold_edit.variable.get())

		settings.filter_type=msfilter.get_filter_type(self.filter_type_menu.variable.get())
		if settings.filter_type==msfilter.FILTER_NONE:
			settings.filter_span=0
		elif settings.filter_type==msfilter.FILTER_GAUSS:
			settings.filter_span=float(self.filter_span_edit.variable.get())
		else:
			settings.filter_span=int(self.filter_span_edit.variable.get())
		settings.filter_order=int(self.filter_order_edit.variable.get())

		settings.rt_penality.set(self.rt_penality_check.variable.get())
		settings.rt_penality_s=float(self.rt_penality_edit.variable.get())

		settings.max_elimination.set(self.max_elimination_check.variable.get())
		settings.max_threshold=float(self.max_threshold_edit.variable.get())

		if self.loaded==0:
			settings.view_rt_penality.set(self.parent_dialog.settings.view_rt_penality.get())
			settings.view_peptide_position.set(self.parent_dialog.settings.view_peptide_position.get())

		self.org_rt=self.parent_dialog.settings.org_rt
		self.org_mz=self.parent_dialog.settings.org_mz

		return settings

	# ---------------------------------------------------------------------------
	#
	def apply_settings_cb(self):

		if self.filter_params_cb()==False:
			mstkutil.warning_dialog(self.top,'Incorrect value detected!!!',title='Warning')
			return False
		
		if not self.check_entries():
			return False

		self.show_window(0)
		self.apply_cb()
		self.close_cb()