###########################################################
#SetContourLevels v5.0 Tymon Rubel, Jacek Sikora 2005-2006#
###########################################################

import sparky
import Tkinter
import mssputil
import mstkutil
import pyutil

# -----------------------------------------------------------------------------
#
class contour_levels_dialog(mstkutil.Dialog):
	
	def __init__(self, session):
		self.session=session
		mstkutil.Dialog.__init__(self, session.tk, 'MS contour levels')

		f=Tkinter.Frame(self.top,borderwidth=2,relief='groove')
		f.pack(side='top',anchor='w',pady=2)

		l=Tkinter.Label(f,text='View:')
		l.grid(row=0,column=0,sticky='w')

		sf=Tkinter.Frame(f,borderwidth=0,relief='flat')
		sf.grid(row=0,column=1,sticky='w')

		m=mssputil.view_menu(session,sf,'',self)
		m.frame.pack(side='left',pady=2)
		self.view_menu=m

		cb=mstkutil.checkbutton(sf,'Apply to all',0)
		cb.button.pack(side='left')
		self.check=cb

		l=Tkinter.Label(f,text='Color:')
		l.grid(row=1,column=0,sticky='w')

		m=mstkutil.option_menu(f,'',('red-blue','green-blue','red-yellow','green','blue','red','yellow','cyan','magenta','orange','brown','gold','pink','purple','white','black','tomato'),'red-blue')
		m.frame.grid(row=1,column=1,sticky='w',pady=2)
		m.add_callback(self.event_cb)
		self.color_menu=m

		l=Tkinter.Label(f,text='Lowest:')
		l.grid(row=2,column=0,sticky='w')

		sf=Tkinter.Frame(f,borderwidth=0,relief='flat')
		sf.grid(row=2,column=1,sticky='w')

		e=mstkutil.entry_field(sf,'','1e+006',8,None,'right')
		e.entry.bind('<Return>',self.event_cb)
		e.frame.pack(side='left')
		self.min_edit=e

		br=mstkutil.button_row(sf,
			   ('E+0', self.min_e0_cb),
			   ('E+1', self.min_e1_cb),
			   ('E+2', self.min_e2_cb),
			   ('E+3', self.min_e3_cb),
			   ('E+4', self.min_e4_cb),
			   ('E+5', self.min_e5_cb),
			   ('E+6', self.min_e6_cb))
		br.frame.pack(side='left',padx=3,pady=2)

		l=Tkinter.Label(f,text='Levels:')
		l.grid(row=3,column=0,sticky='w')

		sf=Tkinter.Frame(f,borderwidth=0,relief='flat')
		sf.grid(row=3,column=1,sticky='w')

		e=mstkutil.entry_field(sf,'','10',8,None,'right')
		e.entry.bind('<Return>',self.event_cb)
		e.frame.pack(side='left')
		self.levels_edit=e

		br=mstkutil.button_row(sf,
			   ('10', self.levels_10_cb),
			   ('20', self.levels_20_cb),
			   ('30', self.levels_30_cb),
			   ('40', self.levels_40_cb),
			   ('50', self.levels_50_cb))
		br.frame.pack(side='left',padx=3,pady=2)

		br=mstkutil.button_row(self.top,
				('Close', self.close_cb),
				('Apply', self.set_levels))
		br.frame.pack(side='bottom',anchor='w',pady=2)

		self.removed_notice=session.notify_me('removed spectrum from project',self.update_view_menu)
		self.added_notice=session.notify_me('added spectrum to project',self.update_view_menu)
		self.top.bind('<Destroy>',self.remove_callbacks,1)

	# ---------------------------------------------------------------------------
	#
	def update_view_menu(self,removed_spectrum):
		views=self.session.project.view_list()
		if views==None or len(views)==0:
			self.top.destroy()
		else:
			self.view_menu.update_menu_cb()
			view=self.session.selected_view()
			if view==None:
				view=views[0]
			
			self.view_menu.set(view.name)

	# ---------------------------------------------------------------------------
	#
	def remove_callbacks(self,event):
		if str(self.top)==str(event.widget):
			if sparky.object_exists(self.session):
				if self.removed_notice!=None:
					self.session.dont_notify_me(self.removed_notice)
				if self.added_notice!=None:
					self.session.dont_notify_me(self.added_notice)
				
			self.removed_notice=None
			self.added_notice=None

	# ---------------------------------------------------------------------------
	#
	def event_cb(self,event):
		self.set_levels()

	# ---------------------------------------------------------------------------
	#
	def show_view_levels(self,spectrum):
		
		self.spectrum=spectrum
		view=mssputil.preferred_view(spectrum)
		if view!=None:
			self.view=view
			self.view_menu.set(self.view.name)

			self.color_menu.set(self.view.positive_levels.color)
			self.levels_edit.variable.set(self.view.positive_levels.levels)
			self.min_edit.variable.set(self.view.positive_levels.lowest)

	# ---------------------------------------------------------------------------
	#
	def set_view_levels(self,view):
		contours=sparky.Contour_Levels()
		contours.lowest=-1000000000000
		contours.levels=0
		contours.color='green'
		contours.factor=1.4
		if view.negative_levels.levels!=contours.levels or view.negative_levels.lowest!=contours.lowest or view.negative_levels.color!=contours.color:
			view.negative_levels=contours

		contours=sparky.Contour_Levels()
		contours.lowest=pyutil.string_to_float(self.min_edit.variable.get(),0)
		contours.levels=pyutil.string_to_int(self.levels_edit.variable.get(),0)
		contours.factor=1.4
		contours.color=self.color_menu.get()

		if view.positive_levels.levels!=contours.levels or view.positive_levels.lowest!=contours.lowest or view.positive_levels.color!=contours.color:
			view.positive_levels=contours

	# ---------------------------------------------------------------------------
	#
	def set_levels(self):
		if self.check.state():
			views=self.session.project.view_list()
			for view in views:
				self.set_view_levels(view)
		else:
			view=self.view_menu.view()	
			if view!=None:
				self.set_view_levels(view)
			else:
				view=self.session.selected_view()
				if view!=None:
					self.set_view_levels(view)

	# ---------------------------------------------------------------------------
	#
	def min_e0_cb(self):
		self.min_edit.variable.set('1e+000')
		self.set_levels()

	# ---------------------------------------------------------------------------
	#
	def min_e1_cb(self):
		self.min_edit.variable.set('1e+001')
		self.set_levels()
	
	# ---------------------------------------------------------------------------
	#
	def min_e2_cb(self):
		self.min_edit.variable.set('1e+002')
		self.set_levels()

	# ---------------------------------------------------------------------------
	#
	def min_e3_cb(self):
		self.min_edit.variable.set('1e+003')
		self.set_levels()

	# ---------------------------------------------------------------------------
	#
	def min_e4_cb(self):
		self.min_edit.variable.set('1e+004')
		self.set_levels()

	# ---------------------------------------------------------------------------
	#
	def min_e5_cb(self):
		self.min_edit.variable.set('1e+005')
		self.set_levels()

	# ---------------------------------------------------------------------------
	#
	def min_e6_cb(self):
		self.min_edit.variable.set('1e+006')
		self.set_levels()

	# ---------------------------------------------------------------------------
	#
	def levels_10_cb(self):
		self.levels_edit.variable.set('10')
		self.set_levels()

	# ---------------------------------------------------------------------------
	#
	def levels_20_cb(self):
		self.levels_edit.variable.set('20')
		self.set_levels()

	# ---------------------------------------------------------------------------
	#
	def levels_30_cb(self):
		self.levels_edit.variable.set('30')
		self.set_levels()

	# ---------------------------------------------------------------------------
	#
	def levels_40_cb(self):
		self.levels_edit.variable.set('40')
		self.set_levels()

	# ---------------------------------------------------------------------------
	#
	def levels_50_cb(self):
		self.levels_edit.variable.set('50')
		self.set_levels()

# -----------------------------------------------------------------------------
#
def show_dialog(session):
	spectrum=session.selected_spectrum()
	if spectrum!=None and hasattr(spectrum,'is_ms'):
		d=mssputil.the_dialog(contour_levels_dialog,session)
		d.show_view_levels(spectrum)
		d.show_window(1)


