import os
import sys
import time

import Tkinter
import numpy

import sparky
import ms
import msisoenvelope
import mssputil
import mstkutil

METHOD_STR=(u'\u03B4m/z',u'\u0394RT','score')
METHOD_MZ=0
METHOD_RT=1
METHOD_SCORE=2

# --------------------------------------------------------------------------
#
def get_remove_method(name):
	remove_type=-1
	for i in range(0,len(METHOD_STR)):
		if name.lower()==METHOD_STR[i].lower():
			remove_type=i
			break
	return remove_type

# -----------------------------------------------------------------------------
#
class duplicated_list_dialog(mstkutil.Dialog,mstkutil.Stoppable):

	def __init__(self,session,parent=None):

		self.session=session
		self.parent=parent
		self.title='MS duplicated assignments:'
		self.settings=duplicated_list_settings(self.session,os.path.join(os.environ.get('SPARKY_INSTALL'),os.path.join('config','duplicatedlist.cfg')))

		mstkutil.Dialog.__init__(self,session.tk,self.title)

		f=Tkinter.Frame(self.top,borderwidth=2,relief='groove')
		f.pack(side='top',anchor='w',fill='x')
		
		pl=mssputil.peak_tablelist(f,columns=None,selected_edit=1,height=20,width=75,editendcommand=self.editend_cb,editstartcommand=self.editstart_cb)
		pl.set_columns((2,'P',40,'Sequence',10,'Protein',9,'m/z',9,u'\u03B4m/z',6,'RT',6,u'\u0394RT',5,'Score'))
		pl.set_columns_align(('center','left','left','right','right','right','right','right'))
		pl.tablelist.columnconfigure(0,editable='yes',editwindow='checkbutton')
		pl.list_size.configure(width=60)

		pl.tablelist.bind('<ButtonRelease-1>',pl.select_peak_cb)
		pl.tablelist.bind('<Double-ButtonRelease-1>',pl.goto_peak_cb)
		pl.tablelist.bind('<Return>',pl.goto_peak_cb)
		pl.tablelist.bind('<Button-3>',self.popup_menu)
		
		pl.frame.pack(side='top',fill='both',expand=1)		
		self.duplicate_tablelist=pl

		f=Tkinter.Frame(self.top,borderwidth=2,relief='groove')
		f.pack(side='top',anchor='w',fill='x')

		l=Tkinter.Label(f,text='Search for:',anchor='w',width=10)
		l.pack(side='left',padx=3)

		e=mstkutil.entry_field(f,'','',45)
		e.frame.pack(side='left',padx=2)
		e.entry.bind('<Return>',self.find_string)
		self.find_edit=e

		m=mstkutil.option_menu(f,'',('Search down','Search up'),'Search down')
		m.frame.pack(side='left',padx=3)
		self.direction_menu=m

		cb=mstkutil.checkbutton(f,'Only exact match',0)
		cb.button.pack(side='left',padx=3)
		self.match_check=cb

		b=Tkinter.Button(f,text="Find",borderwidth=1,width='7',command=self.find_string)
		b.pack(side='right',ipady=2,padx=3,pady=3)
		self.find_button=b

		progress_label=Tkinter.Label(self.top, anchor = 'nw')
		progress_label.pack(side='top',anchor='w')

		br=mstkutil.button_row(self.top,
					('Update', self.update_cb),
					('Stop', self.stop_cb),
					('Save', self.duplicate_tablelist.save_cb),
					('Close', self.closing_window),
#					('Close', self.top.destroy),
					)
		br.frame.pack(side='top',anchor='w',pady=2)
		self.buttons_row=br

		self.top.geometry("+0+0")
		self.top.resizable(1,0)

		self.notice=session.notify_me('removed spectrum from project',self.spectrum_removed_cb)
		self.selection_notice=session.notify_me('selection changed',self.selection_changed_cb)
		self.top.bind('<Destroy>',self.remove_callbacks,1)
		self.top.protocol("WM_DELETE_WINDOW",self.closing_window)

		mstkutil.Stoppable.__init__(self,progress_label,br.buttons[1])

	# ---------------------------------------------------------------------------
	#
	def spectrum_removed_cb(self,removed_spectrum):
		if self.spectrum==removed_spectrum:
			self.top.destroy()

	# ---------------------------------------------------------------------------
	#
	def selection_changed_cb(self):
		peaks=self.session.selected_peaks()
		if peaks!=None and len(peaks)==1:
			self.duplicate_tablelist.mark_peak_cb(peaks[0])

	# ---------------------------------------------------------------------------
	#
	def closing_window(self):
#		print 'closing_window: start'
		if self.settings.lock_callbacks==0:
#			print 'closing_window: no lock'
			if self.parent!=None:
#				print 'closing_window: parent.find_duplicates_dialog=None'
				self.parent.find_duplicates_dialog=None

			self.top.destroy()

	# ---------------------------------------------------------------------------
	#
	def remove_callbacks(self,event):
		
		if str(self.top)==str(event.widget):
			if sparky.object_exists(self.session):
				if self.notice!=None:
					self.session.dont_notify_me(self.notice)
				if self.selection_notice!=None:
					self.session.dont_notify_me(self.selection_notice)
				
			self.notice=None
			self.selection_notice=None

	# ---------------------------------------------------------------------------
	#
	def editstart_cb(self,tbl,row,column,text):
		value=0
		self.duplicate_tablelist.lock_callbacks=1
		if column=='0':
			if self.duplicate_tablelist.get_cell_config(int(row),0,'-text')=='1':
				value=1
			else:
				value=0
		
		return value

	# ---------------------------------------------------------------------------
	#
	def editend_cb(self,tbl,row,column,text):
		value=int(text)
		if column=='0':
			self.check_peak(int(row),value)

		self.duplicate_tablelist.lock_callbacks=0
		return value

	# -----------------------------------------------------------------------------
	#
	def popup_menu(self,event):
		if self.duplicate_tablelist.lock_callbacks==0:
			menu=Tkinter.Menu(self.duplicate_tablelist.tablelist,tearoff=0)

			menu.add_command(label='Show envelope',command=self.isodistrib_cb)
			menu.add_separator()
			menu.add_command(label='Check all',command=self.check_all)
			menu.add_command(label='Uncheck all ',command=self.check_none)

			uncheckby_submenu=Tkinter.Menu(menu,tearoff=0)
			uncheckby_submenu.add_command(label=u'\u03B4m/z',command=self.check_min_dmz)
			uncheckby_submenu.add_command(label=u'\u0394RT',command=self.check_min_drt)
			uncheckby_submenu.add_command(label='score',command=self.check_max_score)
			menu.add_cascade(label='Uncheck by',menu=uncheckby_submenu)

			menu.add_separator()
			menu.add_command(label='Update',command=self.update_cb)

			menu.tk_popup(event.x_root,event.y_root)

	# -----------------------------------------------------------------------------
	#
	def get_queries(self):
		
		self.progress_report('Getting peaks')
		self.duplicated_list=[]

		peaks=self.spectrum.peak_list()
		rt_step=mssputil.get_rt_step(self.spectrum)
		if self.spectrum==mssputil.DOMAIN_FREQ:
			mz_step=mssputil.get_f_step(self.spectrum)
		else:
			mz_step=mssputil.get_mz_step(self.spectrum)

		self.stoppable_loop('peaks',100)
		for peak in peaks:
			self.check_for_stop()
			if mssputil.is_peak_unassigned(peak)!=1 and mssputil.is_peak_present(peak)==1:
				duplicate=ms.duplicated_query()
				duplicate.add(peak)
				duplicate.mz=peak.frequency[1]
				duplicate.rt=peak.frequency[0]
				if self.settings.mz_tol==-1:
					duplicate.mz_tol=mz_step
				else:
					duplicate.mz_tol=self.settings.mz_tol
				if self.settings.rt_tol==-1:
					duplicate.rt_tol=rt_step
				else:
					duplicate.rt_tol=self.settings.rt_tol
				self.duplicated_list.append(duplicate)

	# ---------------------------------------------------------------------------
	#
	def find_string(self,event=None):
		str=self.find_edit.variable.get()
		if str!='':
			self.lock_gui(1)
			if self.direction_menu.variable.get()=='Search down':
				self.duplicate_tablelist.find_string(str,exact_match=self.match_check.state(),y_range=(1,2))
			else:
				self.duplicate_tablelist.find_string_up(str,exact_match=self.match_check.state(),y_range=(1,2))
			self.lock_gui(0)

	# -----------------------------------------------------------------------------
	#
	def get_duplicated(self):
		
		self.progress_report('Getting duplicated assignments')

		self.stoppable_loop('peaks',100)
		duplicated_count=len(self.duplicated_list)
		for i in range(duplicated_count-1,-1,-1):
			self.check_for_stop()
			duplicated=self.duplicated_list[i]
			for j in range(i-1,-1,-1):
				current=self.duplicated_list[j]
				if cmp(current,duplicated):
#					for peak in duplicated.peak_list:
#						current.add(peak)
#					break
#					print duplicated.size()
					for k in range(duplicated.size()-1,-1,-1):
#						print k
						peak=duplicated.get(k)
						current.add(peak)
						duplicated.remove(peak)
					break

	# ---------------------------------------------------------------------------
	#
	def show_duplicated_peaks(self,spectrum):
		self.spectrum=spectrum
		self.title='MS duplicated assignments: ' + self.spectrum.name
		self.top.title(self.title)

		dialogs=self.session.spectrum_dialogs
		key='PL'+self.spectrum.name
		if (dialogs.has_key(key) and not dialogs[key].is_window_destroyed()):
			self.parent=dialogs[key]
			self.parent.find_duplicates_dialog=self

		self.update_cb()

	# ---------------------------------------------------------------------------
	#
	def update_cb(self,event=None):
		self.lock_gui(1)
		self.stoppable_call(self.get_queries)
		self.stoppable_call(self.get_duplicated)
		self.stoppable_call(self.update_tablelist)
		self.lock_gui(0)
		self.duplicate_tablelist.tablelist.focus_set()

	# ---------------------------------------------------------------------------
	#
	def redraw_cb(self,event=None):
		self.lock_gui(1)
		self.stoppable_call(self.update_tablelist)
		self.lock_gui(0)

		self.duplicate_tablelist.tablelist.focus_set()

	# ---------------------------------------------------------------------------
	#
	def update_tablelist(self):
		self.progress_report('Updating table')
		self.duplicate_tablelist.lock_callbacks=1

		counter=-1
		duplicated_counter=0

		bgcolor='gray98'
		self.duplicate_tablelist.clear()
		
		self.stoppable_loop('peptides',50)
		for i in range(len(self.duplicated_list)):
			self.check_for_stop()
			duplicate=self.duplicated_list[i]

			if duplicate.size()>1:
				duplicated_counter=duplicated_counter+1
				if bgcolor=='#d4deec':
					bgcolor='gray98'
				else:
					bgcolor='#d4deec'

				for j in range(duplicate.size()):
					counter=counter+1
					peak=duplicate.get(j)

					charge=mssputil.get_charge(peak)
					sequence='%s (+%d)' % (mssputil.get_sequence(peak),charge)

					#m/z
					mz=mssputil.get_mz(peak)
					cmz=mssputil.m2mz(mssputil.get_cmass(peak),charge)

					if self.settings.delta_mz_correction==1 and self.spectrum.rt_precision!=0:
						dmz_correction=mssputil.get_mz_correction_ppm(self.spectrum,cmz)
					else:
						dmz_correction=0

					mz_str=('%0.4f' % cmz)

					delta_mz_ppm=1e+6*((mz-cmz)/cmz)-dmz_correction
					delta_mz=cmz*delta_mz_ppm/1e+6

					if self.settings.delta_mz_ppm==1:
						delta_mz_str=('%0.2f' % delta_mz_ppm)
					else:
						delta_mz_str=('%0.4f' % delta_mz)


					#RT
					rt=mssputil.get_rt(peak)
					massref_rt=mssputil.get_massref_rt(peak)

					if self.settings.delta_mrt_correction==1:
						drt_correction=mssputil.get_rt_correction(self.spectrum,massref_rt)
					else:
						drt_correction=0

					rt_str=('%0.2f' % rt)
					delta_mrt=rt-massref_rt-drt_correction
					delta_mrt_str=('%0.2f' % delta_mrt)


					self.duplicate_tablelist.append(('',sequence,mssputil.get_protein(peak),mz_str,delta_mz_str,rt_str,delta_mrt_str,mssputil.get_score(peak)),peak)
					self.duplicate_tablelist.tablelist.rowconfigure(counter,background=bgcolor)

					if mssputil.is_peak_present(peak)==0:
						self.duplicate_tablelist.configure_cell(counter,0,image=self.duplicate_tablelist.checkoff_icon,text='0')
						self.duplicate_tablelist.configure_cell(counter,1,foreground='red')
					else:
						self.duplicate_tablelist.configure_cell(counter,0,image=self.duplicate_tablelist.checkon_icon,text='1')

		self.duplicate_tablelist.list_size.configure(text=('%d peaks (%d assignments)' % (counter+1,duplicated_counter)))
		self.duplicate_tablelist.lock_callbacks=0

	# ---------------------------------------------------------------------------
	#
	def isodistrib_cb(self,event=None):
		peak=self.duplicate_tablelist.event_line_data(event)
		if peak!=None:
			msisoenvelope.show_peak_fit(peak,True)

	# ---------------------------------------------------------------------------
	#
	def check_peak(self,row,check,external_call=0):
		peak=self.duplicate_tablelist.line_data[row]
		query=mssputil.get_query(peak)
		query.set_present(check)
		
		if external_call==0 and self.parent!=None:
			line_index=self.parent.peptide_tablelist.find_peak_cb(peak)
			if line_index!=-1:
				self.parent.check_query(line_index,check,1)

		if check==0:
			self.duplicate_tablelist.configure_cell(row,0,image=self.duplicate_tablelist.checkoff_icon,text='0')
			self.duplicate_tablelist.configure_cell(row,1,foreground='red')
		else:
			self.duplicate_tablelist.configure_cell(row,0,image=self.duplicate_tablelist.checkon_icon,text='1')
			self.duplicate_tablelist.configure_cell(row,1,foreground='black')

	# ---------------------------------------------------------------------------
	#
	def check_queries(self):
		line_counter=0
		self.stoppable_loop('queries',50)
		for i in range(len(self.duplicated_list)):
			self.check_for_stop()
			duplicate=self.duplicated_list[i]

			if duplicate.size()>1:
				if self.settings.check<0:
					vals=numpy.zeros(duplicate.size())
					vals2=numpy.zeros(duplicate.size())
					vals3=numpy.zeros(duplicate.size())
					for j in range(duplicate.size()):
#						if mssputil.is_peak_present(duplicate.peak_list[j])==0:
#							vals[j]=numpy.inf
#							continue
						if self.settings.check==-1:
							vals[j]=-1*float(self.duplicate_tablelist.get_cell_config(line_counter+j,7,'-text'))
							vals2[j]=numpy.abs(float(self.duplicate_tablelist.get_cell_config(line_counter+j,4,'-text')))
							vals3[j]=numpy.abs(float(self.duplicate_tablelist.get_cell_config(line_counter+j,6,'-text')))
						elif self.settings.check==-2:
							vals[j]=numpy.abs(float(self.duplicate_tablelist.get_cell_config(line_counter+j,4,'-text')))
							vals2[j]=numpy.abs(float(self.duplicate_tablelist.get_cell_config(line_counter+j,6,'-text')))
							vals3[j]=-1*float(self.duplicate_tablelist.get_cell_config(line_counter+j,7,'-text'))							
						else:
							vals[j]=numpy.abs(float(self.duplicate_tablelist.get_cell_config(line_counter+j,6,'-text')))
							vals2[j]=numpy.abs(float(self.duplicate_tablelist.get_cell_config(line_counter+j,4,'-text')))
							vals3[j]=-1*float(self.duplicate_tablelist.get_cell_config(line_counter+j,7,'-text'))							

					indices=numpy.nonzero(vals==numpy.min(vals))[0]
					if (len(indices)>1):
						indices2=numpy.nonzero(vals2[indices]==numpy.min(vals2[indices]))[0]
						if (len(indices2)>1):
							indices3=numpy.nonzero(vals3[indices[indices2]]==numpy.min(vals3[indices[indices2]]))[0]
							index=indices[indices2[indices3]][0]
						else:
							index=indices[indices2][0]
					else:
						index=indices[0]

					#index=numpy.argmin(vals)
					for j in range(duplicate.size()):
#						self.check_peak(line_counter,(j==index and not numpy.isinf(vals[j])))
						self.check_peak(line_counter,(j==index))
						line_counter=line_counter+1
				else:
					for j in range(duplicate.size()):
						self.check_peak(line_counter,self.settings.check)
						line_counter=line_counter+1

	# ---------------------------------------------------------------------------
	#
	def check_max_score(self):
		self.lock_gui(1)
		self.settings.check=-1
		self.stoppable_call(self.check_queries)
		self.lock_gui(0)

	# ---------------------------------------------------------------------------
	#
	def check_min_dmz(self):
		self.lock_gui(1)
		self.settings.check=-2
		self.stoppable_call(self.check_queries)
		self.lock_gui(0)

	# ---------------------------------------------------------------------------
	#
	def check_min_drt(self):
		self.lock_gui(1)
		self.settings.check=-3
		self.stoppable_call(self.check_queries)
		self.lock_gui(0)

	# ---------------------------------------------------------------------------
	#
	def check_all(self):
		self.lock_gui(1)
		self.settings.check=1
		self.stoppable_call(self.check_queries)
		self.lock_gui(0)

	# ---------------------------------------------------------------------------
	#
	def check_none(self):
		self.lock_gui(1)
		self.settings.check=0
		self.stoppable_call(self.check_queries)
		self.lock_gui(0)

	# ---------------------------------------------------------------------------
	#
	def lock_gui(self,lock):

		if lock==1:
			state='disabled'
			edit='no'
			if sys.platform=='win32':
				cursor='wait'
			else:
				cursor='watch'
		else:
			state='normal'
			edit='yes'
			cursor=''

		self.settings.lock_callbacks=lock

		self.buttons_row.buttons[0]['state']=state
		self.buttons_row.buttons[2]['state']=state
		self.buttons_row.buttons[3]['state']=state
		self.top.config(cursor=cursor)
		self.top.update_idletasks()

# -----------------------------------------------------------------------------
#
class duplicated_list_settings:

	def __init__(self,session,path=None):
		self.session=session
		self.lock_callbacks=0
		self.loaded=0

		if path!=None:
			self.loaded=self.read_settings(path)

		self.delta_mz_ppm=1
		self.check=-2

		try:
			num=int(self.session.project.saved_value('DL_DELTA_MZ_CORRECTION'))
			if num<0 or num>1:
				raise ValueError
			self.delta_mz_correction=num
		except (ValueError,TypeError):
			self.delta_mz_correction=1

		try:
			num=int(self.session.project.saved_value('DL_DELTA_RT_CORRECTION'))
			if num<0 or num>1:
				raise ValueError
			self.delta_mrt_correction=num
		except (ValueError,TypeError):
			self.delta_mrt_correction=1

		try:
			num=float(self.session.project.saved_value('DL_MZ_TOL'))
			if num<0:
				raise ValueError
			self.mz_tol=num
		except (ValueError,TypeError):
			self.mz_tol=-1

		try:
			num=float(self.session.project.saved_value('DL_RT_TOL'))
			if num<0:
				raise ValueError
			self.rt_tol=num
		except (ValueError,TypeError):
			self.rt_tol=-1

	# --------------------------------------------------------------------------
	#
	def read_settings(self,path):
		result=0

		if path!=None:
			try:
				file=open(path,'r')
				lines=file.readlines()

				for i in range(0,len(lines)):
					line=lines[i].strip()
					if len(line)!=0 and line[0]!='#':
						fields=lines[i].split('=')
						if len(fields)==2:
							self.session.project.save_value(fields[0].strip(),fields[1].strip())
				file.close()
				result=1
			except IOError, (errno, strerror):
				print "I/O error(%s): %s" % (errno, strerror)
				result=0
		
		return result

# -----------------------------------------------------------------------------
#
def show_duplicated_list_dialog(parent):
	d=None

	if parent.spectrum==None:
		return d

	if not hasattr(parent.session,'spectrum_dialogs'):
		parent.session.spectrum_dialogs={}
	dialogs=parent.session.spectrum_dialogs

	key='FD'+parent.spectrum.name
	if (dialogs.has_key(key) and not dialogs[key].is_window_destroyed()):
		d=dialogs[key]
		d.show_window(1)
		d.parent=parent
	else:
		d=duplicated_list_dialog(parent.session,parent)
		d.show_window(1)
		d.show_duplicated_peaks(parent.spectrum)
		dialogs[key]=d
	
	return d

# -----------------------------------------------------------------------------
#
def show_duplicated_list(session):

	spectrum=session.selected_spectrum()
	if spectrum==None:
		return

	if not hasattr(session,'spectrum_dialogs'):
		session.spectrum_dialogs={}
	dialogs=session.spectrum_dialogs

	key='FD'+spectrum.name
	if (dialogs.has_key(key) and not dialogs[key].is_window_destroyed()):
		dialogs[key].show_window(1)
	else:
		d=duplicated_list_dialog(session)
		d.show_window(1)
		d.show_duplicated_peaks(spectrum)
		dialogs[key]=d
	
