import sparky
import numpy
import scipy
import scipy.stats
import scipy.optimize
import matplotlib
import matplotlib.pyplot as plt
from matplotlib.figure import Figure

import ms
import mssputil
import msmath
import msfit
import mstkutil


PEAK_SHAPE_GAUSS=0
PEAK_SHAPE_LORENZ=1

# ----------------------------------------------------------------------------
#
class MSIsoDistrib:
	def __init__(self,peak,threshold=0.01):
		self.probs=[]
		self.masses=[]
		self.peaks_count=0;
		self.max_prob=0
		self.max_prob_index=0
		
		isodistrib_str=mssputil.get_isodistrib(peak).split(',')
		peaks_count=len(isodistrib_str)

		if peaks_count>1:
			isomass_str=mssputil.get_isomass(peak).split(',')
#			equal_space=0
#			if len(isomass_str)<peaks_count:
#				equal_space=1
#				mass=mssputil.get_cmass(peak)

			equal_space=1
			mass=mssputil.get_cmass(peak)
			isomass0=float(isomass_str[0])
			if 100*round(mass)/100<100*round(isomass0)/100:
				mass=isomass0

			for i in range(0,peaks_count):
				value=float(isodistrib_str[i])
				if value<threshold:
					continue
				else:
					self.peaks_count=self.peaks_count+1
					self.probs.append(value)
					if equal_space:
						self.masses.append(mass+float(i)*ms.M)
					else:
						self.masses.append(float(isomass_str[i]))

					if value>self.max_prob:
						self.max_prob=value
						self.max_prob_index=i

			self.probs=numpy.array(self.probs)
			self.masses=numpy.array(self.masses)

	# ---------------------------------------------------------------------------
	#
	def get_peak_mz(self,peak_nr,charge,mz_correction=0):
		return(mssputil.m2mz(self.masses[peak_nr],charge)+mz_correction)

	# ---------------------------------------------------------------------------
	#
	def get_peaks_mz(self,charge,mz_correction=0):
		return(mssputil.m2mz_list(self.masses,charge)+mz_correction)

	# ---------------------------------------------------------------------------
	#
	def get_max_peak_index(self):
		return(numpy.argmax(self.probs))

	# ---------------------------------------------------------------------------
	#
	def get_highest_peaks(self,peaks_count,charge,mz_correction=0):

		#[index; uporzadkowane od najwyzszego do najnizszego
		# m/z;
		# prob]
		isopeaks_index=numpy.argsort(self.probs)[-peaks_count:]
		isopeaks_index=isopeaks_index[::-1]
		isopeaks=numpy.array((isopeaks_index,self.get_peaks_mz(charge,mz_correction)[isopeaks_index],self.probs[isopeaks_index]))

		return(isopeaks)

	# ---------------------------------------------------------------------------
	#
	def get_envelope_mz_range(self,charge,mass_margin=[1,1],mz_correction=0):

		min_mz=mssputil.m2mz(self.masses[0]-mass_margin[0],charge)+mz_correction
		max_mz=mssputil.m2mz(self.masses[-1]+mass_margin[1],charge)+mz_correction

		return([min_mz,max_mz])

	# ---------------------------------------------------------------------------
	#
	def get_envelope_mz_abs_range(self,charge,mz_margin=[1,1],mz_correction=0):

		min_mz=mssputil.m2mz(self.masses[0],charge)-mz_margin[0]+mz_correction
		max_mz=mssputil.m2mz(self.masses[-1],charge)+mz_margin[1]+mz_correction

		return([min_mz,max_mz])

	# ---------------------------------------------------------------------------
	#
	def get_peak_mz_range(self,charge,peak_number,mass_margin=[0.5,0.5],mz_correction=0):

		if peak_number<self.peaks_count:
			min_mz=mssputil.m2mz(self.masses[peak_number]-mass_margin[0],charge)+mz_correction
			max_mz=mssputil.m2mz(self.masses[peak_number]+mass_margin[1],charge)+mz_correction
		else:
			min_mz=0
			max_mz=0

		return([min_mz,max_mz])

	# ---------------------------------------------------------------------------
	#
	def get_peak_mz_abs_range(self,charge,peak_number,mz_margin=[0.5,0.5],mz_correction=0):

		if peak_number<self.peaks_count:
			min_mz=mssputil.m2mz(self.masses[peak_number],charge)-mz_margin[0]+mz_correction
			max_mz=mssputil.m2mz(self.masses[peak_number],charge)+mz_margin[1]+mz_correction
		else:
			min_mz=0
			max_mz=0

		return([min_mz,max_mz])


	# ---------------------------------------------------------------------------
	#
	def get_peaks_mz_range(self,charge,peaks_range=[0,-1],mass_margin=[0.5,0.5],mz_correction=0):

		if peaks_range[0]<0:
			peaks_range[0]=0
		if peaks_range[1]>=self.peaks_count:
			peaks_range[1]=-1

		min_mz=mssputil.m2mz(self.masses[peaks_range[0]]-mass_margin[0],charge)+mz_correction
		max_mz=mssputil.m2mz(self.masses[peaks_range[1]]+mass_margin[1],charge)+mz_correction

		return([min_mz,max_mz])

	# ---------------------------------------------------------------------------
	#
	def get_peaks_mz_abs_range(self,charge,peaks_range=[0,-1],mz_margin=[0.5,0.5],mz_correction=0):

		if peaks_range[0]<0:
			peaks_range[0]=0
		if peaks_range[1]>=self.peaks_count:
			peaks_range[1]=-1

		min_mz=mssputil.m2mz(self.masses[peaks_range[0]],charge)-mz_margin[0]+mz_correction
		max_mz=mssputil.m2mz(self.masses[peaks_range[1]],charge)+mz_margin[1]+mz_correction

		return([min_mz,max_mz])

	# ---------------------------------------------------------------------------
	#
	def get_envelope_c(self,session,mz_range,mz_step,charge,resolution,mz_correction=0,scale=1):

		[envelope,mz_points]=session.getEnvelope(self.probs.tolist(),self.masses.tolist(),charge,resolution,mz_range[0],mz_range[1],mz_step,mz_correction,PEAK_SHAPE_GAUSS,scale)
		return(numpy.array(envelope),numpy.array(mz_points))

	# ---------------------------------------------------------------------------
	#
	def get_envelope(self,mz_range,mz_step,charge,resolution,mz_correction=0,scale=1):

		peaks_mz=(self.masses+ms.P*charge)/charge+mz_correction
		peaks_sigma=peaks_mz/(resolution*2*numpy.sqrt(2*numpy.log(2.0)))

		mz_points=self.get_mz_points(mz_range,mz_step)
		envelope=numpy.zeros(mz_points.shape[0])

		for i in range(0,self.peaks_count):
			envelope=envelope+msmath.gauss(mz_points,self.probs[i],peaks_mz[i],peaks_sigma[i])

		if scale!=0:
			max_value=envelope.max()
			envelope=(scale*envelope)/max_value

		return(envelope,mz_points)

	# ---------------------------------------------------------------------------
	#
	def get_envelope_for_points(self,mz_points,charge,resolution,mz_correction=0,scale=1):

		peaks_mz=(self.masses+ms.P*charge)/charge+mz_correction
		peaks_sigma=peaks_mz/(resolution*2*numpy.sqrt(2*numpy.log(2.0)))

		envelope=numpy.zeros(mz_points.shape[0])

		for i in range(0,self.peaks_count):
#			envelope=envelope+self.probs[i]*numpy.exp(-0.5*((mz_points-peaks_mz[i])**2)/(peaks_sigma[i]**2))
			envelope=envelope+msmath.gauss(mz_points,self.probs[i],peaks_mz[i],peaks_sigma[i])

		if scale!=0:
			max_value=envelope.max()
			envelope=(scale*envelope)/max_value

		return(envelope,mz_points)

	# ---------------------------------------------------------------------------
	#
	def get_mz_points(self,mz_range,mz_step):

		points_count=int(numpy.floor((mz_range[1]-mz_range[0])/mz_step)+1)
		mz_points=numpy.zeros(points_count)
		mz_points[0]=mz_range[0]
		for i in range(1,points_count):
			mz_points[i]=mz_points[i-1]+mz_step
		
		return(mz_points)
