import os
import gc
import sys
import copy
import string
import Tkinter
import Pmw

import sparky

import pyutil
import mssputil
import mstkutil
import msfindpeptides
import msfindpeptides_gui
import msfindpeptides_settings

# -----------------------------------------------------------------------------
#
class experiment_dialog(mstkutil.Dialog, mstkutil.Stoppable):

	def __init__(self,session):

		self.session=session
		self.files=[]
		self.peptides=[]
		self.dirname=None
		self.autofind_dialog=None
		self.settings_dialog=None
		self.lock_callbacks=0

		self.settings=msfindpeptides_settings.findpeptides_settings(self.session,os.path.join(os.environ.get('SPARKY_INSTALL'),os.path.join('config','findpeptides.cfg')),False,False)
		self.settings.view_log=False
		self.settings.save_log=True
		self.settings.view_second_pass_params=False
		self.settings.save_second_pass_params=True

		mstkutil.Dialog.__init__(self,session.tk,'MS experiment')

		Pmw.initialise(session.tk)

		f=Pmw.Group(self.top, tag_text='Files:')
		f.pack(fill='both',expand=1,padx=1)

		fl=mstkutil.scrolling_list(f.interior(),None,15,50,0)
		fl.frame.pack(side='top',anchor='w',fill='both',expand=1,pady=2,padx=2)
		fl.listbox.bind('<Double-ButtonRelease-1>',self.double_click_cb)
		fl.listbox.bind('<Button-3>',self.popup_menu)
		fl.listbox.config(selectmode='extended')
#		fl.listbox['exportselection']=0
		self.files_list=fl

		f=Pmw.Group(self.top,tag_text='Settings:')
		f.pack(side='top',fill='both',expand=1,padx=1,pady=2)

		l=Tkinter.Label(f.interior(),text='Directory:',anchor='w',justify='left')
		l.grid(row=0,column=0,sticky='we',padx=3,pady=3)

		e=Tkinter.Entry(f.interior(),borderwidth=1,width=55,justify='left')
		e.grid(row=0,column=1,sticky='we',padx=3,pady=3)
		self.directory_entry=e

		b=Tkinter.Button(f.interior(),text="Browse",borderwidth=1,command=self.read_files_cb)
		b.grid(row=0,column=2,sticky='e',padx=3,pady=3)
		self.directory_button=b

		l=Tkinter.Label(f.interior(),text='Peptides list:',anchor='w',justify='left')
		l.grid(row=1,column=0,sticky='we',padx=3,pady=3)

		e=Tkinter.Entry(f.interior(),borderwidth=1,width=55,justify='left')
		e.grid(row=1,column=1,sticky='we',padx=3,pady=3)
		self.peptides_list_entry=e

		b=Tkinter.Button(f.interior(),text="Browse",borderwidth=1,command=self.read_peptides_cb)
		b.grid(row=1,column=2,sticky='e',padx=3,pady=3)
		self.peptides_list_button=b

		f.interior().columnconfigure(0, weight=1)
		f.interior().columnconfigure(1, weight=1)
		f.interior().columnconfigure(2, weight=1)

		progress_label=Tkinter.Label(self.top,anchor='nw')
		progress_label.pack(side='top',anchor='w',fill='both',expand=1)

		br=mstkutil.button_row(self.top,
					('Start', self.find_peptides_cb),
					('Stop', self.stop_cb),
					('Setup', self.setup_cb),
					('Close', self.closing_window_cb),
					)
		br.frame.pack(side='top',anchor='w',pady=2)
		self.buttons_row=br
		
		self.top.protocol("WM_DELETE_WINDOW",self.closing_window_cb)

		self.top.geometry("+0+0")
#		self.top.resizable(1,0)

		mstkutil.Stoppable.__init__(self, progress_label, br.buttons[1])

	# ---------------------------------------------------------------------------
	#
	def popup_menu(self,event):
		if len(self.files)!=0 and self.lock_callbacks==0:
			menu=Tkinter.Menu(self.files_list.listbox,tearoff=0)

			menu.add_command(label='Show spectrum',command=self.view_spectrum_cb)
			menu.add_command(label='Close spectrum',command=self.close_spectrum_cb)
			menu.add_separator()
			menu.add_command(label='Remove',command=self.remove_cb)
			menu.tk_popup(event.x_root,event.y_root)

	# ---------------------------------------------------------------------------
	#
	def closing_window_cb(self):
		if self.lock_callbacks==0:
			if self.settings_dialog!=None:
				self.settings_dialog.close_cb()
			self.top.destroy()

	#przeniesc do obslugi listy plikow
	# ---------------------------------------------------------------------------
	#
	def double_click_cb(self,event):
		if self.lock_callbacks==0:
			self.view_spectrum_cb()

	# ---------------------------------------------------------------------------
	#
	def view_spectrum_cb(self):
		if len(self.files)!=0:
			selected_index=self.files_list.selected_line_numbers()
			if selected_index!=None:
				spectrum=mssputil.save_path_to_spectrum(self.files[selected_index[0]],self.session)
				if spectrum!=None:
					mssputil.show_spectrum(spectrum)
				else:
					spectrum=self.session.open_spectrum(self.files[selected_index[0]])

	# ---------------------------------------------------------------------------
	#
	def remove_cb(self):
		if len(self.files)!=0:
			selected_index=self.files_list.selected_line_numbers()
			if selected_index!=None:
				for line_index in sorted(selected_index,reverse=True):
					 self.files_list.listbox.delete(str(line_index))
					 del self.files[line_index]
				
	# ---------------------------------------------------------------------------
	#
	def close_spectrum_cb(self):
		selected_index=self.files_list.selected_line_numbers()
		if selected_index!=None:
			spectrum=mssputil.save_path_to_spectrum(self.files[selected_index[0]],self.session)
			if spectrum!=None:
				self.session.project.close_spectrum(spectrum)

	# ---------------------------------------------------------------------------
	#
	def stop_cb(self):
		if self.autofind_dialog!=None:
			self.autofind_dialog.stop_cb()

		mstkutil.Stoppable.stop_cb(self)

	# ---------------------------------------------------------------------------
	#
	def setup_cb(self):
		self.settings_dialog=msfindpeptides_gui.show_peptides_dialog(self.session,self)

	# ---------------------------------------------------------------------------
	#
	def apply_cb(self,settings):
		self.settings=settings

	# ---------------------------------------------------------------------------
	#
	def read_files_cb(self):

		dirname=mstkutil.load_directory(self.top,'Select directory','experimentdir')

		ok=1
		if dirname and os.path.isdir(dirname):
			if not os.access(dirname,os.W_OK):
				dialog=mstkutil.question_dialog(self.top,'The directory %s is read-only.\nDo you want to continue?' % dirname )
				ok=dialog.result
				
			if ok==1:
				self.directory_entry.delete(0,'end')
				self.directory_entry.insert(0,dirname)
				self.files_list.clear()
				self.files=[]
					
				for filename in sorted(os.listdir(dirname)):
					if os.path.isfile(os.path.join(dirname,filename)) and mssputil.check_ext(filename,'save'):
						self.files.append(os.path.join(dirname,filename))
				
				self.dirname=dirname
				self.fill_listbox()
	
	# ---------------------------------------------------------------------------
	#
	def read_peptides_cb(self):

		filename=mstkutil.load_file(self.top,'Open file','peaklist',file_types=[('Sparky peaks', '.4sparky.dat'),('all files', '.*')],initial_dir=self.dirname)
		
		if filename and os.path.isfile(filename):
			self.peptides_list_entry.delete(0,'end')
			self.peptides_list_entry.insert(0,filename)

	# ---------------------------------------------------------------------------
	#
	def find_peptides_cb(self):
		self.lock_gui(1)
		self.stoppable_call(self.find_peptides)
		self.lock_gui(0)

	# ---------------------------------------------------------------------------
	#
	def fill_listbox(self):
		self.files_list.clear()
		
		for filename in self.files:
			self.files_list.listbox.insert('end',mssputil.remove_ext(os.path.basename(filename),'save'))

		if len(self.files)==1:
			self.files_list.list_size.configure(text=('1 spectrum'))
		else:
			self.files_list.list_size.configure(text=('%d spectra') % len(self.files))
					
	# ---------------------------------------------------------------------------
	#
	def find_peptides(self):

		self.settings.view_log=False
		self.settings.save_log=True
		self.settings.view_second_pass_params=False
		self.settings.save_second_pass_params=True
		
		stop=0
		self.progress_report('Processing spectra')
		self.fill_listbox()
		
		pep_filename=self.peptides_list_entry.get()
		if len(pep_filename)>0 and os.path.isfile(pep_filename):
			self.stoppable_loop('File:',1)
			
			for i in range(0,len(self.files)):
				self.check_for_stop()
				if stop==1:
					break
				file=self.files[i]
				self.files_list.select_line(i)

				if os.path.isfile(file) and os.access(file,os.W_OK):
					self.files_list.set(i,mssputil.remove_ext(os.path.basename(file),'save') + " (processing)")
					
					spectrum=self.session.open_spectrum(file)
					self.top.update_idletasks()
					if spectrum!=None:
						#obie rzeczy w zaleznosci od ustawien
						spectrum.reset_calibration()
						mssputil.remove_peaks_from_spectrum(spectrum,False)

						mssputil.read_peaks(pep_filename,spectrum)
						
						peaks=spectrum.peak_list()
						if peaks!=None and len(peaks)>0:
							mssputil.show_peak(peaks[0])
							self.top.update_idletasks()

							self.autofind_dialog=msfindpeptides.find_dialog(self.session,spectrum,copy.copy(self.settings))
							self.autofind_dialog.show_window(1)
							self.autofind_dialog.find_cb()

							stop=self.autofind_dialog.stop_requested
							self.autofind_dialog.top.destroy()

						if stop==0:
							self.files_list.set(i,mssputil.remove_ext(os.path.basename(file),'save') + " (done)")
							
							self.session.save_spectrum(spectrum,0)
							self.session.project.close_spectrum(spectrum)
						else:
							self.files_list.set(i,mssputil.remove_ext(os.path.basename(file),'save') + " (stopped)")
				else:
					self.files_list.set(i,mssputil.remove_ext(os.path.basename(file),'save') + " (skipped)")
					
			self.files_list.clear_selection()
		else:
			mstkutil.warning_dialog(self.top,"Peptides list not loaded")


	# ---------------------------------------------------------------------------
	#
	def lock_gui(self,lock):

		if lock==1:
			state='disabled'
			edit='no'
			if sys.platform=='win32':
				cursor='wait'
			else:
				cursor='watch'
		else:
			state='normal'
			edit='yes'
			cursor=''

		self.lock_callbacks=lock
#		self.files_list.listbox['state']=state
		self.directory_entry['state']=state
		self.directory_button['state']=state
		self.peptides_list_entry['state']=state
		self.peptides_list_button['state']=state
		self.buttons_row.buttons[0]['state']=state
		self.buttons_row.buttons[2]['state']=state
		self.buttons_row.buttons[3]['state']=state
		self.top.config(cursor=cursor)
		self.top.update_idletasks()

# -----------------------------------------------------------------------------
#
def show_experiment_dialog(session):
	mssputil.the_dialog(experiment_dialog,session).show_window(1)
