import math
import sys

import Tkinter

import sparky
import mssputil
import mstkutil
import numpy

# -----------------------------------------------------------------------------
#
class export_peptide_dialog(mstkutil.Dialog,mstkutil.Stoppable):
	
	def __init__(self, session):
		self.session=session
		mstkutil.Dialog.__init__(self,session.tk,'MS export data')

		f=Tkinter.Frame(self.top,borderwidth=2,relief='groove')
		f.pack(side='top',anchor='w',pady=2,fill='x',expand=1)

		cb=mstkutil.checkbutton(f,'Write header with peptide info',0)
		cb.button.grid(row=0,columnspan=2,sticky='w',pady=1)
		cb.button.configure(state='disabled')
		self.peptide_check=cb

		cb=mstkutil.checkbutton(f,'Write header with data info',1)
		cb.button.grid(row=1,columnspan=2,sticky='w',pady=1)
		self.data_check=cb

		l=Tkinter.Label(f,anchor='w',width=20,text='m/z step: ')
		l.grid(row=2,column=0,sticky='w')

		e=mstkutil.entry_field(f,'','0',8,None,'right')
		e.frame.grid(row=2,column=1,sticky='w',padx=2,pady=1)
		self.mz_res_edit=e

		l=Tkinter.Label(f,anchor='w',width=20,text='RT step: ')
		l.grid(row=3,column=0,sticky='w')

		e=mstkutil.entry_field(f,'','0',8,None,'right')
		e.frame.grid(row=3,column=1,sticky='w',padx=2,pady=1)
		self.rt_res_edit=e

		cb=mstkutil.checkbutton(f,'Data smoothing',0)
		cb.button.grid(row=4,columnspan=2,sticky='w',pady=1)
		self.smooth_check=cb

		cb=mstkutil.checkbutton(f,'Data interpolation',0)
		cb.button.grid(row=5,columnspan=2,sticky='w',pady=1)
		cb.add_callback(self.interpolation_cb)
		self.interpolation_check=cb

		l=Tkinter.Label(f,anchor='w',width=20,text='m/z scale: ')
		l.grid(row=6,column=0,sticky='w')

		e=mstkutil.entry_field(f,'','3',8,None,'right')
		e.frame.grid(row=6,column=1,sticky='w',padx=2,pady=1)
		e.entry.configure(state='disabled')
		self.mz_scale_edit=e

		l=Tkinter.Label(f,anchor='w',width=20,text='rt scale: ')
		l.grid(row=7,column=0,sticky='w')

		e=mstkutil.entry_field(f,'','4',8,None,'right')
		e.frame.grid(row=7,column=1,sticky='w',padx=2,pady=1)
		e.entry.configure(state='disabled')
		self.rt_scale_edit=e

		progress_label=Tkinter.Label(self.top, anchor = 'nw')
		progress_label.pack(side='top',anchor='w')

		br=mstkutil.button_row(self.top,
				('Close', self.top.destroy),
				('Stop', self.stop_cb),
				('Save', self.save_cb))
		br.frame.pack(side='bottom',anchor='w',pady=2)
		self.buttons_row=br

		self.top.geometry("+0+0")
		self.top.resizable(1,0)

		self.removed_notice=session.notify_me('removed spectrum from project',self.spectrum_removed_cb)
		self.top.bind('<Destroy>',self.remove_callbacks,1)

		mstkutil.Stoppable.__init__(self,progress_label,br.buttons[1])

	# ---------------------------------------------------------------------------
	#
	def spectrum_removed_cb(self,removed_spectrum):
		if self.spectrum==removed_spectrum:
			self.top.destroy()

	# ---------------------------------------------------------------------------
	#
	def remove_callbacks(self,event):
		if str(self.top)==str(event.widget):
			if sparky.object_exists(self.session):
				if self.removed_notice!=None:
					self.session.dont_notify_me(self.removed_notice)
				
			self.removed_notice=None

	# ---------------------------------------------------------------------------
	#
	def interpolation_cb(self,event):

		if self.interpolation_check.variable.get()==1:
			self.mz_scale_edit.entry.configure(state='normal')
			self.rt_scale_edit.entry.configure(state='normal')
		else:
			self.mz_scale_edit.entry.configure(state='disabled')
			self.rt_scale_edit.entry.configure(state='disabled')

	# --------------------------------------------------------------------------
	#
	def set_view(self,view):
		self.view=view
		self.spectrum=self.view.spectrum

		self.path=None
		self.data_header=None
		self.peptide_header=None
		self.data=None
		self.peak=None

		peaks=self.spectrum.selected_peaks()
		if peaks!=None and len(peaks)==1 and peaks[0].note!=None and peaks[0].note!='':
			peak_mz=mssputil.get_mz(peaks[0])
			peak_rt=mssputil.get_rt(peaks[0])
		else:
			peak_mz=0
			peak_rt=0

		rt_points=mssputil.get_rt_in_range(self.spectrum,(self.view.region[1][0],self.view.region[0][0]),native=False)
		if self.spectrum.domain==mssputil.DOMAIN_FREQ:
			mz_points=mssputil.freq2mz_list(mssputil.get_f_in_range(self.spectrum,(self.view.region[1][1],self.view.region[0][1])),self.spectrum)
		else:
			mz_points=mssputil.get_mz_in_range(self.spectrum,(self.view.region[1][1],self.view.region[0][1]),native=False)

		mz_range=(mz_points[0],mz_points[-1])
		mz_step=(mz_points[1]-mz_points[0])
		rt_range=(rt_points[0],rt_points[-1])
		rt_step=(rt_points[1]-rt_points[0])

		if peak_mz>mz_range[0] and peak_mz<mz_range[1] and peak_rt>rt_range[0] and peak_rt<rt_range[1]:
			self.peak=peaks[0]
			self.peptide_check.button.configure(state='normal')

		self.mz_res_edit.variable.set('%0.5f' % mz_step)
		self.rt_res_edit.variable.set('%0.3f' % rt_step)

	# --------------------------------------------------------------------------
	#
	def save_cb(self):

		path=mstkutil.save_file(self.top,'Save data','save_data')
		if path!=None:
			self.lock_gui(1)
			self.path=path
			self.stoppable_call(self.get_data)
			if len(self.data)>0:
				self.stoppable_call(self.write_file)
			self.lock_gui(0)
			self.top.destroy()

	# ---------------------------------------------------------------------------
	#
	def get_data(self):

		self.progress_report('Getting data')

		rt_points=mssputil.get_rt_in_range(self.spectrum,(self.view.region[1][0],self.view.region[0][0]),native=True)
		if self.spectrum.domain==mssputil.DOMAIN_FREQ:
			mz_points=mssputil.get_f_in_range(self.spectrum,(self.view.region[1][1],self.view.region[0][1]))
		else:
			mz_points=mssputil.get_mz_in_range(self.spectrum,(self.view.region[0][1],self.view.region[1][1]))

		self.data=mssputil.get_datalines(self.spectrum,mz_points,rt_points)

		if self.interpolation_check.variable.get()==1:
			mz_scale=1
			rt_scale=1

#			mz_scale=float(self.mz_scale_edit.variable.get())
#			rt_scale=float(self.rt_scale_edit.variable.get())
#			self.data=self.interpolation(self.data,mz_scale,rt_scale)
		else:
			mz_scale=1
			rt_scale=1

		if self.peak!=None and self.peptide_check.variable.get()==1:
			self.peptide_header='peptide:\t' + mssputil.get_sequence_with_charge(self.peak,1)
			self.peptide_header=self.peptide_header + '\n'
			
			self.peptide_header=self.peptide_header + 'isotopic envelope:\t'
			self.peptide_header=self.peptide_header + mssputil.get_isodistrib(self.peak)
			self.peptide_header=self.peptide_header + '\n'

			self.peptide_header=self.peptide_header + 'isotopic mass:\t'
			self.peptide_header=self.peptide_header + mssputil.get_isomass(self.peak)
			self.peptide_header=self.peptide_header + '\n'

			self.peptide_header=self.peptide_header + 'monoisotopic peak:\t'
			self.peptide_header=self.peptide_header + ('%0.5f,%0.4f\n\n') % (mssputil.get_mz(self.peak),mssputil.get_rt(self.peak))
		else:
			self.peptide_header=None

		if self.data_check.variable.get()==1:
			if self.spectrum.domain==mssputil.DOMAIN_FREQ:
				mz_points=mssputil.freq2mz_list(mz_points,self.spectrum)
			else:
				mz_points=-1*mz_points
			rt_points=-1*rt_points

			mz_range=(mz_points[0],mz_points[-1])
			mz_points_count=len(mz_points)*mz_scale
			mz_step=(mz_points[1]-mz_points[0])/mz_scale

			rt_range=(rt_points[0],rt_points[-1])
			rt_points_count=len(rt_points)*rt_scale
			rt_step=(rt_points[1]-rt_points[0])/rt_scale

			self.data_header=('m/z range:\t%0.5f,%0.5f\n') % (mz_range[0],mz_range[1])
			self.data_header=self.data_header + ('m/z points:\t%d\n') % mz_points_count
			self.data_header=self.data_header + ('m/z step:\t%0.5f\n') % mz_step

			self.data_header=self.data_header + ('rt range:\t%0.5f,%0.5f\n') % (rt_range[0],rt_range[1])
			self.data_header=self.data_header + ('rt points:\t%d\n') % rt_points_count
			self.data_header=self.data_header + ('rt step:\t%0.5f\n') % rt_step
		else:
			self.data_header=None


	# -----------------------------------------------------------------------------
	#
	def interpolation(self,data,xScale,yScale):

		xDataSize=len(data[0])
		xSize=int(round(xScale*xDataSize))
		yDataSize=len(data)
		ySize=int(round(yScale*yDataSize))

		iData=[]

		for y in range(0,ySize):
			yNode=y/yScale
			yLow=int(math.floor(yNode))
			yHigh=yLow+1
			if yHigh>=yDataSize:
				yHigh=yDataSize-1
			yDiff=yNode-yLow

			row=[]
			for x in range(0,xSize):
				xNode=x/xScale
				xLow=int(math.floor(xNode))
				xHigh=xLow+1
				if xHigh>=xDataSize:
					xHigh=xDataSize-1
				xDiff=xNode-xLow
				row.append(data[yLow][xLow]*(1-yDiff)*(1-xDiff)+data[yLow][xHigh]*(1-yDiff)*xDiff+data[yHigh][xLow]*yDiff*(1-xDiff)+data[yHigh][xHigh]*yDiff*xDiff)

			iData.append(row)
	
		return iData

	# -----------------------------------------------------------------------------
	#

	def write_file(self):

		self.progress_report('Saving data')

		if self.path:
			file=open(self.path,'w')
			if self.peptide_header!=None:
				file.write(self.peptide_header+'\n')
			if self.data_header!=None:
				file.write(self.data_header+'\n')
			self.stoppable_loop('rows',50)
			for i in range(0,len(self.data)):
				line=''
				row=self.data[i]
				for j in range(0,len(row)):
					if j>0:
						line=line+'\t'
					line=line+ ('%0.4f' % row[j])

				file.write(line+'\n')

			file.close()

	# ---------------------------------------------------------------------------
	#
	def lock_gui(self,lock):

		if lock==1:
			state='disabled'
			edit='no'
			if sys.platform=='win32':
				cursor='wait'
			else:
				cursor='watch'
		else:
			state='normal'
			edit='yes'
			cursor=''

		self.mz_res_edit.entry['state']=state
		self.rt_res_edit.entry['state']=state
		self.mz_scale_edit.entry['state']=state
		self.rt_scale_edit.entry['state']=state

		self.buttons_row.buttons[0]['state']=state
		self.buttons_row.buttons[2]['state']=state

		self.top.config(cursor=cursor)
		self.top.update_idletasks()

# -----------------------------------------------------------------------------
#
def export_peptide(session):

	view=session.selected_view()
	if view!=None:
		spectrum=view.spectrum
		if spectrum!=None and hasattr(spectrum,'is_ms'):
			d=mssputil.the_dialog(export_peptide_dialog,session)
			d.set_view(view)
			d.show_window(1)



