import time
import math
import gc

import Tkinter
import numpy
from PmwContribD import ProgressMeter as pm

import odict

import sparky
import ms
import msfindpeptides_settings
import msenvelope
import msfit
import mscalib
import mschromatogram
import mssputil
import mstkutil
import msmath
import msfilter
import msduplicated


# -----------------------------------------------------------------------------
#
class find_dialog(mstkutil.Dialog,mstkutil.Stoppable):

	def __init__(self,session,spectrum,settings,peak=None):

		self.session=session
		self.spectrum=spectrum
		self.settings=settings
		self.peak=peak
		self.peptide_list=[]
		
		mstkutil.Dialog.__init__(self,session.tk,'Searching peptides')

		f=Tkinter.Frame(self.top,borderwidth=2,relief='groove')
		f.pack(side='top',anchor='w',pady=2,fill='x',expand=1)

		l=Tkinter.Label(f,text='Processing spectrum: ')
		l.grid(row=0,column=0,sticky='w')

		e=Tkinter.Entry(f,borderwidth=1,width=40,justify='left')
		e.insert(0,self.spectrum.name)
		e.configure(state='readonly')
		e.grid(row=0,column=1,sticky='e',padx=2,pady=2)

		progress_meter=pm.ProgressMeter(f,progresscolor='blue',indicatorcolor='white',shadowthickness=2);
		progress_meter.grid(row=1,columnspan=2,sticky='we',pady=2,padx=2)
		self.progress_meter=progress_meter

		progress_label=Tkinter.Label(f)
		progress_label.grid(row=2,columnspan=2,sticky='w',padx=2,pady=2)

		f.columnconfigure(0, weight=1)
		f.columnconfigure(1, weight=1)

		br=mstkutil.button_row(self.top,
				('Stop', self.stop_cb),
				)
		br.frame.pack(side='top',anchor='w',pady=2)
		self.buttons_row=br

		self.top.geometry('+0+0')
		self.top.resizable(0,0)

		self.top.protocol("WM_DELETE_WINDOW",self.stop_cb)

		mstkutil.Stoppable.__init__(self,progress_label,br.buttons[0])

	# ---------------------------------------------------------------------------
	#
	def find_cb(self):

		self.msg=''

		numpy.random.seed(123)
		
		self.stoppable_call(self.get_peptides)
		if self.stop_requested==0:
			self.settings.iteration=0
			self.stoppable_call(self.find_peptides)

			if self.stop_requested==0:
				if self.settings.calibration==True:
					self.stoppable_call(self.calibrate)
		
					if self.settings.second_pass==True:
						if self.settings.stop!=1 and self.stop_requested==0:
							self.settings.iteration=1
							self.stoppable_call(self.find_peptides)
#						else:
#							pytanie o przywrocenie pozycji pikow
#			else:
#				pytanie o przywrocenie pozycji pikow

				if self.stop_requested==0 and self.settings.remove_duplicated:
					self.stoppable_call(self.remove_duplicated)
				
				if self.stop_requested==0:
					if self.settings.view_log==True:
						d=mstkutil.Text_Dialog(None,('MS find peptides log (%s)' % self.spectrum.name),self.msg,30,100)
						d.show_window(1)

					if self.settings.save_log==True:
						self.save_log(('%s.log' % mssputil.remove_ext(self.spectrum.save_path,'save')),self.msg)

					if self.settings.list_save==True:
						self.save_list(('%s.list' % mssputil.remove_ext(self.spectrum.save_path,'save')))
		
		self.peptide_list=None
		self.settings=None
		gc.collect()

	# -----------------------------------------------------------------------------
	#
	def get_peptides(self):

		if self.peak!=None:
			peptide=ms.peptide(mssputil.get_sequence(self.peak),self.peak)
			peptide.add(mssputil.get_query(self.peak))
			
			self.peptide_list=[peptide]
		else:
			self.peptide_list=mssputil.get_peptides(self.spectrum,self,self.progress_meter)
			
			if self.settings.peptides_sort==msfindpeptides_settings.PEP_SORT_RT:
				self.peptide_list=sorted(self.peptide_list,key=lambda peptide: peptide.query_list[0].massref_rt)
			elif self.settings.peptides_sort==msfindpeptides_settings.PEP_SORT_MZ:
				self.peptide_list=sorted(self.peptide_list,key=lambda peptide: peptide.query_list[0].cmz)
		
	# ---------------------------------------------------------------------------
	#
	def find_peptides(self):

		self.progress_meter.configure(indicatorformat='%(progress)s peptides [%(percentdone)s %%]')
		self.progress_meter.configure(finishvalue=float(len(self.peptide_list)))
		self.progress_report('Searching peptides (iteration %d)' % (self.settings.iteration+1))
		
		start=time.clock()
		
		view=mssputil.preferred_view(self.spectrum)
		if view!=None:
			peptides_count=len(self.peptide_list)
			self.stoppable_loop(None,0)

			#log parametrow przeszukania (tylko gdy nie doszukujemy brakujacych lub pojedynczego)
			msg=self.log_search_settings().strip()
			if self.settings.not_present_only==False and self.peak==None:
				self.spectrum.ms_processing=msg
			fit_msg=''

			#pobranie pelnego zakresu czasu retencji
			full_rt=mssputil.get_rt_in_range(self.spectrum,native=True)

			#pobranie paramtrow zaleznych od iteracji
			ch_peaks_count,ch_filter_span,ch_threshold,ch_max_fits_count,do_full_fit,fit_present_env_rsquare_th,fit_present_rt_rsquare_th,fit_present_peaks_th=self.get_iteration_params()

			#print 'ch peak count: %d' % ch_peaks_count
			#print 'max peak count: %d' % ch_max_fits_count
			#print 'do full fit: %d' % do_full_fit
			#print 'fit present env rsquare th: %f' % fit_present_env_rsquare_th
			
			for i in range(0,peptides_count):
				self.check_for_stop()
				self.progress_meter.updateProgress(i)

				peptide=self.peptide_list[i]
				for j in range(len(peptide.query_list)):
					query=peptide.get(j)
					#print (('%s (+%d)') % (peptide.sequence,query.charge))

					if self.settings.not_present_only==False or (self.settings.not_present_only==True and query.present==0):
						query.set_present(0)
						peak=query.get(0)

						isodistrib=msenvelope.MSIsoDistrib(peak,0.05)

						rp,mz_correction,massref_rt=self.get_query_params(query)
						rt=self.get_rt_scale(full_rt,massref_rt,self.settings.ch_rt_penality_fun,self.settings.ch_rt_penality_s)
			
						(chromat,chr_msg)=mschromatogram.get_chromatogram(self.spectrum,isodistrib,query.charge,mz_correction,self.settings.ch_mz_margin,ch_peaks_count,ch_threshold,self.settings.ch_require_max,self.settings.ch_rt_penality_fun,massref_rt,self.settings.ch_rt_penality_s,rt,self.settings.ch_rt_penality_expand)

						if chromat!=None:
							#wlaczenie/wylaczenie filtracji w zaleznosci od ustawien
							mschromatogram.filter_chromatogram(chromat,self.settings.ch_filter_type,ch_filter_span,self.settings.ch_filter_order)

							mschromatogram.get_chromatogram_peaks(chromat,self.settings.ch_max_elimination*self.settings.ch_max_threshold,self.settings.ch_sort_peaks,query.massref_rt,False,ch_max_fits_count)
							#print 'chromat peaks: %d' % len(chromat.peaks)
							
							self.msg=self.msg+('\n\n#################################################\nSequence: %s (+%d)\n Number of chromatogram peaks: %d' % (peptide.sequence,query.charge,len(chromat.peaks)))
							if len(chr_msg)>0:
								self.msg=self.msg + ('\n %s' % chr_msg)
								
							fit_msg=''
							if len(chromat.peaks)>0:
								peak_fits,fit_msg=msfit.fit_chromatogram_peaks(self.spectrum,chromat,isodistrib,query.charge,self.settings.fit_peaks_count,rp,mz_correction,self.settings.ch_mz_margin,self.settings.fit_rt_monoisotopic,self.settings.fit_env_rsquare_th,peptide.sequence,self.settings.view_fits,do_full_fit,self.settings.fit_scans_margin)
								#print 'fits count: %d' % len(peak_fits)
								msfit.sort_fits(peak_fits,0)
								msfit.set_fits(peak,peak_fits)
								
								if len(peak_fits)>0:
									peak.best_fit=0
									
									#wyrzucic do osobnej funkcji check_fits
									if peak_fits[peak.best_fit].mz_chi>=fit_present_peaks_th:
										if peak_fits[peak.best_fit].mz_env_rsquare>=fit_present_env_rsquare_th:
											if peak_fits[peak.best_fit].rt_rsquare>=fit_present_rt_rsquare_th:
												query.set_present(1)
											else:
												query.set_present(0)
												fit_msg=fit_msg+(('\n WARNING: best RT fit R-square below present threshold (%f) ') % fit_present_rt_rsquare_th)
										else:
											query.set_present(0)
											fit_msg=fit_msg+(('\n WARNING: best m/z fit R-square below present threshold (%f) ') % fit_present_env_rsquare_th)
									else:
										query.set_present(0)
										fit_msg=fit_msg+(('\n WARNING: best fit peaks number below present threshold (%d) ') % fit_present_peaks_th)

									query.set_position(peak_fits[peak.best_fit].rt_pos,peak_fits[peak.best_fit].mz_pos)
									query.mz=mssputil.get_mz(peak)
									query.rt=mssputil.get_rt(peak)
								else:
									if self.settings.move_peaks==True:
										query.restore_position()
							else:
								msfit.set_fits(peak,[])
								if self.settings.move_peaks==True:
									query.restore_position()
						else:
							self.msg=self.msg+('\n\n#################################################\nSequence: %s (+%d)\n Number of chromatogram peaks: 0 (chromatogram out of RT range)' % (peptide.sequence,query.charge))

						self.msg=self.msg+("\n Present: %d%s\n" % (query.present,fit_msg))
		print (time.clock()-start)
		
	# ---------------------------------------------------------------------------
	#
	def get_rt_scale(self,full_rt,rt_mean,penality_fun,rt_fwmh):

		if penality_fun!=mschromatogram.PENALITY_NONE and rt_mean!=0 and rt_fwmh!=0:
			if penality_fun==mschromatogram.PENALITY_GAUSS:
				rt_threshold=1.3383022576488544e-004
				rt_cost=msmath.gauss(full_rt,1,-1*rt_mean,rt_fwmh/(2*numpy.sqrt(2*numpy.log(2.0))))

				rt=full_rt[rt_cost>rt_threshold]
				
			elif penality_fun==mschromatogram.PENALITY_SQUARE:
				rt_cost=numpy.ones(len(full_rt))
				rt_cost[full_rt < -(rt_mean+rt_fwmh/2)]=0
				rt_cost[full_rt > -(rt_mean-rt_fwmh/2)]=0
				
				rt=full_rt[rt_cost==1]
		else:
			rt=full_rt
		return rt

	# ---------------------------------------------------------------------------
	#
	def get_iteration_params(self):
		ch_peaks_count=self.settings.ch_peaks_count
		ch_filter_span=self.settings.ch_filter_span
		ch_threshold=self.settings.ch_threshold
		ch_max_fits_count=0
		do_full_fit=1
		if self.settings.fit_present_env_rsquare==0:
			fit_present_env_rsquare_th=0
		else:
			fit_present_env_rsquare_th=self.settings.fit_present_env_rsquare_th
		if self.settings.fit_present_rt_rsquare==0:
			fit_present_rt_rsquare_th=0
		else:
			fit_present_rt_rsquare_th=self.settings.fit_present_rt_rsquare_th
		if self.settings.fit_present_peaks_count==0:
			fit_present_peaks_th=0
		else:
			fit_present_peaks_th=self.settings.fit_present_peaks_count_th
		
		#ustawienie innych parametrow dla pierwszego kroku przeszukania (jezeli first_pass_restrictive==1)		
		if self.settings.first_pass_restrictive==1 and self.settings.second_pass==1 and self.settings.iteration==0:
			#ch_peaks_count=ch_peaks_count+1									#tworzenie chromatogramu dla wszystkich pikow obwiedni
			#ch_filter_span=2*ch_filter_span									#silniejsze filtrowanie
			#ch_threshold=2*ch_threshold										#wyzszy prog
			ch_max_fits_count=1													#uwzglednianie tylko N pierwszych (najwyzszych) pikow chromatogramu
			do_full_fit=0														#uproszczony fit (bez kierunku RT)				
			fit_present_env_rsquare_th=max(0.95,fit_present_env_rsquare_th)		#ostrzejszy prog dopasowania
			fit_present_rt_rsquare_th=0											#fit RT nie jest robiony, dlatego prog musi byc wyzerowany
			fit_present_peaks_th=self.settings.fit_peaks_count					#wymagane istnienie wszystkich fitpwanych pikow
			
		return ch_peaks_count,ch_filter_span,ch_threshold,ch_max_fits_count,do_full_fit,fit_present_env_rsquare_th,fit_present_rt_rsquare_th,fit_present_peaks_th
	
	# ---------------------------------------------------------------------------
	#
	def get_query_params(self,query):
		
		#rp=mssputil.get_rp_at_mz(self.spectrum,query.cmz,(self.settings.iteration==1 or self.settings.spectrum_rp_correction==True))
		rp=mssputil.get_rp_at_mz(self.spectrum,query.cmz,True)
		
		mz_correction=0
		if self.spectrum.calibrated==1 and (self.settings.iteration==1 or self.settings.spectrum_mz_correction==True):
			mz_correction=mssputil.get_mz_correction(self.spectrum,query.cmz)
						
		massref_rt=query.massref_rt
		if self.spectrum.calibrated==1 and (self.settings.iteration==1 or self.settings.spectrum_rt_correction==True):
			massref_rt=mscalib.get_calibrated_rt(query.massref_rt,self.spectrum)

		return rp,mz_correction,massref_rt
	
	# ---------------------------------------------------------------------------
	#
	def calibrate(self):

#		self.progress_report('Estimating params')

		mz,delta_mz=mscalib.get_mz_delta(self.peptide_list,0,self.spectrum)
		rt,delta_rt=mscalib.get_rt_delta(self.peptide_list,0,self.spectrum)
		rp_mz,rp=mscalib.get_rp_delta(self.peptide_list)
		
		if mz==None or len(mz)<3:
			self.settings.stop=1
		else:
			self.spectrum.calibrated=1
			self.settings.stop=0
			
			#parametry kalibracji m/z
			if len(delta_mz)>0:
				delta_mz=numpy.array(delta_mz)
				mz=numpy.array(mz)
			
				mz,delta_mz,mz_a,mz_b,mz_sf,mz_sa,mz_precision,mz_inliers_index,mz_ransac=mscalib.calibrate(self.session,mz,delta_mz,self.settings.mz_calibration_method,self.settings.mz_estimation_method,self.settings.mz_estimation_m)

				#ustawienia widma zapamietywane dopiero po zakonczeniu przeszukiwania
				self.spectrum.mz_correction_method=self.settings.mz_calibration_method
				self.spectrum.mz_correction_a=numpy.around(mz_a,6)
				self.spectrum.mz_correction_b=numpy.around(mz_b,6)

				if self.settings.save_second_pass_params==True or self.settings.view_second_pass_params==True:
					fig=mscalib.calibration_plot("m/z","Mass error [ppm]",mz,delta_mz,mz_a,mz_b,mz_sf,mz_sa,mz_precision,mz_inliers_index,mz_ransac,(self.settings.ch_mz_margin[0],self.settings.ch_mz_margin[1]))
					d=mstkutil.PlotDialog(self.session,fig,('Mass error vs. m/z: ' + self.spectrum.name))
					
					if self.settings.save_second_pass_params==True:
						(saved,msg)=d.save_figure('%s.mz.png' % mssputil.remove_ext(self.spectrum.save_path,'save'))
						if saved==False:
							mstkutil.warning_dialog(self.top,'Error while saving image:\n%s' %msg)
					if self.settings.view_second_pass_params==True:
						d.show_window(1)
					else:
						d.close()

				if self.settings.estimation==True:
					self.spectrum.mz_precision=numpy.around(min(max(self.settings.ch_mz_margin[0],self.settings.ch_mz_margin[1]),mz_precision),6)
					self.settings.ch_mz_margin=numpy.around((self.spectrum.mz_precision,self.spectrum.mz_precision),6)
					
			#parametry kalibracji RT
			if len(delta_rt)>0:
				rt=numpy.array(rt)
				delta_rt=numpy.array(delta_rt)
				
				if self.settings.ch_rt_penality_fun==mschromatogram.PENALITY_GAUSS:
					rt_m=2*numpy.sqrt(2*numpy.log(2.0))*self.settings.rt_estimation_m
				else:
					rt_m=self.settings.rt_estimation_m

				rt,delta_rt,rt_a,rt_b,rt_sf,rt_sa,rt_precision,rt_inliers_index,rt_ransac=mscalib.calibrate(self.session,rt,delta_rt,self.settings.rt_calibration_method,self.settings.rt_estimation_method,rt_m)

				#ustawienia widma zapamietywane dopiero po zakonczeniu przeszukiwania??????????
				self.spectrum.rt_correction_method=self.settings.rt_calibration_method
				self.spectrum.rt_correction_a=numpy.around(rt_a,6)
				self.spectrum.rt_correction_b=numpy.around(rt_b,6)
				if rt_sa!=None:
					self.spectrum.rt_correction_sa=numpy.around(rt_sa,6).tolist()
					self.spectrum.rt_correction_sf=numpy.around(rt_sf,6).tolist()
				else:
					self.spectrum.rt_correction_sa=None
					self.spectrum.rt_correction_sf=None

				if self.settings.save_second_pass_params==True or self.settings.view_second_pass_params==True:
					fig=mscalib.calibration_plot("RT [min]","RT error [min]",rt,delta_rt,rt_a,rt_b,rt_sf,rt_sa,rt_precision,rt_inliers_index,rt_ransac,(self.settings.ch_rt_penality_s/2.0,self.settings.ch_rt_penality_s/2.0))
					d=mstkutil.PlotDialog(self.session,fig,('RT error vs. RT: ' + self.spectrum.name))

					if self.settings.save_second_pass_params==True:
						(saved,msg)=d.save_figure('%s.rt.png' % mssputil.remove_ext(self.spectrum.save_path,'save'))
						if saved==False:
							mstkutil.warning_dialog(self.top,'Error while saving image:\n%s' %msg)
					if self.settings.view_second_pass_params==True:
						d.show_window(1)
					else:
						d.close()

				if self.settings.estimation==True:
					self.settings.ch_rt_penality_s=numpy.around(min(self.settings.ch_rt_penality_s,2*rt_precision),6)
					self.spectrum.rt_precision=numpy.around(self.settings.ch_rt_penality_s,6)
					
			#parametry kalibracji RP
			if len(rp)>0:
				rp=numpy.array(rp)
				rp_mz=numpy.array(rp_mz)
				
				rp_mz,rp,rp_a,rp_b,rp_sf,rp_sa,rp_precision,rp_inliers_index,rp_ransac=mscalib.calibrate(self.session,rp_mz,rp,self.settings.rp_calibration_method,self.settings.rp_estimation_method,3)
				
				self.spectrum.rp_a=numpy.around(rp_a,6)
				self.spectrum.rp_b=numpy.around(rp_b,6)

				if self.settings.save_second_pass_params==True or self.settings.view_second_pass_params==True:
					fig=mscalib.calibration_plot("m/z","Resolving power",rp_mz,rp,rp_a,rp_b,rp_sf,rp_sa,rp_precision,rp_inliers_index,rp_ransac)
					d=mstkutil.PlotDialog(self.session,fig,('Resolving power vs. m/z: ' + self.spectrum.name))
					
					if self.settings.save_second_pass_params==True:
						(saved,msg)=d.save_figure('%s.rp.png' % mssputil.remove_ext(self.spectrum.save_path,'save'))
						if saved==False:
							mstkutil.warning_dialog(self.top,'Error while saving image:\n%s' %msg)
					if self.settings.view_second_pass_params==True:
						d.show_window(1)
					else:
						d.close()

	# ---------------------------------------------------------------------------
	#
	def log_search_settings(self):

		if self.settings.iteration==0:
			self.msg=self.msg+(('Spectrum: %s') % self.spectrum.name)

		self.msg=self.msg+('\n\n#################################################\n#################################################\nSearch settings (iteration %d):' % (self.settings.iteration+1))
		if self.settings.not_present_only==True:
			self.msg=self.msg+'\n Searching only not present peptides'
		else:
			self.msg=self.msg+'\n Searching all peptides'


		#parametry widma
		if self.settings.estimation==1 and self.settings.iteration==1:
			estimation=self.settings.estimation
		else:
			estimation=0

		#czy w obecnym ukladzie warunek jest sluszny?????????
		if self.spectrum.calibrated==1 and (self.settings.iteration==1 or self.settings.spectrum_mz_correction==True):
			mz_calibration=self.settings.mz_calibration_method
			mz_a=self.spectrum.mz_correction_a
			mz_b=self.spectrum.mz_correction_b
		else:
			mz_calibration=mscalib.CALIB_NONE
			mz_a=0
			mz_b=0
		
		if self.spectrum.calibrated==1 and (self.settings.iteration==1 or self.settings.spectrum_rt_correction==True):
			rt_calibration=self.settings.rt_calibration_method
			rt_sa=self.spectrum.rt_correction_sf
			rt_sf=self.spectrum.rt_correction_sa
			rt_a=self.spectrum.rt_correction_a
			rt_b=self.spectrum.rt_correction_b
		else:
			rt_calibration=mscalib.CALIB_NONE
			rt_a=0
			rt_b=0
			rt_sf=None
			rt_sa=None

		#rp_coeffs=mssputil.get_rp_coeffs(self.spectrum,(self.settings.iteration==1 or self.settings.spectrum_rp_correction==True))
		rp_coeffs=mssputil.get_rp_coeffs(self.spectrum,True)

		params_msg=('\n RP(m/z) calibration method: %s' % mscalib.CALIB_STR[self.settings.rp_calibration_method+1])
		params_msg=params_msg+('\n RP(m/z) coeffs: %f %f' % rp_coeffs)

		params_msg=params_msg+('\n m/z calibration method: %s' % mscalib.CALIB_STR[self.settings.mz_calibration_method+1])
		if not self.settings.mz_calibration_method==mscalib.CALIB_NONE:
			params_msg=params_msg+('\n m/z coeffs.: %f %f' % (mz_a,mz_b))
		params_msg=params_msg+('\n m/z precision estimation: %d' % self.settings.estimation)
		if estimation==1:
			params_msg=params_msg+('\n m/z estimation method: %s' % mscalib.ESTIM_STR[self.settings.mz_estimation_method+1])
			params_msg=params_msg+('\n m/z estimation multiplier: %f' % self.settings.mz_estimation_m)
			
		params_msg=params_msg+('\n RT calibration method: %s' % mscalib.CALIB_STR[rt_calibration+1])
		if rt_calibration==mscalib.CALIB_LOWESS:
			params_msg=params_msg+('\n RT coeffs:')
			for i in range(0,len(rt_sa)):
				params_msg=params_msg+(' (%f;%f)' % (rt_sa[i],rt_sf[i]))
		elif not rt_calibration==mscalib.CALIB_NONE:
			params_msg=params_msg+('\n RT coeffs.: %f * rt + %f [min]' % (rt_a,rt_b))
		params_msg=params_msg+('\n RT precision estimation: %d' % self.settings.estimation)
		if estimation==1:
			params_msg=params_msg+('\n RT estimation method: %s' % mscalib.ESTIM_STR[self.settings.rt_estimation_method+1])
			params_msg=params_msg+('\n RT estimation multiplier: %f' % self.settings.rt_estimation_m)
			
		#parametry chromatogramu
		params_msg=params_msg+('\n Chr. peaks count: %d' % self.settings.ch_peaks_count)
		params_msg=params_msg+('\n Chr. m/z error margin: %f %f' % (self.settings.ch_mz_margin[0],self.settings.ch_mz_margin[1]))

		params_msg=params_msg+('\n Chr. data threshold: %f' % self.settings.ch_threshold)
		params_msg=params_msg+('\n Chr. Require max: %d ' % self.settings.ch_require_max)
		if self.settings.ch_rt_penality==1:
			params_msg=params_msg+('\n Chr. RT penality fun: %s ' % mschromatogram.PENALITY_STR[self.settings.ch_rt_penality_fun])
			params_msg=params_msg+('\n Chr. RT penality width: %f ' % self.settings.ch_rt_penality_s)
			params_msg=params_msg+('\n Chr. RT penality expand: %d %d ' % (self.settings.ch_rt_penality_expand[0],self.settings.ch_rt_penality_expand[1]))
		else:
			params_msg=params_msg+('\n Chr. RT penality: NONE ')

		#parametry filtracji chromatogramu
		if self.settings.ch_filter_type!=msfilter.FILTER_NONE:
			params_msg=params_msg+('\n Chr. filter type: %s' % msfilter.FILTER_STR[self.settings.ch_filter_type])
			params_msg=params_msg+('\n Chr. filter span: %f' % self.settings.ch_filter_span)
			params_msg=params_msg+('\n Chr. filter order: %d' % self.settings.ch_filter_order)
		else:
			params_msg=params_msg+('\n Chr. filter type: NONE')

		#parametry peak-pickingu chromatogramu
		params_msg=params_msg+('\n Chr. max threshold: %f' % (self.settings.ch_max_elimination*self.settings.ch_max_threshold))
		params_msg=params_msg+('\n Chr. max sorting: %s' % mschromatogram.SORT_STR[self.settings.ch_sort_peaks])

		#parametry fitu
		params_msg=params_msg+('\n Fit peaks count: %d' % self.settings.fit_peaks_count)
		params_msg=params_msg+('\n Fit scans margin: %d' % self.settings.fit_scans_margin)
		params_msg=params_msg+('\n Fit RT to monoisotopic: %d' % self.settings.fit_rt_monoisotopic)
		params_msg=params_msg+('\n Fit min. r-square: %f' % self.settings.fit_env_rsquare_th)
		if self.settings.fit_present_env_rsquare==1:
			params_msg=params_msg+('\n Fit min. present r-square: %f' % self.settings.fit_present_env_rsquare_th)
		if self.settings.fit_present_rt_rsquare==1:
			params_msg=params_msg+('\n Fit min. present RT r-square: %f' % self.settings.fit_present_rt_rsquare_th)
		if self.settings.fit_present_peaks_count==1:
			params_msg=params_msg+('\n Fit min. present peaks: %d' % self.settings.fit_present_peaks_count_th)
				
		self.msg=self.msg+params_msg
		
		return params_msg
		
	# ---------------------------------------------------------------------------
	#
	def remove_duplicated(self):		
		duplicated_list=get_duplicated(self.spectrum,self.settings.remove_duplicated_mz_tol,self.settings.remove_duplicated_rt_tol,self,self.progress_meter)
		
		self.progress_meter.configure(indicatorformat='%(progress)s peptides [%(percentdone)s %%]')
		self.progress_meter.configure(finishvalue=float(len(duplicated_list)))
		self.progress_report('Removing duplicated assignments')

		self.stoppable_loop(None,0)
		for i in range(len(duplicated_list)):
			self.check_for_stop()
			self.progress_meter.updateProgress(i)
			duplicate=duplicated_list[i]

			if duplicate.size()>1:
				vals=numpy.zeros(duplicate.size())
				vals2=numpy.zeros(duplicate.size())
				vals3=numpy.zeros(duplicate.size())
				for j in range(duplicate.size()):
					peak=duplicate.get(j)

					#m/z
					charge=mssputil.get_charge(peak)
					mz=mssputil.get_mz(peak)
					cmz=mssputil.m2mz(mssputil.get_cmass(peak),charge)

					#z odpowiednich ustawic
					#if self.settings.delta_mz_correction==1 and self.spectrum.rt_precision!=0:
					#	dmz_correction=mssputil.get_mz_correction_ppm(self.spectrum,cmz)
					#else:
					#	dmz_correction=0
					dmz_correction=mssputil.get_mz_correction_ppm(self.spectrum,cmz)
					delta_mz_ppm=1e+6*((mz-cmz)/cmz)-dmz_correction
					
					#RT
					rt=mssputil.get_rt(peak)
					massref_rt=mssputil.get_massref_rt(peak)

					#if self.settings.delta_mrt_correction==1:
					#	drt_correction=mssputil.get_rt_correction(self.spectrum,massref_rt)
					#else:
					#	drt_correction=0
					drt_correction=mssputil.get_rt_correction(self.spectrum,massref_rt)
					delta_mrt=rt-massref_rt-drt_correction

					score=float(mssputil.get_score(peak))
					
					if self.settings.remove_duplicated_method==msduplicated.METHOD_MZ:
						vals[j]=numpy.abs(delta_mz_ppm)
						vals2[j]=numpy.abs(delta_mrt)
						vals3[j]=-1*score
					elif self.settings.remove_duplicated_method==msduplicated.METHOD_RT:
						vals[j]=numpy.abs(delta_mrt)
						vals2[j]=numpy.abs(delta_mz_ppm)
						vals3[j]=-1*score
					elif self.settings.remove_duplicated_method==msduplicated.METHOD_SCORE:
						vals[j]=-1*score					
						vals2[j]=numpy.abs(delta_mz_ppm)
						vals3[j]=numpy.abs(delta_mrt)

				indices=numpy.nonzero(vals==numpy.min(vals))[0]
				if (len(indices)>1):
					indices2=numpy.nonzero(vals2[indices]==numpy.min(vals2[indices]))[0]
					if (len(indices2)>1):
						indices3=numpy.nonzero(vals3[indices[indices2]]==numpy.min(vals3[indices[indices2]]))[0]
						index=indices[indices2[indices3]][0]
					else:
						index=indices[indices2][0]
				else:
					index=indices[0]

				for j in range(duplicate.size()):
					if j!=index:
						peak=duplicate.get(j)
						mssputil.set_peak_present(peak,0)
						
		#poprawienie statusu present dla queries z listy na podstawie pikow		
		for i in range(len(self.peptide_list)):
			peptide=self.peptide_list[i]
			for j in range(len(peptide.query_list)):
				query=peptide.get(j)
				peak=query.get(0)
				if query.present==1 and mssputil.is_peak_present(peak)!=1:					
					query.set_present(0)
				
	# --------------------------------------------------------------------------
	#
	def save_log(self,path,log):
		
		try:
			file=open(path,'w')
			file.write(log)
			file.close()
		except IOError, ioe:
			mstkutil.warning_dialog(self.top,'Error while saving log:\n%s' % str(ioe))
	
	# --------------------------------------------------------------------------
	#
	def save_list(self,path):
		
		rt_step=mssputil.get_rt_step(self.spectrum)
		if self.spectrum==mssputil.DOMAIN_FREQ:
			mz_step=mssputil.get_f_step(self.spectrum)
		else:
			mz_step=mssputil.get_mz_step(self.spectrum)

		list=[]
		list.append(('F','P','Sequence','Protein','Fit','m/z',(u'\u03B4m/z'),'RT',(u'\u0394RT'),'Volume','RMS(m/z)',(u'R\u00B2(m/z)'),'F(m/z)','RMS(RT)',(u'R\u00B2(RT)'),'F(RT)','RMS(E)',(u'R\u00B2(E)'),'Peaks'))
		
		#self.stoppable_loop('peptides',50)
		for i in range(len(self.peptide_list)):
			#self.check_for_stop()
			peptide=self.peptide_list[i]

			for j in range(len(peptide.query_list)):
				query=peptide.get(j)
				peak=query.get(0)
				fits=msfit.get_fits(peak)

				#if self.settings.delta_mz_correction==1 and self.spectrum.rt_precision!=0:
				#	dmz_correction=mssputil.get_mz_correction_ppm(self.spectrum,query.cmz)
				#else:
				#	dmz_correction=0
				dmz_correction=mssputil.get_mz_correction_ppm(self.spectrum,query.cmz)
				
				#if self.settings.delta_mrt_correction==1:
				#	drt_correction=mssputil.get_rt_correction(self.spectrum,query.massref_rt)
				#else:
				#	drt_correction=0
				drt_correction=mssputil.get_rt_correction(self.spectrum,query.massref_rt)
					
				mz_str=('%0.4f' % query.cmz)
				delta_mz_ppm=1e+6*((query.mz-query.cmz)/query.cmz)-dmz_correction
				delta_mz_str=('%0.2f' % delta_mz_ppm)
				
				rt_str=('%0.2f' % query.rt)
				delta_mrt=query.rt-(query.massref_rt+drt_correction)
				delta_mrt_str=('%0.2f' % delta_mrt)

				fit_str=('- (0)')
				mz_rmse_str='NaN'
				mz_rsquare_str='NaN'
				mz_flag_str='NaN'
				mz_env_rmse_str='NaN'
				mz_env_rsquare_str='NaN'
				rt_rmse_str='NaN'
				rt_rsquare_str='NaN'
				rt_flag_str='NaN'
				fit_rmse_str='NaN'
				value_str='NaN'
				
				present=query.present
				fit=None
				if len(fits)>0:
					fit_nr=peak.best_fit
					if numpy.abs(fits[fit_nr].rt_pos-peak.frequency[0])<=rt_step and numpy.abs(fits[fit_nr].mz_pos-peak.frequency[1])<=mz_step:
						fit=fits[fit_nr]
						fit_str=('%d (%d)' % (fit.fit_nr,len(fits)))

						mz_rmse_str=('%0.2f' % fit.mz_rmse)
						mz_rsquare_str=('%0.2f' % fit.mz_rsquare)
						mz_flag_str=('%d' % fit.mz_flag)

						mz_env_rmse_str=('%0.2f' % fit.mz_env_rmse)
						mz_env_rsquare_str=('%0.2f' % fit.mz_env_rsquare)

						rt_rmse_str=('%0.2f' % fit.rt_rmse)
						rt_rsquare_str=('%0.2f' % fit.rt_rsquare)
						rt_flag_str=('%d' % fit.rt_flag)

						fit_rmse_str=('%d' % fit.peaks_count)
						
						if present==1:
							value=msfit.get_fit_value(peak,self.settings.list_data_height,fit,self.settings.list_only_monoisotopic)
							if value>0:
								if self.settings.list_log_scale==1:
									value=math.log(value,10)
									value_str=('%0.3f' % value)
								else:
									value_str=('%0.1f' % value)
					else:
						fit_str=('- (%d)' % len(fits))

				list.append(('0',('%d' % present),peptide.get_sequence(1,j),peptide.proteins,fit_str,mz_str,delta_mz_str,rt_str,delta_mrt_str,value_str,mz_rmse_str,mz_rsquare_str,mz_flag_str,rt_rmse_str,rt_rsquare_str,rt_flag_str,mz_env_rmse_str,mz_env_rsquare_str,fit_rmse_str))

		file=open(path,'w')
		for i in range(len(list)):
			row=list[i]
			line=unicode('')
			for j in range(len(row)):
				cell=row[j]
				if cell=='':
					cell='-'
				line=line+unicode(cell)
				line=line+'\t'
			file.write(line.encode('utf-8')+'\n')
		file.close()

		
# ---------------------------------------------------------------------------
#
#przerucic do msduplicated
def get_duplicated(spectrum,mz_tol,rt_tol,dialog,progress_meter=None):
		
	duplicated_list=[]

	peaks=spectrum.peak_list()
	rt_step=mssputil.get_rt_step(spectrum)
	if spectrum==mssputil.DOMAIN_FREQ:
		mz_step=mssputil.get_f_step(spectrum)
	else:
		mz_step=mssputil.get_mz_step(spectrum)

	dialog.progress_report('Searching duplicated assignments')
							
	for peak in peaks:
		#dialog.check_for_stop()
		if mssputil.is_peak_unassigned(peak)!=1 and mssputil.is_peak_present(peak)==1:
			duplicate=ms.duplicated_query()
			duplicate.add(peak)
			duplicate.mz=peak.frequency[1]
			duplicate.rt=peak.frequency[0]
			if mz_tol==-1:
				duplicate.mz_tol=mz_step
			else:
				duplicate.mz_tol=mz_tol
				if rt_tol==-1:
					duplicate.rt_tol=rt_step
				else:
					duplicate.rt_tol=rt_tol
			duplicated_list.append(duplicate)
	
	if progress_meter!=None:
		dialog.stoppable_loop(None,0)
		progress_meter.configure(indicatorformat='%(progress)s peaks [%(percentdone)s %%]')
		progress_meter.configure(finishvalue=float(len(peaks)))
	else:
		dialog.stoppable_loop('peaks',100)

	duplicated_count=len(duplicated_list)
	for i in range(duplicated_count-1,-1,-1):
		dialog.check_for_stop()
		duplicated=duplicated_list[i]
		for j in range(i-1,-1,-1):
			current=duplicated_list[j]
			if cmp(current,duplicated):
#				for peak in duplicated.peak_list:
#					current.add(peak)
#				break
#				print duplicated.size()
				for k in range(duplicated.size()-1,-1,-1):
#					print k
					peak=duplicated.get(k)
					current.add(peak)
					duplicated.remove(peak)
				break

	return(duplicated_list)
		
# -----------------------------------------------------------------------------
#
def show_autofind_peptides_dialog(session,spectrum,settings,peak=None):

	if spectrum==None:
		return

	d=find_dialog(session,spectrum,settings,peak)
	d.show_window(1)
	d.find_cb()

	return d

