import os
import Tkinter

import mschromatogram
import msfilter
import mscalib
import msfit
import msduplicated

PEP_SORT_NONE=0
PEP_SORT_MZ=1
PEP_SORT_RT=2


class findpeptides_settings:

	def __init__(self,session,path=None,view_fits=False,view_log=False):

		self.session=session
		self.loaded=0

		if path!=None:
			self.loaded=self.read_settings(path)

		#parametry chromatogramu
		try:
			num=int(self.session.project.saved_value('FP_CH_PEAKS_COUNT'))
			if num<0:
				raise ValueError
			self.ch_peaks_count=num
		except (ValueError,TypeError):
			self.ch_peaks_count=2

		try:
			nums_str=self.session.project.saved_value('FP_CH_MZ_RANGE')
			if nums_str!=None:
				fields=nums_str.split(',')
				if len(fields)==2:
					num1=int(fields[0])
					num2=int(fields[1])
				if num1<0 or num2<0:
					raise ValueError
				self.ch_mz_margin=(num1,num2)
			else:
				raise ValueError
		except (ValueError,TypeError):
			self.ch_mz_margin=(30,30)

		try:
			num=int(self.session.project.saved_value('FP_CH_REQUIRE_MAX'))
			if num<0 or num>1:
				raise ValueError
			if num==0:
				self.ch_require_max=False
			else:
				self.ch_require_max=True
		except (ValueError,TypeError):
			self.ch_require_max=True

		try:
			num=float(self.session.project.saved_value('FP_CH_THRESHOLD'))
			if num<0:
				raise ValueError
			self.ch_threshold=num
		except (ValueError,TypeError):
			self.ch_threshold=500

		try:
			str=self.session.project.saved_value('FP_CH_FILTER_TYPE')
			filter_type=msfilter.get_filter_type(str)
			if filter_type==-1:
				raise ValueError
			self.ch_filter_type=filter_type
		except (ValueError,TypeError):
			self.ch_filter_type=msfilter.FILTER_GAUSS

		try:
			num=float(self.session.project.saved_value('FP_CH_FILTER_SPAN'))
			if num<0:
				raise ValueError
			self.ch_filter_span=num
		except (ValueError,TypeError):
			self.ch_filter_span=1.5

		try:
			num=int(self.session.project.saved_value('FP_CH_FILTER_ORDER'))
			if num<0:
				raise ValueError
			self.ch_filter_order=num
		except (ValueError,TypeError):
			self.ch_filter_order=0

		try:
			num=int(self.session.project.saved_value('FP_CH_RT_PENALITY'))
			if num<0 or num>1:
				raise ValueError
			self.ch_rt_penality=num
		except (ValueError,TypeError):
			self.ch_rt_penality=1

		try:
			num=int(self.session.project.saved_value('FP_RT_PENALITY_FUN'))
			if num<mschromatogram.PENALITY_NONE or num>mschromatogram.PENALITY_SQUARE:
				raise ValueError
			self.ch_rt_penality_fun=num
		except (ValueError,TypeError):
			self.ch_rt_penality_fun=mschromatogram.PENALITY_GAUSS
			
		try:
			num=float(self.session.project.saved_value('FP_CH_RT_PENALITY_S'))
			if num<0:
				raise ValueError
			self.ch_rt_penality_s=num
		except (ValueError,TypeError):
			self.ch_rt_penality_s=5.0

		try:
			nums_str=self.session.project.saved_value('FP_CH_RT_PENALITY_EXPAND')
			if nums_str!=None:
				fields=nums_str.split(',')
				if len(fields)==2:
					num1=int(fields[0])
					num2=int(fields[1])
				if num1<0 or num1>1 or num2<0 or num2>1:
					raise ValueError
				self.ch_rt_penality_expand=(num1,num2)
			else:
				raise ValueError
		except (ValueError,TypeError):
			self.ch_rt_penality_expand=(1,1)
			
		try:
			num=int(self.session.project.saved_value('FP_CH_MAX_ELIMINATION'))
			if num<0 or num>1:
				raise ValueError
			self.ch_max_elimination=num
		except (ValueError,TypeError):
			self.ch_max_elimination=1

		try:
			num=float(self.session.project.saved_value('FP_CH_MAX_THRESHOLD'))
			if num<0:
				raise ValueError
			self.ch_max_threshold=num
		except (ValueError,TypeError):
			self.ch_max_threshold=2

		try:
			num=int(self.session.project.saved_value('FP_CH_SORT_PEAKS'))
			if num<0 or num>2:
				raise ValueError
			self.ch_sort_peaks=num
		except (ValueError,TypeError):
			self.ch_sort_peaks=mschromatogram.SORT_VALUE


		#parametry fitu
		try:
			num=int(self.session.project.saved_value('FP_FIT_PEAKS_COUNT'))
			if num<1:
				raise ValueError
			self.fit_peaks_count=num
		except (ValueError,TypeError):
			self.fit_peaks_count=2

		try:
			num=int(self.session.project.saved_value('FP_FIT_SCAN_MARGIN'))
			if num<0:
				raise ValueError
			self.fit_scans_margin=num
		except (ValueError,TypeError):
			self.fit_scans_margin=0
			
		try:
			num=int(self.session.project.saved_value('FP_FIT_RT_TO_MONOISOTOPIC'))
			if num<0 or num>1:
				raise ValueError
			if num==0:
				self.fit_rt_monoisotopic=False
			else:
				self.fit_rt_monoisotopic=True
		except (ValueError,TypeError):
			self.fit_rt_monoisotopic=False

		try:
			num=int(self.session.project.saved_value('FP_FIT_RSQUARE_THRESHOLD'))
			if num<0 or num>1:
				raise ValueError
			self.fit_env_rsquare_th=num
		except (ValueError,TypeError):
			self.fit_env_rsquare_th=0.1

		#parametry akceptacji fitu
		try:
			num=int(self.session.project.saved_value('FP_FIT_PRESENT_ENV_RSQUARE_USE'))
			if num!=0 and num!=1:
				raise ValueError
			self.fit_present_env_rsquare=num
		except (ValueError,TypeError):
			self.fit_present_env_rsquare=1
			
		try:
			num=float(self.session.project.saved_value('FP_FIT_PRESENT_ENV_RSQUARE_THRESHOLD'))
			if num<0 or num>1:
				raise ValueError
			self.fit_present_env_rsquare_th=num
		except (ValueError,TypeError):
			self.fit_present_env_rsquare_th=0.7

		try:
			num=int(self.session.project.saved_value('FP_FIT_PRESENT_RT_RSQUARE_USE'))
			if num!=0 and num!=1:
				raise ValueError
			self.fit_present_rt_rsquare=num
		except (ValueError,TypeError):
			self.fit_present_rt_rsquare=1
			
		try:
			num=float(self.session.project.saved_value('FP_FIT_PRESENT_RT_RSQUARE_THRESHOLD'))
			if num<0 or num>1:
				raise ValueError
			self.fit_present_rt_rsquare_th=num
		except (ValueError,TypeError):
			self.fit_present_rt_rsquare_th=0.7

		try:
			num=int(self.session.project.saved_value('FP_FIT_PRESENT_PEAKS_USE'))
			if num!=0 and num!=1:
				raise ValueError
			self.fit_present_peaks_count=num
		except (ValueError,TypeError):
			self.fit_present_peaks_count=1
			
		try:
			num=int(self.session.project.saved_value('FP_FIT_PRESENT_PEAKS_THRESHOLD'))
			if num<0 or num>10:
				raise ValueError
			self.fit_present_peaks_count_th=num
		except (ValueError,TypeError):
			self.fit_present_peaks_count_th=2
							
		
		#uzywanie kalibracji obecnego widma
		try:
			num=int(self.session.project.saved_value('FP_SPECTRUM_RT_CORRECTION'))
			if num<0 or num>1:
				raise ValueError
			self.spectrum_rt_correction=num
		except (ValueError,TypeError):
			self.spectrum_rt_correction=0

		try:
			num=int(self.session.project.saved_value('FP_SPECTRUM_MZ_CORRECTION'))
			if num<0 or num>1:
				raise ValueError
			self.spectrum_mz_correction=num
		except (ValueError,TypeError):
			self.spectrum_mz_correction=0

		try:
			num=int(self.session.project.saved_value('FP_SPECTRUM_RP_CORRECTION'))
			if num<0 or num>1:
				raise ValueError
			self.spectrum_rp_correction=num
		except (ValueError,TypeError):
			self.spectrum_rp_correction=0


		#parametry kalibracji m/z, RT i RP
		try:
			num=int(self.session.project.saved_value('FP_CALIBRATION'))
			if num<0 or num>1:
				raise ValueError
			self.calibration=num
		except (ValueError,TypeError):
			self.calibration=1

		try:
			num=int(self.session.project.saved_value('FP_RT_CALIBRATION_METHOD'))
			if num<0 or num>3:
				raise ValueError
			self.rt_calibration_method=num
		except (ValueError,TypeError):
			self.rt_calibration_method=mscalib.CALIB_LOWESS

		try:
			num=int(self.session.project.saved_value('FP_MZ_CALIBRATION_METHOD'))
			if num<0 or num>3:
				raise ValueError
			self.mz_calibration_method=num
		except (ValueError,TypeError):
			self.mz_calibration_method=mscalib.CALIB_RLS

		try:
			num=int(self.session.project.saved_value('FP_RP_CALIBRATION_METHOD'))
			if num<0 or num>3:
				raise ValueError
			self.rp_calibration_method=num
		except (ValueError,TypeError):
			self.rp_calibration_method=mscalib.CALIB_RLS


		#parametry estymacji precyzji m/z, RT i RP
		try:
			num=int(self.session.project.saved_value('FP_ESTIMATION'))
			if num<0 or num>1:
				raise ValueError
			self.estimation=num
		except (ValueError,TypeError):
			self.estimation=1

		try:
			num=int(self.session.project.saved_value('FP_RT_ESTIMATION_METHOD'))
			if num<0 or num>1:
				raise ValueError
			self.rt_estimation_method=num
		except (ValueError,TypeError):
			self.rt_estimation_method=mscalib.ESTIM_SD

		try:
			num=int(self.session.project.saved_value('FP_RT_ESTIMATION_MULTIPLIER'))
			if num<0:
				raise ValueError
			self.rt_estimation_m=num
		except (ValueError,TypeError):
			self.rt_estimation_m=4

		try:
			num=int(self.session.project.saved_value('FP_MZ_ESTIMATION_METHOD'))
			if num<0 or num>1:
				raise ValueError
			self.mz_estimation_method=num
		except (ValueError,TypeError):
			self.mz_estimation_method=mscalib.ESTIM_SD
			
		try:
			num=int(self.session.project.saved_value('FP_MZ_ESTIMATION_MULTIPLIER'))
			if num<0:
				raise ValueError
			self.mz_estimation_m=num
		except (ValueError,TypeError):
			self.mz_estimation_m=3

		try:
			num=int(self.session.project.saved_value('FP_RP_ESTIMATION_METHOD'))
			if num<0 or num>1:
				raise ValueError
			self.rp_estimation_method=num
		except (ValueError,TypeError):
			self.rp_estimation_method=mscalib.ESTIM_MAD

		try:
			num=int(self.session.project.saved_value('FP_RP_ESTIMATION_MULTIPLIER'))
			if num<0:
				raise ValueError
			self.rp_estimation_m=num
		except (ValueError,TypeError):
			self.rp_estimation_m=3


		#parametry przeszukania
		try:
			num=int(self.session.project.saved_value('FP_MOVE_PEAKS'))
			if num<0 or num>1:
				raise ValueError
			if num==0:
				self.move_peaks=False
			else:
				self.move_peaks=True
		except (ValueError,TypeError):
			self.move_peaks=True
			
		try:
			num=int(self.session.project.saved_value('FP_SORT_PEPTIDES'))
			if num<0 or num>2:
				raise ValueError
			self.peptides_sort=num
		except (ValueError,TypeError):
			self.peptides_sort=PEP_SORT_NONE

		try:
			num=int(self.session.project.saved_value('FP_TWO_STEP_SEARCH'))
			if num<0 or num>1:
				raise ValueError
			self.second_pass=num
		except (ValueError,TypeError):
			self.second_pass=1

		try:
			num=int(self.session.project.saved_value('FP_FIRST_STEP_RESTRICTIVE'))
			if num<0 or num>1:
				raise ValueError
			self.first_pass_restrictive=num
		except (ValueError,TypeError):
			self.first_pass_restrictive=1

		try:
			num=int(self.session.project.saved_value('FP_REMOVE_DUPLICATED'))
			if num<0 or num>1:
				raise ValueError
			self.remove_duplicated=num
		except (ValueError,TypeError):
			self.remove_duplicated=1

		try:
			num=int(self.session.project.saved_value('FP_REMOVE_DUPLICATED_MZ_TOL'))
			if num<0:
				raise ValueError
			self.duplicated_mz_tol=num
		except (ValueError,TypeError):
			self.remove_duplicated_mz_tol=-1

		try:
			num=int(self.session.project.saved_value('FP_REMOVE_DUPLICATED_RT_TOL'))
			if num<0:
				raise ValueError
			self.duplicated_rt_tol=num
		except (ValueError,TypeError):
			self.remove_duplicated_rt_tol=-1

		try:
			num=int(self.session.project.saved_value('FP_REMOVE_DUPLICATED_METHOD'))
			if num<0 or num>msduplicated.METHOD_SCORE:
				raise ValueError
			self.remove_duplicated_method=num
		except (ValueError,TypeError):
			self.remove_duplicated_method=msduplicated.METHOD_MZ
		
		#Logs, plots, lists
		try:
			num=int(self.session.project.saved_value('FP_LIST_SAVE'))
			if num<0 or num>1:
				raise ValueError
			if num==0:
				self.list_save=False
			else:
				self.list_save=True
		except (ValueError,TypeError):
			self.list_save=False

		try:
			num=int(self.session.project.saved_value('FP_LIST_VALUE'))
			if num<msfit.VALUE_DATA_HEIGHT or num>msfit.VALUE_SFIT_VOLUME:
				raise ValueError
			self.list_data_height=num
		except (ValueError,TypeError):
			self.list_data_height=msfit.VALUE_DATA_HEIGHT

		try:
			num=int(self.session.project.saved_value('FP_LIST_LOG_SCALE'))
			if num!=0 and num!=1:
				raise ValueError
			self.list_log_scale=num
		except (ValueError,TypeError):
			self.list_log_scale=0

		try:
			num=int(self.session.project.saved_value('FP_LIST_ONLY_MONOISOTOPIC'))
			if num!=0 and num!=1:
				raise ValueError
			self.list_only_monoisotopic=num
		except (ValueError,TypeError):
			self.list_only_monoisotopic=1


		self.view_fits=view_fits
		self.view_log=view_log
		self.view_second_pass_params=True

		self.save_log=True
		self.save_second_pass_params=True

		#
		self.stop=0
		self.iteration=0

		self.not_present_only=False
		
	# --------------------------------------------------------------------------
	#
	def read_settings(self,path):
		result=0

		if path!=None:
			try:
				file=open(path,'r')
				lines=file.readlines()

				for i in range(0,len(lines)):
					line=lines[i].strip()
					if len(line)!=0 and line[0]!='#':
						fields=lines[i].split('=')
						if len(fields)==2:
							self.session.project.save_value(fields[0].strip(),fields[1].strip())
				file.close()
				result=1
			except IOError, (errno, strerror):
				print "I/O error(%s): %s" % (errno, strerror)
				result=0
		
		return result
	
# --------------------------------------------------------------------------
#
def get_search_settings(spectrum):

	mz_margin_from_processing=False
	rt_margin_from_processing=False
	present_env_rsquare_from_processing=False
	present_rt_rsquare_from_processing=False
	present_peaks_count_from_processing=False
	
	settings=findpeptides_settings(spectrum.session,os.path.join(os.environ.get('SPARKY_INSTALL'),os.path.join('config','findpeptides.cfg')))

	if spectrum.processed==1:
		str=spectrum.ms_processing
		if str!=None and len(str)>1:

			lines=str.split('\n')
			for line in lines:
				#???????co zrobic z use_rt_correction i use_mz_correction
				
				#dodac odczyt delete_duplicated z configu  
				
				if line.strip().startswith('m/z precision estimation:') or line.strip().startswith('RT precision estimation:'):
					fields=line.split(':')
					if len(fields)==2:
						settings.estimation=bool(fields[1])

				elif line.strip().startswith('m/z estimation method:'):
					fields=line.split(':')
					if len(fields)==2:
						settings.mz_estimation_method=mscalib.get_estimation_type(fields[1])
						
				elif line.strip().startswith('m/z estimation multiplier:'):
					fields=line.split(':')
					if len(fields)==2:
						settings.mz_estimation_m=float(fields[1])

				elif line.strip().startswith('RT estimation method:'):
					fields=line.split(':')
					if len(fields)==2:
						settings.rt_estimation_method=mscalib.get_estimation_type(fields[1])
						
				elif line.strip().startswith('RT estimation multiplier:'):
					fields=line.split(':')
					if len(fields)==2:
						settings.rt_estimation_m=float(fields[1])

				elif line.strip().startswith('Chr. peaks count:'):
					fields=line.split(':')
					if len(fields)==2:
						settings.ch_peaks_count=int(fields[1])

				elif line.strip().startswith('Chr. m/z error margin:'):
					fields=line.split(':')
					if len(fields)==2:
						fields=fields[1].strip().split(' ')
						if len(fields)==2:
							settings.ch_mz_margin=(float(fields[0]), float(fields[1]))
							mz_margin_from_processing=True

				elif line.strip().startswith('Chr. data threshold:'):
					fields=line.split(':')
					if len(fields)==2:
						settings.ch_threshold=float(fields[1])
						
				elif line.strip().startswith('Chr. Require max:'):
					fields=line.split(':')
					if len(fields)==2:
						settings.ch_require_max=int(fields[1])

				elif line.strip().startswith('Chr. filter type:'):
					fields=line.split(':')
					if len(fields)==2:
						settings.ch_filter_type=msfilter.get_filter_type(fields[1].strip())

				elif line.strip().startswith('Chr. filter span:'):
					fields=line.split(':')
					if len(fields)==2:
						settings.ch_filter_span=float(fields[1])

				elif line.strip().startswith('Chr. filter order:'):
					fields=line.split(':')
					if len(fields)==2:
						settings.ch_filter_order=int(fields[1])

				elif line.strip().startswith('Chr. RT penality fun:'):
					fields=line.split(':')
					if len(fields)==2:
						settings.ch_rt_penality_fun=mschromatogram.get_penality_type(fields[1].strip())
						if settings.ch_rt_penality_fun==mschromatogram.PENALITY_NONE:
							settings.ch_rt_penality=0
						else:
							settings.ch_rt_penality=1

				elif line.strip().startswith('Chr. RT penality width:'):
					fields=line.split(':')
					if len(fields)==2:
						settings.ch_rt_penality_s=float(fields[1])
						rt_margin_from_processing=True

				elif line.strip().startswith('Chr. RT penality expand:'):
					fields=line.split(':')
					if len(fields)==2:
						fields=fields[1].strip().split(' ')
						if len(fields)==2:
							settings.ch_rt_penality_expand=(float(fields[0]), float(fields[1]))

				elif line.strip().startswith('Chr. max threshold:'):
					fields=line.split(':')
					if len(fields)==2:
						settings.ch_max_threshold=float(fields[1])
						if settings.ch_max_threshold!=0:
							settings.ch_max_elimination=1
						else:
							settings.ch_max_elimination=0

				elif line.strip().startswith('Chr. max sorting:'):
					fields=line.split(':')
					if len(fields)==2:
						settings.ch_filter_type=mschromatogram.get_sort_type(fields[1].strip())

				elif line.strip().startswith('Fit peaks count:'):
					fields=line.split(':')
					if len(fields)==2:
						settings.fit_peaks_count=int(fields[1])

				elif line.strip().startswith('Fit scans margin:'):
					fields=line.split(':')
					if len(fields)==2:
						settings.fit_scans_margin=int(fields[1])

				elif line.strip().startswith('Fit RT to monoisotopic:'):
					fields=line.split(':')
					if len(fields)==2:
						settings.fit_rt_monoisotopic=int(fields[1])

				elif line.strip().startswith('Fit min. r-square:'):
					fields=line.split(':')
					if len(fields)==2:
						settings.fit_env_rsquare_th=float(fields[1])

				elif line.strip().startswith('Fit min. present r-square:'):
					fields=line.split(':')
					if len(fields)==2:
						settings.fit_present_env_rsquare_th=float(fields[1])
						present_env_rsquare_from_processing=True

				elif line.strip().startswith('Fit min. present RT r-square:'):
					fields=line.split(':')
					if len(fields)==2:
						settings.fit_present_rt_rsquare_th=float(fields[1])
						present_rt_rsquare_from_processing=True
						
				elif line.strip().startswith('Fit min. present peaks:'):
					fields=line.split(':')
					if len(fields)==2:
						settings.fit_present_peaks_count_th=int(fields[1])
						present_peaks_count_from_processing=True

			if	present_env_rsquare_from_processing==False:
				settings.fit_present_env_rsquare=0			
			if	present_rt_rsquare_from_processing==False:
				settings.fit_present_rt_rsquare=0			
			if	present_peaks_count_from_processing==False:
				settings.fit_present_peaks_count=0
								
	if mz_margin_from_processing==False and spectrum.calibrated==1:
		settings.ch_mz_margin=(spectrum.mz_precision,spectrum.mz_precision)

	if rt_margin_from_processing==False and spectrum.calibrated==1:
		settings.ch_rt_penality_s=spectrum.rt_precision

	return settings