import os
import sys
import Tkinter

import sparky
import mssputil
import mstkutil

# -----------------------------------------------------------------------------
#
class integration_dialog(mstkutil.Dialog,mstkutil.Stoppable):

	def __init__(self,session):
		self.session=session
		self.settings=integration_dialog_settings()

		mstkutil.Dialog.__init__(self,session.tk,'MS integrate peaks')

		f=Tkinter.Frame(self.top,borderwidth=2,relief='groove')
		f.pack(side='top',fill='both',expand='1')

		l=Tkinter.Label(f,text='Spectrum:')
		l.grid(row=0,column=0,sticky='w')

		sf=Tkinter.Frame(f,borderwidth=0,relief='flat')
		sf.grid(row=0,column=1,sticky='w')

		m=mssputil.spectrum_menu(session,sf,'')
		m.frame.pack(side='left',pady=2)
		m.add_callback(self.select_spectrum)
		self.spectrum_menu=m

		cb=mstkutil.checkbutton(f,'Unlock peaks before integration',self.settings.unlock_peaks.get())
		cb.button.configure(variable=self.settings.unlock_peaks)
		cb.button.grid(row=1,columnspan=2,sticky='w')
		self.unlock_check=cb

		cb=mstkutil.checkbutton(f,'Move peaks to maxima before integration',self.settings.center_peaks.get())
		cb.button.configure(variable=self.settings.center_peaks)
		cb.button.grid(row=2,columnspan=2,sticky='w')
		self.center_check=cb

		cb=mstkutil.checkbutton(f,'Lock peaks after integration',self.settings.block_peaks.get())
		cb.button.configure(variable=self.settings.block_peaks)
		cb.button.grid(row=3,columnspan=2,sticky='w')
		self.block_check=cb

		cb=mstkutil.checkbutton(f,'Only selected peaks',self.settings.only_selected.get())
		cb.button.configure(variable=self.settings.only_selected)
		cb.button.grid(row=4,columnspan=2,sticky='w')
		self.block_check=cb

		l=Tkinter.Label(f,text='Number of iterations:')
		l.grid(row=5,column=0,sticky='w')

		e=mstkutil.num_entry_field(f,1,'10',1,10000,'',8,None,'right')
		e.frame.grid(row=5,column=1,sticky='w',pady='2')
		self.integration_edit=e

		l=Tkinter.Label(f,text='Integrated peaks:')
		l.grid(row=6,column=0,sticky='w')

		e=mstkutil.entry_field(f,'','0',8,None,'right')
		e.frame.grid(row=6,column=1,sticky='w',pady='2')
		self.peaks_edit=e

		l=Tkinter.Label(f,text='Mean FRMSE:')
		l.grid(row=7,column=0,sticky='w')

		e=mstkutil.entry_field(f,'','0',8,None,'right')
		e.frame.grid(row=7,column=1,sticky='w',pady='2')
		self.rms_edit=e

		progress_label=Tkinter.Label(self.top, anchor = 'nw')
		progress_label.pack(side='top',anchor='w')

		br=mstkutil.button_row(self.top,
				('Integrate', self.integrate_cb),
				('Center', self.center_cb),
				('Stop', self.stop_cb),
				('Close', self.close_cb),
				)
		br.frame.pack(side='top',anchor='w',pady=2)
		self.buttons_row=br

		self.top.geometry("+0+0")
		self.top.resizable(0,0)

		self.removed_notice=session.notify_me('removed spectrum from project',self.update_spectrum_menu)
		self.added_notice=session.notify_me('added spectrum to project',self.update_spectrum_menu)
		self.top.bind('<Destroy>',self.remove_callbacks,1)
	
		mstkutil.Stoppable.__init__(self,progress_label,br.buttons[2])

	# ---------------------------------------------------------------------------
	#
	def select_spectrum(self,event):
		self.show_spectrum_info(self.spectrum_menu.spectrum())

	# ---------------------------------------------------------------------------
	#
	def update_spectrum_menu(self,removed_spectrum):
		spectra=self.session.project.spectrum_list()
		if spectra==None or len(spectra)==0:
			self.top.destroy()
		else:
			if self.spectrum!=removed_spectrum:
				spectrum=self.spectrum
			else:
				spectrum=spectra[0]

			self.spectrum_menu.update_menu_cb()
			self.show_spectrum_info(spectrum)

	# ---------------------------------------------------------------------------
	#
	def remove_callbacks(self,event):
		if str(self.top)==str(event.widget):
			if sparky.object_exists(self.session):
				if self.removed_notice!=None:
					self.session.dont_notify_me(self.removed_notice)
				if self.added_notice!=None:
					self.session.dont_notify_me(self.added_notice)
				
			self.removed_notice=None
			self.added_notice=None

	# ---------------------------------------------------------------------------
	#
	def show_spectrum_info(self,spectrum):

		if spectrum!=None:
			self.spectrum=spectrum
			self.spectrum_menu.set(spectrum.name)
			self.stoppable_call(self.get_frmse)

	# ---------------------------------------------------------------------------
	#
	def close_cb(self):
		self.top.destroy()

	# ---------------------------------------------------------------------------
	#
	def center_cb(self):
		self.lock_gui(1)
		self.stoppable_call(self.center_peaks)
		self.stoppable_call(self.get_frmse)
		self.lock_gui(0)

	# ---------------------------------------------------------------------------
	#
	def center_peaks(self):
		
		self.progress_report('Centering peaks')

		peaks=self.spectrum.peak_list()

		self.stoppable_loop('peaks',100)
		for peak in peaks:
			self.check_for_stop()
			if self.settings.only_selected.get()==0 or peak.selected==1:
				is_locked=peak.is_lock
				peak.lock(0)
				peak.center()
				peak.lock(is_locked)	

	# ---------------------------------------------------------------------------
	#
	def integrate_cb(self):
		self.lock_gui(1)
		self.stoppable_call(self.integrate_peaks)
		self.stoppable_call(self.get_frmse)
		self.lock_gui(0)

	# ---------------------------------------------------------------------------
	#
	def integrate_peaks(self):
		self.progress_report('Intergating peaks')
		
		view=mssputil.preferred_view(self.spectrum)
		if view!=None:
			peaks=self.spectrum.peak_list()

			itarations=int(self.integration_edit.get())
		
			self.stoppable_loop('iteration',1)
			for iteration in range(itarations):
				self.check_for_stop()
				for peak in peaks:
					if self.settings.only_selected.get()==0 or peak.selected==1:
						if self.settings.unlock_peaks.get()==1:
							peak.lock(0)
							if self.settings.unlock_peaks.get():
								peak.center()
						else:
							peak.lock(1)
	
						peak.fit(view)
		
						if self.settings.block_peaks.get()==1:
							peak.lock(1)

	# ---------------------------------------------------------------------------
	#
	def get_frmse(self):
		self.progress_report('Calculating FRMSE')
		
		peaks=self.spectrum.peak_list()
		total_fit_rms=0
		counter=0

		self.stoppable_loop('peaks',100)
		for peak in peaks:
			self.check_for_stop()
			fit_rms=mssputil.get_float_fit_residual(peak)
			if fit_rms>0:
				total_fit_rms=total_fit_rms+fit_rms
				counter=counter+1

		self.peaks_edit.variable.set('%d' % counter)
		if counter!=0:
			self.rms_edit.variable.set('%10.1f' % (total_fit_rms/counter))
		else:
			self.rms_edit.variable.set('NaN')

	# ---------------------------------------------------------------------------
	#
	def lock_gui(self,lock):

		if lock==1:
			state='disabled'
			if sys.platform=='win32':
      				cursor='wait'
			else:
				cursor='watch'
		else:
			state='normal'
			cursor=''

		self.buttons_row.buttons[0]['state']=state
		self.buttons_row.buttons[1]['state']=state
		self.buttons_row.buttons[3]['state']=state

		self.top.config(cursor=cursor)
		self.top.update_idletasks()

# -----------------------------------------------------------------------------
#
class integration_dialog_settings:

	def __init__(self):
		self.unlock_peaks=Tkinter.IntVar()
		self.unlock_peaks.set(0)
		self.center_peaks=Tkinter.IntVar()
		self.center_peaks.set(0)
		self.block_peaks=Tkinter.IntVar()
		self.block_peaks.set(0)
		self.only_selected=Tkinter.IntVar()
		self.only_selected.set(0)

# -----------------------------------------------------------------------------
#
def show_integration_dialog(session):

	spectrum=session.selected_spectrum()
	if spectrum==None:
		return

	d=integration_dialog(session)
	d.show_window(1)
	d.show_spectrum_info(spectrum)
