import numpy
import os

import ms
import msfit
import msmath
import msfilter
import mstkutil
import mssputil
import msenvelope

import matplotlib
import matplotlib.pyplot as plt
import matplotlib.font_manager as fm
from matplotlib.figure import Figure


# --------------------------------------------------------------------------
#
def plot_fit(spectrum,rt_range,isodistrib,charge,peaks_count,scans_margin,rp,mz_correction,mz_margin,fit_monoisotopic_rt,sequence):

	#wykonanie fitu
	(fit,mz_points,mz_data,mz_fit_data,mz_envelope_data,mz_e,mz_max_range,mz_max_intensity,mz_peaks,mz_noise,rt_points,rt_data,rt_fit_data,rt_fit_range,rt_max,rt_peaks,rt_noise,fit_success,fit_msg)=msfit.create_fit(spectrum,0,rt_range,isodistrib,float(charge),peaks_count,rp,mz_correction,mz_margin,fit_monoisotopic_rt,True,scans_margin)

	#odwrocenie danych RT
	rt_data=rt_data[::-1]
	rt_points=-1*rt_points[::-1]
	rt_max=-1*rt_max			#po co max jak mam to w fitcie???

	# pobieranie, interpolacja i filtracja danych z pelnego zakresu m/z
	full_mz_range=isodistrib.get_envelope_mz_abs_range(charge,[2,1/charge],mz_correction)
	(full_mz_data,full_mz_points)=msfit.get_mz_data(spectrum,full_mz_range,rt_max,scans_margin)
	(full_mz_data,full_mz_points,mz_step)=msmath.interpolate(full_mz_data,full_mz_points,3)
	full_mz_data=msfilter.filter_data(full_mz_data,filter_type=msfilter.FILTER_GAUSS,filter_span=1,filter_order=0)

	if fit_success>0:
		#uzupelnienie punktow m/z o maksima teoreteycznej obwiedni i fitu
		(fit_probs,fit_mzs,fit_sigmas)=msfit.mz_fit_peaks_form_params(fit.mz_params)
		(mz_data,mz_points)=update_points(mz_data,mz_points,fit_mzs,peaks_count)
		isopeaks=isodistrib.get_highest_peaks(peaks_count,charge)
		(mz_data,mz_points)=update_points(mz_data,mz_points,isopeaks[1,:],peaks_count)

		#wyznaczenie danych fitu m/z i obwiedni
		mz_fit_data=msfit.mz_fit_function(fit.mz_params,mz_points)
		mz_fit_data=numpy.maximum(mz_fit_data,numpy.zeros(len(mz_points)))
		(mz_envelope_data,mz_envelope_points)=isodistrib.get_envelope_for_points(mz_points,charge,rp,mz_correction,fit_probs.max())

		#wyznaczanie danych fitu RT
		#!!!!!!!!!uzupelnienic RT o maksima tak samo jak dla m/z
		if fit.rt_params[-1]==msfit.RT_FIT_GAUSS:
			rt_fit_data=msfit.rt_fit_function(fit.rt_params,rt_points,msfit.RT_FIT_GAUSS)
		else:
			rt_fit_data=msfit.rt_fit_function(fit.rt_params,rt_points,msfit.RT_FIT_EMG)
	else:
		#uzupelnienie punktow m/z o maksima teoreteycznej obwiedni i wyznaczenie obwiedni
		isopeaks=isodistrib.get_highest_peaks(peaks_count,charge)
		(mz_data,mz_points)=update_points(mz_data,mz_points,isopeaks[1,:],peaks_count)
		(mz_envelope_data,mz_envelope_points)=isodistrib.get_envelope_for_points(mz_points,charge,rp,mz_correction,mz_max_intensity)

		mz_fit_data=None
		rt_fit_data=None

	#
	# Rysowanie
	fig=Figure(figsize=(15,6))

	yformatter=matplotlib.ticker.ScalarFormatter()
	yformatter.set_scientific(True)
	yformatter.set_powerlimits((0,0))
	
	# Add every font at the specified location
	#font_dir = os.path.join(os.environ.get('SPARKY_INSTALL'),os.path.join('lib','fonts'))
	#for font in fm.findSystemFonts(font_dir):
	#	print dir(fm.fontManager)
	#	fm.fontManager.addfont(font)
	#rcParams['font.family'] = 'Merriweather'
	
	#fpath=os.path.join(os.environ.get('SPARKY_INSTALL'),os.path.join('lib',os.path.join('fonts','arial.ttf')))
	#fprop = fm.FontProperties(fname=fpath)
	
	#matplotlib.rc('font', family='Arial')
	
	#from matplotlib.font_manager import findfont, FontProperties
	#font = findfont(FontProperties(family=['sans-serif']))
	#print font
	
	#m/z
	p=fig.add_subplot(121)

	p.plot(mz_points,mz_envelope_data,color='r')
	p.plot(full_mz_points,full_mz_data,color='b')

		
	if fit_success>0:
		p.plot(mz_points,mz_fit_data,color='g')

		if mz_e==1:
			color='black'
		else:
			color='red'
		
		p.text(0.02,0.99,r'$m/z = %0.5f$' % fit.get_mz(spectrum),transform=p.transAxes,horizontalalignment='left',verticalalignment='top',fontsize=13,style='italic')	#,fontproperties=fprop
		p.text(0.02,0.94,r'$\Delta_{m/z} = %0.5f\/(%0.2f)$' % (fit.mz_delta-mz_correction,mssputil.da2ppm(fit.mz_delta-mz_correction,isodistrib.get_peak_mz(0,charge)+mz_correction)),transform=p.transAxes,horizontalalignment='left',verticalalignment='top',fontsize=13,style='italic')
		p.text(0.02,0.89,r'$R_{E}^2 = %0.3f$' % fit.mz_env_rsquare,transform=p.transAxes,horizontalalignment='left',verticalalignment='top',color=color,fontsize=13,style='italic')
		p.text(0.02,0.84,r'$R^2 = %0.3f$' % fit.mz_rsquare,transform=p.transAxes,horizontalalignment='left',verticalalignment='top',fontsize=13,style='italic')

#		p.text(0.02,0.99,r'm/z = %0.5f' % fit.get_mz(spectrum),transform=p.transAxes,horizontalalignment='left',verticalalignment='top',fontsize=11,style='italic')
#		p.text(0.02,0.94,r'd m/z = %0.5f (%0.2f)' % (fit.mz_delta-mz_correction,mssputil.da2ppm(fit.mz_delta-mz_correction,isodistrib.get_peak_mz(0,charge)+mz_correction)),transform=p.transAxes,horizontalalignment='left',verticalalignment='top',fontsize=11,style='italic')
#		p.text(0.02,0.89,r'RE^2 = %0.3f' % fit.mz_env_rsquare,transform=p.transAxes,horizontalalignment='left',verticalalignment='top',color=color,fontsize=11,style='italic')
#		p.text(0.02,0.84,r'R^2 = %0.3f' % fit.mz_rsquare,transform=p.transAxes,horizontalalignment='left',verticalalignment='top',fontsize=11,style='italic')
		
	#tymczasowe dla sprawdzenie poprawnosci
	#isopeaks_mz=isodistrib.get_peaks_mz(charge)
	#highest_isopeak=numpy.argmax(isodistrib.probs)
	#mono_isopeak_mz=isopeaks_mz[0]
	#rp_mz_margin=mono_isopeak_mz/rp
	#highest_isopeak_mz_range=isodistrib.get_peak_mz_abs_range(charge,highest_isopeak,[2*rp_mz_margin,2*rp_mz_margin],mz_correction)
	#p.plot([highest_isopeak_mz_range[0],highest_isopeak_mz_range[0]],[0,mz_max_intensity],color='black',linestyle='--')
	#p.plot([highest_isopeak_mz_range[-1],highest_isopeak_mz_range[-1]],[0,mz_max_intensity],color='black',linestyle='--')

	#p.plot([mz_max_range[0],mz_max_range[0]],[0,mz_max_intensity],color='0.5',linestyle='--')
	#p.plot([mz_max_range[-1],mz_max_range[-1]],[0,mz_max_intensity],color='0.5',linestyle='--')
	if mz_peaks!=None and mz_peaks.shape[1]>0:
		#p.plot(mz_peaks[0,:],mz_peaks[1,:],'bo')
		p.plot([full_mz_range[0],full_mz_range[-1]],[mz_noise,mz_noise],color='0.5',linestyle='--')


	p.set_xlim([full_mz_range[0],full_mz_range[-1]])
	p.set_title("Isotopic envelope")
	p.set_xlabel("m/z")
	p.yaxis.set_major_formatter(yformatter)
	p.xaxis.set_major_formatter(matplotlib.ticker.FormatStrFormatter('%.3f'))
	for label in p.yaxis.get_ticklabels() + p.xaxis.get_ticklabels():
		label.set_fontsize(10)
	p.set_position([0.04,0.1,0.45,0.8])

	#RT
	p=fig.add_subplot(122)
	p.plot(rt_points,rt_data,color='b')
	if rt_fit_data!=None:
		p.plot(rt_points,rt_fit_data,color='g')

	if rt_peaks!=None and rt_peaks.shape[1]>0:
		#p.plot(-1*rt_peaks[0,:],rt_peaks[1,:],'bo')
		#p.plot(-1*rt_peaks[3,:],rt_peaks[4,:],'ko')
		#p.plot(-1*rt_peaks[6,:],rt_peaks[7,:],'ko')
		p.plot([rt_points[-1],rt_points[0]],[rt_noise,rt_noise],color='0.5',linestyle='--')

	p.set_xlim([rt_points[-1],rt_points[0]])
	p.set_title("Elution profile")
	p.set_xlabel("RT [min.]")
	p.yaxis.set_major_formatter(yformatter)
	for label in p.yaxis.get_ticklabels() + p.xaxis.get_ticklabels():
		label.set_fontsize(10)
	p.set_position([0.53,0.1,0.45,0.8])

	d=mstkutil.PlotDialog(spectrum.session,fig,'Isotopic envelope: %s (+%d)' % (sequence,charge))
	d.show_window(1)
	
# ---------------------------------------------------------------------------
#
def onclick(event):
	print 'button=%d, x=%d, y=%d, xdata=%f, ydata=%f'%(event.button, event.x, event.y, event.xdata, event.ydata)

# ---------------------------------------------------------------------------
#
def update_points(data,points,new_points,points_count=-1):

	if points_count==-1:
		points_count=len(new_points)
	else:
		points_count=min(len(new_points),points_count)

	for i in range(0,points_count):
		below_index=numpy.flatnonzero(points<new_points[i])
		above_index=numpy.flatnonzero(points>new_points[i])

		if len(below_index)==0:
			points=numpy.hstack([new_points[i],points[above_index]])
			data=numpy.hstack([data[0],data[above_index]])

		elif len(above_index)==0:
			points=numpy.hstack([points[above_index],new_points[i]])
			data=numpy.hstack([data[above_index],data[-1]])

		elif (len(below_index)+len(above_index))==len(points):
			new_value=msmath.lin_interpolation(points[below_index[-1]],points[above_index[0]],data[below_index[-1]],data[above_index[0]],new_points[i])

			points=numpy.hstack([numpy.hstack([points[below_index],new_points[i]]),points[above_index]])
			data=numpy.hstack([numpy.hstack([data[below_index],new_value]),data[above_index]])

	return(data,points)

# --------------------------------------------------------------------------
#
def plot_peak_fit(peak,nofit=False):
	#paramtry => musza byc odczytywane z konfiguracji
	peaks_count=3
	fit_monoisotopic_rt=False
	spectrum_mz_correction=1
	rt_margin=(1.5,2.5)
	scans_margin=0
	
	#pobranie informacji o piku
	sequence=mssputil.get_sequence(peak)
	isodistrib=msenvelope.MSIsoDistrib(peak,0.05)
	charge=float(mssputil.get_charge(peak))
		
	isopeaks_mz=isodistrib.get_peaks_mz(charge)	#pobranie m/z najwyzszego piku (do wyznaczenia korekcji m/z i RP)
	mz=isopeaks_mz[numpy.argmax(isodistrib.probs)]

	rt=mssputil.get_rt(peak)

	#wyznaczenie poprawki skali m/z, rozdzieleczosci i toretycznej szerokosci polowkowej
	mz_correction=0
	if spectrum_mz_correction==1:
		mz_correction=mssputil.get_mz_correction(peak.spectrum,mz)
	rp=mssputil.get_rp_at_mz(peak.spectrum,mz)
	#mz_margin powinien byc parametrem
	mz_margin=(mssputil.da2ppm(mz/rp,mz),mssputil.da2ppm(mz/rp,mz))
	
	plot_fit(peak.spectrum,(-1*(rt-rt_margin[0]),-1*(rt+rt_margin[1]),-1*rt),isodistrib,charge,peaks_count,scans_margin,rp,mz_correction,mz_margin,fit_monoisotopic_rt,sequence)

# -----------------------------------------------------------------------------
#
def show_fit(session,nofit=False):

	spectrum=session.selected_spectrum()
	if spectrum!=None and hasattr(spectrum,'is_ms'):
		peaks=spectrum.selected_peaks()
		if peaks!=None and len(peaks)==1 and peaks[0].note!=None and peaks[0].note!='':
			plot_peak_fit(peaks[0],nofit)

# -----------------------------------------------------------------------------
#
def show_peak_fit(peak,nofit=False):

	if peak!=None:
		plot_peak_fit(peak,nofit)
		
