import string
import math
import sys
import os
import time

import Tkinter
import tkFont

import sparky
import mssputil
import mstkutil
import pyutil
import fpconst
import odict

import ms
import msisoenvelope
import mschromatogram

# -----------------------------------------------------------------------------
#
class labelled_list_dialog(mstkutil.Dialog,mstkutil.Stoppable):

	def __init__(self,session):

		self.session=session
		self.title='MS labelled peptides list:'
		self.settings=labelled_list_settings(self.session,os.path.join(os.environ.get('SPARKY_INSTALL'),os.path.join('config','labelledlist.cfg')))
		self.find_duplicates_dialog=None

		mstkutil.Dialog.__init__(self,session.tk,self.title)

		f=Tkinter.Frame(self.top,borderwidth=2,relief='groove')
		f.pack(side='top',anchor='w',fill='x')

		pl=mssputil.peptide_tablelist(f,columns=None,selected_edit=1)
		pl.set_columns((50,'Sequence',8,'Protein',8,'16o',8,'18o',9,'16o18o/18o[%]',8,'o18/o16'))
		pl.set_columns_align(('left','left','right','right','right','right','right'))
		pl.list_size.configure(width=75)

		pl.tablelist.bind('<ButtonRelease-1>',pl.select_peptide_cb)
		pl.tablelist.bind('<Double-ButtonRelease-1>',pl.goto_peptide_cb)
		pl.tablelist.bind('<Return>',pl.goto_peptide_cb)
		pl.tablelist.bind('<Button-3>',self.popup_menu)
		
		pl.frame.pack(side='top',fill='both',expand=1)		
		self.labelled_tablelist=pl

		f=Tkinter.Frame(self.top,borderwidth=2,relief='groove')
		f.pack(side='top',anchor='w',fill='x')

		l=Tkinter.Label(f,text='Search for:',anchor='w',width=10)
		l.pack(side='left',padx=3)

		e=mstkutil.entry_field(f,'','',45)
		e.frame.pack(side='left',padx=2)
		e.entry.bind('<Return>',self.find_string)
		self.find_edit=e

		m=mstkutil.option_menu(f,'',('Search down','Search up'),'Search down')
		m.frame.pack(side='left',padx=3)
		self.direction_menu=m

		cb=mstkutil.checkbutton(f,'Only exact match',0)
		cb.button.pack(side='left',padx=3)
		self.match_check=cb

		b=Tkinter.Button(f,text="Find",borderwidth=1,width='7',command=self.find_string)
		b.pack(side='right',ipady=2,padx=3,pady=3)
		self.find_button=b

		progress_label=Tkinter.Label(self.top, anchor = 'nw')
		progress_label.pack(side='top',anchor='w')

		br=mstkutil.button_row(self.top,
					('Update', self.update_cb),
					('Stop', self.stop_cb),
					('Setup', self.setup_cb),
					('Save', self.save_cb),
					('Close', self.top.destroy),
					)
		br.frame.pack(side='top',anchor='w',pady=2)
		self.buttons_row=br

		self.top.geometry("+0+0")
		self.top.resizable(1,0)

		self.notice=session.notify_me('removed spectrum from project',self.spectrum_removed_cb)
		self.selection_notice=session.notify_me('selection changed',self.selection_changed_cb)
		self.dragging_notice=session.notify_me('dragged peak',self.peak_dragged_cb)
		self.top.bind('<Destroy>',self.remove_callbacks,1)
		self.top.protocol("WM_DELETE_WINDOW",self.closing_window)

		mstkutil.Stoppable.__init__(self,progress_label,br.buttons[1])


	# ---------------------------------------------------------------------------
	#
	def spectrum_removed_cb(self,removed_spectrum):
		if self.spectrum==removed_spectrum:
			self.top.destroy()

	# ---------------------------------------------------------------------------
	#
	def selection_changed_cb(self):
		peaks=self.session.selected_peaks()
		if peaks!=None and len(peaks)==1:
			self.labelled_tablelist.mark_peak_cb(peaks[0])

	# ---------------------------------------------------------------------------
	#
	def peak_dragged_cb(self,peak):
		if peak.spectrum==self.spectrum:
			line_index=self.labelled_tablelist.find_peak_cb(peak)
#			if line_index!=-1 and (self.settings.view_list_only.get()==0 or (self.settings.view_list_only.get()==1 and self.settings.view_list[int(line_index)]==1)):
			if line_index!=-1:
				self.update_query(line_index)
#				if peak.selected==1:
#					self.labelled_tablelist.selected_peak=None
#					self.labelled_tablelist.mark_peak_cb(peak)

	# ---------------------------------------------------------------------------
	#
	def closing_window(self):
		if self.settings.lock_callbacks==0:
			if self.find_duplicates_dialog!=None:
				self.find_duplicates_dialog.parent=None	
			self.top.destroy()
#		else:
#			self.stop_cb()
				
	# ---------------------------------------------------------------------------
	#
	def remove_callbacks(self,event):
		if str(self.top)==str(event.widget):
			if sparky.object_exists(self.session):
				if self.notice!=None:
					self.session.dont_notify_me(self.notice)
				if self.selection_notice!=None:
					self.session.dont_notify_me(self.selection_notice)
				if self.dragging_notice!=None:
					self.session.dont_notify_me(self.dragging_notice)
					
			self.notice=None
			self.selection_notice=None
			self.dragging_notice=None

	# ---------------------------------------------------------------------------
	#
	def get_peptides(self):

		self.progress_report('Getting peptides')
		self.peptide_list=[]

		peaks=self.spectrum.peak_list()
#		pep_dict={}
		pep_dict=odict.OrderedDict()

		self.stoppable_loop('peaks',100)
		for peak in peaks:
			self.check_for_stop()
			if mssputil.is_peak_unassigned(peak)!=1:
				query=ms.query()
				query.charge=mssputil.get_charge(peak)
				query.mz=mssputil.get_mz(peak)
				query.rt=mssputil.get_rt(peak)
				query.present=mssputil.is_peak_present(peak)
				query.add(peak)
			
				seq=mssputil.get_sequence(peak)
				peptide=ms.peptide(seq)
				peptide.isodistrib=mssputil.get_peak_isodistrib(peak,threshold=5,relative_to_first=1)
				peptide.proteins=mssputil.get_protein(peak)
				query.cmz=mssputil.m2mz(peptide.mass,query.charge)
				peptide.add(query)
				if pep_dict.has_key(seq):
					for query in peptide.query_list:
						pep_dict[seq].add(query)
				else:
					pep_dict[seq]=peptide
			else:
				query.add(peak)

		self.peptide_list=pep_dict.values()

	# ---------------------------------------------------------------------------
	#
	def save_cb(self):
		path=mstkutil.save_file(self.labelled_tablelist.tablelist,'Save List','save_list')
		self.top.update_idletasks()
		self.lock_gui(1)
		self.labelled_tablelist.write_file(path,'w')
		self.lock_gui(0)

	# ---------------------------------------------------------------------------
	#
	def setup_cb(self):
		psd=mssputil.the_dialog(labelled_list_settings_dialog,self.session)
		psd.set_parent_dialog(self,self.settings,self.new_settings)
		psd.top.title(self.title + ' settings')
		psd.show_window(1)

	# ---------------------------------------------------------------------------
	#
	def new_settings(self,settings):
		self.settings=settings
		self.update_cb()

	# ---------------------------------------------------------------------------
	#
	def show_spectrum_peaks(self,spectrum):
		self.spectrum=spectrum
		self.title='MS labelled peptides list: ' + self.spectrum.name
		self.top.title(self.title)
		self.update_cb()

	# -----------------------------------------------------------------------------
	#
	def popup_menu(self,event):
		if self.labelled_tablelist.lock_callbacks==0:
			menu=Tkinter.Menu(self.labelled_tablelist.tablelist,tearoff=0)
			
			menu.add_command(label='Show envelope',command=self.isodistrib_cb)
			menu.add_command(label='Show chromatogram',command=self.chromatogram_cb)
			menu.add_separator()
			ratio_submenu=Tkinter.Menu(menu,tearoff=0)
			ratio_submenu.add_radiobutton(label='16o/18o',variable=self.settings.o18,value=0,command=self.redraw_cb)
			ratio_submenu.add_radiobutton(label='18o/16o',variable=self.settings.o18,value=1,command=self.redraw_cb)
			menu.add_cascade(label='Ratio',menu=ratio_submenu)

			view_submenu=Tkinter.Menu(menu,tearoff=0)
			view_submenu.add_radiobutton(label='All',variable=self.settings.view_mode,value=0,command=self.redraw_cb)
			view_submenu.add_separator()
			view_submenu.add_radiobutton(label='Zeros',variable=self.settings.view_mode,value=1,command=self.redraw_cb)
			view_submenu.add_radiobutton(label='Over threshold',variable=self.settings.view_mode,value=2,command=self.redraw_cb)
			view_submenu.add_radiobutton(label='Differantial',variable=self.settings.view_mode,value=3,command=self.redraw_cb)
			view_submenu.add_separator()
			view_submenu.add_checkbutton(label='Presnet',variable=self.settings.view_only_present,command=self.update_cb)
			menu.add_cascade(label='View',menu=view_submenu)

			menu.add_checkbutton(label='Log scale',variable=self.settings.log_scale,command=self.redraw_cb)
			menu.add_separator()
			menu.add_command(label='Update',command=self.update_cb)

			menu.tk_popup(event.x_root,event.y_root)

	# ---------------------------------------------------------------------------
	#
	def update_cb(self,event=None):
		self.lock_gui(1)

		self.stoppable_call(self.get_peptides)
		self.stoppable_call(self.update_tablelist)

		self.lock_gui(0)
		self.labelled_tablelist.tablelist.focus_set()

	# ---------------------------------------------------------------------------
	#
	def redraw_cb(self,event=None):
		self.lock_gui(1)

		self.stoppable_call(self.update_tablelist)

		self.lock_gui(0)
		self.labelled_tablelist.tablelist.focus_set()

	# ---------------------------------------------------------------------------
	#
	def update_tablelist(self):
		self.progress_report('Updating table')

		counter=-1
		query_counter=0
		present_counter=0

		self.set_value_column()

		selected_peak=None
		row_index=int(round(self.labelled_tablelist.tablelist.yview()[0]*self.labelled_tablelist.tablelist.size()))
		self.labelled_tablelist.clear()

		o18_view=self.settings.o18.get()
		log_view=self.settings.log_scale.get()
		if log_view==1:
			format='%0.3f'
		else:
			format='%0.1f'

		low_th=1/self.settings.ratio_th
		high_th=self.settings.ratio_th

		self.stoppable_loop('peptides',50)
		for i in range(len(self.peptide_list)):
			self.check_for_stop()
			peptide=self.peptide_list[i]

			if peptide.size()>1:
				bgcolor='#d4deec'
			else:
				bgcolor='gray98'

			for j in range(len(peptide.query_list)):
				query=peptide.get(j)
				counter=counter+1

				hidden=0
				error_m=1

				o16_value=0
				o16_value_str='0.0'
				o18_value=0
				o18_value_str='0.0'
				o16_o18_value=0
				o16_o18_value_str='NaN'
				ratio=0
				ratio_str='NaN'

				isodistrib=mssputil.get_spectrum_isodistrib(query.peak_list,data_height=self.settings.data_height,relative=0)
				peaks_count=len(isodistrib)
				if peaks_count>4:
					values=mssputil.get_o16_o18_values(peptide.isodistrib,isodistrib,self.settings.check_peaks)

					o16_value=values[0]
					o18_value=values[2]

					if o16_value>0 and log_view==1:
						o16_value_str=format % math.log(o16_value,10)
					else:
						o16_value_str=format % o16_value

					if o18_value!=0:
						if o18_value<0:
							error_m=-1
	
						if log_view:
							o18_value_str=format % math.log(error_m*o18_value,10)
						else:
							o18_value_str=format % (error_m*o18_value)

						if o16_value!=0:
							if o18_view==1:
								ratio=o18_value/o16_value
							else:
								ratio=o16_value/o18_value
		
							if log_view:
								ratio_str=('%0.3f' % math.log(error_m*ratio,10))
							else:
								ratio_str=('%0.3f' % (error_m*ratio))

						if error_m!=-1 and values[1]>0:
							o16_o18_value=(100*values[1]/o18_value)
							o16_o18_value_str=('%0.1f' % o16_o18_value)

				self.labelled_tablelist.append((peptide.get_sequence(1,j),peptide.proteins,o16_value_str,o18_value_str,o16_o18_value_str,ratio_str),(peptide,j))
				self.labelled_tablelist.tablelist.rowconfigure(counter,background=bgcolor)

				if o16_o18_value==0 or o16_o18_value>self.settings.o16_o18_th:
					self.labelled_tablelist.configure_cell(counter,4,foreground='gray55')

				if ratio!=0:
					over_th=0
					if ratio>=high_th:
						self.labelled_tablelist.configure_cell(counter,5,foreground='red')
						over_th=1
					elif ratio<=low_th:
						self.labelled_tablelist.configure_cell(counter,5,foreground='#008201')
						over_th=1

					if self.settings.view_mode.get()==1 or (self.settings.view_mode.get()==2 and over_th==0) or self.settings.view_mode.get()==3:
						hidden=1
					
					if error_m==-1:
						self.labelled_tablelist.configure_cell(counter,3,foreground='gray55')
						if self.settings.nan.get()==1:
							self.labelled_tablelist.configure_cell(counter,5,foreground='gray55')
							self.labelled_tablelist.configure_cell(counter,5,text='NaN')
					
				else:
					self.labelled_tablelist.configure_cell(counter,5,foreground='gray55')
					if self.settings.view_mode.get()==1:
						if o16_value!=0 or o18_value!=0:
							hidden=1

					elif self.settings.view_mode.get()==2:
						hidden=1

					elif self.settings.view_mode.get()==3:
						if o16_value==0 and o18_value==0:
							hidden=1

				if query.present==0:
					self.labelled_tablelist.configure_cell(counter,0,foreground='red')
					if self.settings.view_only_present.get()==1:
						hidden=1
	
				if hidden==0:
					query_counter=query_counter+1
					if query.present==1:
						present_counter=present_counter+1
				else:
					self.labelled_tablelist.tablelist.rowconfigure(counter,hide=1)

				selected=query.get_selected()
				if selected!=None:
					selected_peak=selected

		self.labelled_tablelist.set_short_list_size(queries=query_counter,present=present_counter)

		if selected_peak==None:
			self.labelled_tablelist.tablelist.yview(row_index)
			self.labelled_tablelist.set_sequence('')
		else:
			self.labelled_tablelist.selected_peak=None
			self.labelled_tablelist.lock_callbacks=0
			self.labelled_tablelist.mark_peak_cb(selected_peak)

	# --------------------------------------------------------------------------
	#
	def set_value_column(self):
		name_o16='16o'
		name_o18='18o'
		name_ratio='18o/16o'
		
		if self.settings.o18.get()==0:
			name_ratio='16o/18o'

		if self.settings.log_scale.get()==1:
			name_o16='log('+name_o16
			name_o16=name_o16+')'
			name_o18='log('+name_o18
			name_o18=name_o18+')'
			name_ratio='log('+name_ratio
			name_ratio=name_ratio+')'

		self.labelled_tablelist.tablelist.columnconfigure(2,title=name_o16)
		self.labelled_tablelist.tablelist.columnconfigure(3,title=name_o18)
		self.labelled_tablelist.tablelist.columnconfigure(5,title=name_ratio)

	# ---------------------------------------------------------------------------
	#
	def update_query(self,row):
		print row

	# ---------------------------------------------------------------------------
	#
	def find_string(self,event=None):
		str=self.find_edit.variable.get()
		if str!='':
			self.lock_gui(1)
			if self.direction_menu.variable.get()=='Search down':
				self.labelled_tablelist.find_string(str,exact_match=self.match_check.state(),y_range=(0,1))
			else:
				self.labelled_tablelist.find_string_up(str,exact_match=self.match_check.state(),y_range=(0,1))
			self.lock_gui(0)

	# ---------------------------------------------------------------------------
	#
	def isodistrib_cb(self,event=None):
		data=self.labelled_tablelist.event_line_data(event)
		if data!=None:
			peptide=data[0]
			query_nr=data[1]
			msisoenvelope.show_peak_isotopic_envelope(peptide.get(query_nr).get(0),o18=1)

	# ---------------------------------------------------------------------------
	#
	def chromatogram_cb(self,event=None):
		data=self.labelled_tablelist.event_line_data(event)
		if data!=None:
			peptide=data[0]
			query_nr=data[1]
			mschromatogram.show_peak_chromatogram(peptide.get(query_nr).get(0))

	# ---------------------------------------------------------------------------
	#
	def view_list_cb(self,event=None):
		if self.settings.view_list_only.get()==1:
			if self.settings.view_list==None or len(self.settings.view_list)==0:
				dialog=mstkutil.warning_dialog(self.top,'The list is empty')
				self.settings.view_list_only.set(0)
				return
		self.update_cb()

	# ---------------------------------------------------------------------------
	#
	def lock_gui(self,lock):

		if lock==1:
			state='disabled'
			edit='no'
			if sys.platform=='win32':
				cursor='wait'
			else:
				cursor='watch'
		else:
			state='normal'
			edit='yes'
			cursor=''

		self.settings.lock_callbacks=lock
		self.labelled_tablelist.lock_callbacks=lock

		self.find_button['state']=state
		self.find_edit.entry['state']=state
		self.buttons_row.buttons[0]['state']=state
		self.buttons_row.buttons[2]['state']=state
		self.buttons_row.buttons[3]['state']=state
		self.buttons_row.buttons[4]['state']=state
		self.top.config(cursor=cursor)
		self.top.update_idletasks()

# -----------------------------------------------------------------------------
#
class labelled_list_settings:

	def __init__(self,session,path=None):

		self.session=session
		self.lock_callbacks=0
		self.loaded=0

		if path!=None:
			loaded=self.read_settings(path)

		self.o18=Tkinter.IntVar()
		try:
			num=int(self.session.project.saved_value('LL_O18'))
			if num<0 or num>1:
				raise ValueError
			self.o18.set(num)
		except (ValueError,TypeError):
			self.o18.set(1)

		try:
			num=int(self.session.project.saved_value('LL_DATA_HEIGHT'))
			if num<0 or num>2:
				raise ValueError
			self.data_height=num
		except (ValueError,TypeError):
			self.data_height=0

		self.log_scale=Tkinter.IntVar()
		try:
			num=int(self.session.project.saved_value('LL_LOG_SCALE'))
			if num<0 or num>1:
				raise ValueError
			self.log_scale.set(num)
		except (ValueError,TypeError):
			self.log_scale.set(0)

		try:
			num=float(self.session.project.saved_value('LL_RATIO_TH'))
			if num<1:
				raise ValueError
			self.ratio_th=num
		except (ValueError,TypeError):
			self.ratio_th=2.0

		try:
			num=float(self.session.project.saved_value('LL_O16_O18_TH'))
			if num<0:
				raise ValueError
			self.o16_o18_th=num
		except (ValueError,TypeError):
			self.o16_o18_th=20

		self.view_mode=Tkinter.IntVar()
		try:
			num=int(self.session.project.saved_value('LL_VIEW_MODE'))
			if num<0 or num>3:
				raise ValueError
			self.view_mode.set(num)
		except (ValueError,TypeError):
			self.view_mode.set(0)

		self.view_only_present=Tkinter.IntVar()
		try:
			num=int(self.session.project.saved_value('LL_VIEW_ONLY_PRESENT'))
			if num<0 or num>1:
				raise ValueError
			self.view_only_present.set(num)
		except (ValueError,TypeError):
			self.view_only_present.set(0)

		self.nan=Tkinter.IntVar()
		try:
			num=int(self.session.project.saved_value('LL_NAN'))
			if num<0 or num>1:
				raise ValueError
			self.nan.set(num)
		except (ValueError,TypeError):
			self.nan.set(0)

		try:
			num=int(self.session.project.saved_value('LL_USE_COLORS'))
			if num<0 or num>1:
				raise ValueError
			self.use_colors=num
		except (ValueError,TypeError):
			self.use_colors=1

		try:
			num=int(self.session.project.saved_value('LL_CHECK_PEAKS'))
			if num<1 or num>2:
				raise ValueError
			self.check_peaks=num
		except (ValueError,TypeError):
			self.check_peaks=1

	# --------------------------------------------------------------------------
	#
	def read_settings(self,path):
		result=0

		if path!=None:
			try:
				file=open(path,'r')
				lines=file.readlines()

				for i in range(0,len(lines)):
					line=lines[i].strip()
					if len(line)!=0 and line[0]!='#':
						fields=lines[i].split('=')
						if len(fields)==2:
							self.session.project.save_value(fields[0].strip(),fields[1].strip())
				file.close()
				result=1
			except IOError, (errno, strerror):
				print "I/O error(%s): %s" % (errno, strerror)
				result=0
		
		return result

# -----------------------------------------------------------------------------
#
class labelled_list_settings_dialog(mstkutil.Settings_Dialog):
	
	def __init__(self, session):
		self.session=session
		self.loaded=0
		mstkutil.Settings_Dialog.__init__(self,session.tk,'Settings')

		f=Tkinter.Frame(self.top,borderwidth=2,relief='groove')
		f.pack(side='top',anchor='w',pady=2,fill='x',expand=1)

		l=Tkinter.Label(f,text='Value: ')
		l.grid(row=0,column=0,sticky='w')

		m=mssputil.value_menu(f,'')
		m.frame.grid(row=0,column=1,sticky='w',pady=2,padx=2)
		self.value_menu=m

		l=Tkinter.Label(f,text='Number od checked peaks: ')
		l.grid(row=1,column=0,sticky='w')

		e=mstkutil.num_entry_field(f,0,'',1,2,'',11)
		e.frame.grid(row=1,column=1,sticky='w',pady=2)
		self.peaks_edit=e

		l=Tkinter.Label(f,text='18o/16o ratio thresh.: ')
		l.grid(row=2,column=0,sticky='w')

		e=mstkutil.num_entry_field(f,0,'',1,1000000,'',11)
		e.frame.grid(row=2,column=1,sticky='w',pady=2)
		self.ratio_th_edit=e

		l=Tkinter.Label(f,text='16o18o/18o ratio thresh.: ')
		l.grid(row=3,column=0,sticky='w')

		e=mstkutil.num_entry_field(f,0,'',1,1000000,'',11)
		e.frame.grid(row=3,column=1,sticky='w',pady=2)
		self.o16_o18_th_edit=e

		cb=mstkutil.checkbutton(f,'Use NaN for suspicious 18o/16o ratios',0)
		cb.button.grid(row=4,columnspan=2,sticky='w',pady=1)
		self.nan_value_check=cb

		br=mstkutil.button_row(self.top,
				('Apply', self.apply_settings_cb),
				('Load', self.load_cb),
				('Close', self.close_cb),
				)
		br.frame.pack(side='bottom',anchor='w',pady=2)

		self.top.geometry("+0+0")
		self.top.resizable(0,0)

	# ---------------------------------------------------------------------------
	#
	def load_cb(self):
		path=mstkutil.load_file(self.top,'Open configuration','open_cfg')
		if path!=None:
			settings=labelled_list_settings(self.session,path)
			self.loaded=settings.loaded
			self.show_settings(settings)

	# ---------------------------------------------------------------------------
	#
	def show_settings(self,settings):
		self.value_menu.set_value(settings.data_height)
		self.peaks_edit.variable.set(settings.check_peaks)
		self.ratio_th_edit.variable.set(settings.ratio_th)
		self.o16_o18_th_edit.variable.set(settings.o16_o18_th)
		self.nan_value_check.variable.set(settings.nan.get())

	# ---------------------------------------------------------------------------
	#
	def get_settings(self):
		settings=labelled_list_settings(self.session)

		settings.data_height=self.value_menu.get_value()
		settings.ratio_th=float(self.ratio_th_edit.variable.get())
		settings.check_peaks=int(self.peaks_edit.variable.get())
		settings.o16_o18_th=float(self.o16_o18_th_edit.variable.get())
		settings.nan.set(self.nan_value_check.variable.get())

		if self.loaded==0:
			settings.log_scale.set(self.parent_dialog.settings.log_scale.get())
			settings.o18.set(self.parent_dialog.settings.o18.get())
			settings.view_mode.set(self.parent_dialog.settings.view_mode.get())
			settings.view_only_present.set(self.parent_dialog.settings.view_only_present.get())

		return settings

	# ---------------------------------------------------------------------------
	#
	def apply_settings_cb(self):
		self.show_window(0)
		self.apply_cb()
		self.close_cb()

	# ---------------------------------------------------------------------------
	#
	def clear_list_cb(self,event=None):
		self.parent_dialog.settings.view_list=None
		self.view_list_check.variable.set(0)
		self.view_list_check.button.configure(state='disabled')

# -----------------------------------------------------------------------------
#
def show_labelled_list(session):
	spectrum=session.selected_spectrum()
	if spectrum==None:
		return

	if not hasattr(session,'spectrum_dialogs'):
		session.spectrum_dialogs={}
	dialogs=session.spectrum_dialogs

	key='LL'+spectrum.name
	if (dialogs.has_key(key) and not dialogs[key].is_window_destroyed()):
		dialogs[key].show_window(1)
	else:
		d=labelled_list_dialog(session)
		d.show_window(1)
		d.show_spectrum_peaks(spectrum)
		dialogs[key]=d



