import math
import numpy
import scipy
import scipy.stats
import scipy.signal
import scipy.linalg
import scipy.interpolate
#import scipy.ndimage.filters


# median absolute deviation
# ----------------------------------------------------------------------------
#
def mad(x):
	m=numpy.median(numpy.abs(x-numpy.median(x)))
	return(m)

# standard deviation form MAD
# ----------------------------------------------------------------------------
#
def std_mad(x):
	std=mad(x)/0.6745
	return(std)

# Wartosci gaussoidy (a,mi,sigma) dla punktow x
# ----------------------------------------------------------------------------
#
def gauss(x,a,mi,sigma):

	y=a*numpy.exp(-0.5*((x-mi)**2)/(sigma**2))

	return(y)

# ----------------------------------------------------------------------------
#
def gauss_get_fwhm(sigma):

	fwmh=2*math.sqrt(2*math.log(2))*sigma

	return(fwmh)

# ---------------------------------------------------------------------------
#
def sse(data,fit_data):

	residuals=data-fit_data
	sserror=numpy.sum(numpy.power(residuals,2))

	return sserror


# ---------------------------------------------------------------------------
#
def mse(data,fit_data):

	mserror=sse(data,fit_data)/len(data)

	return mserror

# ---------------------------------------------------------------------------
#
def rmse(data,fit_data):

	rmserror=numpy.sqrt(mse(data,fit_data))

	return rmserror

# ---------------------------------------------------------------------------
#
def tss(data):

	rmserror=numpy.sum(numpy.power(data-numpy.mean(data),2))

	return rmserror

# ---------------------------------------------------------------------------
#
def rsquare(data,fit_data,compute_rmse=False):

	sserror=sse(data,fit_data)
	totalss=tss(data)
	r=1-sserror/totalss
	if compute_rmse==True:
		rmserror=numpy.sqrt(sserror/len(data))
		return r,rmserror
	else:
		return r


#Interpolacja liniowa
# ---------------------------------------------------------------------------
#
def lin_interpolation(node1,node2,value1,value2,point):

	if point>=node1 and point<=node2:
		if node1==node2 or value1==value2:
			pointValue=value1
		elif node2>node1:
			pointValue=((node2-point)*value1+(point-node1)*value2)/(node2-node1)
		else:
			pointValue=((node1-point)*value2+(point-node2)*value1)/(node1-node2)
	else:
		raise ValueError("Point below interpolation range")

	return(pointValue)


# ---------------------------------------------------------------------------
#
def interpolate(data,points,f):

	points_count=len(data)

#	new_points=numpy.linspace(points[0],points[-1],f)
	new_points=numpy.zeros(f*(points_count-1)+1)
	new_step=numpy.abs((points[0]-points[1])/f)
	for i in range(0,points_count):
		new_points[i*f]=points[i]
		if i<points_count-1:
			for j in range(1,f):
				new_points[i*f+j]=new_points[i*f]+float(j)*new_step

	interp_fun=scipy.interpolate.interp1d(points,data)
	new_data=interp_fun(new_points)
	new_data=numpy.maximum(new_data,numpy.zeros(len(new_data)))

	return(new_data,new_points,new_step)

# LOWESS
# --------------------------------------------------------------------------
#
def lowess(session,x,y,f=0.4,nsteps=0,delta=0.0,m=6.0):

	points_count=len(x)

	if not isinstance(x,numpy.ndarray):
		x=numpy.array(x)
		y=numpy.array(y)

	fit=numpy.array(session.lowess(x.tolist(),y.tolist(),f,nsteps,delta))

	residuals=y-fit
	inliers_index=numpy.arange(0,points_count)
	inliers_index=inliers_index[numpy.abs(residuals)<m*mad(residuals)]

	return fit,inliers_index

# --------------------------------------------------------------------------
#
def lowess_val(fit_x,fit_y,x):

#	print x
#	print fit_x
	ind=numpy.flatnonzero(fit_x<=x)
#	print isinstance(ind,numpy.ndarray)
	if ind.size>0:
#		print ind
		ind=ind[-1]
		fit_x1=fit_x[ind]
		fit_y1=fit_y[ind]
		
		if x==fit_x1:
			f=fit_y1
		else:
			ind=numpy.flatnonzero(fit_x>=x)
			if ind.size>0:
				ind=ind[0]
				fit_x2=fit_x[ind]
				fit_y2=fit_y[ind]

#				print ('%f %f %f %f' % (fit_x1,fit_x2,fit_y1,fit_y2))
				f=lin_interpolation(fit_x1,fit_x2,fit_y1,fit_y2,x)
			else:
				f=fit_y[-1]
	else:
		f=fit_y[0]
	
	return(f)
	
# RLS
# --------------------------------------------------------------------------
#
def rls_linfit(x,y,iterations=1):

	(a,b,rr,tt,stderr)=scipy.stats.linregress(x,y)
	inliers_index=numpy.arange(0,len(x))
	
	for i in range(0,iterations):
		points_count=len(x)

		residuals=y-(a*x+b)
#		s=1.4826*(1.0+(5.0/(points_count-2.0)))*numpy.sqrt(numpy.median(numpy.square(residuals)));

		inliers_index=numpy.arange(0,points_count)
#		inliers_index=inliers_index[numpy.abs(residuals/s)<=3.0]
		inliers_index=inliers_index[numpy.abs(residuals)<6.0*mad(residuals)]

		if len(inliers_index)>0:
			(a,b,rr,tt,stderr)=scipy.stats.linregress(x[inliers_index],y[inliers_index])
		else:
			break

	return a,b,inliers_index


# RANSAC
# --------------------------------------------------------------------------
#
def ransac_linfit(x,y,min_n,min_inliers,error_th,iterations,debug=False):

	points_count=len(x)

	X=numpy.hstack((numpy.reshape(x,(points_count,1)),numpy.ones((points_count,1))))
	Y=numpy.reshape(y,(points_count,1))
	data=numpy.hstack((X,Y))

	lin_model=LinearLeastSquaresModel([0,1],[2],debug=debug)

	try:
		ransac_fit,ransac_data=ransac(data,lin_model,min_n,iterations,error_th,min_inliers,debug=debug,return_all=True)
		ransac_succes=True
		inliers_index=ransac_data['inliers']
		a=ransac_fit[0,0]
		b=ransac_fit[1,0]
	except ValueError:
		ransac_succes=False
		inliers_index=None
		a=0
		b=0

	return a,b,inliers_index,ransac_succes

# --------------------------------------------------------------------------
#
def ransac(data,model,n,k,t,d,debug=False,return_all=False):
	"""fit model parameters to data using the RANSAC algorithm
	
	This implementation written from pseudocode found at
	http://en.wikipedia.org/w/index.php?title=RANSAC&oldid=116358182

	{{{
	Given:
		data - a set of observed data points
		model - a model that can be fitted to data points
		n - the minimum number of data values required to fit the model
		k - the maximum number of iterations allowed in the algorithm
		t - a threshold value for determining when a data point fits a model
		d - the number of close data values required to assert that a model fits well to data
	Return:
		bestfit - model parameters which best fit the data (or nil if no good model is found)
	}}}
	"""
	numpy.random.seed(123)
	iterations = 0
	bestfit = None
	besterr = numpy.inf
	best_inlier_idxs = None
	
	while iterations < k:
		maybe_idxs, test_idxs = random_partition(n,data.shape[0])
		maybeinliers = data[maybe_idxs,:]
		test_points = data[test_idxs]
		maybemodel = model.fit(maybeinliers)
		test_err = model.get_error( test_points, maybemodel)
		also_idxs = test_idxs[test_err < t] # select indices of rows with accepted points
		alsoinliers = data[also_idxs,:]
		if debug:
			print 'test_err.min()',test_err.min()
			print 'test_err.max()',test_err.max()
			print 'numpy.mean(test_err)',numpy.mean(test_err)
			print 'iteration %d:len(alsoinliers) = %d'%(iterations,len(alsoinliers))
		if len(alsoinliers) > d:
			betterdata = numpy.concatenate( (maybeinliers, alsoinliers) )
			bettermodel = model.fit(betterdata)
			better_errs = model.get_error( betterdata, bettermodel)
			thiserr = numpy.mean( better_errs )
			if thiserr < besterr:
				bestfit = bettermodel
				besterr = thiserr
				best_inlier_idxs = numpy.concatenate( (maybe_idxs, also_idxs) )
		iterations+=1
	
	if bestfit is None:
		raise ValueError("did not meet fit acceptance criteria")

	if return_all:
		return bestfit, {'inliers':best_inlier_idxs}
	else:
		return bestfit

# --------------------------------------------------------------------------
#
def random_partition(n,n_data):
	"""return n random rows of data (and also the other len(data)-n rows)"""
	all_idxs = numpy.arange( n_data )
	numpy.random.shuffle(all_idxs)
	idxs1 = all_idxs[:n]
	idxs2 = all_idxs[n:]
	return idxs1, idxs2

# --------------------------------------------------------------------------
#
class LinearLeastSquaresModel:
	"""linear system solved using linear least squares

	This class serves as an example that fulfills the model interface
	needed by the ransac() function.
	
	"""
	def __init__(self,input_columns,output_columns,debug=False):
		self.input_columns = input_columns
		self.output_columns = output_columns
		self.debug = debug

	def fit(self, data):
		A = numpy.vstack([data[:,i] for i in self.input_columns]).T
		B = numpy.vstack([data[:,i] for i in self.output_columns]).T
		x,resids,rank,s = scipy.linalg.lstsq(A,B)
		return x

	def get_error( self, data, model):
		A = numpy.vstack([data[:,i] for i in self.input_columns]).T
		B = numpy.vstack([data[:,i] for i in self.output_columns]).T
		B_fit = scipy.dot(A,model)
		err_per_point = numpy.sum((B-B_fit)**2,axis=1) # sum squared error per row
		return err_per_point