import math
import numpy
import scipy

import msmath
import msfilter

NOISE_MAD=-1
NOISE_STD=-2

# --------------------------------------------------------------------------
#
def find_peaks(data,points,filter_type=msfilter.FILTER_NONE,filter_span=7,filter_order=2,noise_estimate_type=NOISE_MAD,trim_peaks=False,trim_threshold=0,np=True):

	peaks=None
	noise_estimate=0
	if data.max()>0:
		if filter_type==msfilter.FILTER_NONE:
			filter_data=numpy.hstack([data,10])
		else:
			filter_data=numpy.hstack([msfilter.filter_data(data,filter_type,filter_span,filter_order),10])

		deriv_data=numpy.diff(filter_data)
		h=numpy.nonzero(deriv_data!=0)[0]
		g=((filter_data[h[0:h.shape[0]-2]] > filter_data[h[1:h.shape[0]-1]]) & (filter_data[h[1:h.shape[0]-1]] < filter_data[h[2:h.shape[0]]]))
		left=h[numpy.hstack([False,g])]
		right=h[g]+1
		if len(left)>1:
			if right[1]<=left[1]:
				right=right[1:]
			if left[-1]>=right[-1]:
				left=left[:-1]

		noise_estimate=estimate_noise(data,noise_estimate_type)
		if trim_threshold<0:
			trim_threshold=noise_estimate

		peaks=[]
		for i in range(0,len(left)):
			max_index=numpy.argmax(data[left[i]:right[i]+1])+left[i]
			max_value=data[max_index]
			if (trim_peaks==True and max_value>=trim_threshold) or trim_peaks==False:
				peaks.append([points[max_index],max_value])

		if np==True:
			peaks_count=len(peaks)
			if peaks_count>0:
				peaks_array=numpy.zeros((2,peaks_count))
				for i in range(0,peaks_count):
					peak=peaks[i]
					peaks_array[0,i]=peak[0]
					peaks_array[1,i]=peak[1]
				peaks=peaks_array
			else:
				peaks=None

	return peaks,noise_estimate

# --------------------------------------------------------------------------
#
#def find_peaks(data,points,filter_type=None,filter_span=7,filter_order=2,noise_estimate_type=NOISE_MAD,trim_peaks=False,trim_threshold=0,np=True):
def find_peaks2(data,points,filter_type=msfilter.FILTER_NONE,filter_span=7,filter_order=2,noise_estimate_type=NOISE_MAD,trim_peaks=False,trim_threshold=0,np=True):

	peaks=None
	noise_estimate=0
	if data.max()>0:
		if filter_type!=msfilter.FILTER_NONE:
			data=msfilter.filter_data(data,filter_type,filter_span,filter_order)

		noise_estimate=estimate_noise(data,noise_estimate_type)
		if trim_threshold<0:
			trim_threshold=noise_estimate

		points_count=len(data)
		deriv_data=numpy.diff(data)
		deriv_data=numpy.hstack([deriv_data,0])

		# szukanie maksimow pikow
		peak_max=[]
		peak_left_min=[]
		peak_right_min=[]
		for i in range(0,points_count-1):
			if deriv_data[i]>0 and deriv_data[i+1]<=0:
				max_index=i+1
				right_min=max(max_index-1,0)
				while right_min>=0 and data[right_min]<data[right_min+1]:
					right_min=right_min-1

				left_min=min(max_index+1,points_count-1)
				while left_min<points_count and data[left_min]<data[left_min-1]:
					left_min=left_min+1

				peak_max.append(max_index)
				peak_right_min.append(right_min+1)
				peak_left_min.append(left_min-1)

		#tworzenie listy pikow
		peaks_count=len(peak_max)
		if peaks_count>0:
			peaks=[]
			for i in range(peaks_count):
				#points[peak_max[i]]	polozenie max
				#data[peak_max[i]]	wartosc max piku
				#peak_max[i]		index maksimum piku
				
				#points[peak_right_min[i]]	polozenie prawego min
				#data[peak_right_min[i]]	wartosc prawego min
				#peak_right_min[i]		index prawego min piku

				#points[peak_left_min[i]]	polozenie lewego min
				#data[peak_left_min[i]]	polozenie lewego min
				#peak_left_min[i]		index lewego min piku

				peaks.append((points[peak_max[i]],data[peak_max[i]],peak_max[i],points[peak_right_min[i]],data[peak_right_min[i]],peak_right_min[i],points[peak_left_min[i]],data[peak_left_min[i]],peak_left_min[i]))

			if np==True:
				peaks_count=len(peaks)
				if peaks_count>0:
					params_count=len(peaks[0])
					peaks_array=numpy.zeros((params_count,peaks_count))
					for i in range(0,peaks_count):
						for j in range(0,params_count):
							peaks_array[j,i]=peaks[i][j]
					peaks=peaks_array
				else:
					peaks=None

	return peaks,noise_estimate
# ---------------------------------------------------------------------------
#
def estimate_noise(data,estimate_type):

	if estimate_type==NOISE_MAD:
		noise_threshold=numpy.sqrt(2*numpy.log(len(data)))*msmath.mad(data[(data>0)])/0.6745
	else:
		noise_threshold=numpy.std(data[(data>0)])

	max_data=data.max()
	if noise_threshold>=0.2*max_data:
		noise_threshold=0.05*max_data
	if noise_threshold<0.05*max_data:
		noise_threshold=0.05*max_data

	return(noise_threshold)
