import string
import math
import sys
import os
import time

import Tkinter
import tkFont

import sparky
import mssputil
import mstkutil
import pyutil
import fpconst

import ms
import msisoenvelope
import mschromatogram_gui
import msduplicated

# -----------------------------------------------------------------------------
#
class peaks_list_dialog(mstkutil.Dialog,mstkutil.Stoppable):

	def __init__(self,session):

		self.session=session
		self.title='MS peaks list:'
		self.settings=peaks_list_settings(self.session,os.path.join(os.environ.get('SPARKY_INSTALL'),os.path.join('config','peakslist.cfg')))
		self.find_duplicates_dialog=None

		mstkutil.Dialog.__init__(self,session.tk,self.title)

		f=Tkinter.Frame(self.top,borderwidth=2,relief='groove')
		f.pack(side='top',anchor='w',fill='x')

		pl=mssputil.peak_tablelist(f,columns=None,height=20,width=75,selected_edit=1)
		pl.set_columns((50,'Sequence',8,'Protein',8,'m/z',6,'RT',8,'Fit Height'))
		pl.set_columns_align(('left','left','right','right','right','right'))
		pl.list_size.configure(width=75)

		pl.tablelist.bind('<ButtonRelease-1>',pl.select_peak_cb)
		pl.tablelist.bind('<Double-ButtonRelease-1>',pl.goto_peak_cb)
		pl.tablelist.bind('<Return>',pl.goto_peak_cb)
		pl.tablelist.bind('<Button-3>',self.popup_menu)
		
		pl.frame.pack(side='top',fill='both',expand=1)		
		self.peaks_tablelist=pl

		f=Tkinter.Frame(self.top,borderwidth=2,relief='groove')
		f.pack(side='top',anchor='w',fill='x')

		l=Tkinter.Label(f,text='Search for:',anchor='w',width=10)
		l.pack(side='left',padx=3)

		e=mstkutil.entry_field(f,'','',45)
		e.frame.pack(side='left',padx=2)
		e.entry.bind('<Return>',self.find_string)
		self.find_edit=e

		m=mstkutil.option_menu(f,'',('Search down','Search up'),'Search down')
		m.frame.pack(side='left',padx=3)
		self.direction_menu=m

		cb=mstkutil.checkbutton(f,'Only exact match',0)
		cb.button.pack(side='left',padx=3)
		self.match_check=cb

		b=Tkinter.Button(f,text="Find",borderwidth=1,width='7',command=self.find_string)
		b.pack(side='right',ipady=2,padx=3,pady=3)
		self.find_button=b

		progress_label=Tkinter.Label(self.top, anchor = 'nw')
		progress_label.pack(side='top',anchor='w')

		br=mstkutil.button_row(self.top,
					('Update', self.update_cb),
					('Stop', self.stop_cb),
					('Setup', self.setup_cb),
					('Save', self.save_cb),
					('Close', self.top.destroy),
					)
		br.frame.pack(side='top',anchor='w',pady=2)
		self.buttons_row=br

		self.top.geometry("+0+0")
		self.top.resizable(1,0)

		self.notice=session.notify_me('removed spectrum from project',self.spectrum_removed_cb)
		self.selection_notice=session.notify_me('selection changed',self.selection_changed_cb)
		self.dragging_notice=session.notify_me('dragged peak',self.peak_dragged_cb)
		self.top.bind('<Destroy>',self.remove_callbacks,1)
		self.top.protocol("WM_DELETE_WINDOW",self.closing_window)

		mstkutil.Stoppable.__init__(self,progress_label,br.buttons[1])


	# ---------------------------------------------------------------------------
	#
	def spectrum_removed_cb(self,removed_spectrum):
		if self.spectrum==removed_spectrum:
			self.top.destroy()

	# ---------------------------------------------------------------------------
	#
	def selection_changed_cb(self):
		peaks=self.session.selected_peaks()
		if peaks!=None and len(peaks)==1:
			self.peaks_tablelist.mark_peak_cb(peaks[0])

	# ---------------------------------------------------------------------------
	#
	def peak_dragged_cb(self,peak):
		pass
#		if peak.spectrum==self.spectrum:
#			line_index=self.peaks_tablelist.find_peak_cb(peak)
#			if line_index!=-1 and (self.settings.view_list_only.get()==0 or (self.settings.view_list_only.get()==1 and self.settings.view_list[int(line_index)]==1)):
#				self.update_query(line_index)
#				if peak.selected==1:
#					self.peaks_tablelist.selected_peak=None
#					self.peaks_tablelist.mark_peak_cb(peak)

	# ---------------------------------------------------------------------------
	#
	def closing_window(self):
		if self.settings.lock_callbacks==0:
			if self.find_duplicates_dialog!=None:
				self.find_duplicates_dialog.parent=None	
			self.top.destroy()
#		else:
#			self.stop_cb()
				
	# ---------------------------------------------------------------------------
	#
	def remove_callbacks(self,event):
		if str(self.top)==str(event.widget):
			if sparky.object_exists(self.session):
				if self.notice!=None:
					self.session.dont_notify_me(self.notice)
				if self.selection_notice!=None:
					self.session.dont_notify_me(self.selection_notice)
				if self.dragging_notice!=None:
					self.session.dont_notify_me(self.dragging_notice)
					
			self.notice=None
			self.selection_notice=None
			self.dragging_notice=None

	# ---------------------------------------------------------------------------
	#
	def save_cb(self):
		path=mstkutil.save_file(self.peaks_tablelist.tablelist,'Save List','save_list')
		self.top.update_idletasks()
		self.lock_gui(1)
		self.peaks_tablelist.write_file(path,'w')
		self.lock_gui(0)

	# ---------------------------------------------------------------------------
	#
	def setup_cb(self):
		psd=mssputil.the_dialog(peaks_list_settings_dialog,self.session)
		psd.set_parent_dialog(self,self.settings,self.new_settings)
		psd.top.title(self.title + ' settings')
		psd.show_window(1)

	# ---------------------------------------------------------------------------
	#
	def new_settings(self,settings):
		self.settings=settings
		self.update_cb()

	# ---------------------------------------------------------------------------
	#
	def show_spectrum_peaks(self,spectrum):
		self.spectrum=spectrum
		self.title='MS peaks list: ' + self.spectrum.name
		self.top.title(self.title)
		self.update_cb()

	# -----------------------------------------------------------------------------
	#
	def popup_menu(self,event):
		if self.peaks_tablelist.lock_callbacks==0:
			menu=Tkinter.Menu(self.peaks_tablelist.tablelist,tearoff=0)

			menu.add_command(label='Show envelope',command=self.isodistrib_cb)
			menu.add_command(label='Show chromatogram',command=self.chromatogram_cb)
			menu.add_separator()

			view_submenu=Tkinter.Menu(menu,tearoff=0)
			view_submenu.add_radiobutton(label='All',variable=self.settings.view_all,value='A',command=self.update_cb)
			view_submenu.add_separator()
			view_submenu.add_radiobutton(label='Present',variable=self.settings.view_all,value='P',command=self.update_cb)
			view_submenu.add_radiobutton(label='Not present',variable=self.settings.view_all,value='NP',command=self.update_cb)
			if self.settings.only_monoisotopic.get()==1:
				menu.add_cascade(label='View',menu=view_submenu,state='normal')
			else:
				menu.add_cascade(label='View',menu=view_submenu,state='disabled')

			menu.add_checkbutton(label='Monoisotopic',variable=self.settings.only_monoisotopic,command=self.update_cb)
			if self.settings.only_monoisotopic.get()==1:
				menu.add_checkbutton(label='Sum values',variable=self.settings.sum,command=self.update_cb,state='normal')
			else:
				menu.add_checkbutton(label='Sum values',variable=self.settings.sum,command=self.update_cb,state='disabled')
			menu.add_checkbutton(label='Log scale',variable=self.settings.log_scale,command=self.update_cb)
			menu.add_separator()
			menu.add_command(label='Update',command=self.update_cb)

			menu.tk_popup(event.x_root,event.y_root)

	# ---------------------------------------------------------------------------
	#
	def update_cb(self,event=None):
		self.lock_gui(1)

		self.peaks_list=self.spectrum.peak_list()

		start=time.clock()
		self.stoppable_call(self.update_tablelist)
		print (time.clock()-start)

		self.lock_gui(0)

		self.peaks_tablelist.tablelist.focus_set()

	# ---------------------------------------------------------------------------
	#
	def update_tablelist(self):
		self.progress_report('Updating table')

		counter=0
		peak_counter=0
		present_counter=0

		self.set_value_column()

		selected_peak=None
		row_index=int(round(self.peaks_tablelist.tablelist.yview()[0]*self.peaks_tablelist.tablelist.size()))
		self.peaks_tablelist.clear()
		
		self.stoppable_loop('peaks',50)
		for i in range(len(self.peaks_list)):
			self.check_for_stop()
			peak=self.peaks_list[i]
			counter=counter+1

			if self.settings.only_monoisotopic.get()==1 and self.settings.sum.get()==1:
				if mssputil.is_peak_unassigned(peak)!=1:
					if counter!=1:
						if self.append_peak(mono_peak,peak_counter,mono_value)==1:
							peak_counter=peak_counter+1

					mono_peak=peak
					mono_value=mssputil.get_peak_height(mono_peak,self.settings.data_height)
				else:
					if mono_value!=0:
						value=mssputil.get_peak_height(peak,self.settings.data_height)
						if value!=0:
							mono_value=mono_value+value
						else:
							mono_value=0

					if counter==len(self.peaks_list):
						if self.append_peak(mono_peak,peak_counter,mono_value)==1:
							peak_counter=peak_counter+1
			else:
				if mssputil.is_peak_unassigned(peak)==1 and self.settings.only_monoisotopic.get()==1:
					continue

				if self.append_peak(peak,peak_counter,mssputil.get_peak_height(peak,self.settings.data_height))==1:
					peak_counter=peak_counter+1
			
			if peak.selected==1:
				selected_peak=peak

		self.peaks_tablelist.set_list_size(peaks=peak_counter)

		if selected_peak==None:
			self.peaks_tablelist.tablelist.yview(row_index)
			self.peaks_tablelist.set_sequence('')
		else:
			self.peaks_tablelist.selected_peak=None
			self.peaks_tablelist.lock_callbacks=0
			self.peaks_tablelist.mark_peak_cb(selected_peak)

	# ---------------------------------------------------------------------------
	#
	def append_peak(self,peak,row,value):

		present=mssputil.is_peak_present(peak)
		monoisotopic=self.settings.only_monoisotopic.get()

		if monoisotopic==1 and ( (present==1 and self.settings.view_all.get()=='NP') or (present==0 and self.settings.view_all.get()=='P')):
			return 0			

		sequence=mssputil.get_sequence_with_charge(peak,1)
		protein=mssputil.get_protein(peak)
		mz_str=('%0.4f' % mssputil.get_mz(peak))
		rt_str=('%0.2f' % mssputil.get_rt(peak))

		value_str='NaN'
		if value!=0:
			if self.settings.log_scale.get()==1:
				value_str=('%0.3f' % math.log(value,10))
			else:
				value_str=('%0.1f' % value)

		self.peaks_tablelist.append((sequence,protein,mz_str,rt_str,value_str),peak)

		if present==0:
			self.peaks_tablelist.configure_cell(row,0,foreground='red')
			if self.settings.nan_checked_value==1:
				self.peaks_tablelist.configure_cell(row,4,text='NaN')
				self.peaks_tablelist.configure_cell(row,4,foreground='red')
		else:
			if value==0:
				self.peaks_tablelist.configure_cell(row,4,foreground='red')

		if self.settings.use_colors==1 and row!=0 and monoisotopic==1:
			prev_sequence=self.peaks_tablelist.get_cell_config(row-1,0,'-text')
			index=sequence.find('(+')
			if index!=-1 and sequence[0:index]==prev_sequence[0:index]:
				self.peaks_tablelist.tablelist.rowconfigure(row-1,background='#d4deec')
				self.peaks_tablelist.tablelist.rowconfigure(row,background='#d4deec')
		return 1

	# --------------------------------------------------------------------------
	#
	def set_value_column(self):
		name='Fit height'
			
		if self.settings.data_height==1:
			name='Data height'
		elif self.settings.data_height==2:
			name='Volume'

		self.peaks_tablelist.tablelist.columnconfigure(4,title=name)

	# ---------------------------------------------------------------------------
	#
	def update_query(self,row):
		print row

	# ---------------------------------------------------------------------------
	#
	def find_string(self,event=None):
		str=self.find_edit.variable.get()
		if str!='':
			self.lock_gui(1)
			if self.direction_menu.variable.get()=='Search down':
				self.peaks_tablelist.find_string(str,exact_match=self.match_check.state(),y_range=(0,1))
			else:
				self.peaks_tablelist.find_string_up(str,exact_match=self.match_check.state(),y_range=(0,1))
			self.lock_gui(0)

	# ---------------------------------------------------------------------------
	#
	def isodistrib_cb(self,event=None):
		peak=self.peaks_tablelist.event_line_data(event)
		if peak!=None:
			msisoenvelope.show_peak_isotopic_envelope(peak)

	# ---------------------------------------------------------------------------
	#
	def chromatogram_cb(self,event=None):
		peak=self.peaks_tablelist.event_line_data(event)
		if peak!=None:
			mschromatogram_gui.show_peak_chromatogram(peak)

	# ---------------------------------------------------------------------------
	#
	def view_list_cb(self,event=None):
		if self.settings.view_list_only.get()==1:
			if self.settings.view_list==None or len(self.settings.view_list)==0:
				dialog=mstkutil.warning_dialog(self.top,'The list is empty')
				self.settings.view_list_only.set(0)
				return
		self.update_cb()

	# ---------------------------------------------------------------------------
	#
	def lock_gui(self,lock):

		if lock==1:
			state='disabled'
			edit='no'
			if sys.platform=='win32':
				cursor='wait'
			else:
				cursor='watch'
		else:
			state='normal'
			edit='yes'
			cursor=''

		self.settings.lock_callbacks=lock
		self.peaks_tablelist.lock_callbacks=lock

		self.find_button['state']=state
		self.find_edit.entry['state']=state
		self.buttons_row.buttons[0]['state']=state
		self.buttons_row.buttons[2]['state']=state
		self.buttons_row.buttons[3]['state']=state
		self.buttons_row.buttons[4]['state']=state
		self.top.config(cursor=cursor)
		self.top.update_idletasks()

# -----------------------------------------------------------------------------
#
class peaks_list_settings:

	def __init__(self,session,path=None):

		self.session=session
		self.lock_callbacks=0
		self.loaded=0

		if path!=None:
			loaded=self.read_settings(path)

		try:
			num=int(self.session.project.saved_value('LT_NAN_CHECKED_VALUE'))
			if num<0 or num>1:
				raise ValueError
			self.nan_checked_value=num
		except (ValueError,TypeError):
			self.nan_checked_value=1

		try:
			num=int(self.session.project.saved_value('LT_DATA_HEIGHT'))
			if num<0 or num>2:
				raise ValueError
			self.data_height=num
		except (ValueError,TypeError):
			self.data_height=0

		self.log_scale=Tkinter.IntVar()
		try:
			num=int(self.session.project.saved_value('LT_LOG_SCALE'))
			if num<0 or num>1:
				raise ValueError
			self.log_scale.set(num)
		except (ValueError,TypeError):
			self.log_scale.set(0)

		self.only_monoisotopic=Tkinter.IntVar()
		try:
			num=int(self.session.project.saved_value('LT_ONLY_MONOISOTOPIC'))
			if num<0 or num>1:
				raise ValueError
			self.only_monoisotopic.set(num)
		except (ValueError,TypeError):
			self.only_monoisotopic.set(1)

		self.sum=Tkinter.IntVar()
		try:
			num=int(self.session.project.saved_value('LT_SUM'))
			if num<0 or num>1:
				raise ValueError
			self.sum.set(num)
		except (ValueError,TypeError):
			self.sum.set(1)

		self.view_all=Tkinter.StringVar()
		self.view_all.set(self.session.project.saved_value('LT_VIEW_ALL'))
		if self.view_all.get()!='A' and self.view_all.get()!='P' and self.view_all.get()!='NP':
			self.view_all.set('A')

		try:
			num=int(self.session.project.saved_value('LT_USE_COLORS'))
			if num<0 or num>1:
				raise ValueError
			self.use_colors=num
		except (ValueError,TypeError):
			self.use_colors=1

	# --------------------------------------------------------------------------
	#
	def read_settings(self,path):
		result=0

		if path!=None:
			try:
				file=open(path,'r')
				lines=file.readlines()

				for i in range(0,len(lines)):
					line=lines[i].strip()
					if len(line)!=0 and line[0]!='#':
						fields=lines[i].split('=')
						if len(fields)==2:
							self.session.project.save_value(fields[0].strip(),fields[1].strip())
				file.close()
				result=1
			except IOError, (errno, strerror):
				print "I/O error(%s): %s" % (errno, strerror)
				result=0
		
		return result

# -----------------------------------------------------------------------------
#
class peaks_list_settings_dialog(mstkutil.Settings_Dialog):
	
	def __init__(self, session):
		self.session=session
		self.loaded=0
		mstkutil.Settings_Dialog.__init__(self,session.tk,'Settings')

		f=Tkinter.Frame(self.top,borderwidth=2,relief='groove')
		f.pack(side='top',anchor='w',pady=2,fill='x',expand=1)

		l=Tkinter.Label(f,text='Value: ')
		l.grid(row=0,column=0,sticky='w')

		m=mssputil.value_menu(f,'')
		m.frame.grid(row=0,column=1,sticky='w',pady=2,padx=2)
		self.value_menu=m

		cb=mstkutil.checkbutton(f,'Value only from monoisotopic peak',0)
		cb.add_callback(self.monoisotopic_cb)
		cb.button.grid(row=1,columnspan=2,sticky='w',pady=1)
		self.only_monoisotopic_check=cb

		cb=mstkutil.checkbutton(f,'Sum peaks',0)
		cb.button.grid(row=2,columnspan=2,sticky='w',pady=1)
		self.sum_check=cb

		cb=mstkutil.checkbutton(f,'Set value to NaN for unchecked',0)
		cb.button.grid(row=3,columnspan=2,sticky='w',pady=1)
		self.nan_value_check=cb

		l=Tkinter.Label(f,text='View:')
		l.grid(row=4,column=0,sticky='w',pady=1)

		m=mstkutil.option_menu(f,'',('All','Present','Not present'),'All')
		m.frame.grid(row=4,column=1,sticky='w',pady=2,padx=2)
		self.view_menu=m

		cb=mstkutil.checkbutton(f,'Use colors',0)
		cb.button.grid(row=5,columnspan=2,sticky='w',pady=1)
		self.colors_check=cb

		br=mstkutil.button_row(self.top,
				('Apply', self.apply_settings_cb),
				('Load', self.load_cb),
				('Close', self.close_cb),
				)
		br.frame.pack(side='bottom',anchor='w',pady=2)

		self.top.geometry("+0+0")
		self.top.resizable(0,0)

	# ---------------------------------------------------------------------------
	#
	def monoisotopic_cb(self,event):
		if self.only_monoisotopic_check.variable.get()==0:
			state='disabled'
		else:
			state='normal'

		self.sum_check.button.configure(state=state)
		self.view_menu.menubutton.configure(state=state)
		self.colors_check.button.configure(state=state)
	# ---------------------------------------------------------------------------
	#
	def load_cb(self):
		path=mstkutil.load_file(self.top,'Open configuration','open_cfg')
		if path!=None:
			settings=peaks_list_settings(self.session,path)
			self.loaded=settings.loaded
			self.show_settings(settings)

	# ---------------------------------------------------------------------------
	#
	def show_settings(self,settings):

		self.value_menu.set_value(settings.data_height)
		self.only_monoisotopic_check.variable.set(settings.only_monoisotopic.get())
		self.sum_check.variable.set(settings.sum.get())
		self.nan_value_check.variable.set(settings.nan_checked_value)

		if settings.view_all.get()=='A':
			entry='All'
		elif settings.view_all.get()=='P':
			entry='Present'
		elif settings.view_all.get()=='NP':
			entry='Not present'
		self.view_menu.set(entry)

		self.colors_check.variable.set(settings.use_colors)

		if settings.only_monoisotopic.get()==0:
			self.sum_check.button.configure(state='disabled')
			self.view_menu.menubutton.configure(state='disabled')
			self.colors_check.button.configure(state='disabled')

	# ---------------------------------------------------------------------------
	#
	def get_settings(self):
		settings=peaks_list_settings(self.session)

		settings.data_height=self.value_menu.get_value()
		settings.only_monoisotopic.set(self.only_monoisotopic_check.variable.get())
		settings.sum.set(self.sum_check.variable.get())
		settings.nan_checked_value=self.nan_value_check.variable.get()

		entry=self.view_menu.get()
		if entry=='All':
			settings.view_all.set('A')
		elif entry=='Present':
			settings.view_all.set('P')
		elif entry=='Not present':
			settings.view_all.set('NP')
	
		settings.use_colors=self.colors_check.variable.get()

		if self.loaded==0:
			settings.log_scale.set(self.parent_dialog.settings.log_scale.get())

		return settings

	# ---------------------------------------------------------------------------
	#
	def apply_settings_cb(self):
		self.show_window(0)
		self.apply_cb()
		self.close_cb()

	# ---------------------------------------------------------------------------
	#
	def clear_list_cb(self,event=None):
		self.parent_dialog.settings.view_list=None
		self.view_list_check.variable.set(0)
		self.view_list_check.button.configure(state='disabled')

# -----------------------------------------------------------------------------
#
def show_spectrum_peaks(session):
	spectrum=session.selected_spectrum()
	if spectrum==None:
		return

	if not hasattr(session,'spectrum_dialogs'):
		session.spectrum_dialogs={}
	dialogs=session.spectrum_dialogs

	key='LT'+spectrum.name
	if (dialogs.has_key(key) and not dialogs[key].is_window_destroyed()):
		dialogs[key].show_window(1)
	else:
		d=peaks_list_dialog(session)
		d.show_window(1)
		d.show_spectrum_peaks(spectrum)
		dialogs[key]=d
