import string
import math
import sys
import os
import time

import Tkinter
import tkFont

import fpconst
import odict

import sparky
import mssputil
import mstkutil
import pyutil

import numpy
import scipy
import scipy.stats
import matplotlib
import matplotlib.pyplot as plt
from matplotlib.figure import Figure

import ms
import msenvelope
import msisoenvelope
import mschromatogram_gui
import msduplicated


flag_icon=Tkinter.PhotoImage(data='R0lGODlhEAAQALMAAAQCBLRqbLyaZKRCRKyCXPSqrKxiZMx2dNS6fOyOjNzGjLSSbMSmdKROTNx+'
				'fMReXCH5BAEAAAAALAAAAAAQABAAAwQkEMhJq53l6uS05YdHcc8gSlxzosYqBa5ExMBCCzRDI7RC'
				'/5oIADs=')

checkon_icon=Tkinter.PhotoImage(data='R0lGODlhCwALALMAAAQCBPz+/Hx+fAAAAAQGAAAFAAARAAAAAPgj3zJx2hYA1AAAdwDwADPrABYS'
				'AAAAACH5BAAAAAAALAAAAAALAAsAAwQfUMhJZbgY23xB2BngWV4nfkJwiiXJlqgJxx0HcmhVRQA7')

checkoff_icon=Tkinter.PhotoImage(data='R0lGODlhCwALALMAAHx+fPz+/BYAAAAAAAQGAAAFAAALAAAAALiT3zul2hYA1AAAd8DwADvrABYS'
				'AAAAACH5BAAAAAAALAAAAAALAAsAAwQXEMhJZbgY25w3v94XcmMHfNqJBmVaVREAOw==')

# -----------------------------------------------------------------------------
#
class peptides_list_dialog(mstkutil.Dialog,mstkutil.Stoppable):

	def __init__(self,session):

		self.session=session
		self.title='MS peptides list:'
		self.settings=peptides_list_settings(self.session,os.path.join(os.environ.get('SPARKY_INSTALL'),os.path.join('config','peptideslist.cfg')))
		self.find_duplicates_dialog=None

		mstkutil.Dialog.__init__(self,session.tk,self.title)

		f=Tkinter.Frame(self.top,borderwidth=2,relief='groove')
		f.pack(side='top',anchor='w',fill='x')

		pl=mssputil.peptide_tablelist(f,columns=None,selected_edit=1,editendcommand=self.editend_cb,editstartcommand=self.editstart_cb)
		pl.set_columns((2,'',2,'P',40,'Sequence',10,'Protein',9,'m/z',9,'D_m/z',6,'RT',6,'D_RT',6,'d_RT',11,'Fit Height',8,'FRMSE',5,'FDIFF',5,'Peaks',4,'Max',7,'ERMSE'))
		pl.set_columns_align(('left','center','left','left','right','right','right','right','right','right','right','right','right','right','right'))
		pl.tablelist.columnconfigure(1,editable='yes',editwindow='checkbutton')

		pl.tablelist.bind('<ButtonRelease-1>',pl.select_peptide_cb)
		pl.tablelist.bind('<Double-ButtonRelease-1>',pl.goto_peptide_cb)
		pl.tablelist.bind('<Return>',pl.goto_peptide_cb)
		pl.tablelist.bind('<Button-3>',self.popup_menu)
		
		pl.frame.pack(side='top',fill='both',expand=1)		
		self.peptide_tablelist=pl

		f=Tkinter.Frame(self.top,borderwidth=2,relief='groove')
		f.pack(side='top',anchor='w',fill='x')

		l=Tkinter.Label(f,text='Search for:',anchor='w',width=10)
		l.pack(side='left',padx=3)

		e=mstkutil.entry_field(f,'','',45)
		e.frame.pack(side='left',padx=2)
		e.entry.bind('<Return>',self.find_string)
		self.find_edit=e

		m=mstkutil.option_menu(f,'',('Search down','Search up'),'Search down')
		m.frame.pack(side='left',padx=3)
		self.direction_menu=m

		cb=mstkutil.checkbutton(f,'Only exact match',0)
		cb.button.pack(side='left',padx=3)
		self.match_check=cb

		b=Tkinter.Button(f,text="Find",borderwidth=1,width='7',command=self.find_string)
		b.pack(side='right',ipady=2,padx=3,pady=3)
		self.find_button=b

		progress_label=Tkinter.Label(self.top, anchor = 'nw')
		progress_label.pack(side='top',anchor='w')

		br=mstkutil.button_row(self.top,
					('Update', self.update_cb),
					('Stop', self.stop_cb),
					('Setup', self.setup_cb),
					('Save', self.save_cb),
					('Close', self.closing_window),
#					('Close', self.top.destroy),
					)
		br.frame.pack(side='top',anchor='w',pady=2)
		self.buttons_row=br

		self.top.geometry("+0+0")
		self.top.resizable(1,0)

		self.notice=session.notify_me('removed spectrum from project',self.spectrum_removed_cb)
		self.selection_notice=session.notify_me('selection changed',self.selection_changed_cb)
		self.dragging_notice=session.notify_me('dragged peak',self.peak_dragged_cb)
		self.top.bind('<Destroy>',self.remove_callbacks,1)
		self.top.protocol("WM_DELETE_WINDOW",self.closing_window)

		mstkutil.Stoppable.__init__(self,progress_label,br.buttons[1])


	# ---------------------------------------------------------------------------
	#
	def spectrum_removed_cb(self,removed_spectrum):
		if self.spectrum==removed_spectrum:
			self.top.destroy()

	# ---------------------------------------------------------------------------
	#
	def selection_changed_cb(self):
		peaks=self.session.selected_peaks()
		if peaks!=None and len(peaks)==1:
			self.peptide_tablelist.mark_peak_cb(peaks[0])

	# ---------------------------------------------------------------------------
	#
	def peak_dragged_cb(self,peak):
		if peak.spectrum==self.spectrum:
			line_index=self.peptide_tablelist.find_peak_cb(peak)
			if line_index!=-1 and (self.settings.view_list_only.get()==0 or (self.settings.view_list_only.get()==1 and self.settings.view_list[int(line_index)]==1)):
				self.update_query(line_index)
				if peak.selected==1:
					self.peptide_tablelist.selected_peak=None
					self.peptide_tablelist.mark_peak_cb(peak)

	# ---------------------------------------------------------------------------
	#
	def closing_window(self):
		if self.settings.lock_callbacks==0:
			if self.find_duplicates_dialog!=None:
				self.find_duplicates_dialog.parent=None	
			self.top.destroy()
#		else:
#			self.stop_cb()
				
	# ---------------------------------------------------------------------------
	#
	def remove_callbacks(self,event):
		if str(self.top)==str(event.widget):
			if sparky.object_exists(self.session):
				if self.notice!=None:
					self.session.dont_notify_me(self.notice)
				if self.selection_notice!=None:
					self.session.dont_notify_me(self.selection_notice)
				if self.dragging_notice!=None:
					self.session.dont_notify_me(self.dragging_notice)
					
			self.notice=None
			self.selection_notice=None
			self.dragging_notice=None

	# -----------------------------------------------------------------------------
	#
	def get_peptides(self):
		self.peptide_list=mssputil.get_peptides(self.spectrum,self,None,'peaks',100)
		
	# ---------------------------------------------------------------------------
	#
	def save_cb(self):
		path=mstkutil.save_file(self.peptide_tablelist.tablelist,'Save List','save_list')
		self.top.update_idletasks()
		self.lock_gui(1)
		self.peptide_tablelist.write_file(path,'w')
		self.lock_gui(0)

	# ---------------------------------------------------------------------------
	#
	def setup_cb(self):
		psd=mssputil.the_dialog(peptides_list_settings_dialog,self.session)
		psd.set_parent_dialog(self,self.settings,self.new_settings)
		psd.top.title(self.title + ' settings')
		psd.show_window(1)

	# ---------------------------------------------------------------------------
	#
	def new_settings(self,settings):
		self.settings=settings
		self.redraw_cb()

	# ---------------------------------------------------------------------------
	#
	def show_spectrum_peptides(self,spectrum):
		self.spectrum=spectrum
		self.title='MS peptides list: ' + self.spectrum.name
		self.top.title(self.title)

		dialogs=self.session.spectrum_dialogs
		key='FD'+self.spectrum.name
		if (dialogs.has_key(key) and not dialogs[key].is_window_destroyed()):
			self.find_duplicates_dialog=dialogs[key]
			self.find_duplicates_dialog.parent=self

		self.update_cb()

	# ---------------------------------------------------------------------------
	#
	def editstart_cb(self,tbl,row,column,text):
#		print 'start'
		value=0
		self.peptide_tablelist.lock_callbacks=1

		if column=='1':
			if self.peptide_tablelist.get_cell_config(int(row),1,'-text')=='1':
				value=1
			else:
				value=0
		return value

	# ---------------------------------------------------------------------------
	#
	def editend_cb(self,tbl,row,column,text):
#		print 'end'
		value=int(text)
		if column=='1':
			self.check_query(int(row),value)
			if text=='0':
				self.peptide_tablelist.set_list_size(present=self.peptide_tablelist.present_count)
			else:
				self.peptide_tablelist.set_list_size(present=self.peptide_tablelist.present_count)

		self.peptide_tablelist.lock_callbacks=0
		return value

	# -----------------------------------------------------------------------------
	#
	def popup_menu(self,event):
		if self.peptide_tablelist.lock_callbacks==0:
			menu=Tkinter.Menu(self.peptide_tablelist.tablelist,tearoff=0)

			menu.add_command(label='Show envelope',command=self.isodistrib_cb)
			menu.add_command(label='Show chromatogram',command=self.chromatogram_cb)
			menu.add_separator()
			menu.add_command(label='Check all',command=self.check_all)
			menu.add_command(label='Uncheck all ',command=self.check_none)
			menu.add_command(label='Uncheck marked',command=self.check_marked)
			menu.add_command(label='Uncheck modified',command=self.check_modified)
			menu.add_separator()

			view_submenu=Tkinter.Menu(menu,tearoff=0)
			view_submenu.add_radiobutton(label='All',variable=self.settings.view_all,value='A',command=self.redraw_cb)
			view_submenu.add_separator()
			view_submenu.add_radiobutton(label='Marked',variable=self.settings.view_all,value='M',command=self.redraw_cb)
			view_submenu.add_radiobutton(label='Not marked',variable=self.settings.view_all,value='NM',command=self.redraw_cb)
			view_submenu.add_separator()
			view_submenu.add_radiobutton(label='Present',variable=self.settings.view_all,value='P',command=self.redraw_cb)
			view_submenu.add_radiobutton(label='Not present',variable=self.settings.view_all,value='NP',command=self.redraw_cb)
			view_submenu.add_separator()
			if self.settings.view_list==None or len(self.settings.view_list)==0:
				view_submenu.add_checkbutton(label='Only from list',variable=self.settings.view_list_only,command=self.view_list_cb,state='disabled')
			else:
				view_submenu.add_checkbutton(label='Only from list',variable=self.settings.view_list_only,command=self.view_list_cb,state='normal')
			menu.add_cascade(label='View',menu=view_submenu)

			menu.add_checkbutton(label='Monoisotopic',variable=self.settings.only_monoisotopic,command=self.redraw_cb)
			menu.add_checkbutton(label='Log scale',variable=self.settings.log_scale,command=self.redraw_cb)
			menu.add_separator()
			menu.add_command(label='Find duplicated',command=self.duplicated_cb)
			masserror_submenu=Tkinter.Menu(menu,tearoff=0)
			masserror_submenu.add_command(label='ME histogram',command=self.mass_error_hist_cb)
			masserror_submenu.add_command(label='ME vs. m/z',command=self.mass_error_mz_cb)
			masserror_submenu.add_command(label='ME vs.RT',command=self.mass_error_rt_cb)
			menu.add_cascade(label='Mass error',menu=masserror_submenu)
			
			rterror_submenu=Tkinter.Menu(menu,tearoff=0)
#			rterror_submenu.add_command(label='RTE Histogram',command=self.rt_error_hist_cb)
#			rterror_submenu.add_command(label='RTE vs. m/z',command=self.rt_error_mz_cb)
			rterror_submenu.add_command(label='RTE vs.RT',command=self.rt_error_rt_cb)
			menu.add_cascade(label='RT error',menu=rterror_submenu)

			signal_submenu=Tkinter.Menu(menu,tearoff=0)
			menu.add_command(label='Peaks histogram',command=self.peaks_hist_cb)
#			menu.add_command(label='Spectrum histogram',command=self.spectrum_histogram_cb)
			menu.add_cascade(label='Values',menu=signal_submenu)

			menu.add_separator()
			menu.add_command(label='Redraw',command=self.redraw_cb)

			menu.tk_popup(event.x_root,event.y_root)

	# ---------------------------------------------------------------------------
	#
	def update_cb(self,event=None):
		self.lock_gui(1)

		start=time.clock()
		self.stoppable_call(self.get_peptides)
		print (time.clock()-start)

		start=time.clock()
		self.stoppable_call(self.update_tablelist)
		print (time.clock()-start)

		self.lock_gui(0)

		self.peptide_tablelist.tablelist.focus_set()

	# ---------------------------------------------------------------------------
	#
	def redraw_cb(self,event=None):
		self.lock_gui(1)
		self.stoppable_call(self.update_tablelist)
		self.lock_gui(0)

		self.peptide_tablelist.tablelist.focus_set()

	# ---------------------------------------------------------------------------
	#
	def update_tablelist(self):
		self.progress_report('Updating table')

		counter=-1
		query_counter=0
		red_flag_counter=0
		present_counter=0

		self.set_value_column()

		selected_peak=None
		row_index=int(round(self.peptide_tablelist.tablelist.yview()[0]*self.peptide_tablelist.tablelist.size()))
		self.peptide_tablelist.clear()
		
		self.stoppable_loop('peptides',50)
		for i in range(len(self.peptide_list)):
			self.check_for_stop()
			peptide=self.peptide_list[i]

			if peptide.size()>1:
				bgcolor='#d4deec'
			else:
				bgcolor='gray98'

			for j in range(len(peptide.query_list)):
				hidden=0
				counter=counter+1
				if self.settings.view_list_only.get()==1 and self.settings.view_list[counter]==0:
					self.peptide_tablelist.append(('','','','','','','','','','','','',''),(peptide,j))
					self.peptide_tablelist.tablelist.rowconfigure(counter,hide=1)
				else:
					query=peptide.get(j)
					if self.settings.cmass==1:
						mz_str=('%0.4f' % query.cmz)
					else:
						mz_str=('%0.4f' % query.mz)
					delta_mz=query.mz-query.cmz
					delta_mz_str=('%0.4f' % delta_mz)
					rt_str=('%0.2f' % query.rt)
					delta_mrt=query.rt-query.massref_rt
					delta_mrt_str=('%0.2f' % delta_mrt)

					peaks_count=0
					value=0
					value_str=('0.0')
					fit_rms=-1
					fit_rms_str='NaN'
					fit_diff=-1
					fit_diff_str='NaN'
					env_rms=1
					env_rms_str='NaN'
					isodistrib_max=0
					delta_rt=-1
					delta_rt_str='NaN'

					isodistrib=mssputil.get_spectrum_isodistrib(query.peak_list,data_height=self.settings.data_height,relative=0)
					peaks_count=mssputil.get_peaks_count(isodistrib)

					if peaks_count!=0 and isodistrib[0]>0:
						value=mssputil.get_value(isodistrib,only_monoisotopic=self.settings.only_monoisotopic.get())
						if value>0:
							if self.settings.log_scale.get()==1:
								value=math.log(value,10)
								value_str=('%0.3f' % value)
							else:
								value_str=('%0.1f' % value)
					
							fit_rms=query.get_fit_rms(self.settings.only_monoisotopic.get())
							if fit_rms>=0 or self.settings.data_height==1:
								if self.settings.data_height!=1:
									fit_rms_str=('%0.1f' % fit_rms)

									if self.settings.data_height!=2:
										fit_diff=mssputil.isodistrib_diff(isodistrib,mssputil.get_spectrum_isodistrib(query.peak_list,data_height=1,relative=0),self.settings.only_monoisotopic.get())
									else:
										fit_diff=mssputil.isodistrib_diff(mssputil.get_spectrum_isodistrib(query.peak_list,data_height=0,relative=0),mssputil.get_spectrum_isodistrib(query.peak_list,data_height=1,relative=0),self.settings.only_monoisotopic.get())
									fit_diff_str='%0.2f' % fit_diff

								if peaks_count>1:
									isodistrib=mssputil.get_relative_isodistrib(isodistrib)
									env_rms=mssputil.isodistrib_rms(peptide.isodistrib,isodistrib)
									env_rms_str=('%0.3f' % env_rms)

									if mssputil.isodistrib_max(isodistrib)==mssputil.isodistrib_max(peptide.isodistrib[0:peaks_count]):
										isodistrib_max=1
									else:
										isodistrib_max=0
								
									delta_rt=abs(mssputil.get_rt(query.get(0))-mssputil.get_rt(query.get(1)))
									delta_rt_str=('%0.2f' % delta_rt)

					self.peptide_tablelist.append(('','',peptide.get_sequence(1,j),peptide.proteins,mz_str,delta_mz_str,rt_str,delta_mrt_str,delta_rt_str,value_str,fit_rms_str,fit_diff_str,peaks_count,isodistrib_max,env_rms_str),(peptide,j))
					self.peptide_tablelist.tablelist.rowconfigure(counter,background=bgcolor)

					red_flag=0

					if self.settings.delta_mz_check and abs(delta_mz/query.cmz)>self.settings.delta_mz_th:
						self.peptide_tablelist.configure_cell(counter,5,foreground='red')
						red_flag=1
					if self.settings.delta_mrt_check and abs(delta_mrt)>self.settings.delta_mrt_th:
						self.peptide_tablelist.configure_cell(counter,7,foreground='red')
						red_flag=1
					if self.settings.delta_rt_check==1 and abs(delta_rt)>self.settings.delta_rt_th:
						self.peptide_tablelist.configure_cell(counter,8,foreground='red')
						red_flag=1
					if value==0:
						self.peptide_tablelist.configure_cell(counter,9,foreground='red')
						red_flag=1
					if self.settings.data_height!=1:
						if self.settings.fit_rms_check==1 and (fit_rms<0 or fit_rms>self.settings.fit_rms_th):
							self.peptide_tablelist.configure_cell(counter,10,foreground='red')
							red_flag=1
						if self.settings.fit_diff_check==1 and (fit_diff==-1 or fit_diff>self.settings.fit_diff_th):
							self.peptide_tablelist.configure_cell(counter,11,foreground='red')
							red_flag=1
					if self.settings.peaks_check==1 and peaks_count<self.settings.peaks_th:
						self.peptide_tablelist.configure_cell(counter,12,foreground='red')
						red_flag=1
					if self.settings.max_check==1 and isodistrib_max==0 and env_rms>self.settings.max_th:
						self.peptide_tablelist.configure_cell(counter,13,foreground='red')
						red_flag=1
					if self.settings.env_rms_check==1 and env_rms>self.settings.env_rms_th:
						self.peptide_tablelist.configure_cell(counter,14,foreground='red')
						red_flag=1

					if red_flag==1:
						self.peptide_tablelist.configure_cell(counter,0,image=flag_icon)
						if self.settings.view_all.get()=='NM':
							self.peptide_tablelist.tablelist.rowconfigure(counter,hide=1)
							hidden=1
					else:
						if self.settings.view_all.get()=='M':
							self.peptide_tablelist.tablelist.rowconfigure(counter,hide=1)
							hidden=1

					if query.present==0:
						self.peptide_tablelist.configure_cell(counter,1,image=checkoff_icon,text='0')
						self.peptide_tablelist.configure_cell(counter,2,foreground='red')
						if self.settings.nan_checked_value==1:
							self.peptide_tablelist.configure_cell(counter,9,text='NaN')
							self.peptide_tablelist.configure_cell(counter,9,foreground='red')

						if self.settings.view_all.get()=='P':
							self.peptide_tablelist.tablelist.rowconfigure(counter,hide=1)
							hidden=1
					else:
						self.peptide_tablelist.configure_cell(counter,1,image=checkon_icon,text='1')
						if self.settings.view_all.get()=='NP':
							self.peptide_tablelist.tablelist.rowconfigure(counter,hide=1)
							hidden=1

					selected=query.get_selected()
					if selected!=None:
						selected_peak=selected
					
					if hidden==0:
						query_counter=query_counter+1
						if red_flag==1:
							red_flag_counter=red_flag_counter+1
						if query.present==1:
							present_counter=present_counter+1
		
		self.peptide_tablelist.set_list_size(queries=query_counter,red_flags=red_flag_counter,present=present_counter)
#		self.peptide_tablelist.tablelist.configure(height=min(int(self.peptide_tablelist.tablelist.cget('-height')),query_counter))

		if selected_peak==None:
			self.peptide_tablelist.tablelist.yview(row_index)
			self.peptide_tablelist.set_sequence('')
		else:
			self.peptide_tablelist.selected_peak=None
			self.peptide_tablelist.lock_callbacks=0
			self.peptide_tablelist.mark_peak_cb(selected_peak)

	# ---------------------------------------------------------------------------
	#
	def find_string(self,event=None):
		str=self.find_edit.variable.get()
		if str!='':
			self.lock_gui(1)
			if self.direction_menu.variable.get()=='Search down':
				self.peptide_tablelist.find_string(str,exact_match=self.match_check.state(),y_range=(2,3))
			else:
				self.peptide_tablelist.find_string_up(str,exact_match=self.match_check.state(),y_range=(2,3))
			self.lock_gui(0)

	# ---------------------------------------------------------------------------
	#
	def isodistrib_cb(self,event=None):
		data=self.peptide_tablelist.event_line_data(event)
		if data!=None:
			peptide=data[0]
			query_nr=data[1]
			query=peptide.get(query_nr)
			peak=query.get(0)
			spectrum=peak.spectrum

			# paramtry
			threshold=0.01
			resolution=50000*(400/query.cmz)
			mz_correction=0
			peaks_count=3
			mz_step=0.001

			charge=query.charge
			rt=peak.frequency[0]
			title=peptide.get_sequence(1,query_nr)

			# pobranie rozkladu izotopowego
			isodistrib=msenvelope.MSIsoDistrib(peak,threshold)
			
			# rysowanie
			msenvelope.plot_envelope(self.session,self.spectrum,isodistrib,charge,peaks_count,mz_correction,resolution,mz_step,rt,title)

	# ---------------------------------------------------------------------------
	#
	def isodistrib2_cb(self,event=None):
		data=self.peptide_tablelist.event_line_data(event)
		if data!=None:
			peptide=data[0]
			query_nr=data[1]
			msisoenvelope.show_peak_isotopic_envelope(peptide.get(query_nr).get(0))

	# ---------------------------------------------------------------------------
	#
	def chromatogram_cb(self,event=None):
		data=self.peptide_tablelist.event_line_data(event)
		if data!=None:
			peptide=data[0]
			query_nr=data[1]
			mschromatogram_gui.show_peak_chromatogram(peptide.get(query_nr).get(0))

	# ---------------------------------------------------------------------------
	#
	def duplicated_cb(self,event=None):
		dialog=msduplicated.show_duplicated_list_dialog(self)
		if dialog!=None:
			self.find_duplicates_dialog=dialog



	# ---------------------------------------------------------------------------
	#
	def peaks_hist_cb(self,event=None):

		if self.peptide_tablelist.present_count>0:
			values=[]
			for i in range(len(self.peptide_list)):
				peptide=self.peptide_list[i]

				for j in range(len(peptide.query_list)):
					query=peptide.get(j)
					if query.present==1:
						isodistrib=mssputil.get_spectrum_isodistrib(query.peak_list,data_height=self.settings.data_height,relative=0)
						peaks_count=mssputil.get_peaks_count(isodistrib)
						if peaks_count!=0 and isodistrib[0]>0:
							value=mssputil.get_value(isodistrib,only_monoisotopic=self.settings.only_monoisotopic.get())
							if value>0:
#								if self.settings.log_scale.get()==1:
#									value=math.log(value,10)
								values.append(value);

			if len(values)>0:
				fig=Figure()
				p=fig.add_subplot(111)
				p.hist(numpy.array(values),50)
				p.set_xlabel("Peak values")
				p.set_ylabel("Counts")

				d=mstkutil.PlotDialog(self.session,fig,('Peak values histogram: ' + self.spectrum.name))
				d.show_window(1)


	# ---------------------------------------------------------------------------
	#
	def mass_error_hist_cb(self,event=None):

		if self.peptide_tablelist.present_count>0:
			delta_mz=[]
			for i in range(len(self.peptide_list)):
				peptide=self.peptide_list[i]

				for j in range(len(peptide.query_list)):
					query=peptide.get(j)
					if query.present==1:
						dmz=query.mz-query.cmz
						if query.cmz!=0 and dmz<1:
							delta_mz.append(1000000*(dmz/query.cmz))

			if len(delta_mz)>0:
				fig=Figure()
				p=fig.add_subplot(111)
				p.hist(numpy.array(delta_mz),100)
				p.set_xlabel("m/z")
				p.set_ylabel("Counts")

				d=mstkutil.PlotDialog(self.session,fig,('Mass error histogram: ' + self.spectrum.name))
				d.show_window(1)

	# ---------------------------------------------------------------------------
	#
	def mass_error_mz_cb(self,event=None):

		if self.peptide_tablelist.present_count>0:
			delta_mz=[]
			mz=[]
			for i in range(len(self.peptide_list)):
				peptide=self.peptide_list[i]
				for j in range(len(peptide.query_list)):
					query=peptide.get(j)
					if query.present==1:
						dmz=query.mz-query.cmz
						if query.cmz!=0 and dmz<1:
							delta_mz.append(1000000*(dmz/query.cmz))
							mz.append(query.cmz)

			if len(delta_mz)>0:
				delta_mz=numpy.array(delta_mz)
				mz=numpy.array(mz)

				fig=Figure()
				p=fig.add_subplot(111)
				p.plot(mz,delta_mz,'ro')
				p.set_xlabel("m/z")
				p.set_ylabel("Mass error [ppm]")

				(a_s,b_s,r,tt,stderr)=scipy.stats.linregress(mz,delta_mz)
				x=numpy.sort(mz)
				p.plot(x,a_s*x+b_s,color='k',linewidth=2)
	
				a=p.axis()
				p.text(a[1],a[2],('a=%f b=%f ' % (a_s,b_s)),horizontalalignment='right',verticalalignment='bottom')

				d=mstkutil.PlotDialog(self.session,fig,('Mass error vs. m/z: ' + self.spectrum.name))
				d.show_window(1)

	# ---------------------------------------------------------------------------
	#
	def mass_error_rt_cb(self,event=None):

		if self.peptide_tablelist.present_count>0:
			delta_mz=[]
			rt=[]
			for i in range(len(self.peptide_list)):
				peptide=self.peptide_list[i]
				for j in range(len(peptide.query_list)):
					query=peptide.get(j)
					if query.present==1:
						dmz=query.mz-query.cmz
						if query.cmz!=0 and dmz<1:
							delta_mz.append(1000000*(dmz/query.cmz))
							rt.append(query.rt)

			if len(delta_mz)>0:
				delta_mz=numpy.array(delta_mz)
				rt=numpy.array(rt)
	
				fig=Figure()
				p=fig.add_subplot(111)
				p.plot(rt,delta_mz,'ro')
				p.set_xlabel("RT [min]")
				p.set_ylabel("Mass error [ppm]")

				(a_s,b_s,r,tt,stderr)=scipy.stats.linregress(rt,delta_mz)
				x=numpy.sort(rt)
				p.plot(x,a_s*x+b_s,color='k',linewidth=2)
				a=p.axis()
				p.text(a[1],a[2],('a=%f b=%f ' % (a_s,b_s)),horizontalalignment='right',verticalalignment='bottom')

				d=mstkutil.PlotDialog(self.session,fig,('Mass error vs. RT: ' + self.spectrum.name))
				d.show_window(1)

	# ---------------------------------------------------------------------------
	#
	def rt_error_rt_cb(self,event=None):

		if self.peptide_tablelist.present_count>0:
			delta_rt=[]
			rt=[]
			for i in range(len(self.peptide_list)):
				peptide=self.peptide_list[i]
				for j in range(len(peptide.query_list)):
					query=peptide.get(j)
					if query.present==1:
						drt=query.rt-query.massref_rt
						if query.massref_rt!=0:
							delta_rt.append(100*(drt/query.massref_rt))
							rt.append(query.rt)

			if len(delta_rt)>0:
				delta_rt=numpy.array(delta_rt)
				rt=numpy.array(rt)

				fig=Figure()
				p=fig.add_subplot(111)
				p.plot(rt,delta_rt,'ro')
				p.set_xlabel("RT [min]")
				p.set_ylabel("RT error [%]")
	
				(a_s,b_s,r,tt,stderr)=scipy.stats.linregress(rt,delta_rt)
				x=numpy.sort(rt)
				p.plot(x,a_s*x+b_s,color='k',linewidth=2)
				a=p.axis()
				p.text(a[1],a[2],('a=%f b=%f ' % (a_s,b_s)),horizontalalignment='right',verticalalignment='bottom')

				d=mstkutil.PlotDialog(self.session,fig,('RT error vs. RT: ' + self.spectrum.name))
				d.show_window(1)

	# --------------------------------------------------------------------------
	#
	def set_value_column(self):
		name='Fit height'
			
		if self.settings.data_height==1:
			name='Data height'
		elif self.settings.data_height==2:
			name='Volume'

		self.peptide_tablelist.tablelist.columnconfigure(9,title=name)

	# ---------------------------------------------------------------------------
	#
	def view_list_cb(self,event=None):
		if self.settings.view_list_only.get()==1:
			if self.settings.view_list==None or len(self.settings.view_list)==0:
				dialog=mstkutil.warning_dialog(self.top,'The list is empty')
				self.settings.view_list_only.set(0)
				return
		self.redraw_cb()


	# ---------------------------------------------------------------------------
	#
	def update_query(self,row):

		#data=self.peptide_tablelist.line_data[row]
		data=self.peptide_tablelist.get_line_data(row)
		peptide=data[0]
		query=peptide.get(data[1])
		query.mz=mssputil.get_mz(query.peak_list[0])
		query.rt=mssputil.get_rt(query.peak_list[0])
				
		if self.settings.cmass==1:
			mz_str=('%0.4f' % query.cmz)
		else:
			mz_str=('%0.4f' % query.mz)
		delta_mz=query.mz-query.cmz
		delta_mz_str=('%0.4f' % delta_mz)
		rt_str=('%0.2f' % query.rt)
		delta_mrt=query.rt-query.massref_rt
		delta_mrt_str=('%0.2f' % delta_mrt)

		peaks_count=0
		value=0
		value_str=('0.0')
		fit_rms=-1
		fit_rms_str='NaN'
		fit_diff=-1
		fit_diff_str='NaN'
		env_rms=1
		env_rms_str='NaN'
		isodistrib_max=0
		delta_rt=-1
		delta_rt_str='NaN'

		isodistrib=mssputil.get_spectrum_isodistrib(query.peak_list,data_height=self.settings.data_height,relative=0)
		peaks_count=mssputil.get_peaks_count(isodistrib)

		if peaks_count!=0 and isodistrib[0]>0:
			value=mssputil.get_value(isodistrib,only_monoisotopic=self.settings.only_monoisotopic.get())
			if value>0:
				if self.settings.log_scale.get()==1:
					value=math.log(value,10)
					value_str=('%0.3f' % value)
				else:
					value_str=('%0.1f' % value)
		
				fit_rms=query.get_fit_rms(self.settings.only_monoisotopic.get())
				if fit_rms>=0 or self.settings.data_height==1:
					if self.settings.data_height!=1:
						fit_rms_str=('%0.1f' % fit_rms)
						
						if self.settings.data_height!=2:
							fit_diff=mssputil.isodistrib_diff(isodistrib,mssputil.get_spectrum_isodistrib(query.peak_list,data_height=1,relative=0),self.settings.only_monoisotopic.get())
						else:
							fit_diff=mssputil.isodistrib_diff(mssputil.get_spectrum_isodistrib(query.peak_list,data_height=0,relative=0),mssputil.get_spectrum_isodistrib(query.peak_list,data_height=1,relative=0),self.settings.only_monoisotopic.get())
						fit_diff_str='%0.2f' % fit_diff

					if peaks_count>1:
						isodistrib=mssputil.get_relative_isodistrib(isodistrib)
						env_rms=mssputil.isodistrib_rms(peptide.isodistrib,isodistrib)
						env_rms_str=('%0.3f' % env_rms)

						if mssputil.isodistrib_max(isodistrib)==mssputil.isodistrib_max(peptide.isodistrib[0:peaks_count]):
							isodistrib_max=1
						else:
							isodistrib_max=0
					
						delta_rt=abs(mssputil.get_rt(query.get(0))-mssputil.get_rt(query.get(1)))
						delta_rt_str=('%0.2f' % delta_rt)

		was_hidden=self.peptide_tablelist.tablelist.rowcget(row,'-hide')
		hidden=0
		was_red=isinstance(self.peptide_tablelist.get_cell_config(row,0,'-image'),str)
		red_flag=0

		self.peptide_tablelist.configure_cell(row,4,text=mz_str)

		self.peptide_tablelist.configure_cell(row,5,text=delta_mz_str,foreground='black')
		if self.settings.delta_mz_check and abs(delta_mz/query.cmz)>self.settings.delta_mz_th:
			self.peptide_tablelist.configure_cell(row,5,foreground='red')
			red_flag=1

		self.peptide_tablelist.configure_cell(row,6,text=rt_str,foreground='black')

		self.peptide_tablelist.configure_cell(row,7,text=delta_mrt_str,foreground='black')
		if self.settings.delta_mrt_check and abs(delta_mrt)>self.settings.delta_mrt_th:
			self.peptide_tablelist.configure_cell(row,7,foreground='red')
			red_flag=1

		self.peptide_tablelist.configure_cell(row,8,text=delta_rt_str,foreground='black')
		if self.settings.delta_rt_check==1 and abs(delta_rt)>self.settings.delta_rt_th:
			self.peptide_tablelist.configure_cell(row,8,foreground='red')
			red_flag=1

		self.peptide_tablelist.configure_cell(row,9,text=value_str,foreground='black')
		if value==0:
			self.peptide_tablelist.configure_cell(row,9,foreground='red')
			red_flag=1
		
		self.peptide_tablelist.configure_cell(row,10,text=fit_rms_str,foreground='black')
		self.peptide_tablelist.configure_cell(row,11,text=fit_diff_str,foreground='black')
		if self.settings.data_height!=1:
			if self.settings.fit_rms_check==1 and (fit_rms<0 or fit_rms>self.settings.fit_rms_th):
				self.peptide_tablelist.configure_cell(row,10,foreground='red')
				red_flag=1
			if self.settings.fit_diff_check==1 and (fit_diff==-1 or fit_diff>self.settings.fit_diff_th):
				self.peptide_tablelist.configure_cell(row,11,foreground='red')
				red_flag=1

		self.peptide_tablelist.configure_cell(row,12,text=peaks_count,foreground='black')
		if self.settings.peaks_check==1 and peaks_count<self.settings.peaks_th:
			self.peptide_tablelist.configure_cell(row,12,foreground='red')
			red_flag=1

		self.peptide_tablelist.configure_cell(row,13,text=isodistrib_max,foreground='black')
		if self.settings.max_check==1 and isodistrib_max==0 and env_rms>self.settings.max_th:
			self.peptide_tablelist.configure_cell(row,13,foreground='red')
			red_flag=1

		self.peptide_tablelist.configure_cell(row,14,text=env_rms_str,foreground='black')
		if self.settings.env_rms_check==1 and env_rms>self.settings.env_rms_th:
			self.peptide_tablelist.configure_cell(row,14,foreground='red')
			red_flag=1

		if red_flag==1:
			self.peptide_tablelist.configure_cell(row,0,image=flag_icon)
			if self.settings.view_all.get()=='NM':
				hidden=1
		else:
			self.peptide_tablelist.configure_cell(row,0,image='')
			if self.settings.view_all.get()=='M':
				hidden=1

		if query.present==0:
			if self.settings.nan_checked_value==1:
				self.peptide_tablelist.configure_cell(row,9,text='NaN')
				self.peptide_tablelist.configure_cell(row,9,foreground='red')
			if self.settings.view_all.get()=='P':
				hidden=1
		else:
			if self.settings.view_all.get()=='NP':
				hidden=1

		self.peptide_tablelist.tablelist.rowconfigure(row,hide=hidden)


		if hidden==0:
			if was_hidden==1:
				self.peptide_tablelist.set_list_size(queries=(self.peptide_tablelist.queries_count+1))
				if query.present:
					self.peptide_tablelist.set_list_size(present=(self.peptide_tablelist.present_count+1))
			if red_flag==1 and was_red==0:
				self.peptide_tablelist.set_list_size(red_flags=(self.peptide_tablelist.red_flag_count+1))
			if red_flag==0 and was_red==1:
				self.peptide_tablelist.set_list_size(red_flags=(self.peptide_tablelist.red_flag_count-1))

		if hidden==1 and was_hidden==0:
			self.peptide_tablelist.set_list_size(queries=(self.peptide_tablelist.queries_count-1))			
			if was_red==1:
				self.peptide_tablelist.set_list_size(red_flags=(self.peptide_tablelist.red_flag_count-1))
			if query.present:
				self.peptide_tablelist.set_list_size(present=(self.peptide_tablelist.present_count-1))


	# -----------------------------------------------------------------------------
	#
	def check_query(self,row,check,external_call=0):
		data=self.peptide_tablelist.line_data[row]
		query=data[0].get(data[1])
		if (query.present==1 and check==0) or (query.present==0 and check==1):
			query.set_present(check)
			if check==0:
				self.peptide_tablelist.configure_cell(row,1,image=checkoff_icon,text='0')
				self.peptide_tablelist.configure_cell(row,2,foreground='red')
				if self.settings.nan_checked_value==1:
					self.peptide_tablelist.configure_cell(row,9,text='NaN')
					self.peptide_tablelist.configure_cell(row,9,foreground='red')

				self.peptide_tablelist.present_count=self.peptide_tablelist.present_count-1
			else:
				self.peptide_tablelist.configure_cell(row,1,image=checkon_icon,text='1')
				self.peptide_tablelist.configure_cell(row,2,foreground='black')
				if self.settings.nan_checked_value==1:
					value=0
					value_str='0.0'
					isodistrib=mssputil.get_spectrum_isodistrib(query.peak_list,data_height=self.settings.data_height,relative=0)
					peaks_count=mssputil.get_peaks_count(isodistrib)
	
					if peaks_count!=0 and isodistrib[0]>0:
						value=mssputil.get_value(isodistrib,only_monoisotopic=self.settings.only_monoisotopic.get())
						if value>0:
							if self.settings.log_scale.get()==1:
								v=math.log(v,10)
								value_str=('%0.3f' % value)
							else:
								value_str=('%0.1f' % value)
	
					self.peptide_tablelist.configure_cell(row,9,text=value_str)
					if value>0:
						self.peptide_tablelist.configure_cell(row,9,foreground='black')
					else:
						self.peptide_tablelist.configure_cell(row,9,foreground='red')

				self.peptide_tablelist.present_count=self.peptide_tablelist.present_count+1

			if external_call==0 and self.find_duplicates_dialog!=None:
				line_index=self.find_duplicates_dialog.duplicate_tablelist.find_peak_cb(query.get(0))
				print line_index
				if line_index!=-1:
					self.find_duplicates_dialog.check_peak(line_index,check,1)

	# ---------------------------------------------------------------------------
	#
	def check_queries(self):
		line_counter=-1
		self.stoppable_loop('peptides',50)
		for i in range(len(self.peptide_list)):
			self.check_for_stop()
			peptide=self.peptide_list[i]

			for j in range(len(peptide.query_list)):
				line_counter=line_counter+1
				if self.settings.check==-1:
					if isinstance(self.peptide_tablelist.get_cell_config(line_counter,0,'-image'),str):
						self.check_query(line_counter,0)

				elif self.settings.check==-2:
					if peptide.sequence.find('+')!=-1:
						self.check_query(line_counter,0)
				else:
					self.check_query(line_counter,self.settings.check)

	# ---------------------------------------------------------------------------
	#
	def check_marked(self):
		self.lock_gui(1)
		self.settings.check=-1
		self.stoppable_call(self.check_queries)
		self.peptide_tablelist.set_list_size(present=self.peptide_tablelist.present_count)
		self.lock_gui(0)

	# ---------------------------------------------------------------------------
	#
	def check_modified(self):
		self.lock_gui(1)
		self.settings.check=-2
		self.stoppable_call(self.check_queries)
		self.peptide_tablelist.set_list_size(present=self.peptide_tablelist.present_count)
		self.lock_gui(0)

	# ---------------------------------------------------------------------------
	#
	def check_all(self):
		self.lock_gui(1)
		self.settings.check=1
		self.stoppable_call(self.check_queries)
		self.peptide_tablelist.set_list_size(present=self.peptide_tablelist.present_count)
		self.lock_gui(0)

	# ---------------------------------------------------------------------------
	#
	def check_none(self):
		self.lock_gui(1)
		self.settings.check=0
		self.stoppable_call(self.check_queries)
		self.peptide_tablelist.set_list_size(present=self.peptide_tablelist.present_count)
		self.lock_gui(0)

	# ---------------------------------------------------------------------------
	#
	def lock_gui(self,lock):

		if lock==1:
			state='disabled'
			edit='no'
			if sys.platform=='win32':
				cursor='wait'
			else:
				cursor='watch'
		else:
			state='normal'
			edit='yes'
			cursor=''

		self.settings.lock_callbacks=lock
		self.peptide_tablelist.lock_callbacks=lock

		self.find_button['state']=state
		self.find_edit.entry['state']=state
		self.buttons_row.buttons[0]['state']=state
		self.buttons_row.buttons[2]['state']=state
		self.buttons_row.buttons[3]['state']=state
		self.buttons_row.buttons[4]['state']=state
		self.top.config(cursor=cursor)
		self.top.update_idletasks()

# -----------------------------------------------------------------------------
#
class peptides_list_settings:

	def __init__(self,session,path=None):

		self.session=session
		self.lock_callbacks=0
		self.loaded=0

		if path!=None:
			self.loaded=self.read_settings(path)

		try:
			num=int(self.session.project.saved_value('PL_CMASS'))
			if num<0 or num>1:
				raise ValueError
			self.cmass=num
		except (ValueError,TypeError):
			self.cmass=0

		try:
			num=int(self.session.project.saved_value('PL_DELTA_MZ'))
			if num<0 or num>1:
				raise ValueError
			self.delta_mz_check=num
		except (ValueError,TypeError):
			self.delta_mz_check=0

		try:
			num=int(self.session.project.saved_value('PL_DELTA_MRT'))
			if num<0 or num>1:
				raise ValueError
			self.delta_mrt_check=num
		except (ValueError,TypeError):
			self.delta_mrt_check=0

		try:
			num=int(self.session.project.saved_value('PL_DELTA_RT'))
			if num<0 or num>1:
				raise ValueError
			self.delta_rt_check=num
		except (ValueError,TypeError):
			self.delta_rt_check=0

		try:
			num=int(self.session.project.saved_value('PL_FIT_RMS'))
			if num<0 or num>1:
				raise ValueError
			self.fit_rms_check=num
		except (ValueError,TypeError):
			self.fit_rms_check=0

		try:
			num=int(self.session.project.saved_value('PL_FIT_DIFF'))
			if num<0 or num>1:
				raise ValueError
			self.fit_diff_check=num
		except (ValueError,TypeError):
			self.fit_diff_check=0

		try:
			num=int(self.session.project.saved_value('PL_ENV_RMS'))
			if num<0 or num>1:
				raise ValueError
			self.env_rms_check=num
		except (ValueError,TypeError):
			self.env_rms_check=0

		try:
			num=int(self.session.project.saved_value('PL_PEAKS'))
			if num<0 or num>1:
				raise ValueError
			self.peaks_check=num
		except (ValueError,TypeError):
			self.peaks_check=0

		try:
			num=int(self.session.project.saved_value('PL_MAX'))
			if num<0 or num>1:
				raise ValueError
			self.max_check=num
		except (ValueError,TypeError):
			self.max_check=0

		try:
			num=float(self.session.project.saved_value('PL_DELTA_MZ_TH'))
			if num<0:
				raise ValueError
			self.delta_mz_th=num/1000000
		except (ValueError,TypeError):
			self.delta_mz_th=20/1000000

		try:
			num=float(self.session.project.saved_value('PL_DELTA_MRT_TH'))
			if num<0:
				raise ValueError
			self.delta_mrt_th=num
		except (ValueError,TypeError):
			self.delta_mrt_th=3

		try:
			num=float(self.session.project.saved_value('PL_DELTA_RT_TH'))
			if num<0:
				raise ValueError
			self.delta_rt_th=num
		except (ValueError,TypeError):
			self.delta_rt_th=0.2

		try:
			num=float(self.session.project.saved_value('PL_FIT_RMS_TH'))
			if num<0:
				raise ValueError
			self.fit_rms_th=num
		except (ValueError,TypeError):
			self.fit_rms_th=1000

		try:
			num=float(self.session.project.saved_value('PL_FIT_DIFF_TH'))
			if num<0:
				raise ValueError
			self.fit_diff_th=num
		except (ValueError,TypeError):
			self.fit_diff_th=0.5

		try:
			num=float(self.session.project.saved_value('PL_ENV_RMS_TH'))
			if num<0:
				raise ValueError
			self.env_rms_th=num
		except (ValueError,TypeError):
			self.env_rms_th=0.3

		try:
			num=float(self.session.project.saved_value('PL_PEAKS_TH'))
			if num<1:
				raise ValueError
			self.peaks_th=num
		except (ValueError,TypeError):
			self.peaks_th=2

		try:
			num=float(self.session.project.saved_value('PL_MAX_TH'))
			if num<0:
				raise ValueError
			self.max_th=num
		except (ValueError,TypeError):
			self.max_th=0.05

		try:
			num=int(self.session.project.saved_value('PL_NAN_CHECKED_VALUE'))
			if num<0 or num>1:
				raise ValueError
			self.nan_checked_value=num
		except (ValueError,TypeError):
			self.nan_checked_value=1

		try:
			num=int(self.session.project.saved_value('PL_DATA_HEIGHT'))
			if num<0 or num>2:
				raise ValueError
			self.data_height=num
		except (ValueError,TypeError):
			self.data_height=0

		self.log_scale=Tkinter.IntVar()
		try:
			num=int(self.session.project.saved_value('PL_LOG_SCALE'))
			if num<0 or num>1:
				raise ValueError
			self.log_scale.set(num)
		except (ValueError,TypeError):
			self.log_scale.set(0)

		self.only_monoisotopic=Tkinter.IntVar()
		try:
			num=int(self.session.project.saved_value('PL_ONLY_MONOISOTOPIC'))
			if num<0 or num>1:
				raise ValueError
			self.only_monoisotopic.set(num)
		except (ValueError,TypeError):
			self.only_monoisotopic.set(1)

		self.view_all=Tkinter.StringVar()
		self.view_all.set(self.session.project.saved_value('PL_VIEW_ALL'))
		if self.view_all.get()!='A' and self.view_all.get()!='M' and self.view_all.get()!='NM' and self.view_all.get()!='P' and self.view_all.get()!='NP':
			self.view_all.set('A')

		self.view_list_only=Tkinter.IntVar()
		self.view_list_only.set(0)
		self.view_list=None

		self.check=-1

	# --------------------------------------------------------------------------
	#
	def read_settings(self,path):
		result=0

		if path!=None:
			try:
				file=open(path,'r')
				lines=file.readlines()

				for i in range(0,len(lines)):
					line=lines[i].strip()
					if len(line)!=0 and line[0]!='#':
						fields=lines[i].split('=')
						if len(fields)==2:
							self.session.project.save_value(fields[0].strip(),fields[1].strip())
				file.close()
				result=1
			except IOError, (errno, strerror):
				print "I/O error(%s): %s" % (errno, strerror)
				result=0
		
		return result

# -----------------------------------------------------------------------------
#
class peptides_list_settings_dialog(mstkutil.Settings_Dialog):
	
	def __init__(self, session):
		self.session=session
		self.loaded=0
		mstkutil.Settings_Dialog.__init__(self,session.tk,'Settings')

		f=Tkinter.Frame(self.top,borderwidth=2,relief='groove')
		f.pack(side='top',anchor='w',pady=2,fill='x',expand=1)

		l=Tkinter.Label(f,text='Value: ')
		l.grid(row=0,column=0,sticky='w')

		m=mssputil.value_menu(f,'')
		m.frame.grid(row=0,column=1,sticky='w',pady=2,padx=2)
		self.value_menu=m

		cb=mstkutil.checkbutton(f,'Value only from monoisotopic peak',0)
		cb.button.grid(row=1,columnspan=2,sticky='w',pady=1)
		self.only_monoisotopic_check=cb

		cb=mstkutil.checkbutton(f,'Set value to NaN for unchecked',0)
		cb.button.grid(row=2,columnspan=2,sticky='w',pady=1)
		self.nan_value_check=cb

		cb=mstkutil.checkbutton(f,'Show theoretical m/z value',0)
		cb.button.grid(row=3,columnspan=2,sticky='w',pady=1)
		self.cmass_check=cb

		cb=mstkutil.checkbutton(f,'Delta m/z (ppm): ',0)
		cb.button.grid(row=4,column=0,sticky='w')
		self.delta_mz_check=cb

		e=mstkutil.entry_field(f,'','0',7,None,'right')
		e.frame.grid(row=4,column=1,sticky='w',padx=2,pady=1)
		self.delta_mz_edit=e

		cb=mstkutil.checkbutton(f,'Fit RMSE: ',0)
		cb.button.grid(row=5,column=0,sticky='w')
		self.frmse_check=cb

		e=mstkutil.entry_field(f,'','0',7,None,'right')
		e.frame.grid(row=5,column=1,sticky='w',padx=2,pady=1)
		self.frmse_edit=e

		cb=mstkutil.checkbutton(f,'Fit DIFF: ',0)
		cb.button.grid(row=6,column=0,sticky='w')
		self.fdiff_check=cb

		e=mstkutil.entry_field(f,'','0',7,None,'right')
		e.frame.grid(row=6,column=1,sticky='w',padx=2,pady=1)
		self.fdiff_edit=e

		cb=mstkutil.checkbutton(f,'Envelope RMSE: ',0)
		cb.button.grid(row=7,column=0,sticky='w')
		self.ermse_check=cb

		e=mstkutil.entry_field(f,'','0',7,None,'right')
		e.frame.grid(row=7,column=1,sticky='w',padx=2,pady=1)
		self.ermse_edit=e

		cb=mstkutil.checkbutton(f,'Massref delta RT: ',0)
		cb.button.grid(row=8,column=0,sticky='w')
		self.delta_mrt_check=cb

		e=mstkutil.entry_field(f,'','0',7,None,'right')
		e.frame.grid(row=8,column=1,sticky='w',padx=2,pady=1)
		self.delta_mrt_edit=e

		cb=mstkutil.checkbutton(f,'Peaks delta RT:',0)
		cb.button.grid(row=9,column=0,sticky='w')
		self.delta_rt_check=cb

		e=mstkutil.entry_field(f,'','0',7,None,'right')
		e.frame.grid(row=9,column=1,sticky='w',padx=2,pady=2)
		self.delta_rt_edit=e

		cb=mstkutil.checkbutton(f,'Peaks in envelope:',0)
		cb.button.grid(row=10,column=0,sticky='w')
		self.peaks_check=cb

		e=mstkutil.entry_field(f,'','0',7,None,'right')
		e.frame.grid(row=10,column=1,sticky='w',padx=2,pady=2)
		self.peaks_edit=e

		cb=mstkutil.checkbutton(f,'Max ERMSE:',0)
		cb.button.grid(row=11,columnspan=2,sticky='w',pady=1)
		self.max_check=cb

		e=mstkutil.entry_field(f,'','0',7,None,'right')
		e.frame.grid(row=11,column=1,sticky='w',padx=2,pady=2)
		self.max_edit=e

		l=Tkinter.Label(f,text='View:')
		l.grid(row=12,column=0,sticky='w',pady=1)

		m=mstkutil.option_menu(f,'',('All','Marked','Not marked','Present','Not present'),'All')
		m.frame.grid(row=12,column=1,sticky='w',pady=2,padx=2)
		self.view_menu=m

		cb=mstkutil.checkbutton(f,'Show only peptides from list',0)
		cb.button.grid(row=13,columnspan=2,sticky='w',pady=1)
		self.view_list_check=cb

		b=Tkinter.Button(f,text='Load list',borderwidth=1,width='12',command=self.load_list_cb)
		b.grid(row=14,column=0,sticky='w',padx=2,pady=2)

		b=Tkinter.Button(f,text='Clear list',borderwidth=1,width='12',command=self.clear_list_cb)
		b.grid(row=14,column=1,sticky='w',padx=2,pady=2)

		cb=mstkutil.checkbutton(f,'Exact charges',0)
		cb.button.grid(row=15,columnspan=2,sticky='w',pady=2)
		self.list_charge_check=cb

		br=mstkutil.button_row(self.top,
				('Apply', self.apply_settings_cb),
				('Load', self.load_cb),
				('Close', self.close_cb),
				)
		br.frame.pack(side='bottom',anchor='w',pady=2)

		self.top.geometry("+0+0")
		self.top.resizable(0,0)

	# ---------------------------------------------------------------------------
	#
	def load_cb(self):
		path=mstkutil.load_file(self.top,'Open configuration','open_cfg')
		if path!=None:
			settings=peptides_list_settings(self.session,path)
			self.loaded=settings.loaded
			self.show_settings(settings)

	# ---------------------------------------------------------------------------
	#
	def show_settings(self,settings):

		self.value_menu.set_value(settings.data_height)
		self.only_monoisotopic_check.variable.set(settings.only_monoisotopic.get())
		self.nan_value_check.variable.set(settings.nan_checked_value)

		self.cmass_check.variable.set(settings.cmass)

		self.delta_mz_check.variable.set(settings.delta_mz_check)
		self.delta_mz_edit.variable.set(settings.delta_mz_th*1000000)
		self.delta_mrt_check.variable.set(settings.delta_mrt_check)
		self.delta_mrt_edit.variable.set(settings.delta_mrt_th)
		self.delta_rt_check.variable.set(settings.delta_rt_check)
		self.delta_rt_edit.variable.set(settings.delta_rt_th)

		self.frmse_check.variable.set(settings.fit_rms_check)
		self.frmse_edit.variable.set(settings.fit_rms_th)
		self.fdiff_check.variable.set(settings.fit_diff_check)
		self.fdiff_edit.variable.set(settings.fit_diff_th)
		self.ermse_check.variable.set(settings.env_rms_check)
		self.ermse_edit.variable.set(settings.env_rms_th)
		self.peaks_check.variable.set(settings.peaks_check)
		self.peaks_edit.variable.set(settings.peaks_th)
		self.max_check.variable.set(settings.max_check)
		self.max_edit.variable.set(settings.max_th)

		if settings.view_all.get()=='A':
			entry='All'
		elif settings.view_all.get()=='M':
			entry='Marked'
		elif settings.view_all.get()=='NM':
			entry='Not marked'
		elif settings.view_all.get()=='P':
			entry='Present'
		elif settings.view_all.get()=='NP':
			entry='Not present'
		self.view_menu.set(entry)

		self.view_list_check.variable.set(settings.view_list_only.get())
		if settings.view_list==None or len(settings.view_list)==0:
			self.view_list_check.button.configure(state='disabled')
		else:
			self.view_list_check.button.configure(state='normal')

	# ---------------------------------------------------------------------------
	#
	def get_settings(self):
		settings=peptides_list_settings(self.session)

		settings.data_height=self.value_menu.get_value()
		settings.only_monoisotopic.set(self.only_monoisotopic_check.variable.get())
		settings.nan_checked_value=self.nan_value_check.variable.get()

		settings.cmass=self.cmass_check.variable.get()

		settings.delta_mz_check=self.delta_mz_check.variable.get()
		settings.delta_mz_th=float(self.delta_mz_edit.variable.get())/1000000
		settings.delta_mrt_check=self.delta_mrt_check.variable.get()
		settings.delta_mrt_th=float(self.delta_mrt_edit.variable.get())
		settings.delta_rt_check=self.delta_rt_check.variable.get()
		settings.delta_rt_th=float(self.delta_rt_edit.variable.get())
		settings.fit_rms_check=self.frmse_check.variable.get()
		settings.fit_rms_th=float(self.frmse_edit.variable.get())
		settings.fit_diff_check=self.fdiff_check.variable.get()
		settings.fit_diff_th=float(self.fdiff_edit.variable.get())
		settings.env_rms_check=self.ermse_check.variable.get()
		settings.env_rms_th=float(self.ermse_edit.variable.get())
		settings.peaks_check=self.peaks_check.variable.get()
		settings.peaks_th=float(self.peaks_edit.variable.get())
		settings.max_check=self.max_check.variable.get()
		settings.max_th=float(self.max_edit.variable.get())

		entry=self.view_menu.get()
		if entry=='All':
			settings.view_all.set('A')
		elif entry=='Marked':
			settings.view_all.set('M')
		elif entry=='Not marked':
			settings.view_all.set('NM')
		elif entry=='Present':
			settings.view_all.set('P')
		elif entry=='Not present':
			settings.view_all.set('NP')
#		if self.loaded==0:
#			settings.log_scale.set(self.parent_dialog.settings.log_scale.get())

		settings.view_list=self.parent_dialog.settings.view_list
		settings.view_list_only.set(self.view_list_check.variable.get())
		
		return settings

	# ---------------------------------------------------------------------------
	#
	def apply_settings_cb(self):
		self.show_window(0)
		self.apply_cb()
		self.close_cb()

	# ---------------------------------------------------------------------------
	#
	def clear_list_cb(self,event=None):
		self.parent_dialog.settings.view_list=None
		self.view_list_check.variable.set(0)
		self.view_list_check.button.configure(state='disabled')

	# ---------------------------------------------------------------------------
	#
	def load_list_cb(self,event=None):
		path=mstkutil.load_file(self.top,'Load list','load_list')
		exact_charge=self.list_charge_check.variable.get()
		sequences=self.load_list(path,exact_charge)

		if sequences!=None and len(sequences)!=0:
			self.parent_dialog.settings.view_list=[]
			peptide_sequences=[]
			for i in range(len(self.parent_dialog.peptide_list)):
				peptide=self.parent_dialog.peptide_list[i]
				for j in range(len(peptide.query_list)):
					peptide_sequences.append(peptide.get_sequence(exact_charge,j))
				
			for i in range(len(peptide_sequences)):
				found=0
				for j in range(len(sequences)):
					if peptide_sequences[i]==sequences[j]:
						found=1
						break
				self.parent_dialog.settings.view_list.append(found)
					
			if max(self.parent_dialog.settings.view_list)==0:
				self.parent_dialog.settings.view_list=[]
				self.view_list_check.button.configure(state='disabled')
				dialog=mstkutil.warning_dialog(self.top,'No peptides from list found')
			else:
				self.view_list_check.button.configure(state='normal')

	# --------------------------------------------------------------------------
	#
	def load_list(self,path,exact_charge=0):
		sequences=None

		if path:
			file=open(path,'r')
			sequences=file.readlines()
			file.close()

			if len(sequences)!=0:
				for i in range(len(sequences)-1,-1,-1):
					sequence=sequences[i].strip()
					if sequence=='':
						sequences[i]=[]
					else:
						sequences[i]=sequence
	
						index=sequence.find(' (+')
						if index!=-1:
							if exact_charge==1:
								index=sequence.find(')',index)+1
							sequences[i]=sequence[:index]	
		return sequences

# -----------------------------------------------------------------------------
#
def show_peptides_list(session):
	spectrum=session.selected_spectrum()
	if spectrum==None:
		return

	if not hasattr(session,'spectrum_dialogs'):
		session.spectrum_dialogs={}
	dialogs=session.spectrum_dialogs

	key='PL'+spectrum.name
	if (dialogs.has_key(key) and not dialogs[key].is_window_destroyed()):
		dialogs[key].show_window(1)
	else:
		d=peptides_list_dialog(session)
		d.show_window(1)
		d.show_spectrum_peptides(spectrum)
		dialogs[key]=d
