import math
import sys
import os
import time
import gc

import Tkinter
import numpy
import odict

import sparky
import ms
import mscalib
import msfit
import mspeptideslist_settings
import msfindpeptides_gui
import mschromatogram_gui
import msduplicated
import msisoenvelope
import msenvelope
import mssputil
import mstkutil

# -----------------------------------------------------------------------------
#
class peptideslist_dialog(mstkutil.Dialog,mstkutil.Stoppable):

	def __init__(self,session,settings=None):

		self.session=session
		self.title='MS peptides list:'
		if settings==None:
			self.settings=mspeptieslist_settings.peptides_list_settings(self.session,os.path.join(os.environ.get('SPARKY_INSTALL'),os.path.join('config','peptideslist2.cfg')))
		else:
			self.settings=settings
		self.find_duplicates_dialog=None

		mstkutil.Dialog.__init__(self,session.tk,self.title)

		f=Tkinter.Frame(self.top,borderwidth=2,relief='groove')
		f.pack(side='top',anchor='w',fill='x')

		pl=mssputil.peptide_tablelist(f,columns=None,selected_edit=1,editendcommand=self.editend_cb,editstartcommand=self.editstart_cb)
		pl.set_columns((2,'F',2,'P',40,'Sequence',10,'Protein',6,'Fit',9,'m/z',9,u'\u03B4m/z',6,'RT',6,u'\u0394RT',11,'Volume',8,'RMS(m/z)',8,u'R\u00B2(m/z)',5,'F(m/z)',8,'RMS(RT)',8,u'R\u00B2(RT)',5,'F(RT)',8,'RMS(E)',8,u'R\u00B2(E)',8,'Peaks',3,'#'))
		pl.set_columns_align(('left','center','left','left','right','right','right','right','right','right','right','right','right','right','right','right','right','right','right','left'))
		pl.tablelist.columnconfigure(1,editable='yes',editwindow='checkbutton')

		pl.sort_index_column=19
		pl.tablelist.columnconfigure(pl.sort_index_column,hide=1)
		pl.tablelist.columnconfigure(0,labelcommand=self.sort_cb,sortmode='ascii')
		pl.tablelist.columnconfigure(1,labelcommand=self.sort_cb,sortmode='ascii')
		pl.tablelist.columnconfigure(2,labelcommand=self.sort_cb,sortmode='ascii')
		pl.tablelist.columnconfigure(3,labelcommand=self.sort_cb,sortmode='ascii')
		pl.tablelist.columnconfigure(4,labelcommand=self.sort_cb,sortmode='command',sortcommand=self.fits_sort)
		pl.tablelist.columnconfigure(5,labelcommand=self.sort_cb,sortmode='real')
		pl.tablelist.columnconfigure(6,labelcommand=self.sort_cb,sortmode='command',sortcommand=pl.abs_real_sort)
		pl.tablelist.columnconfigure(7,labelcommand=self.sort_cb,sortmode='real')
		pl.tablelist.columnconfigure(8,labelcommand=self.sort_cb,sortmode='command',sortcommand=pl.abs_real_sort)
		pl.tablelist.columnconfigure(9,labelcommand=self.sort_cb,sortmode='command',sortcommand=pl.abs_real_sort)
		pl.tablelist.columnconfigure(pl.sort_index_column,labelcommand=self.sort_cb,sortmode='real')
		self.sort_menu_command=Tkinter.IntVar()
		self.sort_menu_command.set(pl.sort_index_column)

		pl.tablelist.bind('<ButtonRelease-1>',pl.select_peptide_cb)
		pl.tablelist.bind('<Double-ButtonRelease-1>',pl.goto_peptide_cb)
		pl.tablelist.bind('<Return>',pl.goto_peptide_cb)
		pl.tablelist.bind('<Button-3>',self.popup_menu)
		pl.tablelist.bind('<Key>', self.keypressed_cb)

		pl.frame.pack(side='top',fill='both',expand=1)
		self.peptide_tablelist=pl

		f=Tkinter.Frame(self.top,borderwidth=2,relief='groove')
		f.pack(side='top',anchor='w',fill='x')

		l=Tkinter.Label(f,text='Search for:',anchor='w',width=10)
		l.pack(side='left',padx=3)

		e=mstkutil.entry_field(f,'','',45)
		e.frame.pack(side='left',padx=2)
		e.entry.bind('<Return>',self.find_string)
		self.find_edit=e

		m=mstkutil.option_menu(f,'',('Search down','Search up'),'Search down')
		m.frame.pack(side='left',padx=3)
		self.direction_menu=m

		cb=mstkutil.checkbutton(f,'Only exact match',0)
		cb.button.pack(side='left',padx=3)
		self.match_check=cb

		b=Tkinter.Button(f,text="Find",borderwidth=1,width='7',command=self.find_string)
		b.pack(side='right',ipady=2,padx=3,pady=3)
		self.find_button=b

		progress_label=Tkinter.Label(self.top, anchor = 'nw')
		progress_label.pack(side='top',anchor='w')

		br=mstkutil.button_row(self.top,
					('Update', self.update_cb),
					('Stop', self.stop_cb),
					('Setup', self.setup_cb),
					('Save', self.save_cb),
					('Close', self.closing_window),
#					('Close', self.top.destroy),
					)
		br.frame.pack(side='top',anchor='w',pady=2)
		self.buttons_row=br

		if sys.platform=='win32':
			self.top.geometry(('%dx%d+%d+%d') % (1280,430,0,0))
		else:
			self.top.geometry(('+%d+%d') % (0,0))
		self.top.resizable(1,0)

		self.notice=session.notify_me('removed spectrum from project',self.spectrum_removed_cb)
		self.selection_notice=session.notify_me('selection changed',self.selection_changed_cb)
		self.dragging_notice=session.notify_me('dragged peak',self.peak_dragged_cb)
		self.top.bind('<Destroy>',self.remove_callbacks,1)
		self.top.protocol("WM_DELETE_WINDOW",self.closing_window)

		mstkutil.Stoppable.__init__(self,progress_label,br.buttons[1])

	# -----------------------------------------------------------------------------
	#
	def popup_menu(self,event):
		if self.peptide_tablelist.lock_callbacks==0:
			menu=Tkinter.Menu(self.peptide_tablelist.tablelist,tearoff=0)

			menu.add_command(label='Best fit plot',command=self.show_fit_cb)
			menu.add_command(label='Isotopic envelope plot',command=self.show_isodistrib_cb)
			menu.add_command(label='Chromatogram plot',command=self.show_chromatogram_cb)
			menu.add_separator()

			menu.add_command(label='Find peptide',command=self.find_peptide_cb)
			menu.add_command(label='Find duplicated',command=self.duplicated_cb)

			menu.add_separator()

			fit_submenu=Tkinter.Menu(menu,tearoff=0)
			fit_submenu.add_command(label='Next fit',accelerator='N',command=self.next_fit_cb)
			fit_submenu.add_command(label='Previous fit',accelerator='P',command=self.prev_fit_cb)
			fit_submenu.add_command(label='Best fit',accelerator='B',command=self.best_fit_cb)
			fit_submenu.add_separator()
			fit_submenu.add_command(label='Create fit',accelerator='C',command=self.create_fit_cb)
			fit_submenu.add_separator()
			fit_submenu.add_command(label='Delete fit',accelerator='D',command=self.delete_fit_cb)
			fit_submenu.add_command(label='Reset position',accelerator='R',command=self.reset_fit_cb)
			menu.add_cascade(label='Fits',menu=fit_submenu)

			menu.add_separator()
			
			check_submenu=Tkinter.Menu(menu,tearoff=0)
			check_submenu.add_command(label='Check all',command=self.check_all)
			check_submenu.add_command(label='Uncheck all ',command=self.check_none)
			check_submenu.add_command(label='Uncheck marked',command=self.check_marked)
			check_submenu.add_command(label='Uncheck modified',command=self.check_modified)
			menu.add_cascade(label='Check/Uncheck',menu=check_submenu)

			view_submenu=Tkinter.Menu(menu,tearoff=0)
			view_submenu.add_radiobutton(label='All',variable=self.settings.view_all,value='A',command=self.redraw_cb)
			view_submenu.add_separator()
			view_submenu.add_radiobutton(label='Marked',variable=self.settings.view_all,value='M',command=self.redraw_cb)
			view_submenu.add_radiobutton(label='Not marked',variable=self.settings.view_all,value='NM',command=self.redraw_cb)
			view_submenu.add_separator()
			view_submenu.add_radiobutton(label='Present',variable=self.settings.view_all,value='P',command=self.redraw_cb)
			view_submenu.add_radiobutton(label='Not present',variable=self.settings.view_all,value='NP',command=self.redraw_cb)
			view_submenu.add_separator()
			if self.settings.view_list==None or len(self.settings.view_list)==0:
				view_submenu.add_checkbutton(label='Only from list',variable=self.settings.view_list_only,command=self.view_list_cb,state='disabled')
			else:
				view_submenu.add_checkbutton(label='Only from list',variable=self.settings.view_list_only,command=self.view_list_cb,state='normal')
			menu.add_cascade(label='View',menu=view_submenu)
			
			sort_submenu=Tkinter.Menu(menu,tearoff=0)
			sort_submenu.add_radiobutton(label='No sorting',variable=self.sort_menu_command,value=self.peptide_tablelist.sort_index_column,command=self.sort_menu_cb)
			sort_submenu.add_separator()
			sort_submenu.add_radiobutton(label='Marked',variable=self.sort_menu_command,value=0,command=self.sort_menu_cb)
			sort_submenu.add_radiobutton(label='Present',variable=self.sort_menu_command,value=1,command=self.sort_menu_cb)
			sort_submenu.add_radiobutton(label='Fits',variable=self.sort_menu_command,value=4,command=self.sort_menu_cb)
			sort_submenu.add_separator()
			sort_submenu.add_radiobutton(label='Sequence',variable=self.sort_menu_command,value=2,command=self.sort_menu_cb)
			sort_submenu.add_radiobutton(label='Protein',variable=self.sort_menu_command,value=3,command=self.sort_menu_cb)
			sort_submenu.add_separator()
			sort_submenu.add_radiobutton(label='m/z',variable=self.sort_menu_command,value=5,command=self.sort_menu_cb)
			sort_submenu.add_radiobutton(label='RT',variable=self.sort_menu_command,value=7,command=self.sort_menu_cb)
			sort_submenu.add_separator()
			sort_submenu.add_radiobutton(label=u'\u03B4m/z',variable=self.sort_menu_command,value=6,command=self.sort_menu_cb)
			sort_submenu.add_radiobutton(label=u'\u0394RT',variable=self.sort_menu_command,value=8,command=self.sort_menu_cb)
			sort_submenu.add_separator()
			sort_submenu.add_radiobutton(label='Value',variable=self.sort_menu_command,value=9,command=self.sort_menu_cb)
			menu.add_cascade(label='Sort',menu=sort_submenu)

			plots_submenu=Tkinter.Menu(menu,tearoff=0)
			plots_submenu.add_command(label='Values histogram',command=self.values_hist_cb)
			plots_submenu.add_separator()
			plots_submenu.add_command(label='Mass error histogram',command=self.mass_error_hist_cb)
			plots_submenu.add_command(label='Mass error vs. m/z',command=self.mass_error_mz_cb)
			plots_submenu.add_command(label='Mass error vs. RT',command=self.mass_error_rt_cb)
			plots_submenu.add_separator()
			plots_submenu.add_command(label='Resolving power vs. m/z',command=self.rp_mz_cb)
			plots_submenu.add_separator()
			plots_submenu.add_command(label='RT error histogram',command=self.rt_error_hist_cb)
			plots_submenu.add_command(label='RT error vs. RT',command=self.rt_error_rt_cb)
#			plots_submenu.add_command(label='RTE vs. m/z',command=self.rt_error_mz_cb)

			menu.add_cascade(label='Plots',menu=plots_submenu)

			menu.add_separator()
			menu.add_command(label='Redraw',command=self.redraw_cb)

			menu.tk_popup(event.x_root,event.y_root)
			
	# ---------------------------------------------------------------------------
	#
	def sort_cb(self,event=None,column=0):

		self.lock_gui(1)
		self.sort_menu_command.set(column)
		self.peptide_tablelist.sortbycolumn(column)
		self.lock_gui(0)

	# ---------------------------------------------------------------------------
	#
	def sort_menu_cb(self):

	    #self.top.update_idletasks()
	    #self.top.after(100)
	    self.sort_cb(column=self.sort_menu_command.get())


	# ---------------------------------------------------------------------------
	#
	def fits_sort(self,a,b):
	    	
	    a=a.split(' (')[1][0:-1]
	    b=b.split(' (')[1][0:-1]

	    return self.peptide_tablelist.abs_real_sort(a,b)
	

	# ---------------------------------------------------------------------------
	#
	def spectrum_removed_cb(self,removed_spectrum):
		if self.spectrum==removed_spectrum:
			self.top.destroy()

	# ---------------------------------------------------------------------------
	#
	def selection_changed_cb(self):
		peaks=self.session.selected_peaks()
		if peaks!=None and len(peaks)==1:
			self.peptide_tablelist.mark_peak_cb(peaks[0])

	# ---------------------------------------------------------------------------
	#
	def peak_dragged_cb(self,peak):
		if peak.spectrum==self.spectrum and mssputil.is_peak_unassigned(peak)!=1:
			line_index=self.peptide_tablelist.find_peak_cb(peak)
			if line_index!=-1 and (self.settings.view_list_only.get()==0 or (self.settings.view_list_only.get()==1 and self.settings.view_list[int(line_index)]==1)):
				self.update_query(line_index)
				if peak.selected==1:
					self.peptide_tablelist.selected_peak=None
					self.peptide_tablelist.mark_peak_cb(peak)

	# ---------------------------------------------------------------------------
	#
	def closing_window(self):
		if self.settings.lock_callbacks==0:
			if self.find_duplicates_dialog!=None:
				self.find_duplicates_dialog.parent=None	
			self.top.destroy()
			self.peptide_list=None
			gc.collect()
#		else:
#			self.stop_cb()
				
	# ---------------------------------------------------------------------------
	#
	def remove_callbacks(self,event):
		if str(self.top)==str(event.widget):
			if sparky.object_exists(self.session):
				if self.notice!=None:
					self.session.dont_notify_me(self.notice)
				if self.selection_notice!=None:
					self.session.dont_notify_me(self.selection_notice)
				if self.dragging_notice!=None:
					self.session.dont_notify_me(self.dragging_notice)
					
			self.notice=None
			self.selection_notice=None
			self.dragging_notice=None

	# -----------------------------------------------------------------------------
	#
	def get_peptides(self):
		self.peptide_list=mssputil.get_peptides(self.spectrum,self,None,'peaks',100)
		
	# ---------------------------------------------------------------------------
	#
	def save_cb(self):
		filetypes=[('List file','.list'),('Text file','.txt')]

		path=mstkutil.save_file(self.peptide_tablelist.tablelist,'Save List','save_list',filetypes)
		if path!=None and len(path)>0:
			self.top.update_idletasks()
			self.lock_gui(1)
			
			format=0
			for i in range(0,len(filetypes)):
				if path.endswith(filetypes[i][1]):
					format=i
					break
			if not path.endswith(filetypes[format][1]):
				path=path+filetypes[format][1]

			try:
				self.peptide_tablelist.write_file(path,'w')
			except IOError, ioe:
				mstkutil.warning_dialog(self.top,'Error while saving log:\n%s' % str(ioe))
			
			self.lock_gui(0)

	# ---------------------------------------------------------------------------
	#
	def setup_cb(self):
		psd=mssputil.the_dialog(mspeptideslist_settings.peptideslist_settings_dialog,self.session)
		psd.set_parent_dialog(self,self.settings,self.new_settings)
		psd.top.title(self.title + ' settings')
		psd.show_window(1)

	# ---------------------------------------------------------------------------
	#
	def new_settings(self,settings):
		self.settings=settings
		self.redraw_cb()

	# ---------------------------------------------------------------------------
	#
	def show_spectrum_peptides(self,spectrum):
		self.spectrum=spectrum

		self.rt_step=mssputil.get_rt_step(self.spectrum)
		if self.spectrum==mssputil.DOMAIN_FREQ:
			self.mz_step=mssputil.get_f_step(self.spectrum)
		else:
			self.mz_step=mssputil.get_mz_step(self.spectrum)

		self.title='MS peptides list: ' + self.spectrum.name
		self.top.title(self.title)

		dialogs=self.session.spectrum_dialogs
		key='FD'+self.spectrum.name
		if (dialogs.has_key(key) and not dialogs[key].is_window_destroyed()):
			self.find_duplicates_dialog=dialogs[key]
			self.find_duplicates_dialog.parent=self

		self.update_cb()

	# ---------------------------------------------------------------------------
	#
	def editstart_cb(self,tbl,row,column,text):
		#print 'start %d %s' % (int(row),column)
		self.edit_start=1
		value=0
		self.peptide_tablelist.lock_callbacks=1

		if column=='1':
			if self.peptide_tablelist.get_cell_config(int(row),1,'-text')=='1':
				value=1
			else:
				value=0
		return value

	# ---------------------------------------------------------------------------
	#
	def editend_cb(self,tbl,row,column,text):
		value=int(text)
		if self.edit_start==1:
			self.edit_start=0
			#print 'end %d %s' % (int(row),column)
			if column=='1':
				#print 'check query: row %d, value %d' % (int(row),value)
				#self.peptide_tablelist.tablelist['editendcommand']=None
				self.check_query(int(row),value)

				#self.peptide_tablelist.tablelist['editendcommand']=self.editend_cb
				if text=='0':
					self.peptide_tablelist.set_list_size(present=self.peptide_tablelist.present_count)
				else:
					self.peptide_tablelist.set_list_size(present=self.peptide_tablelist.present_count)

			self.peptide_tablelist.lock_callbacks=0

		return value

	# ---------------------------------------------------------------------------
	#
	def keypressed_cb(self,event):
		ks=event.keysym.lower()
		if ks=='n':
			self.next_fit_cb()
		if ks=='p':
			self.prev_fit_cb()
		if ks=='b':
			self.best_fit_cb()
		if ks=='r':
			self.reset_fit_cb()
		if ks=='d':
			self.delete_fit_cb()
		if ks=='c':
			self.create_fit_cb()
		if ks=='enter':
			self.peptide_tablelist.goto_peptide_cb()
			#self.peptide_tablelist.tablelist.focus_set()
	# ---------------------------------------------------------------------------
	#
	def update_cb(self,event=None):
		self.lock_gui(1)
		
#		start=time.clock()
		self.stoppable_call(self.get_peptides)
#		print (time.clock()-start)

		self.stoppable_call(self.update_tablelist)
#		print (time.clock()-start)

		if self.sort_menu_command.get()!=self.peptide_tablelist.sort_index_column:
		    self.peptide_tablelist.sort_order=-1*self.peptide_tablelist.sort_order
		    self.sort_menu_cb()

		self.lock_gui(0)

		self.peptide_tablelist.tablelist.focus_set()

	# ---------------------------------------------------------------------------
	#
	def redraw_cb(self,event=None):
		self.lock_gui(1)
		
		self.stoppable_call(self.update_tablelist)

		if self.sort_menu_command.get()!=self.peptide_tablelist.sort_index_column:
		    self.peptide_tablelist.sort_order=-1*self.peptide_tablelist.sort_order
		    self.sort_menu_cb()

		self.lock_gui(0)

		self.peptide_tablelist.tablelist.focus_set()

	# ---------------------------------------------------------------------------
	#
	def update_tablelist(self):
		self.progress_report('Updating table')

		counter=-1
		query_counter=0
		red_flag_counter=0
		present_counter=0
		selected_peak=None

		self.set_value_column()

		row_index=int(round(self.peptide_tablelist.tablelist.yview()[0]*self.peptide_tablelist.tablelist.size()))
		self.peptide_tablelist.clear()

		self.stoppable_loop('peptides',50)
		for i in range(len(self.peptide_list)):
			self.check_for_stop()
			peptide=self.peptide_list[i]

			if peptide.size()>1:
				bgcolor='#d4deec'
			else:
				bgcolor='gray98'

			for j in range(len(peptide.query_list)):
				hidden=0
				counter=counter+1

				if self.settings.view_list_only.get()==1 and self.settings.view_list[counter]==0:
					self.peptide_tablelist.append((' ',' ',' ','- (0)','0','0','0','0','0','0','0','0','0','0','0','0','0','0','0',counter),(peptide,j))
					self.peptide_tablelist.tablelist.rowconfigure(counter,hide=1)
				else: 
					query=peptide.get(j)
					peak=query.get(0)
					fits=msfit.get_fits(peak)

					if self.settings.delta_mz_correction==1 and self.spectrum.rt_precision!=0:
						dmz_correction=mssputil.get_mz_correction_ppm(self.spectrum,query.cmz)
					else:
						dmz_correction=0

					if self.settings.delta_mrt_correction==1:
						drt_correction=mssputil.get_rt_correction(self.spectrum,query.massref_rt)
					else:
						drt_correction=0

					##print query.mz
					mz_str=('%0.4f' % query.cmz)
					delta_mz_ppm=1e+6*((query.mz-query.cmz)/query.cmz)-dmz_correction
					delta_mz=query.cmz*delta_mz_ppm/1e+6
					if self.settings.delta_mz_ppm==1:
						delta_mz_str=('%0.2f' % delta_mz_ppm)
					else:
						delta_mz_str=('%0.4f' % delta_mz)

					rt_str=('%0.2f' % query.rt)
					delta_mrt=query.rt-(query.massref_rt+drt_correction)
					delta_mrt_str=('%0.2f' % delta_mrt)

					fit_str=('- (0)')
					mz_rmse_str='NaN'
					mz_rsquare_str='NaN'
					mz_flag_str='NaN'
					mz_env_rmse_str='NaN'
					mz_env_rsquare_str='NaN'
					rt_rmse_str='NaN'
					rt_rsquare_str='NaN'
					rt_flag_str='NaN'
					fit_rmse_str='NaN'
					value_str='NaN'

					fit=None
					if len(fits)>0:
						fit_nr=peak.best_fit
						if numpy.abs(fits[fit_nr].rt_pos-peak.frequency[0])<=self.rt_step and numpy.abs(fits[fit_nr].mz_pos-peak.frequency[1])<=self.mz_step:
							fit=fits[fit_nr]
							fit_str=('%d (%d)' % (fit.fit_nr,len(fits)))

							mz_rmse_str=('%0.2f' % fit.mz_rmse)
							mz_rsquare_str=('%0.2f' % fit.mz_rsquare)
							mz_flag_str=('%d' % fit.mz_flag)

							mz_env_rmse_str=('%0.2f' % fit.mz_env_rmse)
							mz_env_rsquare_str=('%0.2f' % fit.mz_env_rsquare)

							rt_rmse_str=('%0.2f' % fit.rt_rmse)
							rt_rsquare_str=('%0.2f' % fit.rt_rsquare)
							rt_flag_str=('%d' % fit.rt_flag)

							#fit_rmse_str=('%d' % fit.peaks_count)
							fit_rmse_str=('%d' % fit.mz_chi)
							
							value=msfit.get_fit_value(peak,self.settings.data_height,fit,self.settings.only_monoisotopic.get())
							if value>0:
								if self.settings.log_scale.get()==1:
									value=math.log(value,10)
									value_str=('%0.3f' % value)
								else:
									value_str=('%0.1f' % value)
							
							#im=mssputil.get_isomass(peak).split(',')
							#id=mssputil.get_peak_isodistrib(peak)
						else:
							fit_str=('- (%d)' % len(fits))

					self.peptide_tablelist.append(('','',peptide.get_sequence(1,j),peptide.proteins,fit_str,mz_str,delta_mz_str,rt_str,delta_mrt_str,value_str,mz_rmse_str,mz_rsquare_str,mz_flag_str,rt_rmse_str,rt_rsquare_str,rt_flag_str,mz_env_rmse_str,mz_env_rsquare_str,fit_rmse_str,counter),(peptide,j))
					self.peptide_tablelist.tablelist.rowconfigure(counter,background=bgcolor)

					present=query.present
					red_flag=0
					if fit!=None:
						if self.settings.delta_mz_check and abs(delta_mz_ppm)>self.settings.delta_mz_th:
							self.peptide_tablelist.configure_cell(counter,6,foreground='red')
							red_flag=1
						if self.settings.delta_mrt_check and abs(delta_mrt)>self.settings.delta_mrt_th:
							self.peptide_tablelist.configure_cell(counter,8,foreground='red')
							red_flag=1
						if self.settings.mz_rms_check and fit.mz_rmse>self.settings.mz_rms_th:
							self.peptide_tablelist.configure_cell(counter,10,foreground='red')
							red_flag=1
						if self.settings.mz_rsquare_check and fit.mz_rsquare<self.settings.mz_rsquare_th:
							self.peptide_tablelist.configure_cell(counter,11,foreground='red')
							red_flag=1
						if self.settings.mz_flag_check and fit.mz_flag>4:
							self.peptide_tablelist.configure_cell(counter,12,foreground='red')
							red_flag=1
						if self.settings.rt_rms_check and fit.rt_rmse>self.settings.rt_rms_th:
							self.peptide_tablelist.configure_cell(counter,13,foreground='red')
							red_flag=1
						if self.settings.rt_rsquare_check and fit.rt_rsquare<self.settings.rt_rsquare_th:
							self.peptide_tablelist.configure_cell(counter,14,foreground='red')
							red_flag=1
						if self.settings.rt_flag_check and fit.rt_flag>4:
							self.peptide_tablelist.configure_cell(counter,15,foreground='red')
							red_flag=1
						if self.settings.mz_env_rms_check and fit.mz_env_rmse>self.settings.mz_env_rms_th:
							self.peptide_tablelist.configure_cell(counter,16,foreground='red')
							red_flag=1
						if self.settings.mz_env_rsquare_check and fit.mz_env_rsquare<self.settings.mz_env_rsquare_th:
							self.peptide_tablelist.configure_cell(counter,17,foreground='red')
							red_flag=1
						if self.settings.fit_rms_check and fit.fit_rmse>self.settings.fit_rms_th:
							self.peptide_tablelist.configure_cell(counter,18,foreground='red')
							red_flag=1
					else:
						self.peptide_tablelist.tablelist.rowconfigure(counter,foreground='red')

					if present==1:
						self.peptide_tablelist.configure_cell(counter,1,image=self.peptide_tablelist.checkon_icon,text='1',foreground=bgcolor)
						if self.settings.view_all.get()=='NP':
							self.peptide_tablelist.tablelist.rowconfigure(counter,hide=1)
							hidden=1
					else:
						self.peptide_tablelist.configure_cell(counter,1,image=self.peptide_tablelist.checkoff_icon,text='0',foreground=bgcolor)
						self.peptide_tablelist.configure_cell(counter,2,foreground='red')
						if self.settings.nan_unchecked==1:
							self.peptide_tablelist.configure_cell(counter,9,text='NaN',foreground='red')
						if self.settings.flag_unchecked==1:
							red_flag=1
						else:
							red_flag=0
						if self.settings.view_all.get()=='P':
							self.peptide_tablelist.tablelist.rowconfigure(counter,hide=1)
							hidden=1

					if red_flag==1:
						self.peptide_tablelist.configure_cell(counter,0,image=self.peptide_tablelist.flag_icon,text='1')
						if self.settings.view_all.get()=='NM':
							self.peptide_tablelist.tablelist.rowconfigure(counter,hide=1)
							hidden=1
					else:
						if self.settings.view_all.get()=='M':
							self.peptide_tablelist.tablelist.rowconfigure(counter,hide=1)
							hidden=1


					selected=query.get_selected()
					if selected!=None and selected_peak==None:
						selected_peak=selected

					if hidden==0:
						query_counter=query_counter+1
						if red_flag==1:
							red_flag_counter=red_flag_counter+1
						if present==1:
							present_counter=present_counter+1
		
		self.peptide_tablelist.set_list_size(queries=query_counter,red_flags=red_flag_counter,present=present_counter)
#		self.peptide_tablelist.tablelist.configure(height=min(int(self.peptide_tablelist.tablelist.cget('-height')),query_counter))

		if selected_peak==None:
			self.peptide_tablelist.tablelist.yview(row_index)
			self.peptide_tablelist.set_sequence('')
		else:
			self.peptide_tablelist.selected_peak=None
			self.peptide_tablelist.lock_callbacks=0
			self.peptide_tablelist.mark_peak_cb(selected_peak)

	# ---------------------------------------------------------------------------
	#
	def update_query(self,row):
		
		data=self.peptide_tablelist.get_line_data(row)
		peptide=data[0]
		query=peptide.get(data[1])
		peak=query.get(0)
		query.mz=mssputil.get_mz(peak)
		query.rt=mssputil.get_rt(peak)
		query.present=mssputil.is_peak_present(peak)

		fits=msfit.get_fits(peak)

		if self.settings.delta_mz_correction==1 and self.spectrum.rt_precision!=0:
			dmz_correction=mssputil.get_mz_correction_ppm(self.spectrum,query.cmz)
		else:
			dmz_correction=0

		if self.settings.delta_mrt_correction==1:
			drt_correction=mssputil.get_rt_correction(self.spectrum,query.massref_rt)
		else:
			drt_correction=0

		mz_str=('%0.4f' % query.cmz)
		delta_mz_ppm=1e+6*((query.mz-query.cmz)/query.cmz)-dmz_correction
		delta_mz=query.cmz*delta_mz_ppm/1e+6
		if self.settings.delta_mz_ppm==1:
			delta_mz_str=('%0.2f' % delta_mz_ppm)
		else:
			delta_mz_str=('%0.4f' % delta_mz)

		rt_str=('%0.2f' % query.rt)
		delta_mrt=query.rt-(query.massref_rt+drt_correction)
		delta_mrt_str=('%0.2f' % delta_mrt)

		fit_str=('- (0)')
		mz_rmse_str='NaN'
		mz_rsquare_str='NaN'
		mz_flag_str='NaN'
		mz_env_rmse_str='NaN'
		mz_env_rsquare_str='NaN'
		rt_rmse_str='NaN'
		rt_rsquare_str='NaN'
		rt_flag_str='NaN'
		fit_rmse_str='NaN'
		value_str='NaN'

		fit=None
		if len(fits)>0:
			fit_nr=peak.best_fit
			if numpy.abs(fits[fit_nr].rt_pos-peak.frequency[0])<=self.rt_step and numpy.abs(fits[fit_nr].mz_pos-peak.frequency[1])<=self.mz_step:
				fit=fits[fit_nr]

				fit_str=('%d (%d)' % (fit.fit_nr,len(fits)))
	
				mz_rmse_str=('%0.2f' % fit.mz_rmse)
				mz_rsquare_str=('%0.2f' % fit.mz_rsquare)
				mz_flag_str=('%d' % fit.mz_flag)

				mz_env_rmse_str=('%0.2f' % fit.mz_env_rmse)
				mz_env_rsquare_str=('%0.2f' % fit.mz_env_rsquare)

				rt_rmse_str=('%0.2f' % fit.rt_rmse)
				rt_rsquare_str=('%0.2f' % fit.rt_rsquare)
				rt_flag_str=('%d' % fit.rt_flag)

				fit_rmse_str=('%0.2f' % fit.fit_rmse)

				value=msfit.get_fit_value(peak,self.settings.data_height,fit)
				if value>0:
					if self.settings.log_scale.get()==1:
						value=math.log(value,10)
						value_str=('%0.3f' % value)
					else:
						value_str=('%0.1f' % value)
			else:
				fit_str=('- (%d)' % len(fits))

		self.peptide_tablelist.configure_cell(row,4,text=fit_str)
		self.peptide_tablelist.configure_cell(row,5,text=mz_str)
		self.peptide_tablelist.configure_cell(row,6,text=delta_mz_str)
		self.peptide_tablelist.configure_cell(row,7,text=rt_str)
		self.peptide_tablelist.configure_cell(row,8,text=delta_mrt_str)
		self.peptide_tablelist.configure_cell(row,9,text=value_str)
		self.peptide_tablelist.configure_cell(row,10,text=mz_rmse_str)
		self.peptide_tablelist.configure_cell(row,11,text=mz_rsquare_str)
		self.peptide_tablelist.configure_cell(row,12,text=mz_flag_str)
		self.peptide_tablelist.configure_cell(row,13,text=rt_rmse_str)
		self.peptide_tablelist.configure_cell(row,14,text=rt_rsquare_str)
		self.peptide_tablelist.configure_cell(row,15,text=rt_flag_str)
		self.peptide_tablelist.configure_cell(row,16,text=mz_env_rmse_str)
		self.peptide_tablelist.configure_cell(row,17,text=mz_env_rsquare_str)
		self.peptide_tablelist.configure_cell(row,18,text=fit_rmse_str)
		self.peptide_tablelist.tablelist.rowconfigure(row,foreground='black')

		bgcolor=self.peptide_tablelist.tablelist.rowcget(row,'-background')
		was_hidden=self.peptide_tablelist.tablelist.rowcget(row,'-hide')
		hidden=0
		was_red=isinstance(self.peptide_tablelist.get_cell_config(row,0,'-image'),str)
		red_flag=0
		present=query.present

		if fit!=None:
			self.peptide_tablelist.tablelist.rowconfigure(row,foreground='black')

			if self.settings.delta_mz_check and abs(delta_mz_ppm)>self.settings.delta_mz_th:
				self.peptide_tablelist.configure_cell(row,6,foreground='red')
				red_flag=1
			if self.settings.delta_mrt_check and abs(delta_mrt)>self.settings.delta_mrt_th:
				self.peptide_tablelist.configure_cell(row,8,foreground='red')
				red_flag=1
			if self.settings.mz_rms_check and fit.mz_rmse>self.settings.mz_rms_th:
				self.peptide_tablelist.configure_cell(row,10,foreground='red')
				red_flag=1
			if self.settings.mz_rsquare_check and fit.mz_rsquare<self.settings.mz_rsquare_th:
				self.peptide_tablelist.configure_cell(row,11,foreground='red')
				red_flag=1
			else:
				#dlaczego akurat dla tej kolumny trzeba osobno resetowac kolor???
				self.peptide_tablelist.configure_cell(row,11,foreground='black')
			if self.settings.mz_flag_check and fit.mz_flag>4:
				self.peptide_tablelist.configure_cell(row,12,foreground='red')
				red_flag=1
			if self.settings.rt_rms_check and fit.rt_rmse>self.settings.rt_rms_th:
				self.peptide_tablelist.configure_cell(row,13,foreground='red')
				red_flag=1
			if self.settings.rt_rsquare_check and fit.rt_rsquare<self.settings.rt_rsquare_th:
				self.peptide_tablelist.configure_cell(row,14,foreground='red')
				red_flag=1
			if self.settings.rt_flag_check and fit.rt_flag>4:
				self.peptide_tablelist.configure_cell(row,15,foreground='red')
				red_flag=1
			if self.settings.mz_env_rms_check and fit.mz_env_rmse>self.settings.mz_env_rms_th:
				self.peptide_tablelist.configure_cell(row,16,foreground='red')
				red_flag=1
			if self.settings.mz_env_rsquare_check and fit.mz_env_rsquare<self.settings.mz_env_rsquare_th:
				self.peptide_tablelist.configure_cell(row,17,foreground='red')
				red_flag=1
			if self.settings.fit_rms_check and fit.fit_rmse>self.settings.fit_rms_th:
				self.peptide_tablelist.configure_cell(row,18,foreground='red')
				red_flag=1
		else:
			self.peptide_tablelist.tablelist.rowconfigure(row,foreground='red')

		if present==1:
			self.peptide_tablelist.configure_cell(row,1,image=self.peptide_tablelist.checkon_icon,text='1',foreground=bgcolor)
			if self.settings.view_all.get()=='NP':
				hidden=1
		else:
			self.peptide_tablelist.configure_cell(row,1,image=self.peptide_tablelist.checkoff_icon,text='0',foreground=bgcolor)
			self.peptide_tablelist.configure_cell(row,2,foreground='red')
			if self.settings.nan_unchecked==1:
				self.peptide_tablelist.configure_cell(row,9,text='NaN',foreground='red')
			if self.settings.flag_unchecked==1:
				red_flag=1
			else:
				red_flag=0
			if self.settings.view_all.get()=='P':
				hidden=1

		if red_flag==1:
			self.peptide_tablelist.configure_cell(row,0,image=self.peptide_tablelist.flag_icon,text='1')
			if self.settings.view_all.get()=='NM':
				hidden=1
		else:
			self.peptide_tablelist.configure_cell(row,0,image='',text='')
			if self.settings.view_all.get()=='M':
				hidden=1

		self.peptide_tablelist.tablelist.rowconfigure(row,hide=hidden)

		if hidden==0:
			if was_hidden==1:
				self.peptide_tablelist.set_list_size(queries=(self.peptide_tablelist.queries_count+1))
				if query.present:
					self.peptide_tablelist.set_list_size(present=(self.peptide_tablelist.present_count+1))
			if red_flag==1 and was_red==0:
				self.peptide_tablelist.set_list_size(red_flags=(self.peptide_tablelist.red_flag_count+1))
			if red_flag==0 and was_red==1:
				self.peptide_tablelist.set_list_size(red_flags=(self.peptide_tablelist.red_flag_count-1))

		if hidden==1 and was_hidden==0:
			self.peptide_tablelist.set_list_size(queries=(self.peptide_tablelist.queries_count-1))
			if was_red==1:
				self.peptide_tablelist.set_list_size(red_flags=(self.peptide_tablelist.red_flag_count-1))
			if query.present:
				self.peptide_tablelist.set_list_size(present=(self.peptide_tablelist.present_count-1))

	# --------------------------------------------------------------------------
	#
	def set_value_column(self):
		name='Fit height'
			
		if self.settings.data_height==msfit.VALUE_DATA_HEIGHT:
			name='Data height'
		elif self.settings.data_height==msfit.VALUE_FIT_VOLUME or self.settings.data_height==msfit.VALUE_SFIT_VOLUME:
			name='Volume'

		self.peptide_tablelist.tablelist.columnconfigure(9,title=name)

	# ---------------------------------------------------------------------------
	#
	def view_list_cb(self,event=None):
		if self.settings.view_list_only.get()==1:
			if self.settings.view_list==None or len(self.settings.view_list)==0:
				dialog=mstkutil.warning_dialog(self.top,'The list is empty')
				self.settings.view_list_only.set(0)
				return
		self.redraw_cb()

	# --------------------------------------------------------------------------
	#
	def next_fit_cb(self,event=None):

		data=self.peptide_tablelist.event_line_data(event)
		if data!=None:
			peptide=data[0]
			query=peptide.get(data[1])
			peak=query.get(0)
			fits=msfit.get_fits(peak)
			if len(fits)>0:
				fit_nr_str=self.peptide_tablelist.get_cell_config(self.peptide_tablelist.selected_line_index(),4,'-text')
				num_str=fit_nr_str.split(' (')[0]
				if num_str=='-':
					fit_nr=0
				else:
					fit_nr=min(int(num_str),len(fits)-1)

				fit=fits[fit_nr]
				peak.best_fit=fit_nr
				query.set_position(fit.rt_pos,fit.mz_pos)
				mssputil.show_peak(peak)

	# --------------------------------------------------------------------------
	#
	def prev_fit_cb(self,event=None):
		data=self.peptide_tablelist.event_line_data(event)
		if data!=None:
			peptide=data[0]
			query=peptide.get(data[1])
			peak=query.get(0)
			fits=msfit.get_fits(peak)
			if len(fits)>0:
				fit_nr_str=self.peptide_tablelist.get_cell_config(self.peptide_tablelist.selected_line_index(),4,'-text')
				num_str=fit_nr_str.split(' (')[0]
				if num_str=='-':
					fit_nr=0
				else:
					fit_nr=max(int(num_str)-2,0)
				fit=fits[fit_nr]
				peak.best_fit=fit_nr
				query.set_position(fit.rt_pos,fit.mz_pos)
				mssputil.show_peak(peak)

	# --------------------------------------------------------------------------
	#
	def best_fit_cb(self,event=None):

		data=self.peptide_tablelist.event_line_data(event)
		if data!=None:
			peptide=data[0]
			query=peptide.get(data[1])
			peak=query.get(0)
			fits=msfit.get_fits(peak)
			if len(fits)>0:
				fit=fits[0]
				peak.best_fit=0
				query.set_position(fit.rt_pos,fit.mz_pos)
				mssputil.show_peak(peak)

	# --------------------------------------------------------------------------
	#
	def reset_fit_cb(self,event=None):
		data=self.peptide_tablelist.event_line_data(event)
		if data!=None:
			peptide=data[0]
			query=peptide.get(data[1])
			peak=query.get(0)
			
			rt_pos=-1*query.massref_rt
			if self.spectrum.domain==mssputil.DOMAIN_FREQ:
				mz_pos=mssputil.mz2freq_s(query.cmz,self.spectrum)
			else:
				mz_pos=-1*query.cmz
			peak.set_position(rt_pos,mz_pos)
			mssputil.show_peak(peak)

	# --------------------------------------------------------------------------
	#
	def delete_fit_cb(self,event=None):

		data=self.peptide_tablelist.event_line_data(event)
		if data!=None:
			peptide=data[0]
			query=peptide.get(data[1])
			peak=query.get(0)
			fits=msfit.get_fits(peak)
			if len(fits)>0:
				fit_nr_str=self.peptide_tablelist.get_cell_config(self.peptide_tablelist.selected_line_index(),4,'-text')
				num_str=fit_nr_str.split(' (')[0]
				if num_str!='-':
					fit_nr=max(int(num_str)-1,0)

					if len(fits)>1:
						if fit_nr>0:
							new_fit_nr=fit_nr-1
						else:
							new_fit_nr=fit_nr+1
						for i in range(new_fit_nr,len(fits)):
							fits[i].fit_nr=fits[i].fit_nr-1
							
						new_fit=fits[new_fit_nr]
						peak.best_fit=new_fit_nr

						rt_pos=new_fit.rt_pos
						mz_pos=new_fit.mz_pos
					else:
						rt_pos=-1*query.massref_rt
						if self.spectrum.domain==mssputil.DOMAIN_FREQ:
							mz_pos=mssputil.mz2freq_s(query.cmz,self.spectrum)
						else:
							mz_pos=-1*query.cmz
						query.set_present(0)

					fits.remove(fits[fit_nr])
					msfit.set_fits(peak,fits)

					peak.set_position(rt_pos,mz_pos)
					mssputil.show_peak(peak)

	# --------------------------------------------------------------------------
	#
	def create_fit_cb(self,event=None):

		data=self.peptide_tablelist.event_line_data(event)
		if data!=None:
			peptide=data[0]
			query=peptide.get(data[1])
			peak=query.get(0)
			fits=msfit.get_fits(peak)

			#paramtry => musza byc odczytywane z konfiguracji
			peaks_count=3
			fit_monoisotopic_rt=False
			spectrum_mz_correction=1
			rt_margin=(1.5,2.5)
			scans_margin=0
	
			#pobranie informacji o piku
			sequence=mssputil.get_sequence(peak)
			isodistrib=msenvelope.MSIsoDistrib(peak,0.05)
			charge=float(mssputil.get_charge(peak))
		
			isopeaks_mz=isodistrib.get_peaks_mz(charge)	#pobranie m/z najwyzszego piku (do wyznaczenia korekcji m/z i RP)
			mz=isopeaks_mz[numpy.argmax(isodistrib.probs)]

			rt=mssputil.get_rt(peak)

			#wyznaczenie poprawki skali m/z, rozdzieleczosci i toretycznej szerokosci polowkowej
			mz_correction=0
			if spectrum_mz_correction==1:
				mz_correction=mssputil.get_mz_correction(peak.spectrum,mz)
				rp=mssputil.get_rp_at_mz(peak.spectrum,mz)
				#mz_margin powinien byc parametrem
				mz_margin=(mssputil.da2ppm(mz/rp,mz)/2,mssputil.da2ppm(mz/rp,mz)/2)
			
			(fit,mz_points,mz_data,mz_fit_data,mz_envelope_data,mz_e,mz_max_range,mz_max_intensity,mz_peaks,mz_noise,rt_points,rt_data,rt_fit_data,rt_fit_range,rt_max,rt_peaks,rt_noise,fit_success,fit_msg)=msfit.create_fit(self.spectrum,1,(-1*(rt-rt_margin[0]),-1*(rt+rt_margin[1]),-1*rt),isodistrib,float(charge),peaks_count,rp,mz_correction,mz_margin,fit_monoisotopic_rt,True,scans_margin,True)
			print fit
			
			peak_fits=[]
			peak_fits.append(fit)
			msfit.set_fits(peak,peak_fits)
			peak.best_fit=0
			
			#query.set_present(1)
			query.set_position(peak_fits[peak.best_fit].rt_pos,peak_fits[peak.best_fit].mz_pos)
			query.mz=mssputil.get_mz(peak)
			query.rt=mssputil.get_rt(peak)
			
	# -----------------------------------------------------------------------------
	#
	def check_query(self,row,check,external_call=0):

		#data=self.peptide_tablelist.line_data[row]
		data=self.peptide_tablelist.get_line_data(row)
		query=data[0].get(data[1])
		peak=query.get(0)
		fits=msfit.get_fits(peak)

		bgcolor=self.peptide_tablelist.tablelist.rowcget(row,'-background')

		if len(fits)>0:
			fit_nr_str=self.peptide_tablelist.get_cell_config(row,4,'-text')
			num_str=fit_nr_str.split(' (')[0]
			if num_str!='-':
				if (query.present==1 and check==0) or (query.present==0 and check==1):
					query.set_present(check)
					self.update_query(row)
	
					if external_call==0 and self.find_duplicates_dialog!=None:
						line_index=self.find_duplicates_dialog.duplicate_tablelist.find_peak_cb(query.get(0))
						if line_index!=-1:
							self.find_duplicates_dialog.check_peak(line_index,check,1)

	# ---------------------------------------------------------------------------
	#
	def check_queries(self):
		line_counter=-1
		self.stoppable_loop('peptides',50)
		for i in range(len(self.peptide_list)):
			self.check_for_stop()
			peptide=self.peptide_list[i]

			for j in range(len(peptide.query_list)):
				line_counter=line_counter+1
				if self.settings.check==-1:
					if isinstance(self.peptide_tablelist.get_cell_config(line_counter,0,'-image'),str):
						self.check_query(line_counter,0)

				elif self.settings.check==-2:
					if peptide.sequence.find('+')!=-1:
						self.check_query(line_counter,0)
				else:
					self.check_query(line_counter,self.settings.check)

	# ---------------------------------------------------------------------------
	#
	def check_marked(self):
		self.lock_gui(1)
		self.settings.check=-1
		self.stoppable_call(self.check_queries)
		self.peptide_tablelist.set_list_size(present=self.peptide_tablelist.present_count)
		self.lock_gui(0)

	# ---------------------------------------------------------------------------
	#
	def check_modified(self):
		self.lock_gui(1)
		self.settings.check=-2
		self.stoppable_call(self.check_queries)
		self.peptide_tablelist.set_list_size(present=self.peptide_tablelist.present_count)
		self.lock_gui(0)

	# ---------------------------------------------------------------------------
	#
	def check_all(self):
		self.lock_gui(1)
		self.settings.check=1
		self.stoppable_call(self.check_queries)
		self.peptide_tablelist.set_list_size(present=self.peptide_tablelist.present_count)
		self.lock_gui(0)

	# ---------------------------------------------------------------------------
	#
	def check_none(self):
		self.lock_gui(1)
		self.settings.check=0
		self.stoppable_call(self.check_queries)
		self.peptide_tablelist.set_list_size(present=self.peptide_tablelist.present_count)
		self.lock_gui(0)

	# ---------------------------------------------------------------------------
	#
	def find_string(self,event=None):
		str=self.find_edit.variable.get()
		if str!='':
			self.lock_gui(1)
			if self.direction_menu.variable.get()=='Search down':
				self.peptide_tablelist.find_string(str,exact_match=self.match_check.state(),y_range=(2,3))
			else:
				self.peptide_tablelist.find_string_up(str,exact_match=self.match_check.state(),y_range=(2,3))
			self.lock_gui(0)

	# ---------------------------------------------------------------------------
	#
	def show_fit_cb(self,event=None):
		data=self.peptide_tablelist.event_line_data(event)
		if data!=None:
			peptide=data[0]
			query_nr=data[1]
			msisoenvelope.show_peak_fit(peptide.get(query_nr).get(0),False)

	# ---------------------------------------------------------------------------
	#
	def show_isodistrib_cb(self,event=None):
		data=self.peptide_tablelist.event_line_data(event)
		if data!=None:
			peptide=data[0]
			query_nr=data[1]
			msisoenvelope.show_peak_fit(peptide.get(query_nr).get(0),True)

	# ---------------------------------------------------------------------------
	#
	def show_chromatogram_cb(self,event=None):
		data=self.peptide_tablelist.event_line_data(event)
		if data!=None:
			peptide=data[0]
			query_nr=data[1]
			mschromatogram_gui.show_peak_chromatogram(peptide.get(query_nr).get(0))

	# ---------------------------------------------------------------------------
	#
	def find_peptide_cb(self,event=None):
		data=self.peptide_tablelist.event_line_data(event)
		if data!=None:
			peptide=data[0]
			query_nr=data[1]
			msfindpeptides_gui.show_find_peptide_dialog(peptide.get(query_nr).get(0))

	# ---------------------------------------------------------------------------
	#
	def duplicated_cb(self,event=None):
		dialog=msduplicated.show_duplicated_list_dialog(self)
		if dialog!=None:
			self.find_duplicates_dialog=dialog

	# plots
	#
	# ---------------------------------------------------------------------------
	#
	def values_hist_cb(self,event=None):

		if self.peptide_tablelist.present_count>0:
			values=[]
			for i in range(len(self.peptide_list)):
				peptide=self.peptide_list[i]
				for j in range(len(peptide.query_list)):
					query=peptide.get(j)
					if query.present==1:
						peak=query.get(0)
						fits=msfit.get_fits(peak)
						fit=fits[peak.best_fit]
						value=msfit.get_fit_value(peak,self.settings.data_height,fit)
						if value>0:
							if self.settings.log_scale.get()==1:
								value=math.log(value,10)
							values.append(value)

			if values!=None and len(values)>0:
				name='Fit height'
				if self.settings.data_height==msfit.VALUE_DATA_HEIGHT:
					name='Data height'
				elif self.settings.data_height==msfit.VALUE_FIT_VOLUME:
					name='Volume'
				elif self.settings.data_height==msfit.VALUE_SFIT_HEIGHT:
					name='Sparky fit height'
				elif self.settings.data_height==msfit.VALUE_SFIT_VOLUME:
					name='Sparky fit volume'

				fig=mscalib.histogram_plot(name,"Counts",values)
				d=mstkutil.PlotDialog(self.session,fig,(name + ' histogram: ' + self.spectrum.name))
				d.show_window(1)

	# ---------------------------------------------------------------------------
	#
	def mass_error_hist_cb(self,event=None):

		if self.peptide_tablelist.present_count>0:
			mz,delta_mz=mscalib.get_mz_delta(self.peptide_list,self.settings.delta_mz_correction,self.spectrum)
			
			if delta_mz!=None and len(delta_mz)>0:
				delta_mz=numpy.array(delta_mz)

				fig=mscalib.histogram_plot("m/z error [ppm]","Counts",delta_mz)
				d=mstkutil.PlotDialog(self.session,fig,('Mass error histogram: ' + self.spectrum.name))
				d.show_window(1)

	# ---------------------------------------------------------------------------
	#
	def mass_error_mz_cb(self,event=None):

		if self.peptide_tablelist.present_count>0:
			mz,delta_mz=mscalib.get_mz_delta(self.peptide_list,self.settings.delta_mz_correction,self.spectrum)

			if delta_mz!=None and len(delta_mz)>0:
				delta_mz=numpy.array(delta_mz)
				mz=numpy.array(mz)

				#trzeba przekazywac parametry z widma
				mz,delta_mz,mz_a,mz_b,mz_sf,mz_sa,mz_precision,mz_inliers_index,mz_ransac=mscalib.calibrate(self.session,mz,delta_mz,mscalib.CALIB_RLS,mscalib.ESTIM_SD,3)

				fig=mscalib.calibration_plot("m/z","Mass error [ppm]",mz,delta_mz,mz_a,mz_b,mz_sf,mz_sa,mz_precision,mz_inliers_index,mz_ransac)
				d=mstkutil.PlotDialog(self.session,fig,('Mass error vs. m/z: ' + self.spectrum.name))
				d.show_window(1)
				
	# ---------------------------------------------------------------------------
	#
	def mass_error_rt_cb(self,event=None):

		if self.peptide_tablelist.present_count>0:
			mz,delta_mz=mscalib.get_mz_delta(self.peptide_list,self.settings.delta_mz_correction,self.spectrum)
			rt,delta_rt=mscalib.get_rt_delta(self.peptide_list,self.settings.delta_mrt_correction,self.spectrum)
						
			if delta_mz!=None and rt!=None and len(delta_mz)>0 and len(delta_mz)==len(rt):
				delta_mz=numpy.array(delta_mz)
				rt=numpy.array(rt)
	
				rt,delta_mz,a,b,sf,sa,precision,inliers_index,ransac=mscalib.calibrate(self.session,rt,delta_mz,mscalib.CALIB_LS,mscalib.ESTIM_SD,3)
				
				fig=mscalib.calibration_plot("RT [min]","Mass error [ppm]",rt,delta_mz,a,b,sf,sa,precision,inliers_index,ransac)
				d=mstkutil.PlotDialog(self.session,fig,('Mass error vs. RT: ' + self.spectrum.name))
				d.show_window(1)

	# ---------------------------------------------------------------------------
	#
	def rt_error_hist_cb(self,event=None):

		if self.peptide_tablelist.present_count>0:
			rt,delta_rt=mscalib.get_rt_delta(self.peptide_list,self.settings.delta_mrt_correction,self.spectrum)

			if delta_rt!=None and len(delta_rt)>0:
				delta_rt=numpy.array(delta_rt)

				fig=mscalib.histogram_plot("RT error [min]","Counts",delta_rt)
				d=mstkutil.PlotDialog(self.session,fig,('RT error histogram: ' + self.spectrum.name))
				d.show_window(1)

	# ---------------------------------------------------------------------------
	#
	def rt_error_rt_cb(self,event=None):

		if self.peptide_tablelist.present_count>0:
			rt,delta_rt=mscalib.get_rt_delta(self.peptide_list,self.settings.delta_mrt_correction,self.spectrum)

			if delta_rt!=None and len(delta_rt)>0:
				rt=numpy.array(rt)
				delta_rt=numpy.array(delta_rt)
				
				rt,delta_rt,rt_a,rt_b,rt_sf,rt_sa,rt_precision,rt_inliers_index,rt_ransac=mscalib.calibrate(self.session,rt,delta_rt,mscalib.CALIB_LOWESS,mscalib.ESTIM_SD,4)
				
				fig=mscalib.calibration_plot("RT [min]","RT error [min]",rt,delta_rt,rt_a,rt_b,rt_sf,rt_sa,rt_precision,rt_inliers_index,rt_ransac)
				d=mstkutil.PlotDialog(self.session,fig,('RT error vs. RT: ' + self.spectrum.name))
				d.show_window(1)

	# ---------------------------------------------------------------------------
	#
	def rp_mz_cb(self,event=None):

		if self.peptide_tablelist.present_count>0:
			mz,rp=mscalib.get_rp_delta(self.peptide_list)
			if rp!=None and len(rp)>0:
				mz=numpy.array(mz)
				rp=numpy.array(rp)

				mz,rp,a,b,sf,sa,precision,inliers_index,ransac=mscalib.calibrate(self.session,mz,rp,mscalib.CALIB_RLS,mscalib.ESTIM_MAD,3)

				fig=mscalib.calibration_plot("m/z","Resolving power",mz,rp,a,b,sf,sa,precision,inliers_index,ransac)
				d=mstkutil.PlotDialog(self.session,fig,('Resolving power vs. m/z: ' + self.spectrum.name))
				d.show_window(1)

	# ---------------------------------------------------------------------------
	#
	def lock_gui(self,lock):

		if lock==1:
			state='disabled'
			edit='no'
			if sys.platform=='win32':
				cursor='wait'
			else:
				cursor='watch'
		else:
			state='normal'
			edit='yes'
			cursor=''

		self.settings.lock_callbacks=lock
		self.peptide_tablelist.lock_callbacks=lock

		self.find_button['state']=state
		self.find_edit.entry['state']=state
		self.buttons_row.buttons[0]['state']=state
		self.buttons_row.buttons[2]['state']=state
		self.buttons_row.buttons[3]['state']=state
		self.buttons_row.buttons[4]['state']=state
		self.top.config(cursor=cursor)
		self.top.update_idletasks()


# -----------------------------------------------------------------------------
#
def show_peptides_list(session):
	spectrum=session.selected_spectrum()
	if spectrum==None:
		return

	if not hasattr(session,'spectrum_dialogs'):
		session.spectrum_dialogs={}
	dialogs=session.spectrum_dialogs

	key='PL2'+spectrum.name
	if (dialogs.has_key(key) and not dialogs[key].is_window_destroyed()):
		dialogs[key].show_window(1)
	else:
		spectrum_settings=mspeptideslist_settings.get_peptideslist_settings(spectrum)
		d=peptideslist_dialog(session,spectrum_settings)
		d.show_window(1)
		d.show_spectrum_peptides(spectrum)
		dialogs[key]=d
