import math
import sys
import time

import Tkinter
import tkFont

import sparky
import ms
import mssputil
import mstkutil
import odict


# -----------------------------------------------------------------------------
#
class proteins_list_dialog(mstkutil.Settings_Dialog,mstkutil.Stoppable):

	def __init__(self,session):

		self.session=session
		self.title='Proteins list: '
		self.settings=proteins_list_settings()

		mstkutil.Settings_Dialog.__init__(self,session.tk,self.title)

		f=Tkinter.Frame(self.top,borderwidth=2,relief='groove')
		f.pack(side='top',anchor='w',fill='x')

		pl=mssputil.protein_tablelist(f,columns=None,selected_edit=0)
		pl.set_columns((20,'ID',5,'Peptides',5,'Mean ratio',5,'Ratio SD',5,'p-value'))
		pl.set_columns_align(('left','right','right','right','right'))

		pl.tablelist.bind('<ButtonRelease-1>',self.select_protein_cb)
		pl.tablelist.bind('<Return>',self.select_protein_cb)
		pl.tablelist.bind('<Button-3>',self.protein_popup_menu)
		
		pl.frame.pack(side='top',fill='both',expand=1)		
		self.protein_tablelist=pl

		pl=mssputil.peaks_tablelist(f,columns=None,selected_edit=0)
		pl.tablelist.configure(height=10,width=60)
		pl.set_columns((20,'Sequence',8,'Ratio'))
		pl.set_columns_align(('left','right'))

		pl.tablelist.bind('<ButtonRelease-1>',pl.select_peaks_cb)
		pl.tablelist.bind('<Double-ButtonRelease-1>',pl.goto_peaks_cb)
		pl.tablelist.bind('<Return>',pl.goto_peaks_cb)
		pl.tablelist.bind('<Button-3>',self.peptide_popup_menu)

		pl.frame.pack(side='top',fill='both',expand=1)		
		self.peptide_tablelist=pl

		progress_label=Tkinter.Label(self.top, anchor = 'nw')
		progress_label.pack(side='top',anchor='w')

		br=mstkutil.button_row(self.top,
					('Update', self.update_cb),
					('Stop', self.stop_cb),
					('Save proteins', self.protein_tablelist.save_cb),
					('Save queries', self.peptide_tablelist.save_cb),
					('Close', self.top.destroy),
					)
		br.frame.pack(side='top',anchor='w',pady=2)
		self.buttons_row=br

		self.top.geometry("+0+0")
		self.top.resizable(1,0)

#		self.notice=session.notify_me('removed spectrum from project',self.spectrum_removed_cb)
		self.selection_notice=session.notify_me('selection changed',self.selection_changed_cb)
		self.top.bind('<Destroy>',self.remove_callbacks,1)
		self.top.protocol("WM_DELETE_WINDOW",self.closing_window)

		mstkutil.Stoppable.__init__(self,progress_label,br.buttons[1])

	# ---------------------------------------------------------------------------
	#
	def protein_popup_menu(self,event):
		if self.settings.lock_callbacks==0:
			menu=Tkinter.Menu(self.protein_tablelist.tablelist,tearoff=0)

			menu.add_command(label='Mark protein',command=self.mark_protein_cb)
			menu.add_separator()
			menu.add_command(label='SwissProt',command=self.show_swissprot_cb)
			menu.add_command(label='NCBI',command=self.show_ncbi_cb)
			menu.add_command(label='SGD',command=self.show_sgd_cb)
			menu.add_separator()
			menu.add_command(label='Compute scores',command=self.get_scores_cb,state='disabled')
			menu.tk_popup(event.x_root,event.y_root)

	# ---------------------------------------------------------------------------
	#
	def peptide_popup_menu(self,event):
		if self.settings.lock_callbacks==0:
			menu=Tkinter.Menu(self.peptide_tablelist.tablelist,tearoff=0)

			menu.add_command(label='Mark sequence',command=self.mark_sequence_cb)
			menu.tk_popup(event.x_root,event.y_root)

	# ---------------------------------------------------------------------------
	#
	def show_swissprot_cb(self,event=None):
		protein=self.protein_tablelist.event_line_data(event)
		if protein:
			str='us.expasy.org/uniprot/' + protein.id
			self.session.show_url(str)

	# ---------------------------------------------------------------------------
	#
	def show_ncbi_cb(self,event=None):
		protein=self.protein_tablelist.event_line_data(event)
		if protein:
			index=protein.id.find('gi|')
			if index!=-1:
				str='www.ncbi.nlm.nih.gov/entrez/viewer.fcgi?db=protein&val=' + protein.id[(index+3):]
				self.session.show_url(str)

	# ---------------------------------------------------------------------------
	#
	def show_sgd_cb(self,event=None):
		protein=self.protein_tablelist.event_line_data(event)
		if protein:
			str='db.yeastgenome.org/cgi-bin/locus.pl?locus=' + protein.id
			self.session.show_url(str)

	# ---------------------------------------------------------------------------
	#
	def mark_protein_cb(self,event=None):
		protein=self.protein_tablelist.event_line_data(event)
		if protein:
			self.parent_dialog.mark_protein_edit.variable.set(protein.id)
			self.parent_dialog.update_plot_cb()

	# ---------------------------------------------------------------------------
	#
	def mark_sequence_cb(self,event=None):
		data=self.peptide_tablelist.event_line_data(event)
		if data!=None:
			self.parent_dialog.mark_sequence_edit.variable.set(mssputil.get_sequence(data[0]))
			self.parent_dialog.update_plot_cb()

	# ---------------------------------------------------------------------------
	#
	def get_scores_cb(self,event=None):
		pass

	# ---------------------------------------------------------------------------
	#
#	def spectrum_removed_cb(self,removed_spectrum):
#		if self.spectrum==removed_spectrum:
#			self.top.destroy()

	# ---------------------------------------------------------------------------
	#
	def selection_changed_cb(self):
		peaks=self.session.selected_peaks()
#		if peaks!=None and len(peaks)==1:
#			self.protein_tablelist.mark_peak_cb(peaks[0])

	# ---------------------------------------------------------------------------
	#
	def closing_window(self):
		if self.settings.lock_callbacks==0:
			self.top.destroy()
				
	# ---------------------------------------------------------------------------
	#
	def remove_callbacks(self,event):
		if str(self.top)==str(event.widget):
			if sparky.object_exists(self.session):
				if self.selection_notice!=None:
					self.session.dont_notify_me(self.selection_notice)
				
			self.selection_notice=None
			self.parent_dialog.settings.proteins_list=None

	# ---------------------------------------------------------------------------
	#
	def select_protein_cb(self,event=None):
		if self.settings.lock_callbacks!=1:
			self.lock_gui(1)
			index=self.protein_tablelist.selected_line_index()
	
			self.settings.selected_protein=index
			self.stoppable_call(self.update_peptides_tablelist)
			self.protein_tablelist.tablelist.selection_own()
			self.lock_gui(0)

	# ---------------------------------------------------------------------------
	#
	def show_settings(self,settings):
		self.update_cb()

	# ---------------------------------------------------------------------------
	#
	def update_cb(self,event=None):
		self.lock_gui(1)
	
		#self.settings.view_present_only.set(self.parent_dialog.settings.view_present_only.get())
		#self.settings.list_log.set(self.parent_dialog.settings.list_log.get())
		self.settings.view_present_only.set(1)
		self.settings.list_log.set(1)
		
		self.stoppable_call(self.get_sample)
		self.stoppable_call(self.get_ratios)
		self.stoppable_call(self.update_proteins_tablelist)
		self.stoppable_call(self.update_peptides_tablelist)
		self.lock_gui(0)

		self.protein_tablelist.tablelist.focus_set()

	# ---------------------------------------------------------------------------
	#
	def redraw_cb(self,event=None):
		self.lock_gui(1)
		self.stoppable_call(self.update_proteins_tablelist)
		self.stoppable_call(self.update_peptides_tablelist)
		self.lock_gui(0)

		self.protein_tablelist.tablelist.focus_set()

	# -----------------------------------------------------------------------------
	#
	def get_sample(self):
	
		self.progress_report('Getting peptides')

		self.sample=ms.sample(self.parent_dialog.spectrum1.name)
		self.sample.spectrum=self.parent_dialog.spectrum1.name
		self.top.title(self.title + self.parent_dialog.spectrum2.name + ' vs ' + self.parent_dialog.spectrum1.name)

		counter=-1

		pep_dict=odict.OrderedDict()

		self.stoppable_loop('peaks',200)
		for i in range(len(self.parent_dialog.peaks1)):
			peak=self.parent_dialog.peaks1[i]
			self.check_for_stop()
			counter=counter+1
			if mssputil.is_peak_unassigned(peak)!=1:
				if self.settings.view_present_only.get()==0 or (mssputil.is_peak_present(self.parent_dialog.peaks1[i]) and mssputil.is_peak_present(self.parent_dialog.peaks2[i])):
					query=ms.query()
					query.charge=mssputil.get_charge(peak)
					query.score=mssputil.get_score(peak)
					if self.parent_dialog.x_vals[counter]!=0 and self.parent_dialog.y_vals[counter]!=0:
#						query.value=pow(10,self.parent_dialog.y_vals[counter]-self.parent_dialog.x_vals[counter])
						query.value=self.parent_dialog.y_vals[counter]-self.parent_dialog.x_vals[counter]
					query.add(peak)
			
					seq=mssputil.get_sequence(peak)
					peptide=ms.peptide(seq)
					peptide.proteins=mssputil.get_protein(peak)
					peptide.add(query)
	
					if pep_dict.has_key(seq):
						for query in peptide.query_list:
							pep_dict[seq].add(query)
					else:
						pep_dict[seq]=peptide
				else:
					query.add(peak)

		peptide_list=pep_dict.values()
		
		self.progress_report('Getting peptides')
		prot_dict=odict.OrderedDict()

		self.stoppable_loop('peptides',50)
		for i in range(len(peptide_list)):
			peptide=peptide_list[i]
			proteins_id=peptide.proteins.split(',')
			for id in proteins_id:
				if prot_dict.has_key(id):
					protein=prot_dict[id]
				else:
					protein=ms.protein(id)
					prot_dict[id]=protein

				protein.add(peptide)

		self.sample.protein_list=prot_dict.values()

	# -----------------------------------------------------------------------------
	#
	def get_ratios(self):
		self.progress_report('Getting ratios')
		counter=-1

		self.stoppable_loop('proteins',10)
		for i in range(self.sample.size()):
			self.check_for_stop()
			protein=self.sample.get(i)

			values=[]
			counter=0
			for j in range(protein.size()):
				self.check_for_stop()
				peptide=protein.get(j)
				for k in range(peptide.size()):
					query=peptide.get(k)
					if query.value!=0:
						counter=counter+1
						protein.value=protein.value+query.value
						values.append(query.value)
			if counter>0:
				protein.value=protein.value/counter
				protein.value_sd=0
				if counter>1:
					for i in range(len(values)):
						protein.value_sd=protein.value_sd+math.pow(values[i]-protein.value,2)
					protein.value_sd=math.pow(10,protein.value_sd/counter)

				protein.value=math.pow(10,protein.value/counter)
			else:
				protein.value=0
				protein.value_sd=-1

	# ---------------------------------------------------------------------------
	#
	def update_proteins_tablelist(self):
		self.progress_report('Updating proteins table')

		low_th=1/self.parent_dialog.settings.ratio_th
		high_th=self.parent_dialog.settings.ratio_th

#		low_th=math.log(1/self.parent_dialog.settings.ratio_th,10)
#		high_th=math.log(self.parent_dialog.settings.ratio_th,10)

		selected_protein=self.protein_tablelist.event_line_data(None)
		row_index=int(round(self.protein_tablelist.tablelist.yview()[0]*self.protein_tablelist.tablelist.size()))
		self.protein_tablelist.clear()
		counter=-1

		self.stoppable_loop('proteins',10)
		for i in range(self.sample.size()):
			self.check_for_stop()
			protein=self.sample.get(i)
			counter=counter+1
			
			ratio_str='NaN'
			if protein.value!=0:
				ratio_str=('%0.3f' % protein.value)
			ratio_sd_str='NaN'
			if protein.value_sd!=-1:
				ratio_sd_str=('%0.3f' % protein.value_sd)

			score_str='NaN'

			self.protein_tablelist.append((protein.id,('%d' % protein.size()),ratio_str,ratio_sd_str,score_str),protein)

			if protein.value!=0:
				if protein.value>=high_th:
					self.protein_tablelist.configure_cell(counter,2,foreground='red')
				elif protein.value<=low_th:
					self.protein_tablelist.configure_cell(counter,2,foreground='#008201')
			else:
				self.protein_tablelist.configure_cell(counter,2,foreground='gray55')

		self.protein_tablelist.list_size.configure(text=('%d proteins' % self.sample.size()))

		self.protein_tablelist.lock_callbacks=0
		if not selected_protein:
			self.protein_tablelist.tablelist.yview(row_index)
		else:
			self.protein_tablelist.mark_protein_cb(selected_protein)

	# ---------------------------------------------------------------------------
	#
	def update_peptides_tablelist(self):
		self.progress_report('Updating queries table')

		low_th=1/self.parent_dialog.settings.ratio_th
		high_th=self.parent_dialog.settings.ratio_th

#		low_th=math.log(1/self.parent_dialog.settings.ratio_th,10)
#		high_th=math.log(self.parent_dialog.settings.ratio_th,10)

		selected_peak=self.peptide_tablelist.event_line_data(None)
		if selected_peak!=None:
			selected_peak=selected_peak[0]
		row_index=int(round(self.peptide_tablelist.tablelist.yview()[0]*self.peptide_tablelist.tablelist.size()))
		self.peptide_tablelist.clear()
		counter=-1

		self.stoppable_loop('queries',10)
		protein=self.sample.get(self.settings.selected_protein)
		for i in range(protein.size()):
			self.check_for_stop()
			peptide=protein.get(i)

			if peptide.size()>1:
				bgcolor='#d4deec'
			else:
				bgcolor='gray98'

			for j in range(peptide.size()):
				counter=counter+1
				query=peptide.get(j)

				sequence=peptide.sequence + (' (+%d)' % query.charge)
				ratio=math.pow(10,query.value)
				if query.value!=0:
					ratio_str=('%0.3f' % ratio)
				else:
					ratio_str='NaN'

				self.peptide_tablelist.append((sequence,ratio_str),(query.get(0),None))
				self.peptide_tablelist.tablelist.rowconfigure(counter,background=bgcolor)

				if ratio!=0:
					if ratio>=high_th:
						self.peptide_tablelist.configure_cell(counter,1,foreground='red')
					elif ratio<=low_th:
						self.peptide_tablelist.configure_cell(counter,1,foreground='#008201')
				else:
					self.peptide_tablelist.configure_cell(counter,1,foreground='gray55')

		self.peptide_tablelist.list_size.configure(text=('%d peptides (%d queries)' % (protein.size(),counter+1)))

		self.peptide_tablelist.lock_callbacks=0
		if not selected_peak:
			self.peptide_tablelist.tablelist.yview(row_index)
		else:
			self.peptide_tablelist.mark_peak_cb(selected_peak)

	# ---------------------------------------------------------------------------
	#
	def find_string(self,event=None):
		str=self.find_edit.variable.get()
		if str!='':
			self.lock_gui(1)
			if self.direction_menu.variable.get()=='Search down':
				self.protein_tablelist.find_string(str,exact_match=self.match_check.state(),y_range=(2,3))
			else:
				self.protein_tablelist.find_string_up(str,exact_match=self.match_check.state(),y_range=(2,3))
			self.lock_gui(0)


	# ---------------------------------------------------------------------------
	#
	def lock_gui(self,lock):

		if lock==1:
			state='disabled'
			edit='no'
			if sys.platform=='win32':
				cursor='wait'
			else:
				cursor='watch'
		else:
			state='normal'
			edit='yes'
			cursor=''

		self.settings.lock_callbacks=lock
		self.protein_tablelist.lock_callbacks=lock
		self.peptide_tablelist.lock_callbacks=lock

#		self.find_button['state']=state
#		self.find_edit.entry['state']=state
		self.buttons_row.buttons[0]['state']=state
		self.buttons_row.buttons[2]['state']=state
		self.buttons_row.buttons[3]['state']=state
		self.buttons_row.buttons[4]['state']=state
		self.top.config(cursor=cursor)
		self.top.update_idletasks()

# -----------------------------------------------------------------------------
#
class proteins_list_settings:

	def __init__(self):
		self.lock_callbacks=0

		self.view_present_only=Tkinter.IntVar()
		self.view_present_only.set(0)
		
		self.list_log=Tkinter.IntVar()
		self.list_log.set(1)

		self.selected_protein=0
