import Tkinter
import tkFont

import Pmw

import sparky
import msfit
import msnorm
import mssputil
import mstkutil

VIEW_ALL=0
VIEW_PRESENT=1
VIEW_PRESENT_SINGLE=2
VIEW_PRESENT_BOTH=3
VIEW_PRESENT_NONE=4

class scatterplot_settings:

	def __init__(self,session,path=None):

		self.session=session
		self.lock_callbacks=0
		self.loaded=0

		if path!=None:
			self.loaded=self.read_settings(path)

		self.plot_mva=Tkinter.IntVar()
		try:
			num=int(self.session.project.saved_value('HT_PLOT_MVA'))
			if num<0 or num>1:
				raise ValueError
			self.plot_mva.set(num)
		except (ValueError,TypeError):
			self.plot_mva.set(0)

		self.plot_fit=Tkinter.IntVar()
		try:
			num=int(self.session.project.saved_value('HT_PLOT_FIT'))
			if num<0 or num>1:
				raise ValueError
			self.plot_fit.set(num)
		except (ValueError,TypeError):
			self.plot_fit.set(1)

		self.plot_thr=Tkinter.IntVar()
		try:
			num=int(self.session.project.saved_value('HT_PLOT_THR'))
			if num<0 or num>1:
				raise ValueError
			self.plot_thr.set(num)
		except (ValueError,TypeError):
			self.plot_thr.set(0)

		try:
			num=float(self.session.project.saved_value('HT_FIT_SPAN'))
			if num<0 or num>1:
				raise ValueError
			self.fit_f=num
		except (ValueError,TypeError):
			self.fit_f=0.5

		try:
			num=int(self.session.project.saved_value('HT_FIT_NSTEPS'))
			if num<0:
				raise ValueError
			self.fit_nsteps=num
		except (ValueError,TypeError):
			self.fit_nsteps=2

		try:
			num=float(self.session.project.saved_value('HT_FIT_DELTA'))
			if num<0:
				raise ValueError
			self.fit_delta=num
		except (ValueError,TypeError):
			self.fit_delta=0.0

		try:
			num=int(self.session.project.saved_value('HT_VALUE'))
			if num<-1 or num>msfit.VALUE_SFIT_VOLUME:
				raise ValueError
			self.data_height=num
		except (ValueError,TypeError):
			self.data_height=msfit.VALUE_DATA_HEIGHT

		try:
			num=int(self.session.project.saved_value('HT_NORM_METHOD'))
			if num<msnorm.NORM_NONE or num>msnorm.NORM_LOWESS:
				raise ValueError
			self.norm_method=num
		except (ValueError,TypeError):
			self.norm_method=msnorm.NORM_NONE

		try:
			num=float(self.session.project.saved_value('HT_RATIO_TH'))
			if num<1:
				raise ValueError
			self.ratio_th=num
		except (ValueError,TypeError):
			self.ratio_th=2.0

		self.view_mode=Tkinter.IntVar()
		try:
			num=int(self.session.project.saved_value('HT_VIEW_MODE'))
			if num<VIEW_ALL or num>VIEW_PRESENT_NONE:
				raise ValueError
			self.view_mode.set(num)
		except (ValueError,TypeError):
			self.view_mode.set(VIEW_ALL)

		try:
			num=float(self.session.project.saved_value('HT_DOT_SIZE'))
			if num<1:
				raise ValueError
			self.dot_size=num
		except (ValueError,TypeError):
			self.dot_size=3.0

		self.dot_fill=self.session.project.saved_value('HT_DOT_FILL')
		self.dot_outline=self.session.project.saved_value('HT_DOT_OUTLINE')
		self.dot_selected_fill=self.session.project.saved_value('HT_DOT_SELECTED_FILL')
		self.dot_protein_fill=self.session.project.saved_value('HT_DOT_PROTEIN_FILL')
		self.dot_sequence_fill=self.session.project.saved_value('HT_DOT_SEQUENCE_FILL')
		self.dot_list_fill=self.session.project.saved_value('HT_DOT_LIST_FILL')
		self.ratio_up_color=self.session.project.saved_value('HT_RATIO_UP_COLOR')
		self.ratio_down_color=self.session.project.saved_value('HT_RATIO_DOWN_COLOR')
		self.ratio_nan_color=self.session.project.saved_value('HT_RATIO_NAN_COLOR')
		
		self.view_over_th=Tkinter.IntVar()
		self.view_over_th.set(0)
		
		self.view_list_only=Tkinter.IntVar()
		self.view_list_only.set(0)
		self.view_list=[]

		self.norm_factor=1
		self.norm_protein=''
		
		self.last_selected=None
		self.last_color=self.dot_fill
		self.last_outline=self.dot_outline

		self.mark_protein=None
		self.mark_sequence=None

		self.proteins_list=None

	# --------------------------------------------------------------------------
	#
	def read_settings(self,path):
		result=0

		if path!=None:
			try:
				file=open(path,'r')
				lines=file.readlines()

				for i in range(0,len(lines)):
					line=lines[i].strip()
					if len(line)!=0 and line[0]!='#':
						fields=lines[i].split('=')
						if len(fields)==2:
							self.session.project.save_value(fields[0].strip(),fields[1].strip())
				file.close()
				result=1
			except IOError, (errno, strerror):
				print "I/O error(%s): %s" % (errno, strerror)
				result=0

		return result

# -----------------------------------------------------------------------------
#
class scatterplot_settings_dialog(mstkutil.Settings_Dialog):

	def __init__(self, session):
		self.session=session
		self.loaded=0
		mstkutil.Settings_Dialog.__init__(self,session.tk,'Settings')

		balloon=Pmw.Balloon(self.top,initwait=1500)

		
		#
		f=Pmw.Group(self.top,tag_text='Values:')
		f.pack(side='top',anchor='nw',fill='x',padx=0,pady=0)

#		f=Tkinter.Frame(self.top,borderwidth=2,relief='groove')
#		f.pack(side='top',anchor='w',pady=2,fill='x',expand=1)

		l=Tkinter.Label(f.interior(),text='Value: ')
		l.grid(row=0,column=0,sticky='w')

		m=msfit.value_menu(f.interior(),'',with_rt=1)
		m.add_callback(self.value_menu_cb)
		m.frame.grid(row=0,column=1,sticky='w',pady=2,padx=2)
		self.value_menu=m

		l=Tkinter.Label(f.interior(),text='Ratio thresh.: ')
		l.grid(row=1,column=0,sticky='w')
		self.ratio_th_label=l

		e=mstkutil.num_entry_field(f.interior(),0,'',1,1000000,'',11)
		e.frame.grid(row=1,column=1,sticky='w',pady=2)
		self.ratio_th_edit=e

		f.interior().columnconfigure(0, weight=1)
		f.interior().columnconfigure(1, weight=1)

		#
		f=Pmw.Group(self.top,tag_text='Normalization:')
		f.pack(side='top',anchor='nw',fill='x',padx=0,pady=0)

		l=Tkinter.Label(f.interior(),text='Norm. method: ')
		l.grid(row=0,column=0,sticky='w')

		m=msnorm.normalization_menu(f.interior(),'')
		m.add_callback(self.norm_menu_cb)
		m.frame.grid(row=0,column=1,sticky='w',pady=2)
		self.normalization_menu=m

		l=Tkinter.Label(f.interior(),text='Norm. coeff.: ')
		l.grid(row=1,column=0,sticky='w')
		self.normalization_edit_label=l

		e=mstkutil.num_entry_field(f.interior(),width=11)
		e.frame.grid(row=1,column=1,sticky='w',pady=2)
		self.normalization_edit=e

		l=Tkinter.Label(f.interior(),text='Fit span: ')
		l.grid(row=2,column=0,sticky='w')

		e=mstkutil.num_entry_field(f.interior(),0,'',0,1,'',11)
		e.frame.grid(row=2,column=1,sticky='w',pady=2)
		self.fit_f_edit=e

		l=Tkinter.Label(f.interior(),text='Fit steps: ')
		l.grid(row=3,column=0,sticky='w')

		e=mstkutil.num_entry_field(f.interior(),1,'',0,20,'',11)
		e.frame.grid(row=3,column=1,sticky='w',pady=2)
		self.fit_nsteps_edit=e

		f.interior().columnconfigure(0, weight=1)
		f.interior().columnconfigure(1, weight=1)
		
		#
#		l=Tkinter.Label(f,text='Fit method: ')
#		l.grid(row=5,column=0,sticky='w')

#		m=mstkutil.option_menu(f,'',('Lowess'),'Lowess')
#		m.frame.grid(row=5,column=1,sticky='w',pady=2)
#		self.fit_menu=m
#		self.fit_menu.menubutton.configure(state='disabled')


		#
		f=Pmw.Group(self.top,tag_text='List:')
		f.pack(side='top',anchor='nw',fill='x',padx=0,pady=0)

		cb=mstkutil.checkbutton(f.interior(),'Show only from list',0)
		cb.button.grid(row=0,column=0,columnspan=2,sticky='w',pady=2)
		self.only_list_check=cb

		b=Tkinter.Button(f.interior(),text='Load list',borderwidth=1,width='12',command=self.load_list_cb)
		b.grid(row=1,column=0,sticky='w',padx=2,pady=2)

		b=Tkinter.Button(f.interior(),text='Clear list',borderwidth=1,width='12',command=self.clear_list_cb)
		b.grid(row=1,column=1,sticky='w',padx=2,pady=2)

		cb=mstkutil.checkbutton(f.interior(),'Exact charges',0)
		cb.button.grid(row=2,column=0,columnspan=2,sticky='w',pady=2)
		self.list_charge_check=cb

		f.interior().columnconfigure(0, weight=1)
		f.interior().columnconfigure(1, weight=1)

		#
		br=mstkutil.button_row(self.top,
				('Apply', self.apply_settings_cb),
				('Load', self.load_cb),
				('Close', self.close_cb),
				)
		br.frame.pack(side='bottom',anchor='w',pady=2)

	# ---------------------------------------------------------------------------
	#
	def load_cb(self):
		path=mstkutil.load_file(self.top,'Open configuration','open_cfg')
		if path!=None:
			settings=scatterplot_settings(self.session,path)
			self.loaded=settings.loaded
			self.show_settings(settings)

	# ---------------------------------------------------------------------------
	#
	def value_menu_cb(self,event=None):
		if self.value_menu.get_value()>=0:
			self.ratio_th_edit.min_value=1
			self.ratio_th_label.configure(text='Ratio thresh.')
		else:
			self.ratio_th_edit.min_value=0
			self.ratio_th_label.configure(text='Diff. thresh.')

	# ---------------------------------------------------------------------------
	#
	def norm_menu_cb(self,event=None):
		self.normalization_edit.entry.configure(state='disabled')
		self.normalization_edit.set('')
		self.normalization_edit_label.configure(text='Norm. coeff.:')

		self.fit_f_edit.entry.configure(state='disabled')
		self.fit_nsteps_edit.entry.configure(state='disabled')
		
		norm_method=self.normalization_menu.get_value()
		if norm_method==msnorm.NORM_USER:
			self.normalization_edit.set_entry_type(mstkutil.ENTRY_FP)
			self.normalization_edit.set_range((0,10000000000000))
			self.normalization_edit.entry.configure(state='normal')
		elif norm_method==msnorm.NORM_PROTEIN_MEDIAN or norm_method==msnorm.NORM_PROTEIN_MEAN or norm_method==msnorm.NORM_PROTEIN_GMEAN:
			self.normalization_edit.set_entry_type(mstkutil.ENTRY_STR)
			self.normalization_edit.entry.configure(state='normal')
			self.normalization_edit_label.configure(text='Protein ID:')
		elif norm_method==msnorm.NORM_LOWESS:
			self.fit_f_edit.entry.configure(state='normal')
			self.fit_nsteps_edit.entry.configure(state='normal')

	# ---------------------------------------------------------------------------
	#
	def show_settings(self,settings):
		self.value_menu.set_value(settings.data_height)
		self.normalization_menu.set_value(settings.norm_method)

		if settings.norm_method==msnorm.NORM_USER:
			self.normalization_edit.set(settings.norm_factor)
		if settings.norm_method==msnorm.NORM_PROTEIN_MEDIAN or settings.norm_method==msnorm.NORM_PROTEIN_MEAN or settings.norm_method==msnorm.NORM_PROTEIN_GMEAN:
			self.normalization_edit.set(settings.norm_protein)

		self.fit_f_edit.variable.set(settings.fit_f)
		self.fit_nsteps_edit.variable.set(settings.fit_nsteps)

		self.ratio_th_edit.variable.set(settings.ratio_th)

		self.only_list_check.variable.set(settings.view_list_only.get())

		if settings.view_list==None or len(settings.view_list)==0:
			self.only_list_check.button.configure(state='disabled')
		else:
			self.only_list_check.button.configure(state='normal')

	# ---------------------------------------------------------------------------
	#
	def get_settings(self):
		settings=scatterplot_settings(self.session)

		settings.data_height=self.value_menu.get_value()

		settings.norm_method=self.normalization_menu.get_value()
		if settings.norm_method==msnorm.NORM_USER:
			settings.norm_factor=float(self.normalization_edit.get())
			settings.norm_protein=self.parent_dialog.settings.norm_protein
		elif settings.norm_method==msnorm.NORM_PROTEIN_MEDIAN or settings.norm_method==msnorm.NORM_PROTEIN_MEAN or settings.norm_method==msnorm.NORM_PROTEIN_GMEAN:
			settings.norm_protein=self.normalization_edit.get()
			settings.norm_factor=self.parent_dialog.settings.norm_factor
		else:
			settings.norm_protein=self.parent_dialog.settings.norm_protein
			settings.norm_factor=self.parent_dialog.settings.norm_factor
			
		settings.fit_f=float(self.fit_f_edit.variable.get())
		settings.fit_nsteps=int(self.fit_nsteps_edit.variable.get())

		settings.ratio_th=float(self.ratio_th_edit.variable.get())

		settings.view_list_only.set(self.only_list_check.variable.get())

		if self.loaded==0:
			settings.plot_mva=self.parent_dialog.settings.plot_mva
			settings.plot_fit=self.parent_dialog.settings.plot_fit
			settings.plot_thr=self.parent_dialog.settings.plot_thr
			settings.view_over_th=self.parent_dialog.settings.view_over_th

		settings.last_selected=self.parent_dialog.settings.last_selected
		settings.last_color=self.parent_dialog.settings.last_color
		settings.last_outline=self.parent_dialog.settings.last_outline

		settings.mark_protein=self.parent_dialog.settings.mark_protein
		settings.mark_sequence=self.parent_dialog.settings.mark_sequence
		
		settings.view_list=self.parent_dialog.settings.view_list
		settings.proteins_list=self.parent_dialog.settings.proteins_list

		return settings

	# ---------------------------------------------------------------------------
	#
	def apply_settings_cb(self):
		self.show_window(0)
		self.apply_cb()
		self.close_cb()

	# ---------------------------------------------------------------------------
	#
	def clear_list_cb(self,event=None):
		self.parent_dialog.settings.view_list=None
		self.only_list_check.variable.set(0)
		self.only_list_check.button.configure(state='disabled')

	# ---------------------------------------------------------------------------
	#
	def load_list_cb(self,event=None):
		path=mstkutil.load_file(self.top,'Load list','load_list')
		exact_charge=self.list_charge_check.variable.get()
		sequences=self.load_list(path,exact_charge)

		if sequences!=None and len(sequences)!=0:
			self.parent_dialog.settings.view_list=[]
			for i in range(len(self.parent_dialog.peaks1)):
				found=0
				peak=self.parent_dialog.peaks1[i]
				if mssputil.is_peak_unassigned(peak)==0:
					peptide_sequence=mssputil.get_sequence_with_charge(peak,exact_charge)
					for j in range(len(sequences)):
						if peptide_sequence==sequences[j]:
							found=1
							break
					self.parent_dialog.settings.view_list.append(found)

			if max(self.parent_dialog.settings.view_list)==0:
				self.parent_dialog.settings.view_list=[]
				self.only_list_check.button.configure(state='disabled')
				dialog=mstkutil.warning_dialog(self.top,'No peptides from list found')
			else:
				self.only_list_check.button.configure(state='normal')

	# --------------------------------------------------------------------------
	#
	def load_list(self,path,exact_charge=0):
		sequences=None

		if path:
			file=open(path,'r')
			sequences=file.readlines()
			file.close()

			if len(sequences)!=0:
				for i in range(len(sequences)-1,-1,-1):
					sequence=sequences[i].strip()
					if sequence=='':
						sequences[i]=[]
					else:
						sequences[i]=sequence

						index=sequence.find(' (+')
						if index!=-1:
							if exact_charge==1:
								index=sequence.find(')',index)+1
							sequences[i]=sequence[:index]
		return sequences

