import sparky
import Tkinter
import mssputil
import mstkutil
import pyutil
import fpconst
import math

import Tree
import ms
import os


class MSNode(Tree.Node):
	def __init__(self, *args, **kw_args):
	
		apply(Tree.Node.__init__, (self,)+args, kw_args)

		self.widget.tag_bind(self.label, '<Button-1>', self.PVT_click)

		self.widget.tag_bind(self.symbol,'<Double-Button-1>',self.double_click_cb)
		self.widget.tag_bind(self.label,'<Double-Button-1>',self.double_click_cb)

		self.widget.tag_bind(self.symbol,'<Button-3>',self.popup_menu)
		self.widget.tag_bind(self.label,'<Button-3>',self.popup_menu)


	# -----------------------------------------------------------------------------
	#
	def popup_menu(self, event):
		full_id=self.full_id()

		self.widget.move_cursor(self)

		menu=Tkinter.Menu(self.widget,tearoff=0)
		menu.add_command(label='Select',command=self.select_cb)
		menu.add_command(label='Unselect',command=self.unselect_cb)

		if len(full_id)>1:
			menu.add_command(label='Add to selected',command=self.add_select_cb)

		if len(full_id)==4:
			menu.add_separator()
			menu.add_command(label='Go to',command=self.double_click_cb)

		menu.tk_popup(event.x_root,event.y_root)

	# -----------------------------------------------------------------------------
	#
	def select_cb(self,event=None):
		full_id=self.full_id()
		full_id[0].spectrum.session.unselect_all_ornaments()

		if len(full_id)==1:
			for protein in full_id[0].protein_list:
				for peptide in protein.peptide_list:
					for query in peptide.query_list:
						query.peak_list[0].selected=1

		elif len(full_id)==2:
			protein=self.get_msobject()
			for peptide in protein.peptide_list:
				for query in peptide.query_list:
					query.peak_list[0].selected=1

		elif len(full_id)==3:
			peptide=self.get_msobject()
			for query in peptide.query_list:
				query.peak_list[0].selected=1

		elif len(full_id)==4:
			query=self.get_msobject()
			query.peak_list[0].selected=1

	# -----------------------------------------------------------------------------
	#
	def add_select_cb(self,event=None):
		full_id=self.full_id()

		if len(full_id)==2:
			protein=self.get_msobject()
			for peptide in protein.peptide_list:
				for query in peptide.query_list:
					query.peak_list[0].selected=1

		if len(full_id)==3:
			peptide=self.get_msobject()
			for query in peptide.query_list:
				query.peak_list[0].selected=1

		elif len(full_id)==4:
			query=self.get_msobject()
			query.peak_list[0].selected=1


	# -----------------------------------------------------------------------------
	#
	def unselect_cb(self,event=None):
		full_id=self.full_id()

		if len(full_id)==1:
			full_id[0].spectrum.session.unselect_all_ornaments()

		if len(full_id)==2:
			protein=self.get_msobject()
			for peptide in protein.peptide_list:
				for query in peptide.query_list:
					query.peak_list[0].selected=0

		if len(full_id)==3:
			peptide=self.get_msobject()
			for query in peptide.query_list:
				query.peak_list[0].selected=0

		elif len(full_id)==4:
			query=self.get_msobject()
			query.peak_list[0].selected=0

	# -----------------------------------------------------------------------------
	#
	def double_click_cb(self,event=None):
		full_id=self.full_id()
		if len(full_id)==4:
			query=self.get_msobject()
			mssputil.select_peak(query.get(0))
			mssputil.show_peak(query.get(0))

	# -----------------------------------------------------------------------------
	#
	def get_msobject(self):
		msobject=None

		full_id=self.full_id()
		node_level=len(full_id)
		sample=full_id[0]

		if node_level==1:
			msobject=sample

		elif node_level==2:
			msobject=sample.get_protein(ms.protein(full_id[1]))

		elif node_level==3:
			protein=sample.get_protein(ms.protein(full_id[1]))
			msobject=protein.get_peptide(ms.peptide(full_id[2]))

		elif node_level==4:
			protein=sample.get_protein(ms.protein(full_id[1]))
			peptide=protein.get_peptide(ms.peptide(full_id[2]))
			msobject=peptide.get(full_id[3])

		return msobject 

# -----------------------------------------------------------------------------
#

shut_icon=Tkinter.PhotoImage(data='R0lGODlhCQAQAJH/AMDAwAAAAGnD/wAAACH5BAEAAAAALAAA'
                     'AAAJABAAQAIdhI8hu2EqXIroyQrb\nyRf0VG0UxnSZ5jFjulrhaxQ'
                     'AO6olVwAAOw==')

open_icon=Tkinter.PhotoImage(data='R0lGODlhEAAJAJH/AMDAwAAAAGnD/wAAACH5BAEAAAAALAAA'
                     'AAAQAAkAQAIahI+pyyEPg3KwPrko\nTqH7/yGUJWxcZTapUQAAO8b'
                     'yUgAAOw==')

# -----------------------------------------------------------------------------
#
class spectrum_layout_dialog(mstkutil.Dialog):

	def __init__(self,session):
		self.session=session

		mstkutil.Dialog.__init__(self,session.tk,'MS spectrum layout')

		self.sample=get_sample(self.session.selected_spectrum())

		tr=Tree.Tree(self.top,root_id=self.sample,root_label=self.sample.name,get_contents_callback=self.get_tree,width=250,height=700,collapsed_icon=shut_icon,expanded_icon=open_icon,background='white',node_class=MSNode)
		tr.grid(row=0,column=0,sticky='nsew')
#		tr.bind('<MouseWheel>', self.scrollUp)
#		tr.bind('<Button-5>', self.scrollDown)
		self.tree=tr

		self.top.grid_rowconfigure(0,weight=1)
		self.top.grid_columnconfigure(0,weight=1)

		sb=Tkinter.Scrollbar(self.top)
		sb.grid(row=0,column=1,sticky='ns')
		sb.configure(command=self.tree.yview)
		self.tree.configure(yscrollcommand=sb.set)

		sb=Tkinter.Scrollbar(self.top,orient='horizontal')
		sb.grid(row=1,column=0,sticky='ew')
		sb.configure(command=self.tree.xview)
		self.tree.configure(xscrollcommand=sb.set)

		self.tree.focus_set()

#		br=mstkutil.button_row(self.top,
#				('Close', self.top.destroy))
#		br.frame.pack(side='bottom',anchor='w',pady=2)

		self.top.geometry("+0+0")
#		self.top.resizable(0,0)

	# -----------------------------------------------------------------------------
	#
	def scrollDown(self,event):
		print 'scrolled'
		print event
		print event.delta
		self.tree.yview('scroll',1,'units')

	# -----------------------------------------------------------------------------
	#
	def scrollUp(self,event):
		print 'scrolled'
		print event
		print event.delta
		self.tree.yview('scroll',-2, 'units')

	# -----------------------------------------------------------------------------
	#
	def get_tree(self,node):
		full_id=node.full_id()
		node_level=len(full_id)
		
		if node_level==1:
			for protein in self.sample.protein_list:
				self.tree.add_node(name=protein.id,id=protein.id,flag=1)

		elif node_level==2:
			protein=self.sample.get_protein(ms.protein(full_id[1]))
			for peptide in protein.peptide_list:
				self.tree.add_node(name=peptide.sequence,id=peptide.sequence,flag=1)

		elif node_level==3:
			protein=self.sample.get_protein(ms.protein(full_id[1]))
			peptide=protein.get_peptide(ms.peptide(full_id[2]))
			for i in range(len(peptide.query_list)):
				self.tree.add_node(name=('+' + ('%d' % peptide.query_list[i].charge)),id=i,flag=0)


# -----------------------------------------------------------------------------
#
def show_spectrum_layout(session):
	spectrum=session.selected_spectrum()
	if spectrum==None:
		return

	if not hasattr(session,'spectrum_dialogs'):
		session.spectrum_dialogs={}
	dialogs=session.spectrum_dialogs
	if (dialogs.has_key(spectrum) and not dialogs[spectrum].is_window_destroyed()):
		dialogs[spectrum].show_window(1)
	else:
		d=spectrum_layout_dialog(session)
		d.show_window(1)
		dialogs[spectrum] = d
		

# -----------------------------------------------------------------------------
#
def get_sample(spectrum):
	
	sample=ms.sample(spectrum.name)
	sample.spectrum=spectrum

	peaks=spectrum.peak_list()
	for peak in peaks:
		if mssputil.is_peak_unassigned(peak)!=1:
			query=ms.query()
			query.charge=mssputil.get_charge(peak)
			query.score=mssputil.get_score(peak)
			query.add(peak)

			peptide=ms.peptide(mssputil.get_sequence(peak))
			peptide.add(query)

			proteins_id=mssputil.get_protein(peak).split(',')
			for id in proteins_id:
				protein=ms.protein(id)
				protein=sample.add(protein)
				protein.add(peptide)
		else:
			query.add(peak)

	return sample
					
		
			




