import sys
import string
import pyutil
import sparky
import Tkinter

import mssputil
import mstkutil
import mscalib
import fpconst

import Pmw
from PmwContribD import ProgressMeter as pm

# -----------------------------------------------------------------------------
#
class spectrum_properties_dialog(mstkutil.Dialog,mstkutil.Stoppable):
	
	def __init__(self, session):
		self.session=session
		mstkutil.Dialog.__init__(self,session.tk,'MS spectrum properties')

		self.entries=[]
		
		Pmw.initialise(session.tk)
		balloon=Pmw.Balloon(self.top,initwait=1500)

		#
		f=Pmw.Group(self.top, tag_text='Spectrum selection:')
		f.pack(side='top',anchor='nw',fill='x',padx=1,pady=3)

		m=mssputil.spectrum_menu(session,f.interior(),'')
		m.frame.pack(side='top',padx=2,pady=2,fill='x',expand=1)
		m.add_callback(self.select_spectrum)
		self.spectrum_menu=m
		
		#---------------------------------------------------------------------
		#
		tab_panel=Pmw.NoteBook(self.top,borderwidth=1)
		tab_panel.pack(fill='both',expand=1,padx=1,pady=3)

		#---------------------------------------------------------------------
		#
		tab=tab_panel.add(' Spectrum ')
		tab_panel.tab(' Spectrum ').focus_set()

		#
		f=Pmw.Group(tab, tag_text='Properties:')
		f.pack(side='top',anchor='nw',fill='x',padx=0,pady=0)

		l=Tkinter.Label(f.interior(),text='Instrument:')
		l.grid(row=0,column=0,sticky='w')

		e=mstkutil.entry_field(f.interior(),'','0',30,None,'right')
		e.frame.grid(row=0,column=1,sticky='e',padx=2,pady=2)
#		e.entry.configure(state='readonly')
		self.instrument_edit=e

		l=Tkinter.Label(f.interior(),text='Analyzer:')
		l.grid(row=1,column=0,sticky='w')

		m=mstkutil.option_menu(f.interior(),'',mssputil.MSANALYZER_STR,mssputil.MSANALYZER_STR[0])
		m.menubutton.configure(width='25')
		m.add_callback(self.analyzer_cb)
		m.frame.grid(row=1,column=1,sticky='e',padx=2,pady=2)
		self.analyzer_menu=m

		f.interior().columnconfigure(0, weight=1)
		f.interior().columnconfigure(1, weight=1)

		#
		f=Pmw.Group(tab, tag_text='Ranges:')
		f.pack(side='top',anchor='nw',fill='x',padx=0,pady=0)

		l=Tkinter.Label(f.interior(),text='m/z range [Da]:',justify='left')
		l.grid(row=0,column=0,sticky='w')

		e=mstkutil.entry_field(f.interior(),'','0',16,None,'right')
		e.frame.grid(row=0,column=1,sticky='e',padx=2,pady=2)
		e.entry.configure(state='readonly')
		self.mz_range_edit=e

		l=Tkinter.Label(f.interior(),text='m/z step [Da]:',justify='left')
		l.grid(row=1,column=0,sticky='w')

		e=mstkutil.entry_field(f.interior(),'','0',16,None,'right')
		e.frame.grid(row=1,column=1,sticky='e',padx=2,pady=2)
		e.entry.configure(state='readonly')
		self.mz_step_edit=e

		l=Tkinter.Label(f.interior(),text='RT range [min]:')
		l.grid(row=2,column=0,sticky='w')

		e=mstkutil.entry_field(f.interior(),'','0',16,None,'right')
		e.frame.grid(row=2,column=1,sticky='e',padx=2,pady=2)
		e.entry.configure(state='readonly')
		self.rt_range_edit=e

		l=Tkinter.Label(f.interior(),text='RT step [min]:')
		l.grid(row=3,column=0,sticky='w')

		e=mstkutil.entry_field(f.interior(),'','0',16,None,'right')
		e.frame.grid(row=3,column=1,sticky='e',padx=2,pady=2)
		e.entry.configure(state='readonly')
		self.rt_step_edit=e

		f.interior().columnconfigure(0, weight=1)
		f.interior().columnconfigure(1, weight=1)

		#
		f=Pmw.Group(tab, tag_text='Domain:')
		f.pack(side='top',anchor='nw',fill='x',padx=0,pady=0)

		l=Tkinter.Label(f.interior(),text='Domain:')
		l.grid(row=0,column=0,sticky='w')

		m=mstkutil.option_menu(f.interior(),'',('m/z','frequency'),'m/z')
		m.menubutton.configure(width='10')
		m.add_callback(self.domain_cb)
		m.frame.grid(row=0,column=1,sticky='e',padx=2,pady=2)
		self.domain_menu=m

		l=Tkinter.Label(f.interior(),text='Parameter A:')
		l.grid(row=1,column=0,sticky='w')

		e=mstkutil.entry_field(f.interior(),'','1',16,None,'right')
		e.frame.grid(row=1,column=1,sticky='e',padx=2,pady=2)
		self.para_edit=e

		l=Tkinter.Label(f.interior(),text='Parameter B:')
		l.grid(row=2,column=0,sticky='w')

		e=mstkutil.entry_field(f.interior(),'','0',16,None,'right')
		e.frame.grid(row=2,column=1,sticky='e',padx=2,pady=2)
		self.parb_edit=e

		f.interior().columnconfigure(0, weight=1)
		f.interior().columnconfigure(1, weight=1)

		#
		f=Pmw.Group(tab, tag_text='Statistics:')
		f.pack(side='top',anchor='nw',fill='x',padx=0,pady=0)

		l=Tkinter.Label(f.interior(),text='Total counts:')
		l.grid(row=0,column=0,sticky='w')

		e=mstkutil.entry_field(f.interior(),'','0',16,None,'right')
		e.frame.grid(row=0,column=1,sticky='e',padx=2,pady=2)
		self.total_counts_edit=e

		l=Tkinter.Label(f.interior(),text='Mean counts:')
		l.grid(row=1,column=0,sticky='w')

		e=mstkutil.entry_field(f.interior(),'','0',16,None,'right')
		e.frame.grid(row=1,column=1,sticky='e',padx=2,pady=2)
		self.mean_counts_edit=e

		f.interior().columnconfigure(0, weight=1)
		f.interior().columnconfigure(1, weight=1)

		#---------------------------------------------------------------------
		#
		tab=tab_panel.add(' Peaks ')

		#
		f=Pmw.Group(tab, tag_text='Spectrum peaks:')
		f.pack(side='top',anchor='nw',fill='x',padx=0,pady=0)

		l=Tkinter.Label(f.interior(),text='Number of peaks: ')
		l.grid(row=0,column=0,sticky='w',padx=2,pady=2)

		e=mstkutil.entry_field(f.interior(),'','0',12,None,'right')
		e.frame.grid(row=0,column=1,sticky='e',padx=2,pady=2)
		self.peaks_count_edit=e

		#l=Tkinter.Label(f.interior(),text='')
		#l.grid(row=1,column=0,sticky='w',padx=2,pady=2)
		
		b=Tkinter.Button(f.interior(),text='Remove peaks',width=12,borderwidth=1,command=self.remove_cb)
		b.grid(row=1,column=1,sticky='e',padx=2,pady=2)
		self.remove_button=b

		f.interior().columnconfigure(0, weight=1)
		f.interior().columnconfigure(1, weight=1)


		#
		f=Pmw.Group(tab, tag_text='Read peaks:')
		f.pack(side='top',anchor='nw',fill='x',padx=0,pady=0)

		l=Tkinter.Label(f.interior(),text='File: ')
		l.grid(row=0,column=0,sticky='w',padx=2,pady=2)

		plp=mstkutil.file_field(f.interior(),None,'peaklist',width=35,file_types=[('Sparky peaks', '.4sparky.dat'),('all files', '.*')])
		plp.frame.grid(row=0,column=1,sticky='w',pady=2)
		self.peak_list_path=plp

		#l=Tkinter.Label(f.interior(),text='')
		#l.grid(row=1,column=0,sticky='w',padx=2,pady=2)
		
		b=Tkinter.Button(f.interior(),text='Create peaks',width=12,borderwidth=1,command=self.read_cb)
		b.grid(row=1,column=1,sticky='e',padx=2,pady=2)
		self.create_button=b

		f.interior().columnconfigure(0, weight=1)
		f.interior().columnconfigure(1, weight=1)
		
		#
		f=Pmw.Group(tab, tag_text='Label format:')
		f.pack(side='top',anchor='nw',fill='x',padx=0,pady=0)

		cb=mstkutil.checkbutton(f.interior(),'Include protein Id in label',1)
		cb.button.grid(row=0,columnspan=2,sticky='w',padx=2,pady=2)
		self.label_protein_check=cb

		cb=mstkutil.checkbutton(f.interior(),'Include charge state in label',1)
		cb.button.grid(row=1,columnspan=2,sticky='w',padx=2,pady=2)
		self.label_charge_check=cb

		cb=mstkutil.checkbutton(f.interior(),'Include charge RT in label',0)
		cb.button.grid(row=2,columnspan=2,sticky='w',padx=2,pady=2)
		self.label_rt_check=cb

		cb=mstkutil.checkbutton(f.interior(),'Hide labels of not found peptides',0)
		cb.button.grid(row=3,columnspan=2,sticky='w',padx=2,pady=2)
		self.label_present_check=cb
		
		l=Tkinter.Label(f.interior(),text='')
		l.grid(row=4,column=0,sticky='w',padx=2,pady=2)
		
		b=Tkinter.Button(f.interior(),text='Set format',width=12,borderwidth=1,command=self.label_format_cb)
		b.grid(row=5,column=1,sticky='e',padx=2,pady=2)
		self.format_button=b
		
		f.interior().columnconfigure(0, weight=1)
		f.interior().columnconfigure(1, weight=1)
		
		#
		f=Pmw.Group(tab, tag_text='Progress:')
		f.pack(side='top',anchor='nw',fill='x',padx=0,pady=0)

		progress_label=Tkinter.Label(f.interior(),text='')
		progress_label.grid(row=0,column=0,sticky='w',padx=2,pady=2)
				
		progress_meter=pm.ProgressMeter(f.interior(),progresscolor='blue',indicatorcolor='white',shadowthickness=2);
		progress_meter.grid(row=1,column=0,sticky='we',pady=2,padx=2)
		self.progress_meter=progress_meter
		
		f.interior().columnconfigure(0, weight=1)

		#---------------------------------------------------------------------
		#
		tab=tab_panel.add(' Calibration ')

		#
		f=Pmw.Group(tab, tag_text='m/z calibration:')
		f.pack(side='top',anchor='nw',fill='x',padx=0,pady=0)

		l=Tkinter.Label(f.interior(),text='Calibration method:')
		l.grid(row=0,column=0,sticky='w')

		m=mstkutil.option_menu(f.interior(),'',mscalib.CALIB_STR[0:-1],mscalib.CALIB_STR[0])
		m.menubutton.configure(width='9')
		m.frame.grid(row=0,column=1,sticky='e',padx=2,pady=2)
		m.menubutton.configure(state='disabled')
		self.mz_correction_menu=m

		l=Tkinter.Label(f.interior(),text='Slope:')
		l.grid(row=1,column=0,sticky='w')

		e=mstkutil.entry_field(f.interior(),'','0',15,None,'right')
		e.frame.grid(row=1,column=1,sticky='e',padx=2,pady=2)
		e.entry.configure(state='readonly')
		self.mz_correction_a_edit=e

		l=Tkinter.Label(f.interior(),text='Intercept:')
		l.grid(row=2,column=0,sticky='w')

		e=mstkutil.entry_field(f.interior(),'','0',15,None,'right')
		e.frame.grid(row=2,column=1,sticky='e',padx=2,pady=2)
		e.entry.configure(state='readonly')
		self.mz_correction_b_edit=e

		l=Tkinter.Label(f.interior(),text='Precision [ppm]:')
		l.grid(row=3,column=0,sticky='w')

		#e=mstkutil.entry_field(f.interior(),'','0',15,None,'right')
		e=mstkutil.num_entry_field(f.interior(),mstkutil.ENTRY_FP,'',1,fpconst.PosInf,'',15)
		e.frame.grid(row=3,column=1,sticky='e',padx=2,pady=2)
		self.entries.append(e)
		self.mz_precision_edit=e

		f.interior().columnconfigure(0, weight=1)
		f.interior().columnconfigure(1, weight=1)

		#
		f=Pmw.Group(tab, tag_text='RT calibration:')
		f.pack(side='top',anchor='nw',fill='x',padx=0,pady=0)

		l=Tkinter.Label(f.interior(),text='Calibration method:')
		l.grid(row=0,column=0,sticky='w')

		m=mstkutil.option_menu(f.interior(),'',mscalib.CALIB_STR,mscalib.CALIB_STR[0])
		m.menubutton.configure(width='9')
		m.frame.grid(row=0,column=1,sticky='e',padx=2,pady=2)
		m.menubutton.configure(state='disabled')
		self.rt_correction_menu=m

		l=Tkinter.Label(f.interior(),text='Slope:')
		l.grid(row=1,column=0,sticky='w')

		e=mstkutil.entry_field(f.interior(),'','0',15,None,'right')
		e.frame.grid(row=1,column=1,sticky='e',padx=2,pady=2)
		e.entry.configure(state='readonly')
		self.rt_correction_a_edit=e

		l=Tkinter.Label(f.interior(),text='Intercept:')
		l.grid(row=2,column=0,sticky='w')

		e=mstkutil.entry_field(f.interior(),'','0',15,None,'right')
		e.frame.grid(row=2,column=1,sticky='e',padx=2,pady=2)
		e.entry.configure(state='readonly')
		self.rt_correction_b_edit=e

		l=Tkinter.Label(f.interior(),text='Precision [min]:')
		l.grid(row=3,column=0,sticky='w')

		#e=mstkutil.entry_field(f.interior(),'','0',15,None,'right')
		e=mstkutil.num_entry_field(f.interior(),mstkutil.ENTRY_FP,'',0.01,fpconst.PosInf,'',15)
		self.entries.append(e)
		e.frame.grid(row=3,column=1,sticky='e',padx=2,pady=2)
		self.rt_precision_edit=e

		f.interior().columnconfigure(0, weight=1)
		f.interior().columnconfigure(1, weight=1)

		#
		f=Pmw.Group(tab,tag_text='RP calibration:')
		f.pack(side='top',anchor='nw',fill='x',padx=0,pady=0)

		l=Tkinter.Label(f.interior(),text='Calibration method:')
		l.grid(row=0,column=0,sticky='w')
		
		m=mstkutil.option_menu(f.interior(),'',mscalib.CALIB_STR[0:-1],mscalib.CALIB_STR[0])
		m.menubutton.configure(width='9')
		m.frame.grid(row=0,column=1,sticky='e',padx=2,pady=2)
		m.menubutton.configure(state='disabled')
		self.rp_correction_menu=m

		l=Tkinter.Label(f.interior(),text='Slope:')
		l.grid(row=1,column=0,sticky='w')

		e=mstkutil.entry_field(f.interior(),'','0',15,None,'right')
		e.frame.grid(row=1,column=1,sticky='e',padx=2,pady=2)
		self.rp_correction_a_edit=e

		l=Tkinter.Label(f.interior(),text='Intercept:')
		l.grid(row=2,column=0,sticky='w')

		#e=mstkutil.entry_field(f.interior(),'','0',15,None,'right')
		e=mstkutil.num_entry_field(f.interior(),mstkutil.ENTRY_FP,'',1,fpconst.PosInf,'',15)
		self.entries.append(e)
		e.frame.grid(row=2,column=1,sticky='e',padx=2,pady=2)
		self.rp_correction_b_edit=e

		f.interior().columnconfigure(0, weight=1)
		f.interior().columnconfigure(1, weight=1)

		#
		f=Pmw.Group(tab,tag_text='')
		f.pack(side='top',anchor='nw',fill='x',padx=0,pady=0)
		f.component('ring').configure(relief='flat')

		b=Tkinter.Button(f.interior(),text="Calibration plots",width=15,borderwidth=1,command=self.plot_calibration_cb)
		b.grid(row=0,column=0,sticky='e',padx=2,pady=2)
		
		f.interior().columnconfigure(0, weight=1)
		#f.interior().columnconfigure(1, weight=1)

		#---------------------------------------------------------------------
		#
		tab=tab_panel.add(' Processing ')

		t=Pmw.ScrolledText(tab,text_wrap='none')
		t.pack(side='top',anchor='nw',fill='both',expand=1,padx=0,pady=0)
		self.processing_edit=t

		#---------------------------------------------------------------------
		#
		tab_panel.setnaturalsize(pageNames=(' Spectrum ',' Calibration ',' Peaks '))

		#---------------------------------------------------------------------
		#
		
		f=Tkinter.Frame(self.top)
		f.pack(side='top',anchor='w',fill='x')
		
		b=Tkinter.Button(f,text="Reset settings",width=15,borderwidth=1,command=self.reset_calibration_cb)
		b.grid(row=0,column=0,sticky='w',padx=6,pady=2)
		
		br=mstkutil.button_row(f,
				('Apply', self.set_spectrum_properties),
				('Close', self.close_cb))
		br.frame.grid(row=0,column=1,sticky='e',padx=6,pady=2)

		f.columnconfigure(0, weight=1)
		f.columnconfigure(1, weight=1)
		
		#
		self.removed_notice=session.notify_me('removed spectrum from project',self.update_spectrum_menu)
		self.added_notice=session.notify_me('added spectrum to project',self.update_spectrum_menu)
		self.top.bind('<Destroy>',self.remove_callbacks,1)

		mstkutil.Stoppable.__init__(self,progress_label,None)

#		self.top.geometry("+0+0")
		self.top.resizable(0,0)

	# ---------------------------------------------------------------------------
	#
	def select_spectrum(self,event):
		self.show_spectrum_properties(self.spectrum_menu.spectrum())

	# ---------------------------------------------------------------------------
	#
	def update_spectrum_menu(self,removed_spectrum):
		spectra=self.session.project.spectrum_list()
		if spectra==None or len(spectra)==0:
			self.top.destroy()
		else:
			if self.spectrum!=removed_spectrum:
				spectrum=self.spectrum
			else:
				spectrum=spectra[0]

			self.spectrum_menu.update_menu_cb()
			self.show_spectrum_properties(spectrum)

	# ---------------------------------------------------------------------------
	#
	def remove_callbacks(self,event):
		if str(self.top)==str(event.widget):
			if sparky.object_exists(self.session):
				if self.removed_notice!=None:
					self.session.dont_notify_me(self.removed_notice)
				if self.added_notice!=None:
					self.session.dont_notify_me(self.added_notice)
				
			self.removed_notice=None
			self.added_notice=None

	# ---------------------------------------------------------------------------
	#
	def domain_cb(self,event):
	
		if self.domain_menu.get()=='m/z':
			state='disabled'
		else:
			state='normal'

		self.para_edit.entry.configure(state=state)
		self.parb_edit.entry.configure(state=state)

	# ---------------------------------------------------------------------------
	#
	def analyzer_cb(self,event):

		ms_analyzer=mssputil.get_anlyzer_type(self.analyzer_menu.get())

		self.mz_precision_edit.variable.set(mssputil.get_def_mz_precision(ms_analyzer))
		rp_coeffs=mssputil.get_def_rp_coeffs(ms_analyzer)
		self.rp_correction_a_edit.variable.set(rp_coeffs[0])
		self.rp_correction_b_edit.variable.set(rp_coeffs[1])

	# ---------------------------------------------------------------------------
	#
	def show_spectrum_properties(self,spectrum):

		if spectrum!=None:
			self.spectrum=spectrum

			#spectrum
			self.spectrum_menu.set(spectrum.name)

			self.instrument_edit.variable.set(spectrum.ms_instrument)

			self.analyzer_menu.set(mssputil.MSANALYZER_STR[spectrum.ms_analyzer+1])

			mz_range=mssputil.get_mz_range(spectrum,native=False)
			self.mz_range_edit.variable.set('%.3f - %.3f' % (mz_range[0],mz_range[1]))
			if self.spectrum.domain==0:
				self.mz_step_edit.variable.set(mssputil.get_mz_step(spectrum))
			else:
				self.mz_step_edit.variable.set(mssputil.get_f_step(spectrum))

			rt_range=mssputil.get_rt_range(spectrum,native=False)
			self.rt_range_edit.variable.set('%.3f - %.3f' % (rt_range[0],rt_range[1]))
			self.rt_step_edit.variable.set(mssputil.get_rt_step(spectrum))

			if self.spectrum.domain==0:
				self.domain_menu.set('m/z')
				state='disabled'
			else:
				self.domain_menu.set('frequency')
				state='normal'

			self.para_edit.variable.set(spectrum.para)
			self.para_edit.entry.configure(state=state)
			self.parb_edit.variable.set(spectrum.parb)
			self.parb_edit.entry.configure(state=state)

			self.total_counts_edit.variable.set(spectrum.total_counts)
			self.mean_counts_edit.variable.set(spectrum.mean_counts)

			#peaks
			self.update_peaks_count()
			
			#calibration
			self.mz_correction_menu.set(mscalib.CALIB_STR[spectrum.mz_correction_method+1])
			self.mz_correction_a_edit.variable.set(spectrum.mz_correction_a)
			self.mz_correction_b_edit.variable.set(spectrum.mz_correction_b)
			self.mz_precision_edit.variable.set(mssputil.get_mz_precision(spectrum,True))

			self.rt_correction_menu.set(mscalib.CALIB_STR[spectrum.rt_correction_method+1])
			self.rt_correction_a_edit.variable.set(spectrum.rt_correction_a)
			self.rt_correction_b_edit.variable.set(spectrum.rt_correction_b)
			if spectrum.rt_precision!=0:
				self.rt_precision_edit.variable.set(spectrum.rt_precision)
			else:
				self.rt_precision_edit.variable.set(20)

			if spectrum.calibrated:
				self.rp_correction_menu.set(mscalib.CALIB_STR[mscalib.CALIB_RLS+1])
			else:
				self.rp_correction_menu.set(mscalib.CALIB_STR[mscalib.CALIB_NONE+1])
			rp_coeffs=mssputil.get_rp_coeffs(spectrum,True)
			self.rp_correction_a_edit.variable.set(rp_coeffs[0])
			self.rp_correction_b_edit.variable.set(rp_coeffs[1])

			#processing
			self.processing_edit.setvalue(spectrum.ms_processing)

	# ---------------------------------------------------------------------------
	#
	def set_spectrum_properties(self):

		if not self.check_entries():
			return False
		
		self.spectrum.ms_instrument=self.instrument_edit.variable.get()
		self.spectrum.ms_analyzer=mssputil.get_anlyzer_type(self.analyzer_menu.get())

		if self.domain_menu.get()=='m/z':
			self.spectrum.domain=0
		else:
			self.spectrum.domain=1

		if self.spectrum.domain==1:
			self.spectrum.para=float(self.para_edit.variable.get())
			self.spectrum.parb=float(self.parb_edit.variable.get())

		self.spectrum.total_counts=float(self.total_counts_edit.variable.get())
		self.spectrum.mean_counts=float(self.mean_counts_edit.variable.get())

		self.spectrum.mz_precision=float(self.mz_precision_edit.variable.get())/2
		self.spectrum.rt_precision=float(self.rt_precision_edit.variable.get())

		self.spectrum.rp_a=float(self.rp_correction_a_edit.variable.get())
		self.spectrum.rp_b=float(self.rp_correction_b_edit.variable.get())

		self.spectrum.ms_processing=self.processing_edit.getvalue().strip()

		self.close_cb()
		
	# ---------------------------------------------------------------------------
	#
	def reset_calibration_cb(self):

		dialog=mstkutil.question_dialog(self.top,"Are you sure you want to reset the calibration parameters?")
		if dialog.result==1:
			self.spectrum.ms_analyzer=mssputil.get_anlyzer_type(self.analyzer_menu.get())

			self.spectrum.reset_calibration()
			self.spectrum.mz_precision=mssputil.get_mz_precision(self.spectrum,True)/2
			self.spectrum.rt_precision=20
			rp_coeffs=mssputil.get_rp_coeffs(self.spectrum,True)
			self.spectrum.rp_a=rp_coeffs[0]
			self.spectrum.rp_b=rp_coeffs[1]
			self.reset_processing_cb()
			
			self.show_spectrum_properties(self.spectrum)

	# ---------------------------------------------------------------------------
	#
	def reset_processing_cb(self):
		processing=self.processing_edit.getvalue().strip()
		if processing!='':
			dialog=mstkutil.question_dialog(self.top,'Are you sure you want to reset the processing settings?')
			if dialog.result==1:
				self.spectrum.ms_processing=''
				self.processing_edit.setvalue(self.spectrum.ms_processing)

	# ---------------------------------------------------------------------------
	#
	def plot_calibration_cb(self):
		fig=mscalib.mz_calibration_params_plot(self.spectrum)
		d=mstkutil.PlotDialog(self.session,fig,('m/z calibration: ' + self.spectrum.name))
		d.show_window(1)		
		
		fig=mscalib.rt_calibration_params_plot(self.spectrum)
		d=mstkutil.PlotDialog(self.session,fig,('RT calibration: ' + self.spectrum.name))
		d.show_window(1)
		
		fig=mscalib.rp_calibration_params_plot(self.spectrum)
		d=mstkutil.PlotDialog(self.session,fig,('RP calibration: ' + self.spectrum.name))
		d.show_window(1)

	# ---------------------------------------------------------------------------
	#
	def update_peaks_count(self):
		self.peaks_count_edit.variable.set(0)		
		if self.spectrum!=None:
			peaks=self.spectrum.peak_list()
			if peaks!=None:
				self.peaks_count_edit.variable.set(len(peaks))
				
	# ---------------------------------------------------------------------------
	#
	def label_format_cb(self):
		peaks=self.spectrum.peak_list()
		
		if peaks!=None and len(peaks)>0:
			self.progress_report('Setting labels format')
			self.progress_meter.configure(indicatorformat='%(progress)s peaks')
			self.progress_meter.configure(finishvalue=float(len(peaks)))
			self.lock_gui(1)
	
			only_present=self.label_present_check.state()
			show_protein=self.label_protein_check.state()
			show_charge=self.label_charge_check.state()
			show_rt=self.label_rt_check.state()
					
			self.stoppable_loop(None,0)
			for i in range(0,len(peaks)):
				peak=peaks[i]
				self.progress_meter.updateProgress(i)
				
				if mssputil.is_peak_unassigned(peak)!=1:
					query=mssputil.get_query(peak)
					#query.set_present(mssputil.is_peak_present(peak))
					
					if only_present==0 or (only_present==1 and query.present==1):
						label=mssputil.get_peak_label(peak,show_protein,show_charge,show_rt)
					else:
						label=''
					
					if label!=peak.label.text:
						peak.show_label(label)
			
			self.progress_report('')
			self.progress_meter.updateProgress(0)
			self.lock_gui(0)		

	# ---------------------------------------------------------------------------
	#
	def read_cb(self):
		self.lock_gui(1)
		
		path=self.peak_list_path.get()
		if self.spectrum and path:
			mssputil.read_peaks(path,self.spectrum,self,self.progress_meter)

		self.update_peaks_count()
		self.lock_gui(0)			

	# ---------------------------------------------------------------------------
	#
	def remove_cb(self):
		self.lock_gui(1)		
		
		mssputil.remove_peaks_from_spectrum(self.spectrum,False,self,self.progress_meter)
		self.update_peaks_count()
		
		self.lock_gui(0)
		
	# ---------------------------------------------------------------------------
	#
	def lock_gui(self,lock):

		pass
		if lock==1:
			state='disabled'
			if sys.platform=='win32':
				cursor='wait'
			else:
				cursor='watch'
		else:
			state='normal'
			cursor=''

		self.label_present_check.button['state']=state
		self.label_protein_check.button['state']=state
		self.label_rt_check.button['state']=state
		self.label_charge_check.button['state']=state
		self.create_button['state']=state
		self.format_button['state']=state
		self.remove_button['state']=state
		self.peaks_count_edit.entry['state']=state		
		self.peak_list_path.entry.entry['state']=state
		self.peak_list_path.button['state']=state
		
		self.top.config(cursor=cursor)
		self.top.update_idletasks()
		
# -----------------------------------------------------------------------------
#
def show_spectrum_properties(session):

	spectrum=session.selected_spectrum()
	if spectrum==None or not hasattr(spectrum,'is_ms'):
		return

	d=mssputil.the_dialog(spectrum_properties_dialog,session)
	d.show_spectrum_properties(spectrum)
	d.show_window(1)



