import pyutil
import re
import string
import math
import Tkinter

import sparky
import fpconst
import numpy
import odict

import ms
import msfit
import msmath
import mscalib
import mstkutil

#!!!!!!!wywalic gdzie indziej

# -----------------------------------------------------------------------------
#
def check_ext(filename,ext):

	return(filename.upper().endswith(('.' + ext.upper())))

# -----------------------------------------------------------------------------
#
def remove_ext(filename,ext):

	if filename.upper().endswith(('.' + ext.upper())):
		index=filename.upper().find(('.' + ext.upper()))
		filename=filename[0:index]
	
	return(filename)

# --------------------------------------------------------------------------
#
P=1.007276
DOMAIN_MASS=0
DOMAIN_FREQ=1

MSANALYZER_STR=('UNKNOWN','Q-TOF','FT-ICR','ORBITRAP-VELOS','ORBITRAP-VELOS-MS/MS','ORBITRAP-ELITE','ORBITRAP-ELITE-MS/MS','ORBITRAP-QEXACTIVE','ORBITRAP-QEXACTIVE-MS/MS','ORBITRAP-EXPLORIS','ORBITRAP-EXPLORIS-MS/MS')
MSANALYZER_NONE=-1
MSANALYZER_QTOF=0
MSANALYZER_FTICR=1
MSANALYZER_ORBITRAP=2
MSANALYZER_ORBITRAP_MSMS=3
MSANALYZER_ORBITRAP_ELITE=4
MSANALYZER_ORBITRAP_ELITE_MSMS=5
MSANALYZER_ORBITRAP_QEXACTIVE=6
MSANALYZER_ORBITRAP_QEXACTIVE_MSMS=7
MSANALYZER_ORBITRAP_EXPLORIS=8
MSANALYZER_ORBITRAP_EXPLORIS_MSMS=9

# --------------------------------------------------------------------------
#
def get_anlyzer_type(name):
	anlyzer_type=-1
	for i in range(0,len(MSANALYZER_STR)):
		if name.lower()==MSANALYZER_STR[i].lower():
			anlyzer_type=i-1
			break
	return anlyzer_type

# Zamina Da na ppm
# --------------------------------------------------------------------------
#
def da2ppm(delta_m,m):
	return (1000000*delta_m/m)

# Zamina ppm na Da
# --------------------------------------------------------------------------
#
def ppm2da(delta_m_ppm,m):
	return (delta_m_ppm*m/1000000)

# Zamina masy na m/z
# --------------------------------------------------------------------------
#
def m2mz(mass,charge):

	return ((mass+float(charge)*P)/float(charge))

# Zamina listy masy na liste m/z
# ----------------------------------------------------------------------------
#
def m2mz_list(mass_list,charge):

	points_count=len(mass_list)
	mz=numpy.zeros(points_count)
	i=0
	for mass in mass_list:
		mz[i]=(mass+float(charge)*P)/float(charge)
		i=i+1

	return(mz)

# Zamina m/z na mase
# ----------------------------------------------------------------------------
#
def mz2m(mz,charge):

	return (charge*(mz-P))

# Zamina listy masy na liste m/z
# ----------------------------------------------------------------------------
#
def mz2m_list(mz_list,charge):

	points_count=len(mz_list)
	mass=numpy.zeros(points_count)
	i=0
	for mz in mz_list:
		mass[i]=(charge*(mz-P))
		i=i+1

	return(mass)

# Zamina m/z na f
# ----------------------------------------------------------------------------
#
def mz2freq(mz,a,b):
	
	f=(a+math.sqrt(a*a+4*mz*b))/(2*mz)

	return(f)

# Zamina m/z na f
# ----------------------------------------------------------------------------
#
def mz2freq_s(mz,spectrum):
	
	f=(spectrum.para+math.sqrt(spectrum.para*spectrum.para+4*mz*spectrum.parb))/(2*mz)

	return(f)

# Zamina listy m/z na liste f
# ----------------------------------------------------------------------------
#
def mz2freq_list(mz_list,spectrum):

	points_count=len(mz_list)
	f=numpy.zeros(points_count)
	i=0
	for mz in mz_list:
		f[i]=(spectrum.para+math.sqrt(spectrum.para*spectrum.para+4*mz*spectrum.parb))/(2*mz)
		i=i+1

	return(f)

# Zamina f na m/z
# ----------------------------------------------------------------------------
#
def freq2mz(f,a,b):
	mz=a/f+b/(f*f)

	return(mz)

# Zamina f na m/z
# ----------------------------------------------------------------------------
#
def freq2mz_s(f,spectrum):
	mz=spectrum.para/f+spectrum.parb/(f*f)

	return(mz)

# Zamina listy f na liste m/z
# ----------------------------------------------------------------------------
#
def freq2mz_list(f_list,spectrum):

	points_count=len(f_list)
	mz=numpy.zeros(points_count)
	i=0
	for f in f_list:
		mz[i]=spectrum.para/f+spectrum.parb/(f*f)
		i=i+1

	return(mz)


# Rozdzielczosc widma przy danym m/z
# ----------------------------------------------------------------------------
#
def get_rp_at_mz(spectrum,mz,use_calibration=1):

	rp_coeffs=get_rp_coeffs(spectrum,use_calibration)
	rp=rp_coeffs[0]*mz+rp_coeffs[1]

	return(rp)

# Wspolczynniki RP(m/z) - wynikajace z kalibracji lub domyslne
# ----------------------------------------------------------------------------
#
def get_rp_coeffs(spectrum,use_spectrum):

	if use_spectrum==1 and (spectrum.rp_a!=0 or spectrum.rp_b!=0):
		rp_coeffs=(spectrum.rp_a,spectrum.rp_b)
	else:
		rp_coeffs=get_def_rp_coeffs(spectrum.ms_analyzer)

	return(rp_coeffs)

# Domyslne wspolczynniki RP(m/z)
# ----------------------------------------------------------------------------
#
def get_def_rp_coeffs(ms_analyzer):

	if ms_analyzer==MSANALYZER_FTICR:
		rp_coeffs=(-20,41000)
	elif ms_analyzer==MSANALYZER_QTOF:
		rp_coeffs=(0,5000)
	elif ms_analyzer==MSANALYZER_ORBITRAP_MSMS:
		rp_coeffs=(-8,18000)
	elif ms_analyzer==MSANALYZER_ORBITRAP_ELITE:
		rp_coeffs=(-13.5,51000)
	elif ms_analyzer==MSANALYZER_ORBITRAP_ELITE_MSMS:
		rp_coeffs=(-16.5,33000)	#25500, -7.0 dla kratki 0.01
	elif ms_analyzer==MSANALYZER_ORBITRAP_QEXACTIVE:
		rp_coeffs=(-14,47500)
	elif ms_analyzer==MSANALYZER_ORBITRAP_QEXACTIVE_MSMS:
		rp_coeffs=(-12.5,47000)
	elif ms_analyzer==MSANALYZER_ORBITRAP_EXPLORIS:
		rp_coeffs=(-4.5,31000)
	elif ms_analyzer==MSANALYZER_ORBITRAP_EXPLORIS_MSMS:
		rp_coeffs=(-4.5,31000)
	else:
		rp_coeffs=(-10,30000)

	return(rp_coeffs)

# Precyzja widma - wynikajace z kalibracji lub domyslne
# ----------------------------------------------------------------------------
#
def get_mz_precision(spectrum,use_spectrum):

	if use_spectrum==1 and spectrum.mz_precision!=0:
		mz_precision=2*spectrum.mz_precision
	else:
		mz_precision=get_def_mz_precision(spectrum.ms_analyzer)

	return(mz_precision)

# Domyslna precyzja widma
# ----------------------------------------------------------------------------
#
def get_def_mz_precision(ms_analyzer):

	if ms_analyzer==MSANALYZER_QTOF:
		mz_precision=120
	else:
		mz_precision=70

	return(mz_precision)

# Poprawka masy widma przy danym m/z w ppm
# ----------------------------------------------------------------------------
#
def get_mz_correction_ppm(spectrum,mz):

	mz_correction=0.0
	if spectrum.calibrated==1:
		mz_correction=spectrum.mz_correction_a*mz+spectrum.mz_correction_b

	return(mz_correction)

# Poprawka masy widma przy danym m/z
# ----------------------------------------------------------------------------
#
def get_mz_correction(spectrum,mz):

	mz_correction=0.0
	if spectrum.calibrated==1:
		mz_correction=mz*get_mz_correction_ppm(spectrum,mz)/1e+6

	return(mz_correction)

# Poprawka czasu widma przy danym rt
# ----------------------------------------------------------------------------
#
def get_rt_correction(spectrum,rt):

	rt_correction=0.0
	if spectrum.calibrated==1:
		if spectrum.rt_correction_method!=mscalib.CALIB_LOWESS:
			rt_correction=spectrum.rt_correction_a*rt+spectrum.rt_correction_b
		else:
			if spectrum.rt_correction_sa!=None and spectrum.rt_correction_sf!=None:
				rt_correction=msmath.lowess_val(numpy.array(spectrum.rt_correction_sa),numpy.array(spectrum.rt_correction_sf),rt)
		
	return(rt_correction)

# Zakres widma w m/z
# ----------------------------------------------------------------------------
#
def get_mz_range(spectrum,native=False):

	mz_range=[spectrum.region[1][1],spectrum.region[0][1]]
	if spectrum.domain==DOMAIN_FREQ:
		mz_range=freq2mz_list(mz_range,spectrum)
	else:
		if native==False and mz_range[0]<0:
			mz_range[0]=-1*mz_range[0]
			mz_range[1]=-1*mz_range[1]
	return(mz_range)

# Zakres widma w f
# ----------------------------------------------------------------------------
#
def get_f_range(spectrum):

	return([spectrum.region[1][1],spectrum.region[0][1]])

# Zakres widma w RT
# ----------------------------------------------------------------------------
#
def get_rt_range(spectrum,native=False):
	
	rt_range=[spectrum.region[1][0],spectrum.region[0][0]]
	if native==False and rt_range[0]<0:
		rt_range[0]=-1*rt_range[0]
		rt_range[1]=-1*rt_range[1]

	return(rt_range)

# Kwant osi m/z
# ----------------------------------------------------------------------------
#
def get_mz_step(spectrum):
	mz_step=spectrum.spectrum_width[1]/spectrum.data_size[1]

	return(mz_step)

# Kwant osi f
# ----------------------------------------------------------------------------
#
def get_f_step(spectrum):
	f=spectrum.spectrum_width[1]/spectrum.data_size[1]

	return(f)

# Kwant osi RT
# ----------------------------------------------------------------------------
#
def get_rt_step(spectrum):
	rt_step=spectrum.spectrum_width[0]/spectrum.data_size[0]

	return(rt_step)

# Lista punktow f z zakresie f_range (jak f_range==None to pelen zakres widma)
# ----------------------------------------------------------------------------
#
def get_f_in_range(spectrum,f_range=None):

	if f_range==None:
		f_range=get_f_range(spectrum)
	f_step=get_f_step(spectrum)

	f=[]
	current_f=f_range[0]
	while current_f>=f_range[1]:
		f.append(current_f)
		current_f=current_f-f_step

	return(numpy.array(f))

# Lista punktow mz z zakresie mz_range (jak mz_range==None to pelen zakres widma)
# ----------------------------------------------------------------------------
#
def get_mz_in_range(spectrum,mz_range=None,native=False):

	if mz_range==None:
		mz_range=get_mz_range(spectrum,native=True)
	mz_step=get_mz_step(spectrum)

	if mz_range[0]>0:
		min_mz=-1*mz_range[0]
		max_mz=-1*mz_range[1]
	else:
		min_mz=mz_range[0]
		max_mz=mz_range[1]

	mz=[]
	current_mz=min_mz
	while current_mz>=max_mz:
		mz.append(current_mz)
		current_mz=current_mz-mz_step
	mz=numpy.array(mz)

	if native==False and mz[0]<0:
		mz=-1*mz

	return(mz)

# Lista punktow rt z zakresie rt_range (jak rt_range==None to pelen zakres widma)
# ----------------------------------------------------------------------------
#
def get_rt_in_range(spectrum,rt_range=None,native=False):

	if rt_range==None:
		rt_range=get_rt_range(spectrum,native=True)
	
	rt_step=get_rt_step(spectrum)
	
	if rt_range[0]>0:
		min_rt=-1*rt_range[0]
		max_rt=-1*rt_range[1]
	else:
		min_rt=rt_range[0]
		max_rt=rt_range[1]

	rt=[]
	current_rt=min_rt
	while current_rt>=max_rt:
		rt.append(current_rt)
		current_rt=current_rt-rt_step
	rt=numpy.array(rt)

	if native==False and rt[0]<0:
		rt=-1*rt
	
	return(rt)

# Pobranie danych dla czasu rt
# ----------------------------------------------------------------------------
#
def get_dataline(spectrum,mz_list,rt):

	points_count=len(mz_list)
	dataline=numpy.zeros(points_count)
	
	i=0
	for mz in mz_list:
		dataline[i]=spectrum.data_height((rt,mz))
		i=i+1

	return(dataline)

# Pobranie danych dla listy czasow rt_list
# ----------------------------------------------------------------------------
#
def get_datalines(spectrum,mz_list,rt_list):

	mz_points_count=len(mz_list)
	rt_points_count=len(rt_list)
	datalines=numpy.zeros((rt_points_count,mz_points_count))
	
	i=0
	for rt in rt_list:
		if rt>0:
			rt=-1*rt
		j=0
		for mz in mz_list:
			datalines[i,j]=spectrum.data_height((rt,mz))
			j=j+1
		i=i+1

	return(datalines)


# Pobranie przypisanych pikow
# -----------------------------------------------------------------------------
#
def get_assinged_peaks(spectrum):

	assigned_peaks=[]

	peaks=spectrum.peak_list()
	if peaks!=None and len(peaks)!=0:
		for peak in peaks:
			if is_peak_unassigned(peak)==0:
				assigned_peaks.append(peak)
	return assigned_peaks


# Pobranie peptydow
# -----------------------------------------------------------------------------
#
def get_peptides(spectrum,dialog=None,progress_meter=None,str=None,value=0):

	if dialog!=None:
		dialog.progress_report('Getting peptides')
		
	peaks=spectrum.peak_list()
	#peaks=get_assinged_peaks(spectrum)
	pep_dict=odict.OrderedDict()

	if dialog!=None:
		if progress_meter!=None:
			progress_meter.configure(indicatorformat='%(progress)s peaks [%(percentdone)s %%]',finishvalue=float(len(peaks)))
		dialog.stoppable_loop(str,value)

	query=None
	for peak in peaks:
		if dialog!=None:
			dialog.check_for_stop()
			if progress_meter!=None:
				progress_meter.updateProgress(progress_meter.progress+1)
				
		if is_peak_unassigned(peak)!=1:
			seq=get_sequence(peak)
			if pep_dict.has_key(seq):
				peptide=pep_dict[seq]
			else:
				peptide=ms.peptide(seq,peak)
				pep_dict[seq]=peptide

			query=ms.query(peak)
			peptide.add(query)
		else:
			if query!=None:
				query.add(peak)

	return(pep_dict.values())

# Pobranie peptydu
# -----------------------------------------------------------------------------
#
def get_peptide(peak):

	seq=get_sequence(peak)
	peptide=None
	query=None
	
	peaks=peak.spectrum.peak_list()
	if peaks!=None and len(peaks)!=0:
		for peak in peaks:
			if is_peak_unassigned(peak)==0 and get_sequence(peak)==seq:
				if peptide==None:
					peptide=ms.peptide(seq,peak)
				query=ms.query(peak)
				peptide.add(query)
			else:
				if query!=None:
					query.add(peak)
	return peptide

#Pobranie query
# --------------------------------------------------------------------------
#
def get_query(peak):
	q=None

	if is_peak_unassigned(peak)==0:
		spectrum=peak.spectrum
		peak_list=spectrum.peak_list()
		try:
			index=peak_list.index(peak)
			q=ms.query(peak)
			#q.add(peak)
			for i in range(index+1,len(peak_list)):
				peak=peak_list[i]
				if is_peak_unassigned(peak)==0:
					break
				else:
					q.add(peak)
		except ValueError:
			return q

	return q

# --------------------------------------------------------------------------
#
def shift_query(peak,rt):

	if peak!=None and peak.note!=None and peak.note!='':
		spectrum=peak.spectrum
		peak_list=spectrum.peak_list()
		if peak_list!=None and len(peak_list)!=0:
			peaks=[]
			peaks.append(peak)
			for i in range(0,len(peak_list)):
				if peak_list[i]==peaks[0]:
					for j in range(i+1,len(peak_list)):
						if peak_list[j].note=='':
							peaks.append(peak_list[j])
						else:
							break
					break

			for i in range(0,len(peaks)):
				peaks[i].set_position(rt,peaks[i].frequency[1])

#
#
#
#
# ----------------------------------------------------------------------------
#
def spectra_for_peaks(peaks):
	return pyutil.unique_attribute_values(peaks, 'spectrum')

# ----------------------------------------------------------------------------
#
def selected_condition(session):
	s = session.selected_spectrum()
	if s:
		return s.condition
	return None

# ----------------------------------------------------------------------------
# If an assignment along an axis has the same group as the previous axis
# then the group name is not shown.
#
def assignment_name(resonances):
  a = ''
  last_group = None
  for r in resonances:
    if r:
      if r.group == last_group:
	a = a + r.atom.name + '-'
      else:
	a = a + r.name + '-'
      last_group = r.group
    else:
      a = a + '?-'
  return a[:-4]


# --------------------------------------------------------------------------
#
def center_peptide(session):

	spectrum=session.selected_spectrum()
	peak_list=spectrum.selected_peaks()
	if peak_list!=None and len(peak_list)!=0 and peak_list[0].note!=None and peak_list[0].note!='':
		peaks=[]
		peaks.append(peak_list[0])

		peak_list=spectrum.peak_list()
		for i in range(0,len(peak_list)):
			if peak_list[i]==peaks[0]:
				for j in range(i+1,len(peak_list)):
					if peak_list[j].note=='':
						peaks.append(peak_list[j])
					else:
						break
				break

		is_locked=peaks[0].is_lock
		peaks[0].lock(0)
		peaks[0].center()
		peaks[0].lock(is_locked)	

		for i in range(1,len(peaks)):
			peaks[i].set_position(peaks[0].frequency[0],peaks[i].frequency[1])

# ----------------------------------------------------------------------------
#
def show_charge(peak,charge):
	mass=get_mass(peak)
	mz=m2mz(mass,charge)
	rt=peak.frequency[0]

	if peak.spectrum.domain==1:
		mz=mz2freq(mz,peak.spectrum.para,peak.spectrum.parb)

	show_spectrum_position(peak.spectrum,(rt,mz))

# ----------------------------------------------------------------------------
#
def show_charge(session,charge):
	spectrum=session.selected_spectrum()
	peak_list=spectrum.selected_peaks()
	if peak_list!=None and len(peak_list)!=0:
		peak=peak_list[0]
		mass=get_mass(peak)
		mz=m2mz(mass,charge)
		rt=peak.frequency[0]

		if spectrum.domain==1:
			mz=mz2freq(mz,spectrum.para,spectrum.parb)
		else:
			mz=-1*mz
		show_spectrum_position(spectrum,(rt,mz))

# ----------------------------------------------------------------------------
#
def integrate_query(session,iterations):
	spectrum=session.selected_spectrum()
	peak_list=spectrum.selected_peaks()
	if peak_list!=None and not is_peak_unassigned(peak_list[0]):
		selected_peak=peak_list[0]
		peak_list=spectrum.peak_list()
		try:
			index=peak_list.index(selected_peak)
			peaks=[]
			peaks.append(peak_list[index])
			for i in range(index+1,len(peak_list)):
				if is_peak_unassigned(peak_list[i])==1:
					peaks.append(peak_list[i])
				else:
					break

#			for i in range(iterations):
			for j in range(len(peaks)):
				peaks[j].mfit(session.selected_view(),iterations)
		except ValueError:
			return
		
# ----------------------------------------------------------------------------
#
def is_peak_integrated(peak):
	if hasattr(peak,'is_ms'):
		return not (peak.volume==None or peak.fit_height==None or peak.fit_residual==None or peak.fit_height<0 or fpconst.isNaN(peak.fit_residual))
	else:
		return not (peak.volume==None or peak.fit_height==None or peak.fit_height==1000000 or peak.fit_height<0)

# ----------------------------------------------------------------------------
#
def is_peak_present(peak):
	present=0

	if peak.note!=None:
		index=peak.note.find('found=')
		if index!=-1:
			present=int(peak.note[index+6])

	return present

# ----------------------------------------------------------------------------
#
def set_peak_present(peak,present):
	if peak.note!=None:
		index=peak.note.find('found=')
		if index!=-1:
			peak.note=peak.note[0:index+6]+('%d' % present)
		else:
			peak.note='%s_found=%d' % (peak.note.strip(),present)
	else:
		peak.note='found='+('%d' % present)

# ----------------------------------------------------------------------------
#
def set_peak_color(peak,color):
	peak.color=color
	if peak.label!=None:
		peak.label.color=color

# ----------------------------------------------------------------------------
#
def get_peak_label(peak,show_protein,show_charge,show_rt):
	
	label=get_sequence(peak)
					
	if not (label==None):
		if show_protein:
			protein=get_protein(peak)+":"
			label=protein+label
		if show_charge:
			charge='(+%d)' % get_charge(peak)
			label=label+charge
		if show_rt:
			rt=';%3.2f' % get_massref_rt(peak)
			label=label+rt

	return label

#przerobic na inne wartosci
# ---------------------------------------------------------------------------
#
def get_peak_height(peak,data_height):
	height=0

	if data_height==1:
		height=peak.spectrum.data_height(peak.frequency)
	else:
		if is_peak_integrated(peak):
			if data_height==0:
				height=peak.fit_height
			else:
				height=peak.volume
	return height

# ---------------------------------------------------------------------------
#
def get_spectrum_isodistrib(peaks,data_height=0,relative=0):
	isodistrib=[]

	if peaks!=None:
		for peak in peaks:
			isodistrib.append(get_peak_height(peak,data_height))

		if relative==1:
			max_height=max(isodistrib)
			if max_height!=0:
				for i in range(0,len(isodistrib)):
					isodistrib[i]=isodistrib[i]/max_height

	return isodistrib		

# ---------------------------------------------------------------------------
#
def get_relative_isodistrib(isodistrib):
	max_height=max(isodistrib)
	if max_height!=0:
		for i in range(0,len(isodistrib)):
			isodistrib[i]=isodistrib[i]/max_height

	return isodistrib		

# ---------------------------------------------------------------------------
#
def get_o18_ratio(isodistrib,spectrum_isodistrib,o18=0):
	ratio=0
	
	ratio=spectrum_isodistrib[4]-spectrum_isodistrib[0]*isodistrib[4]+spectrum_isodistrib[2]*(1-isodistrib[2])-spectrum_isodistrib[0]*isodistrib[2]*(1-isodistrib[2])

	if o18==1:
		ratio=ratio/spectrum_isodistrib[0]
	else:
		ratio=spectrum_isodistrib[0]/ratio
	return(ratio)

# ---------------------------------------------------------------------------
#
def get_o18_value(isodistrib,spectrum_isodistrib):
	value=0
	
	v2=spectrum_isodistrib[2]-spectrum_isodistrib[0]*isodistrib[2]
	v4=spectrum_isodistrib[0]*isodistrib[4]

	if v2<0:
		value=-1*spectrum_isodistrib[4]
	else:
		value=spectrum_isodistrib[4]-v4-isodistrib[2]*v2+v2
	
	return(value)

# ---------------------------------------------------------------------------
#
def get_o16_o18_values(isodistrib,spectrum_isodistrib,check_peaks=1):
	values=[0,0,0]
	
	peaks_count=len(spectrum_isodistrib)
	if peaks_count==5:
		check_peaks=1

	is_o16=((check_peaks==1 and spectrum_isodistrib[0]!=0) or (check_peaks==2 and spectrum_isodistrib[0]!=0 and spectrum_isodistrib[1]!=0))
	is_o18=((check_peaks==1 and spectrum_isodistrib[4]!=0) or (check_peaks==2 and spectrum_isodistrib[4]!=0 and spectrum_isodistrib[5]!=0))
	if is_o16==1 and is_o18==1:
		v2=spectrum_isodistrib[2]-spectrum_isodistrib[0]*isodistrib[2]
		v4=spectrum_isodistrib[0]*isodistrib[4]

		values[0]=spectrum_isodistrib[0]
		values[1]=v2
		if v2<0:
			values[2]=-1*spectrum_isodistrib[4]
		else:
			values[2]=spectrum_isodistrib[4]-v4-isodistrib[2]*v2+v2

	elif is_o16==1 and is_o18==0:
		values[0]=spectrum_isodistrib[0]

	elif is_o16==0 and is_o18==1:
		values[1]=spectrum_isodistrib[2]
		values[2]=spectrum_isodistrib[4]+spectrum_isodistrib[2]

	return(values)

# ---------------------------------------------------------------------------
#
def get_peak_isodistrib(peak,threshold=0.05,relative_to_first=0):
	isodistrib=[]
	isodistrib_str=get_isodistrib(peak).split(',')

	if threshold<=1:
		for i in range(0,len(isodistrib_str)):
			if float(isodistrib_str[i])>=threshold:
				isodistrib.append(float(isodistrib_str[i]))
	else:
		peaks_count=threshold
		if relative_to_first==1:
			first=float(isodistrib_str[0])
		else:
			first==1.0

		for i in range(peaks_count):
			isodistrib.append(float(0))
		peaks_count=min(len(isodistrib_str),peaks_count)
		for i in range(0,peaks_count):
			isodistrib[i]=float(isodistrib_str[i])/first

	return isodistrib

# ----------------------------------------------------------------------------
#
def get_peaks_count(isodistrib):
	peaks_count=0

	for i in range(len(isodistrib)):
		if isodistrib[i]>0:
			peaks_count=peaks_count+1
	return peaks_count

# ----------------------------------------------------------------------------
#
def get_value(isodistrib,only_monoisotopic=1):
	value=0

	if only_monoisotopic==1:
		peaks_count=1
	else:
		peaks_count=len(isodistrib)

	for i in range(peaks_count):
		if isodistrib[i]>0:
			value=value+isodistrib[i]
		else:
			value=0
			break

	return value

# ---------------------------------------------------------------------------
#
def isodistrib_rms(isodistrib,spectrum_isodistrib):
	rms=0
	iso_height=0	

	peaks_count=min(len(isodistrib),len(spectrum_isodistrib))
	for i in range(0,peaks_count):
		iso_height=iso_height+isodistrib[i]
#		rms=rms+((isodistrib[i]-spectrum_isodistrib[i])*isodistrib[i])*((isodistrib[i]-spectrum_isodistrib[i])*isodistrib[i])
		rms=rms+((isodistrib[i]-spectrum_isodistrib[i])/isodistrib[i])*((isodistrib[i]-spectrum_isodistrib[i])/isodistrib[i])
#		rms=rms+(isodistrib[i]-spectrum_isodistrib[i])*(isodistrib[i]-spectrum_isodistrib[i])
	rms=math.sqrt(rms)/peaks_count
#	rms=math.sqrt(rms/(iso_height*peaks_count))
	return rms

#	rms=math.sqrt(rms/peaks_count)
#	iso_height=iso_height/peaks_count
#	return rms/iso_height

# --------------------------------------------------------------------------
#
def isodistrib_diff(isodistrib,data_isodistrib,only_monoisotopic=1):
	if only_monoisotopic:
		peaks_count=1
	else:
		peaks_count=len(isodistrib)

	fit_diff=0.0
	for i in range(peaks_count):
		if data_isodistrib[i]!=0:
			fit_diff=fit_diff+abs(isodistrib[i]-data_isodistrib[i])/data_isodistrib[i]
	fit_diff=fit_diff/peaks_count
			
	return fit_diff

# ---------------------------------------------------------------------------
#
def isodistrib_max(isodistrib):
	max_index=0
	max_value=isodistrib[0]
		
	for i in range(1,len(isodistrib)):
		if isodistrib[i]>max_value:
			max_value=isodistrib[i]
			max_index=i
		
	return max_index


# ----------------------------------------------------------------------------
#
def get_note_field(peak,field_name):
	field=None
	
	if peak.note!=None:
		index1=peak.note.find(field_name)
		if index1!=-1:
			index1=index1+len(field_name)
			index2=peak.note.find('_',index1)
			if index2==-1:
				field=peak.note[index1:]
			else:
				field=peak.note[index1:index2]	
	return field

# ----------------------------------------------------------------------------
#
def get_sequence(peak):
	sequence='N/A '
	note_str=None

#	lata na zamiane ; na _ przez ms2pipe
#	note_str=get_note_field(peak,'sequence=')

	if peak.note!=None:
		index1=peak.note.find('sequence=')
		if index1!=-1:
			index1=index1+len('sequence=')
			index2=peak.note.find('_cmass',index1)
			if index2==-1:
				note_str=peak.note[index1:]
			else:
				note_str=peak.note[index1:index2]	

	if note_str!=None:
#		sequence=note_str
		sequence=note_str.replace('_',';')
	else:
		if peak.label!=None:
			sequence=peak.label.text
			tokens=sequence.split(';')
			if len(tokens)>0:
				sequence=tokens[0]

	return sequence

# ----------------------------------------------------------------------------
#
def get_sequence_with_charge(peak,charge=0):
	sequence='N/A '
	note_str=None

#	lata na zamiane ; na _ przez ms2pipe
#	note_str=get_note_field(peak,'sequence=')

	if peak.note!=None:
		index1=peak.note.find('sequence=')
		if index1!=-1:
			index1=index1+len('sequence=')
			index2=peak.note.find('_cmass',index1)
			if index2==-1:
				note_str=peak.note[index1:]
			else:
				note_str=peak.note[index1:index2]	

#	note_str=get_note_field(peak,'sequence=')
	if note_str!=None:
#		sequence=note_str
		sequence=note_str.replace('_',';')
		if charge==1:
			sequence=sequence + (' (+%d)' % get_charge(peak))
	else:
		if peak.label!=None:
			sequence=peak.label.text
			tokens=sequence.split(';')
			if len(tokens)>0:
				sequence=tokens[0]
				if charge==1:
					sequence=sequence + (' (+%d)' % get_charge(peak))
	return sequence

# ----------------------------------------------------------------------------
#
def get_protein(peak):
	protein='N/A '

	note_str=get_note_field(peak,'proteins=')
	if note_str!=None:
		protein=note_str
	else:
		if peak.label!=None:
			tokens=peak.label.text.split(';')
			if len(tokens)==4:
				protein=tokens[1]

	return protein

# ----------------------------------------------------------------------------
#
def get_label_charge(label):
	charge=None

	index=label.find('(+')
	if index!=-1:
		charge=label[index+2:index+3]

	return charge

# ----------------------------------------------------------------------------
#
def get_note_charge(note):
	charge=None

	if note!=None:
		index1=note.find('charge=')
		if index1!=-1:
			index1=index1+len('charge=')
			index2=note.find('_',index1)
			if index2==-1:
				charge=note[index1:]
			else:
				charge=note[index1:index2]	

	return charge

# ----------------------------------------------------------------------------
#
def get_assignment_charge(peak):
	charge=0
	
	resonances=peak.resonances()
	if resonances:
		r=resonances[1]
		if r and r.name and r.name.find('(')==-1:
			charge=int(r.name[1:3])
			if charge>=10:
				charge=charge/10

	return charge

# ----------------------------------------------------------------------------
#
def get_charge(peak):
	charge=0

	note_str=get_note_field(peak,'charge=')
	if note_str!=None:
		charge=int(note_str)
	else:
		charge=get_assignment_charge(peak)

	return charge

# ----------------------------------------------------------------------------
#
def get_mz(peak):
	mz=0
	
	#fits=msfit.get_fits(peak)
	#if len(fits)>0:
	#	fit_nr=peak.best_fit
	#	mz=fits[peak.best_fit].mz_pos
	#else:						
	#	mz=peak.frequency[1]
	
	mz=peak.frequency[1]
	if mz<0:
		mz=-1*mz
	
	if hasattr(peak,'is_ms') and peak.spectrum.domain==1:
		mz=peak.spectrum.para/mz+peak.spectrum.parb/(mz*mz)

	return mz
	
# ----------------------------------------------------------------------------
#
def get_massref_mz(peak):
	massref_mz=0

	charge=get_charge(peak)
	if charge!=0:
		massref_mz=(get_massref_mass(peak)+charge*P)/charge

	return massref_mz

# ----------------------------------------------------------------------------
#
def get_dmz(peak):
	dmz=0

	if not is_peak_unassigned(peak):
		dmz=get_mz(peak)-get_massref_mz(peak)

	return dmz

# ----------------------------------------------------------------------------
#
def get_cmz(peak):
	cmz=0


	charge=get_charge(peak)
	if charge!=0:
		cmz=(get_cmass(peak)+charge*P)/charge

	return cmz

# ----------------------------------------------------------------------------
#
def get_mass(peak):

	return float(get_charge(peak))*(get_mz(peak)-P)

# ----------------------------------------------------------------------------
#
def get_massref_mass(peak):
	massref_mass=0
	
	note_str=get_note_field(peak,'mmass=')
	if note_str!=None:
		massref_mass=float(note_str)
	
	return massref_mass

# ----------------------------------------------------------------------------
#
def get_dmass(peak):

	return get_mass(peak)-get_massref_mass(peak)

# ----------------------------------------------------------------------------
#
def get_cmass(peak):
	mass=0

	note_str=get_note_field(peak,'cmass=')
	if note_str!=None:
		mass=float(note_str)

	return mass

# ----------------------------------------------------------------------------
#
def get_emass(peak):
	mass=0

	note_str=get_note_field(peak,'emass=')
	if note_str!=None:
		mass=float(note_str)

	return mass

# ----------------------------------------------------------------------------
#
def get_rt(peak):
	
	rt=peak.frequency[0]
	if rt<0:
		rt=-1*rt
	
	return rt

# ----------------------------------------------------------------------------
#
def get_massref_rt(peak):
	rt=0

	note_str=get_note_field(peak,'mrt=')
	if note_str!=None:
		rt=float(note_str)
	else:
		if peak.label!=None:
			tokens=peak.label.text.split(';')
			if len(tokens)==3:
				rt=float(tokens[1])
			if len(tokens)==4:
				rt=float(tokens[2])
	return rt

# ----------------------------------------------------------------------------
#
def get_drt(peak):
	drt=0

	if not is_peak_unassigned(peak):
		drt=get_rt(peak)-get_massref_rt(peak)

	return drt

# ----------------------------------------------------------------------------
#
def get_score(peak):
	score=' 0'

	note_str=get_note_field(peak,'score=')
	if note_str!=None:
		score=note_str

	return score

# ----------------------------------------------------------------------------
#
def get_isodistrib(peak):
	isodistrib='1 '

	note_str=get_note_field(peak,'isodistrib=')
	if note_str!=None:
		isodistrib=note_str

	return isodistrib

# ----------------------------------------------------------------------------
#
def get_isomass(peak):
	isomass='1 '

	note_str=get_note_field(peak,'isomass=')
	if note_str!=None:
		isomass=note_str

	return isomass

# ----------------------------------------------------------------------------
#
def get_fit_residual(peak):
	fit_residual=' N/A'

	if hasattr(peak,'is_ms'):
		if peak.fit_residual==None:
			fit_residual=' N/A'
		else:
			if fpconst.isNaN(peak.fit_residual):
				fit_residual=' NaN'			
			else:
				fr=100*peak.fit_residual
				fit_residual=' %8.1f' % fr
	return fit_residual

# ----------------------------------------------------------------------------
#
def get_float_fit_residual(peak):
	fit_residual=-2.0

	if hasattr(peak,'is_ms'):
		if peak.fit_residual==None:
			fit_residual=-2.0
		else:
			if fpconst.isNaN(peak.fit_residual):
				fit_residual=-1.0			
			else:
				fit_residual=100*peak.fit_residual

	return fit_residual

# -----------------------------------------------------------------------------
# Are all axes unassigned?
#
def is_peak_unassigned(peak):
	for r in peak.resonances():
		if r != None:
			return 0
	return 1

# -----------------------------------------------------------------------------
#
def unassign_spectrum(spectrum):
  for p in spectrum.peak_list():
    rlist = p.resonances()
    for axis in range(spectrum.dimension):
      if rlist[axis]:
        p.assign(axis, '', '')

# -----------------------------------------------------------------------------
#
def assignment_range(assignment):

  gmin = None
  gmax = None
  for r in assignment:
    g = r.group.number
    if g != None:
      if gmin == None or g < gmin: gmin = g
      if gmax == None or g > gmax: gmax = g

  if gmin == None:
    return None

  return gmax - gmin

# -----------------------------------------------------------------------------
#
def molecule_group_atom_list(molecule):
  group_atom_list = []
  for a in molecule.atom_list():
    group_atom_list.append((a.group.name, a.name))
  return group_atom_list

# -----------------------------------------------------------------------------
#
def resonance_group_atom(r):
  return (r.group.name, r.atom.name)

# -----------------------------------------------------------------------------
#
def group_atom_assignment(res):
  return tuple(map(resonance_group_atom, res))

# -----------------------------------------------------------------------------
# Assignment is tuple of (group name, atom name) pairs
#
def group_atom_assignment_name(assignment):
  text = ''
  for group, atom in assignment:
    text = text + group + atom + '-'
  return text[:-1]

# -----------------------------------------------------------------------------
# Assignment is tuple of (group name, atom name) pairs
#
def group_atom_resonances(condition, assignment):
  molecule = condition.molecule
  res = []
  for group_atom in assignment:
    r = group_atom_resonance(group_atom, condition)
    if r == None:
      return None
    res.append(r)
  return res

# -----------------------------------------------------------------------------
#
def group_atom_resonance(group_atom, condition):
  return apply(condition.find_resonance, group_atom)

# -----------------------------------------------------------------------------
#
def number_group_atom(ga):
  return (group_number(ga[0]),) + ga

# -----------------------------------------------------------------------------
#
def group_number(g):
  if g:
    s = re.search('[0-9]+', g)
    if s:
      return string.atoi(s.group())
  return None

# -----------------------------------------------------------------------------
#
def compare_groups(g1, g2):
  return cmp(g1.number, g2.number) or cmp(g1.name, g2.name)

# -----------------------------------------------------------------------------
#
def compare_atoms(a1, a2):
  return compare_groups(a1.group, a2.group) or cmp(a1.name, a2.name)

# -----------------------------------------------------------------------------
#
def compare_resonances(r1, r2):
  return compare_groups(r1.group, r2.group) or cmp(r1.atom.name, r2.atom.name)

# -----------------------------------------------------------------------------
#
def sort_peaks_by_assignment(peaks, stoppable):
  return mstkutil.stoppable_sort_by_key(peaks, assignment_sort_key, stoppable)

# -----------------------------------------------------------------------------
# Return assignment as ((residue #, residue symbol, atom name), ...)
# for sorting.
#
def assignment_sort_key(peak):
  a = []
  for r in peak.resonances():
    if r == None:
      a.append(None)
    else:
      a.append((r.group.number, r.group.symbol, r.atom.name))
  return tuple(a)

# -----------------------------------------------------------------------------
#
def parse_group_name(g):
  s = re.search('[0-9]+', g)
  if s:
    return g[:s.start()], string.atoi(s.group())
  return g, None

# ---------------------------------------------------------------------------
# Parse assignment of form g1a1-g2a2 to get ((g1, a1), (g2, a2))
# Or g1a1-a2 to get ((g1, a1), (g1, a2))
# A '?' component is translated to ('', '')
#
def parse_assignment(assignment):

  res = string.split(assignment, '-')
  assignment = []
  last_group = None
  for s in res:
    ga = split_group_atom(s)
    if ga == None:
      if last_group:
	ga = (last_group, s)
      else:
	return None
    assignment.append(ga)
    last_group = ga[0]

  return tuple(assignment)

# -----------------------------------------------------------------------------
# Skip to first digit, then skip to first H|C|N to find start of atom name.
#
def split_group_atom(group_atom):

  s = re.search('[0-9]', group_atom)
  if s:
    first_digit = s.start()
    s = re.search('[hHcCnNqQmM]', group_atom[first_digit:])
    if s:
      HCNQM_offset = s.start()
      d = first_digit + HCNQM_offset
      return (group_atom[:d], group_atom[d:])
  if group_atom == '?':
    return ('', '')
  return None

# ----------------------------------------------------------------------------
#
def is_complete_assignment(resonances):
  for r in resonances:
    if r == None:
      return 0
  return 1

# -----------------------------------------------------------------------------
#
def selected_peak(session):
  
  peaks = session.selected_peaks()
  if len(peaks) == 0:
    session.stdout.write('selected_peak(): 0 peaks selected.\n')
  elif len(peaks) > 1:
    session.stdout.write('selected_peak(): 2 or more peaks selected.\n')
  else:
    return peaks[0]
  return None

# ---------------------------------------------------------------------------
#
#def find_spectrum(session,path):
#
#	spectrum=None
#	spectra=session.project.spectrum_list()
#	for sp in spectra:
#		if sp.save_path==path:
#			spectrum=sp
#			break;
#
#	return spectrum

# ----------------------------------------------------------------------------
#
def reset_spectrum_calibration(spectrum):
	spectrum.calibrated=0
	spectrum.rp_a=0
	spectrum.rp_b
	spectrum.mz_correction_method=CALIB_NONE
	spectrum.mz_correction_a=0
	spectrum.mz_correction_b=0
	#uzaleznic od rodzaju spektrometru
	spectrum.mz_precision=0
	spectrum.rt_correction_method=CALIB_NONE
	spectrum.rt_correction_a=0
	spectrum.rt_correction_b=0
	spectrum.rt_precision=0
	#prawidlowo czyscic lowess
	spectrum.rt_correction_sa=None
	spectrum.rt_correction_sf=None

# ----------------------------------------------------------------------------
#
def name_to_spectrum(name,session):

	slist = session.project.spectrum_list()
	for s in slist:
		if s.name == name:
			return s
	return None

# ----------------------------------------------------------------------------
#
def save_path_to_spectrum(path,session):

	slist=session.project.spectrum_list()
	for s in slist:
		if s.save_path==path:
			return s
	return None

# -----------------------------------------------------------------------------
#
def show_spectrum(spectrum):
	v=preferred_view(spectrum)
	if v:
		tk=v.session.tk
		mstkutil.raise_named_widget(tk,v.frame)

# -----------------------------------------------------------------------------
#
def show_spectrum_position(spectrum,pos):
	v=preferred_view(spectrum)
	if v:
		v.center=pos
		v.set_crosshair_position(pos)
		tk=v.session.tk
		mstkutil.raise_named_widget(tk,v.frame)

# -----------------------------------------------------------------------------
#
#def create_peaks_for_spectrum(spectrum,unassigned_only=False,parent_dialog=None):

# -----------------------------------------------------------------------------
#
def remove_peaks_from_spectrum(spectrum,unassigned_only=False,parent_dialog=None,progress_meter=None):

	peaks=spectrum.peak_list()

	if peaks!=None and len(peaks)>0:
		if parent_dialog!=None:
			if unassigned_only==True:
				parent_dialog.progress_report('Removing unassigned peaks')
			else:
				parent_dialog.progress_report('Removing peaks')
			parent_dialog.stoppable_loop(None,0)
		if progress_meter!=None:
			progress_meter.configure(indicatorformat='%(progress)s peaks')
			progress_meter.configure(finishvalue=float(len(peaks)))
						
		for i in range(0,len(peaks)):
			peak=peaks[i]
			if parent_dialog!=None:
				parent_dialog.check_for_stop()
			if progress_meter!=None:
				progress_meter.updateProgress(i)
				
			if unassigned_only==False or (unassigned_only==True and is_peak_unassigned(peak)==1):
				spectrum.remove_peak(peak)
	
		if parent_dialog!=None:
			parent_dialog.progress_report('')
		if progress_meter!=None:
			progress_meter.updateProgress(0)

# ---------------------------------------------------------------------------
#
def read_peaks(path,spectrum,parent_dialog=None,progress_meter=None):

	input=open(path,'r')
	lines=input.readlines()
	input.close()

	if parent_dialog!=None:
		parent_dialog.progress_report('Creating peaks')
		parent_dialog.stoppable_loop(None,0)
	if progress_meter!=None:
		progress_meter.configure(indicatorformat='%(progress)s peaks')
		progress_meter.configure(finishvalue=float(len(lines)))

	#to musi byc inteligentniejsze
	remove_peaks_from_spectrum(spectrum,False)

	count=0
	for i in range(0,len(lines)):
		if parent_dialog!=None:
			parent_dialog.check_for_stop()
		if progress_meter!=None:
			progress_meter.updateProgress(i)
		
		count=count+create_peak(lines[i],spectrum)

	if parent_dialog!=None:
		parent_dialog.progress_report('')
	if progress_meter!=None:
		progress_meter.updateProgress(0)
		
		#msg=('%d peaks created\n' % count + '%d unreadable lines shown below' % (len(lines) - count))
		#self.unreadable.heading['text']=msg

# ---------------------------------------------------------------------------
#
def create_peak(line,spectrum):

	pinfo=parse_peak_line(line,spectrum.dimension,spectrum)
	if pinfo:
		assignment,frequency,note,label=pinfo
		peak=spectrum.place_peak(frequency)
		move_peak_onto_spectrum(peak)
			
		assigned = 0
		for a in range(spectrum.dimension):
			group_name,atom_name=assignment[a]
			if group_name or atom_name:
				peak.assign(a,group_name,atom_name)
				assigned=1

		if assigned:
			peak.show_label(label)

		if note:
			peak.note = note
		peak.selected = 1
			
		set_peak_color(peak,'red')
		return 1
	return 0

# ---------------------------------------------------------------------------
# If the peak is off the edge of the spectrum, fold it onto the spectrum
# and set the alias to keep its frequency the same.
#
def move_peak_onto_spectrum(peak):

	freq = peak.frequency
	pos = alias_onto_spectrum(freq, peak.spectrum)
	if pos != freq:
		peak.position = pos
		peak.alias = pyutil.subtract_tuples(freq, pos)

# ---------------------------------------------------------------------------
#
def parse_peak_line(line,dim,spectrum):

	fields=string.split(line,None,dim+1)
	if len(fields)<dim+1:
		return None

	frequency=[]
	for a in range(dim):
		f=pyutil.string_to_float(fields[a+1])
		if f==None:
			#self.unreadable.listbox.insert('end', line)
			return None
		else:
			if a==1 and hasattr(spectrum,'is_ms') and spectrum.domain==1:
				f=mz2freq(-1*f,spectrum.para,spectrum.parb)
		frequency.append(f)

	label=fields[0]
	note=''
	if len(fields)>dim+1:
		note=fields[dim+1]

	charge=get_note_charge(note)
	if charge==None:
		charge=get_label_charge(label)

	if charge!=None:
		assignment=[('M','1000'),('C',charge)]
	else:
		assignment=parse_assignment(fields[0])

	if assignment==None or len(assignment)!=dim:
		#self.unreadable.listbox.insert('end', line)
		return None

	return (assignment,frequency,note,label)

# -----------------------------------------------------------------------------
#
def select_peak(peak):
	if peak:
		peak.spectrum.session.unselect_all_ornaments()
		peak.selected=1

# -----------------------------------------------------------------------------
#
def show_peak(peak):
	if peak:
		show_spectrum_position(peak.spectrum,peak.position)

# -----------------------------------------------------------------------------
#
def peak_height(peak):

  h = peak.data_height
  if h == None:
    # peak group
    h = 0
    for p in peak.peaklets():
      h += p.data_height

  return h

# -----------------------------------------------------------------------------
#
def select_assignment(assignment, spectrum):

  if assignment and spectrum:
    select_peak(spectrum.find_peak(assignment))

# -----------------------------------------------------------------------------
#
def show_assignment(assignment, spectrum):

  if assignment and spectrum:
    peak = spectrum.find_peak(assignment)
    if peak:
      show_peak(peak)
    else:
      pos = assignment_position(assignment, spectrum)
      show_spectrum_position(spectrum, pos)

# -----------------------------------------------------------------------------
# Alias assignment frequency onto spectrum.
#
def assignment_position(assignment, spectrum):

  freq = pyutil.attribute_values(assignment, 'frequency')
  return alias_onto_spectrum(freq, spectrum)

# -----------------------------------------------------------------------------
# Alias frequency onto spectrum returning the position.
# If the frequency is on the spectrum then the returned position
# exactly equals the specified frequency.
#
def alias_onto_spectrum(freq, spectrum):
  
  region = spectrum.region
  pos = []
  for a in range(len(freq)):
    swidth = spectrum.sweep_width[a]
    max_ppm = region[1][a]
    min_ppm = max_ppm - swidth
    f = freq[a]
    if f >= min_ppm and f <= max_ppm:
      p = f
    else:
      p = min_ppm + (f - min_ppm) % swidth
    pos.append(p)
  return tuple(pos)

# -----------------------------------------------------------------------------
# Alias frequency onto spectrum.
# If the frequency is on the spectrum then the returned position
# exactly equals the specified frequency.
#
def alias_axis_onto_spectrum(freq, axis, spectrum):
  
  region = spectrum.region
  swidth = spectrum.sweep_width[axis]
  max_ppm = region[1][axis]
  min_ppm = max_ppm - swidth
  if freq >= min_ppm and freq <= max_ppm:
    return freq
  return min_ppm + (freq - min_ppm) % swidth

# -----------------------------------------------------------------------------
#
def closest_peak(freq, peak_list, ranges):

  min_offset = None
  closest = None
  for p in peak_list:
    peak_freq = p.frequency
    offset = 0
    for axis in range(len(freq)):
      offset = offset + abs(peak_freq[axis] - freq[axis]) / ranges[axis]
    if min_offset == None or offset < min_offset:
      min_offset = offset
      closest = p
  return closest

# -----------------------------------------------------------------------------
# 
def resonance_offset_string(frequency, resonances):
  offset = ''
  for a in range(len(frequency)):
    r = resonances[a]
    if r:
      offset = offset + ' %7.4f' % (frequency[a] - r.frequency)
    else:
      offset = offset + ' %7s' % ''
  return offset

# -----------------------------------------------------------------------------
#
def preferred_view(spectrum):

  session = spectrum.session
  v = session.selected_view()
  if v and v.spectrum == spectrum and v.is_top_level_window:
    return v
  vlist = session.project.view_list()
  vlist = filter(lambda v, s=spectrum:
                 v.spectrum == s and v.is_top_level_window,
                 vlist)
  for v in vlist:
    if v.is_shown:
      return v
  if vlist:
    return vlist[0]
  return None

# -----------------------------------------------------------------------------
#
def hydrogen_atoms(molecule):
  atoms = molecule.atom_list()
  protons = []
  for atom in atoms:
    if atom.name[0] in 'HMQ':
      protons.append(atom)
  return protons

# -----------------------------------------------------------------------------
#
def inter_residue_peaks(peaks, axis1, axis2):

  inter = []
  for peak in peaks:
    r = peak.resonances()
    r1 = r[axis1]
    r2 = r[axis2]
    if r1 and r2:
      n1 = r1.group.number
      n2 = r2.group.number
      if n1 != None and n2 != None and n1 != n2:
	inter.append(peak)
  return inter

# ----------------------------------------------------------------------------
#
class value_menu(mstkutil.option_menu):

	def __init__(self,parent,label,allow_no_choice=0):
		self.allow_no_choice=allow_no_choice
		mstkutil.option_menu.__init__(self,parent,label,('Fit height ','Data height','Volume     '),'Fit height ')

	# --------------------------------------------------------------------------
	#
	def get_value(self):
		entry=self.get()
		if entry=='Fit height ':
			value=0
		elif entry=='Data height':
			value=1
		else:
			value=2

		return value

	# --------------------------------------------------------------------------
	#
	def set_value(self,value):
		entry='Fit height '
			
		if value==1:
			entry='Data height'
		elif value==2:
			entry='Volume     '

		entry=self.set(entry)

# ----------------------------------------------------------------------------
#
class charge_menu(mstkutil.option_menu):

	def __init__(self,parent,label,allow_no_choice=0):
		self.allow_no_choice=allow_no_choice
		mstkutil.option_menu.__init__(self,parent,label,('+1','+2','+3','+4','+5','+6'),'+3')

	# --------------------------------------------------------------------------
	#
	def get_value(self):
		entry=self.get()
		if entry=='+1':
			value=1
		elif entry=='+2':
			value=2
		elif entry=='+3':
			value=3
		elif entry=='+4':
			value=4
		elif entry=='+5':
			value=5
		elif entry=='+6':
			value=6

		return value

	# --------------------------------------------------------------------------
	#
	def set_value(self,value):
		entry='+1'
			
		if value==2:
			entry='+2'
		elif value==3:
			entry='+3'
		elif value==4:
			entry='+4'
		elif value==5:
			entry='+5'
		elif value==6:
			entry='+6'
		entry=self.set(entry)

	# --------------------------------------------------------------------------
	#
	def get_min_charge(self):
		return(1)

	# --------------------------------------------------------------------------
	#
	def get_max_charge(self):
		return(6)

# ----------------------------------------------------------------------------
# Create an option menu listing spectra in the current project.
#
class spectrum_menu(mstkutil.option_menu):

	def __init__(self, session, parent, label, allow_no_choice = 0):

		self.session = session
		self.allow_no_choice = allow_no_choice
		self.names = self.spectrum_names()
		mstkutil.option_menu.__init__(self,parent,label,self.names,self.default_spectrum_name())
		self.menu['postcommand']=self.update_menu_cb

	# --------------------------------------------------------------------------
	#
	def spectrum_names(self):

		spectra=self.session.project.spectrum_list()
		names=pyutil.attribute_values(spectra, 'name')
		if self.allow_no_choice:
			names=('',) + names
		return names

	# --------------------------------------------------------------------------
	#
	def default_spectrum_name(self):

		return pyutil.attribute(self.session.selected_spectrum(), 'name', '')

	# --------------------------------------------------------------------------
	#
	def update_menu_cb(self):

		current_names = self.spectrum_names()
		if current_names != self.names:
			self.names = current_names
			self.remove_all_entries()
			for name in self.names:
				self.add_entry(name)
			if not self.get() in self.names:
				self.set(self.default_spectrum_name())

	# --------------------------------------------------------------------------
	#
	def spectrum(self):

		return name_to_spectrum(self.get(), self.session)

# ----------------------------------------------------------------------------
# Create an option menu listing views in the current project.
#
class view_menu(mstkutil.option_menu):

	def __init__(self,session, parent, label, dialog):

		self.session = session
		self.names = self.view_names()
		self.dialog = dialog
		mstkutil.option_menu.__init__(self, parent, label, self.names, self.default_view_name())
		self.menu['postcommand'] = self.update_menu_cb
		self.add_callback(self.menu_cb)

	# --------------------------------------------------------------------------
	#
	def view_names(self):
		views = self.session.project.view_list()
		return pyutil.attribute_values(views, 'name')

	# --------------------------------------------------------------------------
	#
	def default_view_name(self):
		return pyutil.attribute(self.session.selected_view(), 'name', '')

	# --------------------------------------------------------------------------
	#
	def update_menu_cb(self):
		current_names = self.view_names()
		if current_names != self.names:
			self.names = current_names
			self.remove_all_entries()
			for name in self.names:
				self.add_entry(name)
			if not self.get() in self.names:
				self.set(self.default_view_name())

	# --------------------------------------------------------------------------
	#
	def menu_cb(self,variable):
		if variable!=None:
			view=name_to_view(variable,self.session)
			if view!=None:
				self.dialog.min_edit.variable.set(pyutil.number_string('%5.2e',view.positive_levels.lowest))
				levels=view.positive_levels.levels
				if levels<10:
					levels=10
				self.dialog.levels_edit.variable.set(pyutil.number_string('%u',levels))
				self.dialog.color_menu.set(view.positive_levels.color)

	# --------------------------------------------------------------------------
	#
	def view(self):
		return name_to_view(self.get(),self.session)

# ----------------------------------------------------------------------------
#
def name_to_view(name, session):
	vlist = session.project.view_list()
	for v in vlist:
		if v.name == name:
			return v
	return None

# ----------------------------------------------------------------------------
# Used for color coding spectra.  Provides the same colors for spectra to
# all extensions that want to color code (eg spin graphs and strip plots).
#
def spectrum_color(spectrum):

  if not hasattr(spectrum, 'key_color'):

    spectrum_colors = ('green', 'blue', 'red', 'cyan',
                       'magenta', 'yellow', 'orange',
                       'brown', 'gold', 'pink', 'purple')

    session = spectrum.session
    if not hasattr(session, 'next_spectrum_color'):
      session.next_spectrum_color = 0

    spectrum.key_color = spectrum_colors[session.next_spectrum_color]
    ncolor = len(spectrum_colors)
    session.next_spectrum_color = (session.next_spectrum_color + 1) % ncolor
    
  return spectrum.key_color

# ----------------------------------------------------------------------------
# Make a table of spectra, one spectrum per row.
# Callback routines create the widgets for each row.
#
class spectrum_table:

  def __init__(self, session, parent, headings,
               add_spectrum_cb, remove_spectrum_cb):

    f = Tkinter.Frame(parent, borderwidth = 5)
    self.frame = f

    for h in range(len(headings)):
      lbl = Tkinter.Label(f, text = headings[h])
      lbl.grid(row = 0, column = h)

    if headings:
      self.next_row = 1
    else:
      self.next_row = 0

    self.add_spectrum_cb = add_spectrum_cb
    self.remove_spectrum_cb = remove_spectrum_cb

    self.session = session
    
    rn = session.notify_me('removed spectrum from project',
                           self.remove_spectrum)
    an = session.notify_me('added spectrum to project', self.add_spectrum)
    self.notices = (rn, an)
    f.bind('<Destroy>', self.window_destroyed_cb, 1)
    
  # --------------------------------------------------------------------------
  #
  def add_spectrum(self, spectrum):

    row = self.next_row
    self.next_row = row + 1
    self.add_spectrum_cb(spectrum, self, row)
    
  # --------------------------------------------------------------------------
  #
  def remove_spectrum(self, spectrum):

    self.remove_spectrum_cb(spectrum, self)

  # ---------------------------------------------------------------------------
  #
  def window_destroyed_cb(self, event):

    if str(event.widget) == str(self.frame):
      if sparky.object_exists(self.session):
        for notice in self.notices:
          self.session.dont_notify_me(notice)
      self.notices = ()

# ----------------------------------------------------------------------------
# A single column of spectrum checkbuttons.
#
class spectrum_checkbuttons(spectrum_table):
  
  def __init__(self, session, parent, heading):

    if heading:
      headings = (heading,)
    else:
      headings = ()

    spectrum_table.__init__(self, session, parent, headings,
                            self.add_spect, self.remove_spect)
    self.spectrum_to_checkbutton = {}
    self.chosen_spectra = []

    spectra = session.project.spectrum_list()
    for spectrum in spectra:
      self.add_spectrum(spectrum)

  # ---------------------------------------------------------------------------
  #
  def add_spect(self, spectrum, table, row):
    
    #
    # Make spectrum checkbutton
    #
    cb = mstkutil.checkbutton(table.frame, spectrum.name, 0)
    cb.button['selectcolor'] = spectrum_color(spectrum)
    choose_cb = pyutil.precompose(choose_spectrum_cb, spectrum,
                                  table.chosen_spectra)
    cb.add_callback(choose_cb)
    cb.button.grid(row = row, column = 0, sticky = 'w')
    table.spectrum_to_checkbutton[spectrum] = cb

  # ---------------------------------------------------------------------------
  #
  def remove_spect(self, spectrum, table):

    if not table.spectrum_to_checkbutton.has_key(spectrum):
      return
    
    cb = table.spectrum_to_checkbutton[spectrum]
    cb.set_state(0)
    cb.button.destroy()
    del table.spectrum_to_checkbutton[spectrum]

# -----------------------------------------------------------------------------
#
def choose_spectrum_cb(name, names, chosen):

  if chosen:
    if not name in names:
      names.append(name)
  else:
    if name in names:
      names.remove(name)

# ----------------------------------------------------------------------------
# Create an option menu listing conditions.
#
class condition_menu(mstkutil.option_menu):

  def __init__(self, session, parent, label):

    self.session = session
    self.names = self.condition_names()
    mstkutil.option_menu.__init__(self, parent, label, self.names,
                                self.selected_condition_name())
    self.menu['postcommand'] = self.update_menu_cb

  # --------------------------------------------------------------------------
  #
  def update_menu_cb(self):

    current_names = self.condition_names()
    if current_names != self.names:
      self.names = current_names
      self.remove_all_entries()
      for name in self.names:
        self.add_entry(name)
      if not self.get() in self.names:
        self.set(self.selected_condition_name())

  # --------------------------------------------------------------------------
  #
  def selected_condition_name(self):

    return condition_full_name(selected_condition(self.session))

  # --------------------------------------------------------------------------
  #
  def condition_names(self):

    return map(condition_full_name, self.session.project.condition_list())

  # --------------------------------------------------------------------------
  #
  def condition(self):

    return name_to_condition(self.get(), self.session)
  
# ----------------------------------------------------------------------------
#
def name_to_condition(name, session):

  clist = session.project.condition_list()
  for c in clist:
    if condition_full_name(c) == name:
      return c
  return None

# ----------------------------------------------------------------------------
#
def condition_full_name(c):
  if c and (c.name or c.molecule.name):
    return c.molecule.name + ' / ' + c.name
  return ''

# ---------------------------------------------------------------------------
#
class peak_tablelist(mstkutil.scrolling_tablelist):

	# ---------------------------------------------------------------------------
	#
	def __init__(self,parent,columns,height=0,width=0,selected_edit=1,editstartcommand=None,editendcommand=None):
		mstkutil.scrolling_tablelist.__init__(self,parent,columns,height,width,selected_edit,editstartcommand,editendcommand)
		
		self.lock_callbacks=0
		self.selected_peak=None

		self.peaks_count=0

	# ---------------------------------------------------------------------------
	#
	def set_list_size(self,peaks=-1):
		if peaks!=-1:
			self.peaks_count=peaks

		self.list_size.configure(text=('%d peaks' % (self.peaks_count)))


	# ---------------------------------------------------------------------------
	#
	def set_sequence(self,peak):
		if self.selected_edit!=None and peak!=None:
			if peak!=None and (hasattr(peak,'note') or hasattr(peak,'label')):
				self.selected_edit.variable.set(get_sequence(peak))
			else:
				self.selected_edit.variable.set('')

	# ---------------------------------------------------------------------------
	#
	def select_peak_cb(self,event):
		if self.lock_callbacks==0:
			peak=self.event_line_data(event)
			if peak!=None:
				self.selected_peak=peak
				self.set_sequence(peak)
				select_peak(peak)
			self.tablelist.selection_own()

	# ---------------------------------------------------------------------------
	#
	def goto_peak_cb(self,event):
		if self.lock_callbacks==0:
			peak=self.event_line_data(event)
			if peak!=None:
				self.selected_peak=peak
				show_peak(peak)
			self.tablelist.selection_own()

	# ---------------------------------------------------------------------------
	#
	def mark_peak_cb(self,peak):
		if self.lock_callbacks==0 and peak!=None and peak!=self.selected_peak:
			for index in range(len(self.line_data)):
				if peak==self.line_data[index]:
					self.tablelist.select_clear(0,'end')
					self.tablelist.selection_set(index)
					self.tablelist.activate(index)
					self.tablelist.see(index)

					self.selected_peak=peak
					if self.tablelist.rowcget(index,'-hide')==0:
						self.set_sequence(peak)
					else:
						self.set_sequence(None)
					break
			self.tablelist.selection_own()

	# ---------------------------------------------------------------------------
	#
	def find_peak_cb(self,peak):
		line_index=-1
		if self.lock_callbacks==0 and peak!=None:
			for index in range(len(self.line_data)):
				if self.line_data[index]==peak:
					line_index=index
					break
		return line_index

# ---------------------------------------------------------------------------
#
class peaks_tablelist(mstkutil.scrolling_tablelist):

	# ---------------------------------------------------------------------------
	#
	def __init__(self,parent,columns,height=0,width=0,selected_edit=1,editstartcommand=None,editendcommand=None):
		mstkutil.scrolling_tablelist.__init__(self,parent,columns,height,width,selected_edit,editstartcommand,editendcommand)
		
		self.lock_callbacks=0

	# ---------------------------------------------------------------------------
	#
	def set_sequence(self,peak):
		if self.selected_edit!=None:
			if peak!=None and (hasattr(peak,'note') or hasattr(peak,'label')):
				self.selected_edit.variable.set(get_sequence(peak))
			else:
				self.selected_edit.variable.set('')

	# ---------------------------------------------------------------------------
	#
	def mark_peak_cb(self,peak):
		if self.select_line_data(peak)!=0:
			self.set_sequence(peak)
			self.tablelist.selection_own()

	# ---------------------------------------------------------------------------
	#
	def select_peaks_cb(self,event):
		if self.lock_callbacks==0:
			data=self.event_line_data(event)
			if data!=None and data[0]!=None:
				self.set_sequence(data[0])
				if data[1]!=None:
					select_peak(data[1])
				select_peak(data[0])
			self.tablelist.selection_own()

	# ---------------------------------------------------------------------------
	#
	def goto_peaks_cb(self,event):
		if self.lock_callbacks==0:
			data=self.event_line_data(event)
			if data!=None and data[0]!=None:
				if data[1]!=None:
					show_peak(data[1])
				show_peak(data[0])
			self.tablelist.selection_own()

	# --------------------------------------------------------------------------
	#
	def select_line_data(self,peak):
		result=0
		if peak!=None:
			for index in range(len(self.line_data)):
				data=self.line_data[index]
				if data[0]==peak or data[1]==peak:
					if self.tablelist.rowcget(index,'-hide')==0:
						self.tablelist.select_clear(0,'end')
						self.tablelist.selection_set(index)
						self.tablelist.activate(index)
						self.tablelist.see(index)
						result=1
					break
		return result

# ---------------------------------------------------------------------------
#
class peptide_tablelist(mstkutil.scrolling_tablelist):

	# ---------------------------------------------------------------------------
	#
	def __init__(self,parent,columns,selected_edit=1,editstartcommand=None,editendcommand=None):
		mstkutil.scrolling_tablelist.__init__(self,parent,columns,20,170,selected_edit,editstartcommand,editendcommand)
		
		self.lock_callbacks=0
		self.selected_peak=None

		self.queries_count=0
		self.red_flag_count=0
		self.present_count=0

	# ---------------------------------------------------------------------------
	#
	def set_list_size(self,queries=-1,red_flags=-1,present=-1):
		if queries!=-1:
			self.queries_count=queries
		if red_flags!=-1:
			self.red_flag_count=red_flags
		if present!=-1:
			self.present_count=present

		self.list_size.configure(text=('%d PSMs (%d marked, %d present)' % (self.queries_count,self.red_flag_count,self.present_count)))

	# ---------------------------------------------------------------------------
	#
	def set_short_list_size(self,queries=-1,present=-1):
		if queries!=-1:
			self.queries_count=queries
		if present!=-1:
			self.present_count=present

		self.list_size.configure(text=('%d PSMs (%d present)' % (self.queries_count,self.present_count)))

	# ---------------------------------------------------------------------------
	#
	def select_peptide_cb(self,event):
		if self.lock_callbacks==0:
			data=self.event_line_data(event)
			if data!=None:
				peptide=data[0]
				query_nr=data[1]
				self.selected_peak=peptide.get(query_nr).get(0)
				self.set_sequence(peptide.sequence)
				select_peak(self.selected_peak)
			self.tablelist.selection_own()

	# ---------------------------------------------------------------------------
	#
	def goto_peptide_cb(self,event):
		if self.lock_callbacks==0:
			data=self.event_line_data(event)
			if data!=None:
				peptide=data[0]
				query_nr=data[1]
				self.selected_peak=peptide.get(query_nr).get(0)
				show_peak(self.selected_peak)
			self.tablelist.selection_own()


	# ---------------------------------------------------------------------------
	#
	def mark_peak_cb(self,peak):
		if self.lock_callbacks==0 and peak!=None and peak!=self.selected_peak:
			for index in range(len(self.line_data)):
				data=self.line_data[index]
				query=data[0].get(data[1])
				for i in range(query.size()):
					if query.get(i)==peak:
						self.tablelist.select_clear(0,'end')

						index=self.translate_data_line_index(index)
						self.tablelist.selection_set(index)
						self.tablelist.activate(index)
						self.tablelist.see(index)

						self.selected_peak=peak
						if self.tablelist.rowcget(index,'-hide')==0:
							self.set_sequence(data[0].sequence)
						else:
							self.set_sequence('')
						break
			self.tablelist.selection_own()

	# ---------------------------------------------------------------------------
	#
	def find_peak_cb(self,peak):
		line_index=-1
		if self.lock_callbacks==0 and peak!=None:
			for index in range(len(self.line_data)):
				data=self.line_data[index]
				query=data[0].get(data[1])
				for i in range(query.size()):
#					if query.get(i)==peak and self.tablelist.rowcget(index,'-hide')==0:
					if query.get(i)==peak:
						line_index=self.translate_data_line_index(index)
						break
		return line_index

	# ---------------------------------------------------------------------------
	#
	def set_sequence(self,sequence):
		if self.selected_edit!=None:
			self.selected_edit.variable.set(sequence)

# ---------------------------------------------------------------------------
#
class protein_tablelist(mstkutil.scrolling_tablelist):

	# ---------------------------------------------------------------------------
	#
	def __init__(self,parent,columns,selected_edit=1,editstartcommand=None,editendcommand=None):
		mstkutil.scrolling_tablelist.__init__(self,parent,columns,10,60,selected_edit,editstartcommand,editendcommand)
		
		self.lock_callbacks=0

	# ---------------------------------------------------------------------------
	#
#	def set_list_size(self,queries=-1,red_flags=-1,present=-1):
#		if queries!=-1:
#			self.queries_count=queries
#		if red_flags!=-1:
#			self.red_flag_count=red_flags
#		if present!=-1:
#			self.present_count=present
#
#		self.list_size.configure(text=('%d queries (%d marked, %d present)' % (self.queries_count,self.red_flag_count,self.present_count)))

	# ---------------------------------------------------------------------------
	#
#	def select_peptide_cb(self,event):
#		if self.lock_callbacks==0:
#			data=self.event_line_data(event)
#			if data!=None:
#				peptide=data[0]
#				query_nr=data[1]
#				self.selected_peak=peptide.get(query_nr).get(0)
#				self.set_sequence(peptide.sequence)
#				select_peak(self.selected_peak)
#			self.tablelist.selection_own()

	# ---------------------------------------------------------------------------
	#
#	def get_peptide_cb(self,event):
#		if self.lock_callbacks==0:
#			data=self.event_line_data(event)
#			if data!=None:
#				peptide=data[0]
#				query_nr=data[1]
#				self.selected_peak=peptide.get(query_nr).get(0)
#				show_peak(self.selected_peak)
#			self.tablelist.selection_own()

	# ---------------------------------------------------------------------------
	#
	def mark_protein_cb(self,protein):
		if self.lock_callbacks==0 and protein:
			for index in range(len(self.line_data)):
				if protein==self.line_data[index]:
					self.tablelist.select_clear(0,'end')
					self.tablelist.selection_set(index)
					self.tablelist.activate(index)
					self.tablelist.see(index)
					break
			self.tablelist.selection_own()

	# ---------------------------------------------------------------------------
	#
#	def set_sequence(self,sequence):
#		if self.selected_edit!=None:
#			self.selected_edit.variable.set(sequence)

# ---------------------------------------------------------------------------
#
class peak_listbox(mstkutil.scrolling_list):

	def __init__(self,parent,selected_field=1):

		mstkutil.scrolling_list.__init__(self, parent, '',15,40,selected_field)

	# ---------------------------------------------------------------------------
	#
	def set_sequence(self,peak):
		if self.selected_edit!=None:
			if peak!=None and (hasattr(peak,'note') or hasattr(peak,'label')):
				self.selected_edit.variable.set(get_sequence(peak))
			else:
				self.selected_edit.variable.set('')

	# ---------------------------------------------------------------------------
	#
	def select_peak_cb(self,event):
		peak=self.event_line_data(event)
		if peak!=None:
			self.set_sequence(peak)
			select_peak(peak)
			self.listbox.focus_set()

	# ---------------------------------------------------------------------------
	#
	def mark_peak_cb(self,peak):
		if self.select_line_data(peak)!=0:
			self.set_sequence(peak)

	# ---------------------------------------------------------------------------
	#
	def return_cb(self, event):
		selection=self.selected_line_numbers()
		if len(selection)==0:
			index=-1
		else:
			index=selection[0]
		if index!=-1:
			peak=self.line_data[index]
			self.set_sequence(peak)
			select_peak(peak)
			show_peak(peak)

	# ---------------------------------------------------------------------------
	#
	def goto_peak_cb(self,event):
		show_peak(self.event_peak(event))
#		self.listbox.focus_set()

	# ---------------------------------------------------------------------------
	#
	def event_peak(self, event):
		return self.event_line_data(event)


# ---------------------------------------------------------------------------
#
class assignment_listbox(mstkutil.scrolling_list):

  def __init__(self, session, parent):

    self.session = session
    mstkutil.scrolling_list.__init__(self, parent, '', 5)

  # ---------------------------------------------------------------------------
  #
  def select_assignment_cb(self, event):
    a = self.event_assignment(event)
    s = self.session.selected_spectrum()
    if a and s:
      select_peak(s.find_peak(a))

  # ---------------------------------------------------------------------------
  #
  def goto_assignment_cb(self, event):
    a = self.event_assignment(event)
    show_assignment(a, self.session.selected_spectrum())

  # ---------------------------------------------------------------------------
  #
  def event_assignment(self, event):
    return self.event_line_data(event)

# -----------------------------------------------------------------------------
#
def cluster_peaks(peaks, axis1, range1, axis2, range2):

  c = pyutil.cluster((range1, range2))
  for peak in peaks:
    p = (peak.frequency[axis1], peak.frequency[axis2])
    c.add_point(p, peak)

  return c.clusters()

# -----------------------------------------------------------------------------
# Currently doesn't handle function key events.
#
def command_keypress_cb(session, event):

  session.command_characters(event.char)

# -----------------------------------------------------------------------------
#
def the_dialog(dialog_class, session):

  if not hasattr(session, 'dialogs'):
    session.dialogs = {}
  dialog_table = session.dialogs
    
  if (not dialog_table.has_key(dialog_class) or
      dialog_table[dialog_class].is_window_destroyed()):
    dialog_table[dialog_class] = dialog_class(session)

  return dialog_table[dialog_class]
  
# -----------------------------------------------------------------------------
# Return a function to be used as a help button callback.
#
def help_cb(session, extension_topic):

  manual_url = 'extensions.html#' + extension_topic
  def show_url(url = manual_url, session = session):
    session.show_manual_url(url)
  return show_url
