# -----------------------------------------------------------------------------
# Sparky site initialization file.  The initialize_session routine in this
# file is invoked when a Sparky session is started.  We add menu entries
# for standard extensions.
#

import sys
import os
import sparky
import Tkinter

# -----------------------------------------------------------------------------
#
def initialize_session(session):

	if sys.platform=='win32':
		tk = Tkinter.Tk()
		tk.call('wm', 'iconbitmap', tk._w, '-default',os.path.join(os.environ.get('SPARKY_INSTALL'),os.path.join('bin','msparky.ico')))
		tk.destroy()

	sme = standard_menu_entries()
	add_menu_entries(session, sme)

# -----------------------------------------------------------------------------
#
def standard_menu_entries():

  integration_menu = (
    ('ip', 'MS integrate peaks',         ('msintegrate','show_integration_dialog')),
    )

  integrate_query_menu = (
    ('i1', 'x1',         ('mssputil','integrate_query',1)),
    ('i2', 'x2',         ('mssputil','integrate_query',2)),
    ('i3', 'x3',         ('mssputil','integrate_query',3)),
    ('i4', 'x4',         ('mssputil','integrate_query',4)),
    ('i5', 'x5',         ('mssputil','integrate_query',5)),
    ('i6', 'x6',         ('mssputil','integrate_query',6)),
    ('i7', 'x7',         ('mssputil','integrate_query',7)),
    ('i8', 'x8',         ('mssputil','integrate_query',8)),
    ('i9', 'x9',         ('mssputil','integrate_query',9)),
    ('ii', 'x10',         ('mssputil','integrate_query',10)),
    )

  experiment_menu = (
    ('ee', 'Experiment',		('msexperiment','show_experiment_dialog')),
    )

  peak_menu = (
    ('LT', 'Peak list',		('mspeaklist','show_spectrum_peaks')),
#    ('rp', 'MS read peak list',		('msreadpeaks','read_peak_list')),
    )

  peptide_menu = (
    ('PL', 'Peptides list',		('mspeptideslist','show_peptides_list')),
    ('pl', 'Peptides list',		('mspeptideslist2','show_peptides_list')),
#    ('ll', 'MS labelled list',		('mslabelledlist','show_labelled_list')),
    ('pp', 'Center peptide',		('mssputil','center_peptide')),
    ('ch', 'Chromatogram',		('mschromatogram_gui','show_chromatogram')),
#    ('ex', 'Export peptide',		('msexport','export_peptide')),
    ('ie', 'Show isotopic envelope',	('msisoenvelope','show_fit',True)),
#    ('if', 'MS show fit',		('msfit','show_fit',False)),
    ('ff', 'Find peptide',		('msfindpeptides_gui','show_find_selected_peptide_dialog')),
    ('fp', 'Find peptides',		('msfindpeptides_gui','show_find_peptides_dialog')),
    ('fd', 'Find duplicated',	('msduplicated','show_duplicated_list')),
    )

  charge_menu = (
    ('c0', 'Find shifted charges',	('mscharges','show_duplicated_list')),
    ('c1', '+1',		        ('mssputil','show_charge', 1)),
    ('c2', '+2',			('mssputil','show_charge', 2)),
    ('c3', '+3',			('mssputil','show_charge', 3)),
    ('c4', '+4',			('mssputil','show_charge', 4)),
    ('c5', '+5',			('mssputil','show_charge', 5)),
    ('c6', '+6',			('mssputil','show_charge', 6)),
    )

  spectrum_menu = (
    ('ms', 'Spectrum properties',    ('msspectprop','show_spectrum_properties')),
#    ('lf', 'MS label format',		('mslabelformat','show_label_format')),
    ('ht', 'Scatter plot',		('msscatterplot','show_scatterplot')),

    ('al', 'Align spectrum',            ('align','show_shift_dialog')),
    ('fm', 'Open multiple files', 	('openspectra','show_file_dialog')),
    )

  view_menu = (
    ('cc', 'Contour Levels',     ('mscontourlevels','show_dialog')),
    ('cv', 'Center view setup',         ('centerview','center_view_setup')),
    )

  top_menu = (
    ('py', 'Python shell',              ('pythonshell','show_python_shell')),
    )
  info_menu = (
    ('ii', 'About MSparky',              ('msinfo','show_info_dialog')),
    )

  #
  # The menu list consists of pairs having a menu path and entry list.
  # The menu path describes where under the Extension menu the entries should
  # appear.  An empty menu path puts the entries in the Extension menu.  If
  # the menu path is 'Spectrum' the entries are put in a cascaded menu called
  # Spectrum.  A menu path of 'Spectrum/Fold' puts the entries in a Fold menu
  # cascaded under the spectrum menu.
  # 
  menus = (
    ('',                        top_menu),
#    ('Integration',             integration_menu),
#    ('Integration/Integrate query',             integrate_query_menu),
    ('Experiment',              experiment_menu),
    ('Peak',                    peak_menu),
    ('Peptide',                 peptide_menu),
    ('Peptide/Charge',       charge_menu),
    ('Spectrum',                spectrum_menu),
    ('View',                    view_menu),
    ('',                        info_menu),
    )

  return menus

# -----------------------------------------------------------------------------
#
def add_menu_entries(session, menu_list):

  for menu_path, entries in menu_list:
    for accelerator, menu_text, function_spec in entries:

      if menu_path:
        path = menu_path + '/' + menu_text
      else:
        path = menu_text

      def func(module_name = function_spec[0],function_name = function_spec[1],session = session,extra_args = function_spec[2:]):
	import pythonshell
        pythonshell.invoke_module_function(session, module_name, function_name,(session,) + extra_args)

      session.add_command(accelerator, path, func)

