#!/usr/bin/env python
#
# $Id: OptionColorButton.py,v 1.6 2001/11/03 11:05:22 doughellmann Exp $
#
# Copyright 2001 Doug Hellmann.
#
#
#                         All Rights Reserved
#
# Permission to use, copy, modify, and distribute this software and
# its documentation for any purpose and without fee is hereby
# granted, provided that the above copyright notice appear in all
# copies and that both that copyright notice and this permission
# notice appear in supporting documentation, and that the name of Doug
# Hellmann not be used in advertising or publicity pertaining to
# distribution of the software without specific, written prior
# permission.
#
# DOUG HELLMANN DISCLAIMS ALL WARRANTIES WITH REGARD TO THIS SOFTWARE,
# INCLUDING ALL IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS, IN
# NO EVENT SHALL DOUG HELLMANN BE LIABLE FOR ANY SPECIAL, INDIRECT OR
# CONSEQUENTIAL DAMAGES OR ANY DAMAGES WHATSOEVER RESULTING FROM LOSS
# OF USE, DATA OR PROFITS, WHETHER IN AN ACTION OF CONTRACT,
# NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT OF OR IN
# CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
#

"""Button which controls a color chooser dialog.

  This button type is intended to be used in a preferences dialog.
  The 'PrefsDialog' and 'TagAttributeChooserDialog' use this.

"""

__rcs_info__ = {
    #
    #  Creation Information
    #
    'module_name'  : '$RCSfile: OptionColorButton.py,v $',
    'rcs_id'       : '$Id: OptionColorButton.py,v 1.6 2001/11/03 11:05:22 doughellmann Exp $',
    'creator'      : 'Doug Hellmann <doug@hellfly.net>',
    'project'      : 'PmwContribD',
    'created'      : 'Sun, 01-Apr-2001 13:05:23 EDT',

    #
    #  Current Information
    #
    'author'       : '$Author: doughellmann $',
    'version'      : '$Revision: 1.6 $',
    'date'         : '$Date: 2001/11/03 11:05:22 $',
}

#
# Import system modules
#
import Pmw
from Tkinter import *
from tkColorChooser import askcolor


#
# Import Local modules
#


#
# Module
#


class OptionColorButton(Pmw.LabeledWidget):
    """Button to use in option dialog to let the user choose a color.

    Options

      'variable' -- The Tkinter StringVar instance which should be
      keep up to date with the option value.
      
    """
    
    def __init__(self, parent=None, **kw):
        optiondefs = (
            ('variable', None, Pmw.INITOPT),
            ('command', None, Pmw.INITOPT),
            )
        self.internal_variable = StringVar()
        self.internal_variable.trace('w', self.variable_trace)
        self.defineoptions(kw, optiondefs)
        Pmw.LabeledWidget.__init__(self, parent=parent)
        interior = self.interior()
        l = self.createcomponent(
            'label',
            (), None,
            Label,
            (interior,),
            text='    ',
            relief='raised',
            )
        l.pack(
            side=LEFT,
            fill=X,
            )
        w = self.createcomponent(
            'button',
            (), None,
            Button,
            (interior,),
            command=self.command,
            )
        w.pack(
            side=LEFT,
            expand=YES,
            fill=X,
            )
        w.grid_columnconfigure( 1, weight=1 )
        self.initialiseoptions(self.__class__)
        if self['variable']:
            self.internal_variable.set(self['variable'].get())
        return

    def variable_trace(self, *args):
        "Follow the variable changes as associated with user actions."
        #print 'tracing ', args
        current_val = self.internal_variable.get()
        self.configure(button_text=current_val,
                       label_background=current_val)
        if self['variable']:
            self['variable'].set(current_val)
        return
    
    def command(self, *args):
        "Callback called when user clicks on button."
        default = self.internal_variable.get()
        rgb, newColor = askcolor(default)
        #print 'newColor is ', newColor
        if newColor:
            self.internal_variable.set(str(newColor))
        if self['command']:
            self['command']()
        return
    
    def get(self):
        "Return the color value."
        return self.internal_variable.get()

    def set(self, newVal):
        "Set the color value."
        self.internal_variable.set(newVal)
        return
    
