#!/usr/bin/env python
#
# $Id: OptionFontButton.py,v 1.4 2001/11/03 11:05:22 doughellmann Exp $
#
# Copyright 2001 Doug Hellmann.
#
#
#                         All Rights Reserved
#
# Permission to use, copy, modify, and distribute this software and
# its documentation for any purpose and without fee is hereby
# granted, provided that the above copyright notice appear in all
# copies and that both that copyright notice and this permission
# notice appear in supporting documentation, and that the name of Doug
# Hellmann not be used in advertising or publicity pertaining to
# distribution of the software without specific, written prior
# permission.
#
# DOUG HELLMANN DISCLAIMS ALL WARRANTIES WITH REGARD TO THIS SOFTWARE,
# INCLUDING ALL IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS, IN
# NO EVENT SHALL DOUG HELLMANN BE LIABLE FOR ANY SPECIAL, INDIRECT OR
# CONSEQUENTIAL DAMAGES OR ANY DAMAGES WHATSOEVER RESULTING FROM LOSS
# OF USE, DATA OR PROFITS, WHETHER IN AN ACTION OF CONTRACT,
# NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT OF OR IN
# CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
#

"""Button which controls a font chooser dialog.

  This button type is intended to be used in preferencess dialog to
  bring up an interface for editing a font specification.  The
  'PrefsDialog' and 'TagAttributeChooserDialog' use this.

"""

__rcs_info__ = {
    #
    #  Creation Information
    #
    'module_name'  : '$RCSfile: OptionFontButton.py,v $',
    'rcs_id'       : '$Id: OptionFontButton.py,v 1.4 2001/11/03 11:05:22 doughellmann Exp $',
    'creator'      : 'Doug Hellmann <doug@hellfly.net>',
    'project'      : 'PmwContribD',
    'created'      : 'Sat, 05-May-2001 11:34:05 EDT',

    #
    #  Current Information
    #
    'author'       : '$Author: doughellmann $',
    'version'      : '$Revision: 1.4 $',
    'date'         : '$Date: 2001/11/03 11:05:22 $',
}

#
# Import system modules
#
import Pmw
from Tkinter import *
import string


#
# Import Local modules
#
from TagAttributeChooserDialog import TagAttributeChooserDialog


#
# Module
#


class OptionFontButton(Pmw.LabeledWidget):
    """Button which controls a font chooser dialog.

    This button type is intended to be used in preferencess dialog to
    bring up an interface for editing a font specification.  The
    'PrefsDialog' and 'TagAttributeChooserDialog' use this.

    Options

      'command' -- Callback to be executed when value changes.

      'variable' -- Tkinter StringVar instance to be kept up to date
      based on the value of the font specification.

    """

    def __init__(self, parent=None, **kw):
        optiondefs = (
            ('command', None, Pmw.INITOPT),
            ('variable', None, Pmw.INITOPT),
            ('title', 'Click to set', Pmw.INITOPT),
            )
        self.defineoptions(kw, optiondefs)
        Pmw.LabeledWidget.__init__(self, parent)
        interior = self.interior()
        b = self.createcomponent(
            'button', (), None,
            Button, (interior,),
            text=self['title'],
            command=self.command,
            relief=FLAT,
            )
        b.pack(side=LEFT, fill=X)
        self.dlg = TagAttributeChooserDialog(self.interior())
        self.dlg.withdraw()
        if self['variable']:
            self['variable'].trace('w', self.variable_trace)
        self.initialiseoptions(OptionFontButton)
        self.variable_trace()
        return

    def variable_trace(self, *args):
        "Follow the changes to the variable."
        
        #print 'tracing ', args
        var = self['variable']
        current_val = var.get()
        #print 'current val="%s"' % str(current_val)
        if not current_val:
            return
        
        spec = eval(current_val)

        try:
            fontspec, tagspec = spec
        except ValueError:
            fontspec = spec
            tagspec = {}
        #print '\tfontspec=', fontspec
        #print '\ttagspec=', tagspec
        
        self.configure(button_font=fontspec)
        apply(self.component('button').configure, (), tagspec)
        
        dlg_params = {}
        dlg_params.update(tagspec)
        try:
            dlg_params['family'] = fontspec[0]
            dlg_params['size'] = fontspec[1]
        except IndexError:
            # tried for too many items
            pass
        
        for item in fontspec[2:]:
            dlg_params[string.lower(item)] = 1
        #print '\tdlg_params=', dlg_params
        if dlg_params.has_key('font'):
            del dlg_params['font']
            
        apply(self.dlg.configure, (), dlg_params)
        return
    
    def command(self, *args):
        "Callback for when button is pressed."
        #print 'button pressed'
        #print '\t', self['variable'].get()
        result = self.dlg.activate()
        if not result:
            return
        #fontspec, tagspec = result
        self['variable'].set(str(result))
        return
        
