#!/usr/bin/env python
#
# $Id: OptionFontSelector.py,v 1.4 2001/11/03 11:05:22 doughellmann Exp $
#
# Copyright 2001 Doug Hellmann.
#
#
#                         All Rights Reserved
#
# Permission to use, copy, modify, and distribute this software and
# its documentation for any purpose and without fee is hereby
# granted, provided that the above copyright notice appear in all
# copies and that both that copyright notice and this permission
# notice appear in supporting documentation, and that the name of Doug
# Hellmann not be used in advertising or publicity pertaining to
# distribution of the software without specific, written prior
# permission.
#
# DOUG HELLMANN DISCLAIMS ALL WARRANTIES WITH REGARD TO THIS SOFTWARE,
# INCLUDING ALL IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS, IN
# NO EVENT SHALL DOUG HELLMANN BE LIABLE FOR ANY SPECIAL, INDIRECT OR
# CONSEQUENTIAL DAMAGES OR ANY DAMAGES WHATSOEVER RESULTING FROM LOSS
# OF USE, DATA OR PROFITS, WHETHER IN AN ACTION OF CONTRACT,
# NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT OF OR IN
# CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
#

"""Font selector dialog.

"""

__rcs_info__ = {
    #
    #  Creation Information
    #
    'module_name'  : '$RCSfile: OptionFontSelector.py,v $',
    'rcs_id'       : '$Id: OptionFontSelector.py,v 1.4 2001/11/03 11:05:22 doughellmann Exp $',
    'creator'      : 'Doug Hellmann <doug@hellfly.net>',
    'project'      : 'PmwContribD',
    'created'      : 'Sun, 01-Apr-2001 13:08:28 EDT',

    #
    #  Current Information
    #
    'author'       : '$Author: doughellmann $',
    'version'      : '$Revision: 1.4 $',
    'date'         : '$Date: 2001/11/03 11:05:22 $',
}

#
# Import system modules
#
import Pmw
from Tkinter import *
from tkFont import *


#
# Import Local modules
#


#
# Module
#

class OptionFontSelector(Pmw.LabeledWidget):
    """Font selection dialog.

    Options

      'variable' -- Tkinter StringVar instance in which the font
      specification should be stored.
    
    """
    
    def __init__(self, parent=None, **kw):
        optiondefs = (
            ('variable', None, Pmw.INITOPT),
            )
        self.internal_variable = StringVar()
        self.internal_variable.trace('w', self.variable_trace)
        self.defineoptions(kw, optiondefs)
        Pmw.LabeledWidget.__init__(self, parent=parent)
        interior = self.interior()
        w = self.createcomponent(
            'button',
            (), None,
            Button,
            (interior,),
            command=self.command,
            )
        w.pack(
            side=LEFT,
            expand=YES,
            fill=X,
            )
        self.initialiseoptions(self.__class__)
        if self['variable']:
            self.internal_variable.set(self['variable'].get())
        return

    def getFont(self, str):
        "Does nothing."
        return
    
    def variable_trace(self, *args):
        "Trace variable changes."
        #print 'tracing ', args
        current_val = self.internal_variable.get()
        self.configure(button_text=current_val,
                       button_font=current_val)
        if self['variable']:
            self['variable'].set(current_val)
        return
    
    def command(self, *args):
        "Callback for when button is pressed."
        default = self.internal_variable.get()
        rgb, newColor = askcolor(default)
        #print 'newColor is ', newColor
        if newColor:
            self.internal_variable.set(str(newColor))
        return
    
    def get(self):
        "Return the font value."
        return self.internal_variable.get()

    def set(self):
        "Set the font value."
        self.internal_variable.set()
        return
    
