# ------------------------------------------------------------------
# Author: Khalid A. Bakr
# email: khabkr at yahoo dot com
# Copyright: Copyright (C) 2004-2006 Khalid A. Bakr. All rights reserved.
# Project: pymingw.py
# ------------------------------------------------------------------
import os

prog_name = 'pyMinGW tools'
prog_ver_no = '0.0.3'
prog_short_version = "%s %s" % (prog_name, prog_ver_no)
prog_py_version = "%s (May-04-2006) running in \nPython %s" % \
    (prog_short_version, os.sys.version)
prog_copyright = 'Copyright (C) 2004-2006 KAB. All rights reserved.'
prog_full_version = "%s %s" % (prog_short_version, prog_copyright)

prog_license = '''
This software is licensed as part of pyMinGW and under the same
conditions. Please refer to pyMinGW's license for details.


DISCLAIMER:
----------
THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. 
IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY 
CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, 
TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE 
SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE. 
'''
IGNORED = 'IGNORED'
#-----------------------------------------
# The first 2 functions are modified from 
# test_glob.py, and rmpyc.py respectively. 
# They are mainly used to help 
# cleanly uninstall Python.
#-----------------------------------------
def deltree(fname, at='cwd'):
    if at == 'cwd':
        fname = os.path.join(os.getcwd(), fname)
    elif at == 'parent':
        a = os.getcwd()
        a = a[:a.rfind('\\')+1]
        fname = os.path.join(a, fname)
        del a
    else: at = 'cwd'
    if not os.path.exists(fname):
        return
    for f in os.listdir(fname):
        fullname = os.path.join(fname, f)
        if os.path.isdir(fullname):
            deltree(fullname)
        else:
            try:
                os.unlink(fullname)
            except:
                pass
    try:
        os.rmdir(fname)
    except:
        pass
#---------------------------------
def rmpyc(root):
    from os.path import join

    npyc = npyo = 0
    for root, dirs, files in os.walk(root):
        for name in files:
            deleteit = False
            if name.endswith('.pyc'):
                deleteit = True
                npyc += 1
            elif name.endswith('.pyo'):
                deleteit = True
                npyo += 1

            if deleteit:
                os.remove(join(root, name))

    return npyc, npyo
#---------------------------------
def uninstall(id=None):
    """Clean after idle; then remove all .pyc and .pyo files in \Lib"""
    # A limited guard.
    if 'Inno Setup' or id == 'su':
        deltree(r'Idlerc~1', at='cwd')
        deltree(r'Idlerc~1', at='parent')
        a, b = rmpyc('.')

#-----------------------------------------
# Code to "change" the mode of the MinGW 
# compiled Python. Binaries, for example, 
# exe, pyd, dll, are here forced to "switch"
# their runtime library and use another.
#-----------------------------------------
safe_files = ['msvcr71.dll', 'msvcrt.dll']

runtimes = ['msvcrt.dll', 'msvcr71.dll', 'msvcr80.dll', 'msvcr81.dll']
pymodes = {}
pymodes['msvcrt'] = {'dll': 'msvcr71.dll', 'new_dll': 'msvcrt.dll'}
pymodes['msvcr71'] = {'dll': 'msvcrt.dll', 'new_dll': 'msvcr71.dll'}

pymode = 'msvcr71'
dll = pymodes[pymode]['dll']
_new_dll = pymodes[pymode]['new_dll']
#---------------------------------
def print_help(detailed=0): 
    pass
#---------------------------------
def version(Exit=1, v=prog_full_version):
    if v == prog_full_version:
    	print prog_full_version
    elif v == prog_short_version:
        print prog_short_version
    elif v == prog_py_version:
        print prog_py_version

    if Exit: os.sys.exit(1)
#---------------------------------
def copyright():
	version(Exit=0, v=prog_full_version)
	#print prog_copyright
	print prog_license
	os.sys.exit(1)
#---------------------------------
def examples():
	print """

    ------------------------------------
    Examples of how to call the program:
    ------------------------------------

    [1] python pymingw.py -V
    [2] python pymingw.py -v
    [3] python pymingw.py -help
    [4] python pymingw.py -copyright
    [5] python pymingw.py -file=python.exe -newmode=msvcr70 -debug
    [6] python pymingw.py -file=python24.dll -newmode=msvcrt -d
    [7] python pymingw.py -file=python23.dll -newmode=msvcr71
    [8] python pymingw.py -file=python25.dll -newmode=msvcr5000 -force
    [9] python pymingw.py -dir=C:\python24 -newmode=msvcr71
    """
	os.sys.exit(1)
#-----------------------
def print_help(detailed=0):

    help1 = """ 
    ------
    Usage:
    ------ 
    python pymingw.py [options]

    """


    help2 = """
    ------
    Usage:
    ------
    python pymingw.py [options]



    Available options:
    ------------------
    [-h|-help] [V] [-v|-version] [-copyright] [-file=infile] 
    [-newmode=mode] [-force] [-examples] [-d|-debug] [-dir=somedir]



    Detailed explaination:
    ----------------------
    -h:             Prints this screen; optional
    -help:          Same as -h above
    -V:             Prints program name and version number; optional
    -v:             Prints detailed program and version info, and
                    the Python version running the program; optional
    -version:       Same as -v above
    -copyright:     Shows copyright information and disclaimer
    -file=infile:   Sets the binary file to work on. If supplied,
                    the -dir=somedir switch is ignored.
    -newmode=mode:  Changes runtime of -file=infile to mode
    -force:         Forces -newmode=mode to be accepted even if mode 
                    seems invalid; optional 
    -examples:      Prints some usage examples
    -d:             Sets debug mode on; optional
    -debug:         Same as -d above
    -dir=somedir    Sets directory of files to change old runtimes in 
                    to -newmode=mode; optional; not implemented yet

	"""

    if not detailed: 
        os.sys.stderr.write(help1)
    elif detailed:
	    os.sys.stderr.write(help2)
    os.sys.exit(1)
#---------------------------------
def print_error(msg=''):
	print
	print 'Error: '+ msg
	print "Please refer to the usage help below and try again."
	print_help(1)
#---------------------------------
def get_options(args):

	# No arguments supplied
    len_args = len(args)
    if len_args <= 1:
        print_help()

    # Asking for help?
    if '-help' in args or '-h' in args:
        print_help(detailed=1)

    # Asking for version info?
    if '-V' in args:
        version(v=prog_short_version)

    # Asking for detailed version info?
    if '-v' in args or '-version' in args:
        version(v=prog_py_version)

    # Asking for copyright info?
    if '-copyright' in args:
        copyright()

    # Asking for examples?
    if '-examples' in args:
        examples()

    newmode = ''
    mode_error = 0
    file_to_fix = ''
    filename_error = 0
    dir_to_fix = ''
    dir_error = 0
    infile_supplied = 0

    for item in args:
        #print "Itemizing: ", item
        if item[0:9] == '-newmode=':
            newmode = item[9:] + '.dll'
            if newmode not in runtimes:
                if "-force" not in args:
                    mode_error = 1
                    print_error("Unrecognized mode '"+newmode+"'")
                

        elif item[0:6] == '-file=':
            file_to_fix = item[6:]
            if file_to_fix == '':
                filename_error = 1
                print_error("File name error '"+file_to_fix+"'")
            else:
                infile_supplied = 1
                dir_to_fix = IGNORED

        elif item[0:5] == '-dir=':
            if not infile_supplied:
                dir_to_fix = item[5:]
                if dir_to_fix == '':
                    dir_error = 1
                    print_error("Directory name error '"+dir_to_fix+"'")


    """
    print "Xx"*30
    print "newmode: %s\ndir_to_fix: %s\nfile_to_fix: %s\n" % \
        (newmode, dir_to_fix, file_to_fix)
    print "Xx"*30
    """
    
    # Asking for debugging? Set default to 0
    debugOn = 0
    if '-d' in args:
        # Debug on
        debugOn = 1

    if (mode_error or (filename_error and dir_error)):
        print "Error: error mode was set in newmode, dir(ectory), or infile."
        print_help(detailed=1)

    if (newmode == '' or (file_to_fix == '' and dir_to_fix == '')):
        print "Error: newmode, dir(ectory), or infile empty."
        print_help(detailed=1)

    if (file_to_fix != '' and (dir_to_fix != '' and dir_to_fix != IGNORED)):
        print "Error: problem in newmode, dir(ectory), or infile."
        #print file_to_fix, dir_to_fix
        print_help(detailed=1)

    return [newmode, dir_to_fix, file_to_fix, debugOn]
#---------------------------------
def dll_fix_name(this_dll=None):

    # Fix dll if passed just as mode: 'msvcrt',
    # i.e. suffix '.dll' is not passed.
    if this_dll != None:
        # We use upper case, because this is 
        # what seems to be used inside DLLs, 
        # example: SOMEDLL.dll
        if not this_dll.endswith('.dll'):
            this_dll = this_dll.upper() + '.dll'
        # .pyd files, or other DLLs?
        else:
            split_dll_name = this_dll.split('.')
            this_dll = split_dll_name[0].upper() + '.' + split_dll_name[1].lower()
    return this_dll
#---------------------------------
def get_lines(file_to_fix, debug=0):
    if debug:
        print "Opening file %s ..." % file_to_fix
    f = open(file_to_fix, 'rb')

    if debug:
        print "Reading file %s ..." % file_to_fix
    lines = f.readlines()

    if debug:
        print "Closing file %s ..." % file_to_fix
    f.close()
    return lines
#---------------------------------
def dll_on_line(dll_name, lines, debug=0):
    if debug: 
        print "Searching file %s ..." % dll_name
    i = 0
    line_no = -1
    for line in lines:
    	if dll_name.lower() in line.lower():
            line_no = i
            if debug:
                print "Found dll name at line %d" % i

            return line_no
    	i += 1
    if debug:
        if line_no == -1:
            print "Not found"
    return line_no
#---------------------------------
def runtime_used(dll_file, get_first=0, debug=0):
    if debug: 
        print
        print "Searching for runtime used in file %s ..." % dll_file

    lines = get_lines(dll_file, debug)
    i = 0
    found_dll = False
    i_locations = []
    found_one_already = False

    #for dll_name in runtimes:
    #    for line in lines:
    for line in lines:
        for dll_name in runtimes:
            if dll_name.lower() in line.lower():
                if debug:
                    print "Found a dll name at line %d" % i
        
                k = runtimes.index(dll_name)
                found_dll = True
                i_locations.append(k)
                break
    	    i += 1
        if found_dll:
            if found_one_already:
                found_one_already = True
                print "Trouble! Found another runtime!!!"
                if debug:                    
                    l = len(i_locations)
                    k = 0                    
                    while k < l:
                        if debug:
                            print "Found %s at %d" % (runtimes[k], i_locations[k])
                raise SystemExit
            else:
                if get_first:
                    break
                continue                
                
    if found_dll:
        if len(i_locations) == 1:
            runtime_found = runtimes[i_locations[0]]
            if debug:
                print "Found '%s' at line: %d" % (runtime_found, i)
            return runtime_found
    else:
        if debug:
            print "No runtime found!"
        return
#---------------------------------
def _fix_file(file_to_fix, new_dll, old_dll=None, debug=0):

    if old_dll != None:
        new_dll_namebackup = new_dll
        old_dll_namebackup = old_dll
        old_dll += chr(0)
        new_name = file_to_fix
    else:
        new_dll_namebackup = new_dll
        old_dll = file_to_fix
        old_dll_namebackup = old_dll
        old_dll += chr(0)
        new_name = new_dll_namebackup

    lines = get_lines(file_to_fix, debug)
    line_no = dll_on_line(old_dll, lines)

    if debug:
        print "_fix_file"
        print "File to fix: '%s', new_dll: '%s', old_dll: '%s', debug: %d" % (file_to_fix,
        new_dll, old_dll, debug)
        print "new_name:", new_name
        print "Found %s at: %d." % (old_dll, line_no)

    if line_no == -1:
        if debug:
            print "Not found"
        return

    new_dll += chr(0)

    fixed_file = file_to_fix + '.fix'

    if line_no == -1:
        #print "DLL name not (or no longer) in file."
        #raise SystemExit
        return

    if debug:
        print "Creating file %s" % fixed_file

    f = open(fixed_file, 'wb')
    
    i = 0
    if debug:
        print "Writing to file %s upto %d" % (fixed_file, line_no)

    while i < line_no:
     	linel = lines[i]
       	f.write(linel)
       	i += 1
    
    linel = lines[line_no]

    if debug:
        print "Searching line ..."
        print "%s in line? %d" % (old_dll, old_dll in linel)

    pos_dll_name = linel.lower().index(old_dll.lower())

    if debug:
        print "Found dll_name at char no: %d" % pos_dll_name
        print "proof: '%s'" % linel[pos_dll_name: pos_dll_name + len(old_dll)+2]
        print "Fixing line..."

    null_margin = len(new_dll_namebackup) - len(old_dll_namebackup)
    tempstr = linel[0 : pos_dll_name]
    tempstr += new_dll
    tempstr += linel[pos_dll_name + len(old_dll) + null_margin:]
    f.write(tempstr)
        
    len_lines = len(lines)
    i = line_no + 1
    
    if debug:
        print "Writing rest of %s file" % fixed_file

    while i < len_lines:
        linel = lines[i]
        i += 1
        f.write(linel)
    
    f.close()
    
    os.unlink(file_to_fix)
    os.rename(fixed_file, new_name.lower())
#---------------------------------
def change_id(old_id, new_id, debug=0):    
    old_id = dll_fix_name(old_id)
    new_id = dll_fix_name(new_id)
    _fix_file(old_id, new_id, debug=debug)
#---------------------------------
def fix_dll_file(file_to_fix, old_dll=None,
    new_dll=_new_dll, debug=0):

    if new_dll.lower() != old_dll.lower():
        if debug:
            print "Runtimes '%s' != '%s' and need fixing..." % (new_dll, old_dll)
        _fix_file(file_to_fix, new_dll, old_dll, debug=debug)
    else:
        if debug:
            print "Nothing else to do"
#---------------------------------
def fix_file(file_to_fix, new_dll=_new_dll, old_dll=None, 
    qdebug=1, debug=0):

    # Using default, i.e. check runtime ourselves?
    if old_dll == None:
        old_dll = runtime_used(file_to_fix, debug)

    # Couldn't find runtime?
    if old_dll == None:
        if debug:
            print
            print "Error. Unknown runtime in %s" % file_to_fix
            os.sys.exit(-1)

    # Fix dll names.
    new_dll = dll_fix_name(new_dll)
    old_dll = dll_fix_name(old_dll)
 
    if debug:
        print
        print "Current runtime dll: '%s'" % old_dll
        print "Runtime new_dll: '%s'" % new_dll

    # Make three passes to trap and fix
    # every occurance.
    if qdebug: print file_to_fix ,
    for i in xrange(3):
        fix_dll_file(file_to_fix, old_dll, new_dll, debug)
        if qdebug: os.sys.stdout.write('.')
    if debug: print
#---------------------------------
def fix_pyd_file(file_to_fix, new_dll='msvcr71',
    old_dll='msvcrt', qdebug=1, debug=0):

    fix_file(file_to_fix, new_dll, old_dll, qdebug, debug)
#---------------------------------
def fix_all(sdir='.', old_dll=None, new_dll=_new_dll, debug=0):
    import glob

    # Glob files
    pyds = glob.glob1(sdir, '*.pyd')
    exes = glob.glob1(sdir, '*.exe')
    dlls = glob.glob1(sdir, '*.dll')

    # Group them
    all_files = pyds + exes + dlls
    files_to_fix = []
    for ffile in all_files:
        fname = os.path.join(sdir, ffile)
        files_to_fix.append(fname)

    # Now fix them
    for ffile in files_to_fix:
        if ffile not in safe_files:
            fix_file(ffile, new_dll, old_dll, qdebug=1, debug=debug)
            print
#---------------------------------
if __name__ == '__main__':

    newmode, dir_to_fix, file_to_fix, debug = get_options(os.sys.argv)

    if debug:
        print "newmode: %s\ndir_to_fix: %s\nfile_to_fix: %s\ndebug: %d" % (newmode, 
            dir_to_fix, file_to_fix, debug)

    print "WARNING: script mode not yet complete and is buggy!"
    #os.sys.exit(-1)

    if dir_to_fix != IGNORED:
        fix_all(dir_to_fix, new_dll=newmode, debug=debug)
    else:
        fix_file(file_to_fix, new_dll=newmode, debug=debug)

    #print "Used runtime: %s " % runtime_used('libpython24.a')        
    #print "Don't know what to do. Exiting."

